﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lims.Entities
{
    [Alias(Configs.TableNameB2bLink)]
    [Api(Description = "电商客户关系表")]
    public class B2bLink: EntityWithMetadata, IEntityInAssemblyDb,
        IEntityHasCreator,IEntityHasUpdater
    {
        [ApiMember(Description = "所属客户Kvid")]
        [IgnoreUpdate]
        [Required]
        public Guid OwnerKvid { get; set; }

        [ApiMember(Description = "所属客户名称")]
        [Default("")]
        [StringLength(500)]
        public string OwnerName { get; set; }

        [ApiMember(Description = "关联类型")]
        [StringLength(100)]
        public LinkType LinkType { get; set; }

        [ApiMember(Description = "关联客户Kvid")]
        [IgnoreUpdate]
        [Required]
        public Guid LinkKvid { get; set; }

        [ApiMember(Description = "关联客户名称")]
        [Default("")]
        [StringLength(500)]
        public string LinkName { get; set; }


        [ApiMember(Description = "配置参数")]
        [StringLength(int.MaxValue)]//必须要配置StringLength,因为序列化的文本比较大
        //Default(""), 不能赋空值,在mySql中会报错
        public string Parameters { get; set; }

        [ApiMember(Description = "配置查询")]
        [StringLength(int.MaxValue)]//必须要配置StringLength,因为序列化的文本比较大
        public string Filters { get; set; }


        [ApiMember(Description = "备注")]
        [StringLength(2000)]
        public string Remark { get; set; }

        #region 创建/更新人V1.0
        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion
    }

    public enum LinkType
    {
        Unsupported = 0,// 不支持的类型

        /// <summary>
        /// 委托单位
        /// </summary>
        Demander,

        /// <summary>
        /// 生产单位
        /// </summary>
        Manufacturer,

        /// <summary>
        /// 付款单位
        /// </summary>
        Payer
    }
}
