﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lims.Entities
{
    /// <summary>
    /// Status约束：100（草稿状态），150（审批中），200（审批通过）
    /// </summary>
    [Alias(Configs.TableNameReportChange)]
    [Api(Description = "报告更改")]
    public class ReportChange: EntityWithMetadata
    {
        #region 第三方关联
        [ApiMember(Description = "业务Kvid")]
        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型,指实体类型全名称")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string BizType { get; set; }
        #endregion

        #region 关联信息
        /// <summary>
        /// 关联报告Kvid
        /// </summary>
        [ApiMember(Description = "报告Kvid")]
        [DefaultEmptyGuid]
        public Guid ReportKvid { get; set; }
        #endregion

        #region 原报告信息，冗余信息
        /// <summary>
        /// 报告编号
        /// </summary>
        [ApiMember(Description = "报告编号")]
        [StringLength(50), Default("")]
        public string ReportId { get; set; }

        /// <summary>
        /// 委托单位
        /// </summary>
        [ApiMember(Description = "需方Kvid")]
        [DefaultEmptyGuid]
        public Guid ReportDemanderKvid { get; set; }

        [ApiMember(Description = "需方名称,即检测单位")]
        [StringLength(100), Required]
        public string ReportDemanderName { get; set; }

        [DefaultEmptyGuid]
        public Guid ReportDemanderContactKvid { get; set; }

        [ApiMember(Description = "需方联系人")]
        [StringLength(100), Default("")]
        public string ReportDemanderContactName { get; set; }

        /// <summary>
        /// 生产单位
        /// </summary>
        [ApiMember(Description = "生产单位Kvid")]
        [DefaultEmptyGuid]
        public Guid ReportSampleManufacturerKvid { get; set; }

        [ApiMember(Description = "生产单位")]
        [StringLength(100), Default("")]
        public string ReportSampleManufacturerName { get; set; }

        /// <summary>
        /// 样品名称
        /// </summary>
        [ApiMember(Description = "样品名称")]
        [StringLength(200), Required]
        public string ReportSampleName { get; set; }

        /// <summary>
        /// 样品批次号
        /// </summary>
        [ApiMember(Description = "样品批次号")]
        [StringLength(200), Default("")]
        public string ReportSampleBatchNumber { get; set; }

        /// <summary>
        /// 检验依据
        /// </summary>
        [ApiMember(Description = "检验依据")]
        [StringLength(1000), Default("")]
        public string ReportTestStandards { get; set; }

        /// <summary>
        /// 原报告付款单位Kvid
        /// </summary>
        [DefaultEmptyGuid]
        public Guid ReportPayerKvid { get; set; }

        /// <summary>
        /// 原报告付款单位
        /// </summary>
        [ApiMember(Description = "付款单位,贷方")]
        [StringLength(100)]
        public string ReportPayerName { get; set; }

        /// <summary>
        /// 报告受理人Kvid
        /// </summary>
        [ApiMember(Description = "原报告受理人Kvid")]
        [DefaultEmptyGuid]
        public Guid ReportOperatorKvid { get; set; }
        /// <summary>
        /// 报告接待人(受理人)
        /// </summary>
        [ApiMember(Description = "受理人姓名")]
        [StringLength(50), Default("")]
        public string ReportOperatorName { get; set; }
        /// <summary>
        /// 报告签发人Kvid
        /// </summary>
        [ApiMember(Description = "原报告签发人Kvid")]
        [DefaultEmptyGuid]
        public Guid ReportIssuerKvid { get; set; }
        /// <summary>
        /// 报告签发人
        /// </summary>
        [ApiMember(Description = "签发人姓名")]
        [StringLength(50), Default("")]
        public string ReportIssuerName { get; set; }

        /// <summary>
        /// 原报告委托日期
        /// </summary>
        [ApiMember(Description = "委托日期")]
        public DateTime? ReportDealDate { get; set; }

        /// <summary>
        /// 原报告要求完成日期
        /// </summary>
        [ApiMember(Description = "完成日期")]
        public DateTime? ReportDeadDate { get; set; }

        /// <summary>
        /// 原报告备注
        /// </summary>
        [ApiMember(Description = "报告备注")]
        [StringLength(2000), Default("")]
        public string ReportRemark { get; set; }

        #endregion

        #region 基本信息
        [ApiMember(Description = "编号")]
        [SerialNumber]
        [StringLength(100), Default("")]
        public string SerialNumber { get; set; }

        /// <summary>
        /// 报告份数
        /// </summary>
        [ApiMember(Description = "报告份数")]
        [DecimalLength(4, 2), Default(0)]
        public decimal Quantity { get; set; }

        [ApiMember(Description = "数量单位")]
        [Ignore]
        public string QuantityUnit
        {
            get
            {
                return "份";
            }
        }
        /// <summary>
        /// 更改类型
        /// </summary>
        [ApiMember(Description = "更改类型")]
        [StringLength(200)]
        [Default("")]
        public string Type { get; set; }
        /// <summary>
        /// 更改报告内容
        /// </summary>
        [ApiMember(Description = "更改内容")]
        [StringLength(2000)]
        public string Content { get; set; }

        /// <summary>
        /// 费用信息
        /// </summary>
        [ApiMember(Description = "货币单位")]
        [IgnoreUpdate]
        [StringLength(50), Required]
        public CurrencyUnit Currency { get; set; }

        [ApiMember(Description = "更改费")]
        [DecimalLength(10, 2), Default(0)]
        public decimal Amount { get; set; }

        /// <summary>
        /// 经办人Kvid
        /// </summary>
        [DefaultEmptyGuid]
        public Guid OperatorKvid { get; set; }

        /// <summary>
        /// 更改报告受理人(经办人)
        /// </summary>
        [ApiMember(Description = "经办人姓名")]
        [StringLength(20), Default("")]
        public string OperatorName { get; set; }

        /// <summary>
        /// 责任人
        /// </summary>
        [ApiMember(Description = "责任人Kvid")]
        [DefaultEmptyGuid]
        public Guid PersonLiableKvid { get; set; }

        [ApiMember(Description = "责任人姓名")]
        [StringLength(50), Default("")]
        public string PersonLiableName { get; set; }

        /// <summary>
        /// 审批人
        /// </summary>
        [ApiMember(Description = "审批人Kvid")]
        [DefaultEmptyGuid]
        public Guid ApproverKvid { get; set; }

        [ApiMember(Description = "审批人姓名")]
        [StringLength(50), Default("")]
        public string ApproverName { get; set; }

        /// <summary>
        /// 受理日期:默认为当天日期
        /// </summary>
        [ApiMember(Description = "受理日期")]
        public DateTime? OperateDate { get; set; }

        /// <summary>
        /// 要求完成日期
        /// </summary>
        [ApiMember(Description = "完成日期")]
        public DateTime? DeadDate { get; set; }

        /// <summary>
        /// 备注
        /// </summary>
        [ApiMember(Description = "备注")]
        [StringLength(2000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 创建/更新人V1.0
        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion

    }
}
