using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lims.Entities
{
    [Alias(Configs.TableNameSample)]
    [Api(Description = "样品信息")]
    public class Sample: EntityWithMetadata, IEntityInAssemblyDb,IEntityHasRemark,IEntityHasCreator, IEntityHasUpdater
    {
        #region 关联信息

        [ApiMember(Description = "样品来源Kvid")]
        [DefaultEmptyGuid]
        public Guid SourceKvid { get; set; }

        [InternalSetter]
        [DefaultEmptyGuid]
        public Guid RootKvid { get; set; }

        [ApiMember(Description ="父级样品Kvid")]
        [DefaultEmptyGuid]
        public Guid ParentKvid { get; set; }
        #endregion

        #region 样品来源信息（冗余字段）
        /// <summary>
        /// 来源编号，若为抽样，则记录计划编号，若为邮寄到样，则记录快递单号
        /// </summary>
        [ApiMember(Description = "来源编号")]
        [StringLength(200), Default("")]
        public string SourceBatchNumber { get; set; }
        
        [ApiMember(Description = "来样方式")]
        [StringLength(50), Default("")]
        public string SourceType { get; set; }
        #endregion

        #region 基本信息

        [ApiMember(Description = "样品名称")]
        [Required]
        [StringLength(200)]
        public string Name { get; set; }

        [ApiMember(Description ="样品类型")]
        [StringLength(50),Default("")]
        public string Type { get; set; }

        [ApiMember(Description = "样品批次编号")]
        [StringLength(200), Default("")]
        public string BatchNumber { get; set; }

        [ApiMember(Description = "样品编号")]
        [SerialNumber]
        [StringLength(100), Default("")]
        public string SerialNumber { get; set; }
              
        [ApiMember(Description = "样品品牌")]
        [StringLength(200),Default("")]
        public string Brand { get; set; }

        [ApiMember(Description = "样品规格")]
        [StringLength(200), Default("")]
        public string Specification { get; set; }

        [ApiMember(Description = "样品型号")]
        [StringLength(200), Default("")]
        public string Model { get; set; }

        [ApiMember(Description = "样品材质")]
        [StringLength(200), Default("")]
        public string Material { get; set; }

        /// <summary>
        /// 样品品级
        /// </summary>
        public string Grade { get; set; }

        /// <summary>
        /// 安全类别
        /// </summary>
        public string GradeSafety { get; set; }

        [ApiMember(Description = "样品状况")]
        [StringLength(200), Default("")]
        public string Condition { get; set; }

        #endregion

        #region 数量相关
        [ApiMember(Description = "样品数量")]
        [DecimalLength(10, 2),Default(0)]
        public decimal Quantity { get; set; }

        [ApiMember(Description = "数量单位")]
        [StringLength(50), Default("")]
        public string QuantityUnit { get; set; }
        #endregion

        #region 生产单位信息及生产日期
        [ApiMember(Description = "生产单位Kvid")]
        [DefaultEmptyGuid]
        [IgnoreUpdate]
        public Guid ManufacturerKvid { get; set; }

        [ApiMember(Description = "生产单位")]
        [StringLength(100), Default("")]
        public string ManufacturerName { get; set; }
        
        [ApiMember(Description = "生产单位联系人Kvid")]
        [DefaultEmptyGuid]
        [IgnoreUpdate]
        public Guid ManufacturerContactKvid { get; set; }

        [ApiMember(Description = "生产单位联系人姓名")]
        [StringLength(50), Default("")]
        public string ManufacturerContactName { get; set; }

        [ApiMember(Description = "生产单位联系人电话")]
        [StringLength(50), Default("")]
        //正则表达式验证：(^(\d{3,4}-)?\d{7,8})$|(13[0-9]{9})
        public string ManufacturerContactNumber { get; set; }

        [ApiMember(Description = "生产单位地址")]
        [StringLength(200), Default("")]
        public string ManufacturerAddress { get; set; }

        [ApiMember(Description = "生产日期")]
        public DateTime? ProductionDate { get; set; }

        #endregion

        #region 文本 备注
        [ApiMember(Description = "备注")]
        [StringLength(1000), Default("")]
        public string Remark { get; set; }
        #endregion

        #region 创建/更新人V1.0
        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion


    }
}
