﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lims.Entities
{
    [Alias(Configs.TableNameSampleSource)]
    [Api(Description = "样品来源信息")]
    public class SampleSource: EntityWithMetadata,IEntityHasBiz, IEntityInAssemblyDb, IEntityHasRemark, IEntityHasCreator, IEntityHasUpdater
    {
        #region 第三方关联V1.0
        //可关联快递+省局抽样任务等信息

        [ApiMember(Description = "业务Kvid")]
        [IgnoreUpdate]
        [DefaultEmptyGuid]
        public Guid BizKvid { get; set; }

        [ApiMember(Description = "业务流水号")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string BizId { get; set; }

        [ApiMember(Description = "业务类型,指实体类型全名称")]
        [IgnoreUpdate]
        [StringLength(200), Default("")]
        public string BizType { get; set; }
        #endregion

        #region 基本信息
        /// <summary>
        /// 送样，快递等
        /// </summary>
        [ApiMember(Description = "来源类型")]
        [StringLength(100), Required]
        public string Type { get; set; }

        /// <summary>
        /// 来源编号:若为邮寄到样，则记录快递单号
        /// </summary>
        [ApiMember(Description = "来源编号")]
        [StringLength(200), Default("")]
        public string BatchNumber { get; set; }
        #endregion

        #region 日期单位
        /// <summary>
        /// 快递或送样默认为到样时间
        /// </summary>
        [ApiMember(Description = "到样日期")]
        public DateTime EffectiveDate { get; set; }

        #endregion

        #region 文本描述和 备注

        [ApiMember(Description = "备注")]
        [StringLength(1000)]
        public string Remark { get; set; }
        #endregion

        #region 创建/更新人V1.0
        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion

    }
}
