﻿using Kivii.DataAnnotations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lims.Entities
{
    public enum TemplateType
    {
        Unsupported = 0,
        Common = 1,
        EnterPrise = 2,
        Supervise = 3
    }

    public enum TemplateCategoryType
    {
        Unsupported = 0,
        Report = 1,//报告模板
        SDMS = 2,//原始记录模板
        ReportItem = 3//检验项目模板
    }

    /// <summary>
    /// 模板的流程状态设置，暂定6个状态
    /// </summary>
    public enum StatusType
    {
        Unsupported = 0,
        First = 100,
        Second = 200,
        Third = 300,
        Fourth = 400,
        Fifth = 500,
        Sixth = 600
    }

    /// <summary>
    /// 可管理报告模板、原始记录模板、检验项目模板
    /// </summary>
    [Api(Description = "模版管理")]
    [Alias(Configs.TableNameTemplate)]
    public class Template:EntityWithMetadata, IEntityInAssemblyDb
    {
        //2021/01/15因项目模板需要树形结构显示，新增树形结构
        [DefaultEmptyGuid]
        public Guid RootKvid { get; set; }

        [DefaultEmptyGuid]
        public Guid ParentKvid { get; set; }

        [Required]
        [ApiMember(Description = "模版名称")]
        public string Name { get; set; }

        /// <summary>
        /// 用于记录通用模板、企业模板等
        /// 在Type为Report时使用,枚举类型
        /// </summary>
        [Required]
        [ApiMember(Description = "模板类型")]
        public TemplateType Type { get; set; }

        /// <summary>
        /// 用于记录报告模板，还是原始记录模板，还是检验项目模板
        /// 枚举类型
        /// </summary>
        [Required]
        [ApiMember(Description = "模板类别")]
        public TemplateCategoryType Category { get; set; }

        [Default(0)]
        [ApiMember(Description = "排序号")]
        public int SortId { get; set; }

        [ApiMember(Description = "生效日期")]
        public DateTime? EffectiveTime { get; set; }

        [ApiMember(Description = "失效日期")]
        public DateTime? ExpiredTime { get; set; }

        [ApiMember(Description = "是否受控")]
        [Default(0)]
        public bool IsUnderController { get; set; }

        /// <summary>
        /// 一个受控号可能存在多个模板，所以不能唯一
        /// </summary>
        [Default("")]
        [ApiMember(Description = "受控号")]
        public string UnderControllerId { get; set; }

        [Default("")]
        [ApiMember(Description ="备注")]
        public string Remark { get; set; }

        #region 创建/更新人V1.0
        [ApiMember(Description = "创建人Kvid ")]
        [IgnoreUpdate]
        [CurrentMemberKvid]
        public Guid CreatorKvid { get; set; }

        [ApiMember(Description = "创建人")]
        [IgnoreUpdate]
        [StringLength(50), CurrentMemberName]
        public string CreatorName { get; set; }

        [ApiMember(Description = "更新人Kvid ")]
        [CurrentMemberKvid]
        public Guid UpdaterKvid { get; set; }

        [ApiMember(Description = "更新人")]
        [StringLength(50), CurrentMemberName]
        public string UpdaterName { get; set; }
        #endregion
    }
}
