﻿using Kivii.Lims.Entities;
using Kivii.Linq;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Kivii.Lac.Lims.Extensions
{
    public static class ReportItemExtension
    {
        /// <summary>
        /// 用于生成色号时 更新Test的RootKvid
        /// </summary>
        /// <param name="targetTest">要被更改的目标Test</param>
        /// <param name="parentTest">指定的父级Test</param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static ReportItem ModifyRoot(this ReportItem targetTest, ReportItem parentTest, IDbConnection conn = null)
        {
            targetTest.ThrowIfNull("请传入要被更改的目标Test!");
            targetTest.Kvid.ThrowIfEmpty("要被更改的目标TestKvid不能为空!");

            parentTest.ThrowIfNull("请传入指定的父级Test!");
            parentTest.Kvid.ThrowIfEmpty("指定的父级TestKvid不能为空!");

            bool useTransaction = conn == null;//是否启用事务，如果外部未传入conn,启用内部事务
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            }
            //targetTest = conn.SingleById<ReportItem>(targetTest.Kvid);
            targetTest.ThrowIfNull("未找到要被更改的目标Test!");

            //parentTest = conn.SingleById<ReportItem>(parentTest.Kvid);
            parentTest.ThrowIfNull("未找到指定的父级Test!");
            parentTest.SampleKvid.ThrowIfEmpty("指定的父级Test的样品来源SampleKvid不能为空!");

            var nodesTargetTests = conn.Select<ReportItem>(o => o.ParentKvid == targetTest.Kvid);

            var rtns = new ReportItem();

            IDbTransaction trans = null;//事务，如果外部来了Connection，不生成事务
            if (useTransaction) trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                var newReportItem = new ReportItem();
                newReportItem.PopulateWith(targetTest);
                newReportItem.Kvid = Guid.NewGuid();
                newReportItem.ParentKvid = parentTest.Kvid;
                newReportItem.SampleKvid = parentTest.SampleKvid;
                newReportItem.SampleRootKvid = parentTest.SampleRootKvid;
                conn.Insert(newReportItem);
                if (!nodesTargetTests.IsNullOrEmpty())
                {
                    foreach (var node in nodesTargetTests)
                    {
                        node.ModifyRoot(newReportItem, conn);
                    }
                }
                newReportItem.RemoveAllOnlyProperties();
                rtns = newReportItem;
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }

        /// <summary>
        ///  从字符串中提取数字
        /// </summary>
        /// <param name="str"></param>
        /// <returns></returns>
        public static string IsNumber(this string str)
        {
            string result = "";
            for (int i = 0; i < str.Length; i++)
            {
                if (Char.IsNumber(str, i) == true)
                {
                    result += str.Substring(i, 1);
                }
                else if (char.IsPunctuation(str, i) == true)
                {
                    result += str.Substring(i, 1) == "." ? "." : null;
                }
                else
                {
                    result += " ";
                }
            }
            return result;
        }
        /// <summary>
        /// Test和TestFolder类型检测项目结果判定
        /// 通过对比标准值和实测值,计算当前的项目是否合格，
        /// 条件一：根据传入的ReportItem查report，获取字段TestNeedJudge，若为判定，则；若为数据，则；
        /// </summary>
        /// <param name="reportItem"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        
        [Obsolete("该方法不再使用")]
        public static bool CalculateJudgement(this ReportItem reportItem, IDbConnection conn = null)
        {
            reportItem.ThrowIfNull("未传入ReportItem对象!");
            reportItem.Kvid.ThrowIfEmpty("ReportItem的Kvid不能为空");

            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            Report report = conn.SingleById<Report>(reportItem.ReportKvid);
            report.ThrowIfNull("未查询到对应的检测报告！");

            bool rtns = false;
            try
            {
                if (reportItem.Type == ReportItemType.Test)
                {
                    string left = reportItem.DetectionValueQualified.IsNullOrEmpty() ? "" : reportItem.DetectionValueQualified.Replace(" ", "");
                    string right = reportItem.TestValue.IsNullOrEmpty() ? "" : reportItem.TestValue.Replace(" ", "");
                    //判断实测值是不是类似于：3-4等
                    if (right.Contains("-") && !right.StartsWith("-"))
                    {
                        try
                        {
                            string[] array = right.Split('-');
                            right = ((decimal.Parse(array[0]) + decimal.Parse(array[1])) / 2).ToString();
                        }
                        catch { }
                    }
                    if (right.IsNullOrEmpty())
                    {
                        reportItem.Judgement = "";
                        return true;
                    }
                    else if (right == "+" || right == "未检出" || right == "未检出#not detected" || (right.Contains("符合") && !right.Contains("不符合")) || left == right)
                    {
                        reportItem.Judgement = "合格";
                        return true;
                    }
                    else if (right == "/")
                    {
                        reportItem.Judgement = "/";
                        return true;
                    }
                    //else if ((parentTest == null || (parentTest != null && parentTest.Type != TestType.TestFolder)) && (right.Contains("不符合") || (test.Title.Contains("标志标识") && right != "")))
                    else if (right.Contains("不符合") || (reportItem.Title.Contains("标志标识") && right != ""))
                    {
                        reportItem.Judgement = "不合格";
                        return true;
                    }
                    //else if (parentTest != null && parentTest.Type == TestType.TestFolder && (right.Contains("不符合") || (parentTest.Title.Contains("标志标识") && right != "")))
                    else if (right.Contains("不符合") || (reportItem.Title.Contains("标志标识") && right != ""))
                    {
                        reportItem.Judgement = "不合格";
                        return true;
                    }
                    decimal l1 = 0, l2 = 0, r = 0;

                    if (right.Contains("≥")) r = decimal.Parse(right.Substring(1));
                    else if (right.Contains(">")) r = decimal.Parse(right.Substring(1)) + 1;
                    else if (right.Contains("≤")) r = decimal.Parse(right.Substring(1));
                    else if (right.Contains("<")) r = decimal.Parse(right.Substring(1)) - 1;
                    else if (!decimal.TryParse(right, out r))
                    {
                        reportItem.Judgement = "";
                        return true;
                    }

                    Expression expRight = Expression.Constant(r);

                    char[] split = new char[] { '≥', '>', '≤', '<', '±', '～'};

                    string[] args = left.Split(split);
                    if (args.Length != 2) return false;

                    if (left.Contains("±"))
                    {
                        if (args[0] == "") args[0] = "0";
                        l1 = decimal.Parse(args[0]) - decimal.Parse(args[1]);
                        l2 = decimal.Parse(args[0]) + decimal.Parse(args[1]);

                        Expression expLeft1 = Expression.Constant(l1);
                        Expression exp1 = Expression.LessThanOrEqual(expLeft1, expRight);

                        Expression expLeft2 = Expression.Constant(l2);
                        Expression exp2 = Expression.GreaterThanOrEqual(expLeft2, expRight);

                        reportItem.Judgement = (bool)Expression.Lambda(Expression.And(exp1, exp2)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        return true;
                    }
                    else if (left.Contains("～"))
                    {
                        l1 = decimal.Parse(args[0]);
                        l2 = decimal.Parse(args[1]);

                        Expression expLeft1 = Expression.Constant(l1);
                        Expression exp1 = Expression.LessThanOrEqual(expLeft1, expRight);

                        Expression expLeft2 = Expression.Constant(l2);
                        Expression exp2 = Expression.GreaterThanOrEqual(expLeft2, expRight);

                        reportItem.Judgement = (bool)Expression.Lambda(Expression.And(exp1, exp2)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        return true;
                    }
                    else
                    {
                        if(left.Contains("×") && left.Contains("%"))
                        {
                            Match match = Regex.Match(left, @"(\d+)×(\d+)%");
                            if (match.Success)
                            {
                                // 提取数字部分并转换为整数
                                int num1 = int.Parse(match.Groups[1].Value);
                                int num2 = int.Parse(match.Groups[2].Value);

                                // 将百分比转换为小数（除以100）
                                decimal percentage = (decimal)(num2 / 100.0);

                                // 计算结果
                                decimal result = (decimal)(num1 * percentage);
                                l1 = result;
                            }
                        }
                        //此处要添加个条件：args[1]包含“-”并且不是以“-”开始的
                        else if (args[1].Contains("-") && !args[1].StartsWith("-"))
                        {
                            string[] vs = args[1].Split('-');
                            if (decimal.Parse(vs[0]) > decimal.Parse(vs[1]))
                            {
                                l1 = (decimal.Parse(vs[0]) - decimal.Parse(vs[1]));
                            }
                            else
                            {
                                l1 = (decimal.Parse(vs[0]) + decimal.Parse(vs[1])) / 2;
                            }
                        }
                        else
                        {
                            l1 = decimal.Parse(args[1]);
                        }
                        Expression expLeft = Expression.Constant(l1);

                        if (left.Contains("≥")) reportItem.Judgement = (bool)Expression.Lambda(Expression.LessThanOrEqual(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        else if (left.Contains(">")) reportItem.Judgement = (bool)Expression.Lambda(Expression.LessThan(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        else if (left.Contains("≤")) reportItem.Judgement = (bool)Expression.Lambda(Expression.GreaterThanOrEqual(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        else if (left.Contains("<")) reportItem.Judgement = (bool)Expression.Lambda(Expression.GreaterThan(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                    }

                    //若报告中的检验要求为判定，则处理
                    if (!report.TestNeedJudge) reportItem.Judgement = string.Empty;
                    rtns = true;
                }
                if (reportItem.Type == ReportItemType.TestFolder)
                {
                    var nodeTests = conn.Select<ReportItem>(o => o.ParentKvid == reportItem.Kvid && o.Type == ReportItemType.Test);
                    if (nodeTests.IsNullOrEmpty()) return false;

                    //已经完成判定项的个数
                    var judgedTasksCount = nodeTests.Count(o => !o.Judgement.IsNullOrEmpty());

                    //当前小项合格项的个数
                    var qualifiedTasksCount = nodeTests.Count(o => o.Judgement == "合格" || o.Judgement == "/");

                    //已经录入实测值的个数
                    var testValuesCount = nodeTests.Count(o => !o.TestValue.IsNullOrEmpty());

                    if (nodeTests.Count != judgedTasksCount)
                    {
                        reportItem.Judgement = "";
                        rtns = false;
                    }
                    else if (judgedTasksCount > qualifiedTasksCount)
                    {
                        reportItem.Judgement = "不合格";
                        rtns = true;
                    }
                    else
                    {
                        reportItem.Judgement = "合格";
                        rtns = true;
                    }
                    //若报告中的检验要求为判定，则处理
                    if (!report.TestNeedJudge) reportItem.Judgement = string.Empty;
                    //说明子项全部录入实测值了 大项不管有没有判定需要更新检测人信息
                    if (nodeTests.Count == testValuesCount)
                    {
                        reportItem.TestCreateTime = nodeTests.Min(o => o.TestCreateTime);
                        reportItem.TestCreatorName = nodeTests.FirstOrDefault().TestCreatorName;
                        reportItem.TestCreatorKvid = nodeTests.FirstOrDefault().TestCreatorKvid;
                        rtns = true;
                    }
                }
            }
            catch
            {
                rtns = false;
            }
            return rtns;
        }

        public static bool CalculateJudgement2(this ReportItem reportItem, Report report, List<ReportItem> nodeTests = null)
        {
            reportItem.ThrowIfNull("未传入ReportItem对象!");
            reportItem.Kvid.ThrowIfEmpty("ReportItem的Kvid不能为空");

            report.ThrowIfNull("未传入报告信息！");

            bool rtns = false;
            var originalDetectionValueQualified = reportItem.DetectionValueQualified;
            if (reportItem.Metadata.ContainsKey("DetectionValueCustomer") && reportItem.Metadata["DetectionValueCustomer"] != "")
            {
                reportItem.DetectionValueQualified = reportItem.Metadata["DetectionValueCustomer"];
            }
            try
            {
                if (reportItem.Type == ReportItemType.Test)
                {
                    string right = reportItem.TestValue.IsNullOrEmpty() ? "" : reportItem.TestValue.Replace(" ", "");
                    //如果实测值为“/”，不管标准值是什么，都判定为“/”
                    if(right == "/")
                    {
                        reportItem.Judgement = "/";
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return true;
                    }
                    //如果实测值为“-”，不管标准值是什么，都判定为“-”
                    if (right == "—"|| right == "-")
                    {
                        reportItem.Judgement = "—";
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return true;
                    }
                    string left = reportItem.DetectionValueQualified.IsNullOrEmpty() ? "" : reportItem.DetectionValueQualified.Replace(" ", "").Replace("＞",">").Replace("＜","<");
                    //如果标准值为空，不管实测值是什么都不判定
                    if (left.IsNullOrEmpty())
                    {
                        reportItem.Judgement = "";
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return true;
                    }
                    //判断实测值是不是类似于：3-4等
                    if (right.Contains("-") && !right.StartsWith("-"))
                    {
                        try
                        {
                            string[] array = right.Split('-');
                            right = ((decimal.Parse(array[0]) + decimal.Parse(array[1])) / 2).ToString();
                        }
                        catch { }
                    }
                    //判断实测值是不是指数表示，如2.1×10^(3)
                    if (right.Contains("^"))
                    {
                        try
                        {
                            string[] array = right.Split('×');
                            //a×10^(b)
                            //获取a
                            var aValue = double.Parse(array[0]);
                            //获取b
                            var bValue = double.Parse(Regex.Replace(array[1].Split('^')[1], @"[^0-9]+", ""));
                            right = (aValue * Math.Pow(10, bValue)).ToString();
                        }
                        catch { }
                    }
                    //判断实测值为带斜杠的实测值，如100/2
                    if (right.Contains("/")&&right.Length>1)
                    {
                        try
                        {
                            string[] array = right.Split('/');
                            right = array[0];
                        }
                        catch { }
                    }
                    if (right.IsNullOrEmpty())
                    {
                        reportItem.Judgement = "";
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return true;
                    }
                    else if (right == "+" || right == "未检出" || right == "未检出#not detected" || right == "经向沿纬向断裂" || right == "纬向沿经向断裂" || (right.Contains("符合") && !right.Contains("不符合")) || left == right)
                    {
                        reportItem.Judgement = "合格";
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return true;
                    }
                    else if (right == "/")
                    {
                        reportItem.Judgement = "/";
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return true;
                    }
                    //else if ((parentTest == null || (parentTest != null && parentTest.Type != TestType.TestFolder)) && (right.Contains("不符合") || (test.Title.Contains("标志标识") && right != "")))
                    else if (right.Contains("不符合") || (reportItem.Title.Contains("标志标识") && right != ""))
                    {
                        reportItem.Judgement = "不合格";
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return true;
                    }
                    //else if (parentTest != null && parentTest.Type == TestType.TestFolder && (right.Contains("不符合") || (parentTest.Title.Contains("标志标识") && right != "")))
                    else if (right.Contains("不符合") || (reportItem.Title.Contains("标志标识") && right != ""))
                    {
                        reportItem.Judgement = "不合格";
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return true;
                    }
                    else if (right == "无绳带" || right == "无绳圈")//31701的项目
                    {
                        reportItem.Judgement = "合格";
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return true;
                    }
                    decimal l1 = 0, l2 = 0, r = 0;

                    if (right.Contains("≥")) r = decimal.Parse(right.Substring(1));
                    else if (right.Contains(">")) r = decimal.Parse(right.Substring(1)) + 1;
                    else if (right.Contains("≤")) r = decimal.Parse(right.Substring(1));
                    else if (right.Contains("<")) r = decimal.Parse(right.Substring(1)) - 1;
                    else if (!decimal.TryParse(right, out r))
                    {
                        reportItem.Judgement = "";
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return true;
                    }

                    Expression expRight = Expression.Constant(r);

                    char[] split = new char[] { '≥', '>', '≤', '<', '±', '～' ,'~'};

                    string[] args = left.Split(split);
                    if (args.Length != 2) 
                    {
                        reportItem.DetectionValueQualified = originalDetectionValueQualified;
                        return false;
                    }
                    

                    if (left.Contains("±"))
                    {
                        if (args[0] == "") args[0] = "0";
                        if (args[0].Contains("/"))
                        {
                            args[0] = args[0].Split('/')[0];
                        }
                        if (args[0].Contains('.'))
                        {
                            args[0] = Regex.Replace(args[0], @"[^\d.\d]", "");
                        }
                        else
                        {
                            args[0] = Regex.Replace(args[0], @"[^0-9]+", "");
                        }
                        //处理args[1]为百分比的情况
                        if (args[1].Contains("%"))
                        {
                            args[1] = (decimal.Parse(new Regex(@"\d+\.\d+|\d+").Matches(args[1])[0].Value) / 100 * decimal.Parse(args[0])).ToString();
                        }
                        l1 = decimal.Parse(args[0]) - decimal.Parse(args[1]);
                        l2 = decimal.Parse(args[0]) + decimal.Parse(args[1]);

                        Expression expLeft1 = Expression.Constant(l1);
                        Expression exp1 = Expression.LessThanOrEqual(expLeft1, expRight);

                        Expression expLeft2 = Expression.Constant(l2);
                        Expression exp2 = Expression.GreaterThanOrEqual(expLeft2, expRight);

                        reportItem.Judgement = (bool)Expression.Lambda(Expression.And(exp1, exp2)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        return true;
                    }
                    else if (left.Contains("～")|| left.Contains("~"))
                    {
                        l1 = decimal.Parse(args[0]);
                        l2 = decimal.Parse(args[1]);
                        if (l1 > l2)
                        {
                            var tmp = l1;
                            l1 = l2;
                            l2 = tmp;
                        }

                        Expression expLeft1 = Expression.Constant(l1);
                        Expression exp1 = Expression.LessThanOrEqual(expLeft1, expRight);

                        Expression expLeft2 = Expression.Constant(l2);
                        Expression exp2 = Expression.GreaterThanOrEqual(expLeft2, expRight);

                        reportItem.Judgement = (bool)Expression.Lambda(Expression.And(exp1, exp2)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        return true;
                    }
                    else
                    {
                        if (left.Contains("×") && left.Contains("%")&&!left.Contains("标称值"))
                        {
                            //Match match = Regex.Match(left, @"(\d+)×(\d+)%");
                            Match match = Regex.Match(left, @"(\d+(?:\.\d+)?)(?:[×%]*)(\d+(?:\.\d+)?)(?:%)");
                            if (match.Success)
                            {
                                // 提取数字部分并转换为整数
                                decimal num1 = decimal.Parse(match.Groups[1].Value);
                                decimal num2 = decimal.Parse(match.Groups[2].Value);

                                // 将百分比转换为小数（除以100）
                                decimal percentage = (decimal)(num2 / 100);

                                // 计算结果
                                decimal result = (decimal)(num1 * percentage);
                                l1 = result;
                            }
                        }

                        //处理百分比
                        else if (args[1].Contains("%"))
                        {
                            if (args[1].Contains("-"))
                            {
                                string[] vs = args[1].Split('-');
                                if (vs.Length != 2) return false;
                                if (vs[0].Contains("/"))
                                {
                                    vs[0] = args[0].Split('/')[0];
                                }
                                l1 = decimal.Parse(Regex.Replace(vs[0], @"[^0-9]+", "")) - decimal.Parse(Regex.Replace(vs[0], @"[^0-9]+", "")) * decimal.Parse(Regex.Replace(vs[1], @"[^\d.\d]", "")) / 100;
                            }
                            else if (args[1].Contains("+"))
                            {
                                string[] vs = args[1].Split('+');
                                if (vs.Length != 2) return false;
                                if (vs[0].Contains("/"))
                                {
                                    vs[0] = args[0].Split('/')[0];
                                }
                                l1 = decimal.Parse(Regex.Replace(vs[0], @"[^0-9]+", "")) + decimal.Parse(Regex.Replace(vs[0], @"[^0-9]+", "")) * decimal.Parse(Regex.Replace(vs[1], @"[^0-9]+", "")) / 100;
                            }
                            else
                            {
                                reportItem.DetectionValueQualified = originalDetectionValueQualified;
                                return false;
                            }
                        }
                        //此处要添加个条件：args[1]包含“-”并且不是以“-”开始的
                        else if (args[1].Contains("-") && !args[1].StartsWith("-"))
                        {
                            string[] vs = args[1].Split('-');
                            if (decimal.Parse(vs[0]) > decimal.Parse(vs[1]))
                            {
                                l1 = (decimal.Parse(vs[0]) - decimal.Parse(vs[1]));
                            }
                            else
                            {
                                l1 = (decimal.Parse(vs[0]) + decimal.Parse(vs[1])) / 2;
                            }
                        }
                        else if (args[1].Contains("/"))
                        {
                            l1 = decimal.Parse(args[1].Split('/')[0]);
                        }
                        else
                        {
                            l1 = decimal.Parse(args[1]);
                        }
                        Expression expLeft = Expression.Constant(l1);

                        if (left.Contains("≥")) reportItem.Judgement = (bool)Expression.Lambda(Expression.LessThanOrEqual(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        else if (left.Contains(">")) reportItem.Judgement = (bool)Expression.Lambda(Expression.LessThan(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        else if (left.Contains("≤")) reportItem.Judgement = (bool)Expression.Lambda(Expression.GreaterThanOrEqual(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        else if (left.Contains("<")) reportItem.Judgement = (bool)Expression.Lambda(Expression.GreaterThan(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                    }

                    //若报告中的检验要求为判定，则处理
                    if (!report.TestNeedJudge) reportItem.Judgement = string.Empty;
                    rtns = true;
                }
                if (reportItem.Type == ReportItemType.TestFolder)
                {
                    //var nodeTests = conn.Select<ReportItem>(o => o.ParentKvid == reportItem.Kvid & o.Type == ReportItemType.Test);
                    nodeTests.ThrowIfNullOrEmpty("父级项目判定请传入子项列表");
                    //过滤一遍传入参数，至处理当前大项的子项
                    nodeTests = nodeTests.Where(o => o.ParentKvid == reportItem.Kvid).ToList();
                    //已经完成判定项的个数
                    var judgedTasksCount = nodeTests.Count(o => !o.Judgement.IsNullOrEmpty());

                    //当前小项合格项的个数
                    var qualifiedTasksCount1 = nodeTests.Count(o => o.Judgement == "合格");
                    //记斜杠和横杠数
                    var qualifiedTasksCount2 = nodeTests.Count(o=>o.Judgement == "/");
                    var qualifiedTasksCount3 = nodeTests.Count(o=>o.Judgement == "—");
                    //已经录入实测值的个数
                    var testValuesCount = nodeTests.Count(o => !o.TestValue.IsNullOrEmpty());

                    if (nodeTests.Count != judgedTasksCount)
                    {
                        reportItem.Judgement = "";
                        rtns = false;
                    }
                    else if (judgedTasksCount > (qualifiedTasksCount1+ qualifiedTasksCount2+ qualifiedTasksCount3))
                    {
                        reportItem.Judgement = "不合格";
                        rtns = true;
                    }
                    else if(judgedTasksCount == qualifiedTasksCount2)
                    {
                        reportItem.Judgement = "/";
                        rtns = true;
                    }
                    else if (judgedTasksCount == qualifiedTasksCount3)
                    {
                        reportItem.Judgement = "—";
                        rtns = true;
                    }
                    else
                    {
                        reportItem.Judgement = "合格";
                        rtns = true;
                    }
                    //若报告中的检验要求为判定，则处理
                    if (!report.TestNeedJudge) reportItem.Judgement = string.Empty;
                    //说明子项全部录入实测值了 大项不管有没有判定需要更新检测人信息
                    if (nodeTests.Count == testValuesCount)
                    {
                        //reportItem.TestCreateTime = nodeTests.Min(o => o.TestCreateTime);
                        //reportItem.TestCreatorName = nodeTests.FirstOrDefault().TestCreatorName;
                        //reportItem.TestCreatorKvid = nodeTests.FirstOrDefault().TestCreatorKvid;
                        rtns = true;
                    }
                }
            }
            catch
            {
                reportItem.DetectionValueQualified = originalDetectionValueQualified;
                rtns = false;
            }
            reportItem.DetectionValueQualified = originalDetectionValueQualified;
            return rtns;
        }
        public static bool CalculateJudgementBackup(this ReportItem reportItem, Report report, List<ReportItem> nodeTests = null)
        {
            reportItem.ThrowIfNull("未传入ReportItem对象!");
            reportItem.Kvid.ThrowIfEmpty("ReportItem的Kvid不能为空");

            report.ThrowIfNull("未传入报告信息！");

            bool rtns = false;
            try
            {
                if (reportItem.Type == ReportItemType.Test)
                {
                    string right = reportItem.TestValue.IsNullOrEmpty() ? "" : reportItem.TestValue.Replace(" ", "");
                    //如果实测值为“/”，不管标准值是什么，都判定为“/”
                    if (right == "/")
                    {
                        reportItem.Judgement = "/";
                        return true;
                    }
                    //如果实测值为“-”，不管标准值是什么，都判定为“-”
                    if (right == "—" || right == "-")
                    {
                        reportItem.Judgement = "—";
                        return true;
                    }
                    string left = reportItem.DetectionValueQualified.IsNullOrEmpty() ? "" : reportItem.DetectionValueQualified.Replace(" ", "");
                    //如果标准值为空，不管实测值是什么都不判定
                    if (left.IsNullOrEmpty())
                    {
                        reportItem.Judgement = "";
                        return true;
                    }
                    //判断实测值是不是类似于：3-4等
                    if (right.Contains("-") && !right.StartsWith("-"))
                    {
                        try
                        {
                            string[] array = right.Split('-');
                            right = ((decimal.Parse(array[0]) + decimal.Parse(array[1])) / 2).ToString();
                        }
                        catch { }
                    }
                    //判断实测值是不是指数表示，如2.1×10^(3)
                    if (right.Contains("^"))
                    {
                        try
                        {
                            string[] array = right.Split('×');
                            //a×10^(b)
                            //获取a
                            var aValue = double.Parse(array[0]);
                            //获取b
                            var bValue = double.Parse(Regex.Replace(array[1].Split('^')[1], @"[^0-9]+", ""));
                            right = (aValue * Math.Pow(10, bValue)).ToString();
                        }
                        catch { }
                    }
                    //判断实测值为带斜杠的实测值，如100/2
                    if (right.Contains("/") && right.Length > 1)
                    {
                        try
                        {
                            string[] array = right.Split('/');
                            right = array[0];
                        }
                        catch { }
                    }
                    if (right.IsNullOrEmpty())
                    {
                        reportItem.Judgement = "";
                        return true;
                    }
                    else if (right == "+" || right == "未检出" || right == "未检出#not detected" || right == "经向沿纬向断裂" || right == "纬向沿经向断裂" || (right.Contains("符合") && !right.Contains("不符合")) || left == right)
                    {
                        reportItem.Judgement = "合格";
                        return true;
                    }
                    else if (right == "/")
                    {
                        reportItem.Judgement = "/";
                        return true;
                    }
                    //else if ((parentTest == null || (parentTest != null && parentTest.Type != TestType.TestFolder)) && (right.Contains("不符合") || (test.Title.Contains("标志标识") && right != "")))
                    else if (right.Contains("不符合") || (reportItem.Title.Contains("标志标识") && right != ""))
                    {
                        reportItem.Judgement = "不合格";
                        return true;
                    }
                    //else if (parentTest != null && parentTest.Type == TestType.TestFolder && (right.Contains("不符合") || (parentTest.Title.Contains("标志标识") && right != "")))
                    else if (right.Contains("不符合") || (reportItem.Title.Contains("标志标识") && right != ""))
                    {
                        reportItem.Judgement = "不合格";
                        return true;
                    }
                    else if (right == "无绳带" || right == "无绳圈")//31701的项目
                    {
                        reportItem.Judgement = "合格";
                        return true;
                    }
                    decimal l1 = 0, l2 = 0, r = 0;

                    if (right.Contains("≥")) r = decimal.Parse(right.Substring(1));
                    else if (right.Contains(">")) r = decimal.Parse(right.Substring(1)) + 1;
                    else if (right.Contains("≤")) r = decimal.Parse(right.Substring(1));
                    else if (right.Contains("<")) r = decimal.Parse(right.Substring(1)) - 1;
                    else if (!decimal.TryParse(right, out r))
                    {
                        reportItem.Judgement = "";
                        return true;
                    }

                    Expression expRight = Expression.Constant(r);

                    char[] split = new char[] { '≥', '>', '≤', '<', '±', '～', '~' };

                    string[] args = left.Split(split);
                    if (args.Length != 2) return false;

                    if (left.Contains("±"))
                    {
                        if (args[0] == "") args[0] = "0";
                        if (args[0].Contains("/"))
                        {
                            args[0] = args[0].Split('/')[0];
                        }
                        if (args[0].Contains('.'))
                        {
                            args[0] = Regex.Replace(args[0], @"[^\d.\d]", "");
                        }
                        else
                        {
                            args[0] = Regex.Replace(args[0], @"[^0-9]+", "");
                        }
                        //处理args[1]为百分比的情况
                        if (args[1].Contains("%"))
                        {
                            args[1] = (decimal.Parse(new Regex(@"\d+\.\d+|\d+").Matches(args[1])[0].Value) / 100 * decimal.Parse(args[0])).ToString();
                        }
                        l1 = decimal.Parse(args[0]) - decimal.Parse(args[1]);
                        l2 = decimal.Parse(args[0]) + decimal.Parse(args[1]);

                        Expression expLeft1 = Expression.Constant(l1);
                        Expression exp1 = Expression.LessThanOrEqual(expLeft1, expRight);

                        Expression expLeft2 = Expression.Constant(l2);
                        Expression exp2 = Expression.GreaterThanOrEqual(expLeft2, expRight);

                        reportItem.Judgement = (bool)Expression.Lambda(Expression.And(exp1, exp2)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        return true;
                    }
                    else if (left.Contains("～") || left.Contains("~"))
                    {
                        l1 = decimal.Parse(args[0]);
                        l2 = decimal.Parse(args[1]);
                        if (l1 > l2)
                        {
                            var tmp = l1;
                            l1 = l2;
                            l2 = tmp;
                        }

                        Expression expLeft1 = Expression.Constant(l1);
                        Expression exp1 = Expression.LessThanOrEqual(expLeft1, expRight);

                        Expression expLeft2 = Expression.Constant(l2);
                        Expression exp2 = Expression.GreaterThanOrEqual(expLeft2, expRight);

                        reportItem.Judgement = (bool)Expression.Lambda(Expression.And(exp1, exp2)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        return true;
                    }
                    else
                    {
                        if (left.Contains("×") && left.Contains("%") && !left.Contains("标称值"))
                        {
                            Match match = Regex.Match(left, @"(\d+)×(\d+)%");
                            if (match.Success)
                            {
                                // 提取数字部分并转换为整数
                                int num1 = int.Parse(match.Groups[1].Value);
                                int num2 = int.Parse(match.Groups[2].Value);

                                // 将百分比转换为小数（除以100）
                                decimal percentage = (decimal)(num2 / 100.0);

                                // 计算结果
                                decimal result = (decimal)(num1 * percentage);
                                l1 = result;
                            }
                        }

                        //处理百分比
                        else if (args[1].Contains("%"))
                        {
                            if (args[1].Contains("-"))
                            {
                                string[] vs = args[1].Split('-');
                                if (vs.Length != 2) return false;
                                if (vs[0].Contains("/"))
                                {
                                    vs[0] = args[0].Split('/')[0];
                                }
                                l1 = decimal.Parse(Regex.Replace(vs[0], @"[^0-9]+", "")) - decimal.Parse(Regex.Replace(vs[0], @"[^0-9]+", "")) * decimal.Parse(Regex.Replace(vs[1], @"[^\d.\d]", "")) / 100;
                            }
                            else if (args[1].Contains("+"))
                            {
                                string[] vs = args[1].Split('+');
                                if (vs.Length != 2) return false;
                                if (vs[0].Contains("/"))
                                {
                                    vs[0] = args[0].Split('/')[0];
                                }
                                l1 = decimal.Parse(Regex.Replace(vs[0], @"[^0-9]+", "")) + decimal.Parse(Regex.Replace(vs[0], @"[^0-9]+", "")) * decimal.Parse(Regex.Replace(vs[1], @"[^0-9]+", "")) / 100;
                            }
                            else return false;

                        }
                        //此处要添加个条件：args[1]包含“-”并且不是以“-”开始的
                        else if (args[1].Contains("-") && !args[1].StartsWith("-"))
                        {
                            string[] vs = args[1].Split('-');
                            if (decimal.Parse(vs[0]) > decimal.Parse(vs[1]))
                            {
                                l1 = (decimal.Parse(vs[0]) - decimal.Parse(vs[1]));
                            }
                            else
                            {
                                l1 = (decimal.Parse(vs[0]) + decimal.Parse(vs[1])) / 2;
                            }
                        }
                        else if (args[1].Contains("/"))
                        {
                            l1 = decimal.Parse(args[1].Split('/')[0]);
                        }
                        else
                        {
                            l1 = decimal.Parse(args[1]);
                        }
                        Expression expLeft = Expression.Constant(l1);

                        if (left.Contains("≥")) reportItem.Judgement = (bool)Expression.Lambda(Expression.LessThanOrEqual(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        else if (left.Contains(">")) reportItem.Judgement = (bool)Expression.Lambda(Expression.LessThan(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        else if (left.Contains("≤")) reportItem.Judgement = (bool)Expression.Lambda(Expression.GreaterThanOrEqual(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                        else if (left.Contains("<")) reportItem.Judgement = (bool)Expression.Lambda(Expression.GreaterThan(expLeft, expRight)).Compile().DynamicInvoke() ? "合格" : "不合格";
                    }

                    //若报告中的检验要求为判定，则处理
                    if (!report.TestNeedJudge) reportItem.Judgement = string.Empty;
                    rtns = true;
                }
                if (reportItem.Type == ReportItemType.TestFolder)
                {
                    //var nodeTests = conn.Select<ReportItem>(o => o.ParentKvid == reportItem.Kvid & o.Type == ReportItemType.Test);
                    nodeTests.ThrowIfNullOrEmpty("父级项目判定请传入子项列表");
                    //过滤一遍传入参数，至处理当前大项的子项
                    nodeTests = nodeTests.Where(o => o.ParentKvid == reportItem.Kvid).ToList();
                    //已经完成判定项的个数
                    var judgedTasksCount = nodeTests.Count(o => !o.Judgement.IsNullOrEmpty());

                    //当前小项合格项的个数
                    var qualifiedTasksCount1 = nodeTests.Count(o => o.Judgement == "合格");
                    //记斜杠和横杠数
                    var qualifiedTasksCount2 = nodeTests.Count(o => o.Judgement == "/");
                    var qualifiedTasksCount3 = nodeTests.Count(o => o.Judgement == "—");
                    //已经录入实测值的个数
                    var testValuesCount = nodeTests.Count(o => !o.TestValue.IsNullOrEmpty());

                    if (nodeTests.Count != judgedTasksCount)
                    {
                        reportItem.Judgement = "";
                        rtns = false;
                    }
                    else if (judgedTasksCount > (qualifiedTasksCount1 + qualifiedTasksCount2 + qualifiedTasksCount3))
                    {
                        reportItem.Judgement = "不合格";
                        rtns = true;
                    }
                    else if (judgedTasksCount == qualifiedTasksCount2)
                    {
                        reportItem.Judgement = "/";
                        rtns = true;
                    }
                    else if (judgedTasksCount == qualifiedTasksCount3)
                    {
                        reportItem.Judgement = "—";
                        rtns = true;
                    }
                    else
                    {
                        reportItem.Judgement = "合格";
                        rtns = true;
                    }
                    //若报告中的检验要求为判定，则处理
                    if (!report.TestNeedJudge) reportItem.Judgement = string.Empty;
                    //说明子项全部录入实测值了 大项不管有没有判定需要更新检测人信息
                    if (nodeTests.Count == testValuesCount)
                    {
                        //reportItem.TestCreateTime = nodeTests.Min(o => o.TestCreateTime);
                        //reportItem.TestCreatorName = nodeTests.FirstOrDefault().TestCreatorName;
                        //reportItem.TestCreatorKvid = nodeTests.FirstOrDefault().TestCreatorKvid;
                        rtns = true;
                    }
                }
            }
            catch
            {
                rtns = false;
            }
            return rtns;
        }
        public static List<ReportItem> UpdateChildrenSortId(this List<ReportItem> reportItems, int originalSortId, int newSortId, IDbConnection conn = null)
        {
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            }
            var rtns = new List<ReportItem>();
            foreach (var reportItem in reportItems)
            {
                reportItem.SortIdEx = reportItem.SortIdEx - originalSortId * 100 + newSortId * 100;
                reportItem.AddOnlyProperties(o => o.SortIdEx);
                conn.UpdateOnly(reportItem);
                rtns.Add(reportItem);
            }
            return rtns;
        }

        ///// <summary>
        ///// 本方法筛选出类型为Test的大项检测项目，或类型为TestFolder的大项
        ///// </summary>
        ///// <param name="reportItems"></param>
        ///// <returns></returns>
        //public static List<ReportItem> filterReportItem(this List<ReportItem> reportItems)
        //{
        //    var rtnReportItems = new List<ReportItem>();
        //    foreach (var en in reportItems)
        //    {
        //        if (en.Type == ReportItemType.TestFolder)
        //        {
        //            rtnReportItems.Add(en);
        //            continue;
        //        }
        //        if (en.Type == ReportItemType.Test)
        //        {
        //            if (en.ParentKvid == Guid.Empty)
        //            {
        //                rtnReportItems.Add(en);
        //                continue;
        //            }
        //            var parentTest = reportItems.Where(o => o.Kvid == en.ParentKvid).FirstOrDefault();
        //            if (parentTest == null) continue;
        //            if (parentTest.Type != ReportItemType.TestFolder)
        //            {
        //                rtnReportItems.Add(en);
        //                continue;
        //            }
        //        }
        //    }
        //    return rtnReportItems;
        //}
        /// <summary>
        /// 本方法筛选出类型为Test的大项检测项目，或类型为TestFolder的大项
        /// </summary>
        /// <param name="reportItems"></param>
        /// <returns></returns>
        public static List<ReportItem> filterReportItem(this List<ReportItem> reportItems)
        {
            var rtns = new List<ReportItem>();
            if (reportItems.IsNullOrEmpty()) return rtns;
            IDbConnection conn = null;
            
            try
            {
                foreach (var item in reportItems)
                {
                    if (item.Type == ReportItemType.TestFolder)
                    {
                        rtns.Add(item);
                        continue;
                    }
                    if (item.Type == ReportItemType.Test)
                    {
                        if (item.ParentKvid == Guid.Empty)
                        {
                            rtns.Add(item);
                            continue;
                        }
                        var parentTest = reportItems.FirstOrDefault(o => o.Kvid == item.ParentKvid);
                        ////如果原列表里未找到父级，需要去数据库里找到
                        if (parentTest == null)
                        {
                            if (conn == null) conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
                            parentTest = conn.Single<ReportItem>(o => o.Kvid == item.ParentKvid);
                        }
                        //要是仍然为空再跳过
                        if (parentTest == null) continue;
                        if (parentTest.Type != ReportItemType.TestFolder)
                        {
                            rtns.Add(item);
                            continue;
                        }
                    }
                }
            }
            finally
            {
                conn?.Close();
            }
            return rtns;
        }

        /// <summary>
        /// 本方法用作删除检验项目验证，只验证数据结构方面是否允许删除
        /// 只处理同一报告下的数据删除，不支持跨报告删除
        /// </summary>
        /// <param name="reportItems"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static bool ValidReportItemDelete_Stract(this List<ReportItem> reportItems, List<ReportItem> allReportItems)
        {
            reportItems.ThrowIfNullOrEmpty("空数据，无法处理");
            if (reportItems.Select(o => o.ReportKvid).Distinct().ToList().Count > 1) throw new Exception("验证不通过！暂不支持跨报告数据删除！");

            //待删除的项目 与所有项目对比，验证是否存在待删除项目的子项未在待删除中
            foreach(var en in reportItems)
            {
                var subReportItems = reportItems.Where(o => o.ParentKvid == en.Kvid);
                //2021-10-17升级，由于删除带部位的项目不会删除部位记录，所以验证这边需要处理此类脏数据
                var subAllReportItems = allReportItems.Where(o => o.ParentKvid == en.Kvid&&o.Type != ReportItemType.SamplePart);
                if (subReportItems.Count() < subAllReportItems.Count()) throw new Exception("验证不通过！此次删除会导致无父节点的子项存在！");
            }
            return true;
        }

        //2021-08-24 review replaced by up code
        //public static bool ValidReportItemDelete_Stract(this List<ReportItem> reportItems, IDbConnection conn = null)
        //{
        //    if (conn == null)
        //    {
        //        conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
        //    }
        //    reportItems.ThrowIfNullOrEmpty("空数据，无法处理");
        //    if (reportItems.Select(o => o.ReportKvid).Distinct().ToList().Count > 1) throw new Exception("验证不通过！暂不支持跨报告数据删除！");
        //    //查出当前报告下的所有的检验项目
        //    var queryAllReportItems = conn.From<ReportItem>();
        //    queryAllReportItems.Where(o => o.ReportKvid == reportItems.FirstOrDefault().ReportKvid);
        //    var allReportItems = conn.Select(queryAllReportItems);

        //    //待删除的项目 与所有项目对比，验证是否存在待删除项目的子项未在待删除中
        //    foreach (var en in reportItems)
        //    {
        //        var subReportItems = reportItems.Where(o => o.ParentKvid == en.Kvid);
        //        var subAllReportItems = allReportItems.Where(o => o.ParentKvid == en.Kvid);
        //        if (subReportItems.Count() != subAllReportItems.Count()) throw new Exception("验证不通过！此次删除会导致无父节点的子项存在！");
        //    }
        //    return true;
        //}

        /// <summary>
        /// 本方法用作删除检验项目验证，验证业务逻辑是否允许删除
        /// </summary>
        /// <param name="reportItems"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static bool ValidReportItemDelete_Biz(this Report report , List<ReportItem> reportItems)
        {
            reportItems.ThrowIfNullOrEmpty("空数据，无法处理");
            report.ThrowIfNull("报告为空无法处理！");
            //验证项目状态
            //if (reportItems.Exists(o => o.Type == ReportItemType.SampleSelf || o.Type == ReportItemType.SamplePart)) throw new Exception("验证不通过，请先删除色号或部位后，再删除项目！");
            //不验证色号项目或者部位项目的数据
            reportItems = reportItems.Where(o => o.Type != ReportItemType.SamplePart && o.Type != ReportItemType.SampleSelf).ToList();
            if (reportItems.Exists(o => o.StatusType != ReportItemStatusType.BeforeTest && o.StatusType != ReportItemStatusType.TestCollected)) throw new Exception("验证不通过！待删除项目存在非委托状态项目，禁止删除！");
            if(reportItems.Exists(o=>o.StatusType == ReportItemStatusType.TestCollected))
            {
                //处理删除X报告，不验证项目是否归档
                //处理引用报告逻辑，引用的项目，要支持自由删除项目，无论归档与否（）
                if (report.BizType != "H" && report.BizType != "X") throw new Exception("验证不通过！待删除项目中存在归档项目，禁止删除！");
            }
            foreach(var en in reportItems)
            {
                if (en.StatusType == ReportItemStatusType.BeforeTest && en.TestKvid != Guid.Empty) throw new Exception($"验证不通过！待删除的项目《{en.Title}》存在检测记录！");
            }
            return true;
        }

        //2021-08-24 review replaced by up code
        //public static bool ValidReportItemDelete_Biz(this List<ReportItem> reportItems, IDbConnection conn = null)
        //{
        //    if (conn == null)
        //    {
        //        conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
        //    }
        //    reportItems.ThrowIfNullOrEmpty("空数据，无法处理");
        //    //验证项目状态
        //    //if (reportItems.Exists(o => o.Type == ReportItemType.SampleSelf || o.Type == ReportItemType.SamplePart)) throw new Exception("验证不通过，请先删除色号或部位后，再删除项目！");
        //    if (reportItems.Exists(o => o.StatusType != ReportItemStatusType.BeforeTest && o.StatusType != ReportItemStatusType.TestCollected)) throw new Exception("验证不通过！待删除项目存在非委托状态项目，禁止删除！");
        //    if (reportItems.Exists(o => o.StatusType == ReportItemStatusType.TestCollected))
        //    {
        //        var report = conn.SingleById<Report>(reportItems.FirstOrDefault().ReportKvid);
        //        if (report.TestStartDate != null) throw new Exception("验证不通过！待删除项目中存在归档项目，禁止删除！");
        //    }
        //    foreach (var en in reportItems)
        //    {
        //        if (en.StatusType == ReportItemStatusType.BeforeTest && en.TestKvid != Guid.Empty) throw new Exception($"验证不通过！待删除的项目《{en.Title}》存在检测记录！");
        //    }
        //    return true;
        //}

        /// <summary>
        /// 报告项目流程进度描述
        /// </summary>
        /// <param name="report"></param>
        /// <returns></returns>
        public static string StatusDescription(int Status)
        {
            string description = "";
            switch (Status)
            {
                case 100:
                    description = "委托受理";
                    break;
                case 150:
                    description = "任务分派";
                    break;
                case 200:
                    description = "项目检验";
                    break;
                case 250:
                    description = "待复核";
                    break;
                case 300:
                    description = "检验完成";
                    break;
                case int.MaxValue:
                    description = "项目归档";
                    break;
                default:
                    description = "项目流程异常，请与管理员联系";
                    break;
            }
            return description;
        }

        #region 对ReportItems进行一些处理的方法，update by leo 
        /// <summary>
        /// 对ReportItems进行合并，把SamplePart合并至TitlePart中
        /// </summary>
        /// <param name="reportItems"></param>
        /// <returns>返回新的集合，必须再赋值给调用方</returns>
        internal static List<ReportItem> CombinSamplePart(this List<ReportItem> reportItems)
        {
            List<ReportItem> rtns = reportItems.ToList();//先复制，不影响原来的数据集
            //所有的部位信息
            var sampleParts = rtns.Where(o => o.Type == ReportItemType.SamplePart);
            if (sampleParts.Count() < 1) return rtns;//如果没找到部位信息，直接返回原对象

            foreach (var samplePart in sampleParts)
            {
                var tests = rtns.Where(o => o.ParentKvid == samplePart.Kvid);//找到所有部位下的子对象集合
                //把所有的子对象的属性值进行重新连接
                tests.Each(o => {
                    o.TitlePart = samplePart.Title;
                    o.ParentKvid = samplePart.ParentKvid;
                    o.PartKvid = samplePart.Kvid;
                });

            }
            //把所有的样品信息从集合中删除
            sampleParts.Each(o=>rtns.Remove(o));

            return rtns;
        }
        /// <summary>
        /// 移除目录中没有实际检测项的样品或样品部位
        /// </summary>
        /// <param name="reportItems"></param>
        /// <returns>返回新的集合，必须再赋值给调用方</returns>
        internal static List<ReportItem> RemoveEmptyFolder(this List<ReportItem> reportItems)
        {
            List<ReportItem> rtns = reportItems.ToList();//先复制，不影响原来的数据集

            //样品或样品部位都是树型的根（目录节点），当这些目录节点下没有检测项，就不需要显示
            //根据RootKvid进行分组，再查这些分组下有没有检测项，如果没有，直接可以从集合中移除
            var groups = rtns.GroupBy(o => o.RootKvid);
            foreach (var kv in groups)
            {
                //如果这个组中存在Test或TestFolder,说明就有检测项，跳过
                if (kv.Any(o => o.Type == ReportItemType.Test || o.Type == ReportItemType.TestFolder)) continue;
                //没有检测项的情况，把整个根都移除了，即RootKvid相同的都移除了
                rtns.RemoveAll(o => o.RootKvid == kv.Key);
            }
            return rtns;
        }

        /// <summary>
        /// 验证树形结构的数据有严谨性，如果数据上下级结构不正确，可能会引起客户端显示有问题
        /// </summary>
        /// <param name="reportItems"></param>
        /// <returns></returns>
        internal static bool ValidIsTreeStruct(this List<ReportItem> reportItems)
        {
            bool rtns = false;
            var kvids = reportItems.Select(o => o.ParentKvid);//先找到所有的上级Kvid
            if (kvids.Count() != kvids.Distinct().Count()) return rtns;//如果kvids集合的长度不等于Distince的长度，说明数据集中有重复的Kvid
            var parentKvids = reportItems.Select(o => o.ParentKvid).Distinct();//先找到所有的上级Kvid
            rtns=!parentKvids.Any(o => !kvids.Contains(o));//只要Kvids中不存在任何一个ParentKvid的数据，就说明找不到上级的对象，
            return rtns;
        }
        /// <summary>
        /// 返回所有的大项，Test单项，或包含Test的TestFolder
        /// </summary>
        /// <param name="reportItems"></param>
        /// <returns></returns>
        internal static List<ReportItem> GetMainTest(this List<ReportItem> reportItems)
        {
            List<ReportItem> rtns = new List<ReportItem>();

            //对所有项目循环
            foreach (var reportItem in reportItems)
            {
                //如果是TestFolder,直接加入返回数据集中
                if (reportItem.Type == ReportItemType.TestFolder)
                {
                    rtns.Add(reportItem);
                    continue;
                }
                //如果不是Test,直接继续，因为只判断TestFolder和Test,
                if (reportItem.Type!= ReportItemType.Test) continue;

                //如果上级ParentKvid为空，根项目，加入返回集，继续
                if (reportItem.ParentKvid == Guid.Empty)
                {
                    rtns.Add(reportItem);
                    continue;
                }
                //如果ParentKvid为不空，有可能是Sample或SamplePart的上级，这种也是需要加入数据集的，只要不是TestFolder就可以
                var parentTest = reportItems.FirstOrDefault(o => o.Kvid == reportItem.ParentKvid);
                parentTest.ThrowIfNull("传入的数据集不完整或数据结构错误");//不做额外的处理，需要在传入的数据集上传完整。

                if (parentTest.Type== ReportItemType.SampleSelf|| parentTest.Type == ReportItemType.SamplePart)
                {
                    rtns.Add(reportItem);
                    continue;
                }
            }
            //下面可以对TitlePart进行赋值，合成SampleSelf和SamplePart的信息
            //可以对ParentKvid进行集中清空，建立一个完整的数据结构
            rtns.Each(o => o.ParentKvid = Guid.Empty);
            return rtns;
        }
        #endregion

        #region 对ReportItem的检验组相关处理的一些方法，update by rty
        internal static List<ReportItem> SyncWorkGroup(this List<ReportItem> reportItems)
        {
            List<ReportItem> rtns = new List<ReportItem>();
            //遍历所有项目
            foreach(var kv in reportItems)
            {
                //只需要处理带子项的大项
                if (kv.Type != ReportItemType.TestFolder) continue;
                var subItems = new List<ReportItem>();
                subItems = reportItems.Where(o => o.ParentKvid == kv.Kvid).ToList();
                if (subItems == null) continue;
                if (subItems.Exists(o => o.WorkGroupName != kv.WorkGroupName && o.WorkGroupName != null && o.WorkGroupName != string.Empty))
                {
                    foreach (var en in subItems)
                    {
                        if (en.WorkGroupName == kv.WorkGroupName) continue;
                        en.WorkGroupName = kv.WorkGroupName;
                        en.WorkGroupKvid = kv.WorkGroupKvid;
                        en.AddOnlyProperties(o => o.WorkGroupName);
                        en.AddOnlyProperties(o => o.WorkGroupKvid);
                        rtns.Add(en);
                    }
                }
            }
            return rtns;
        }
        #endregion

        #region 对检验项目的Amount字段更新，添加日志记录的方法
        public static void SetAmountLog(this ReportItem reportItem,Report report,string type,decimal originalAmount,decimal finalAmount, IDbConnection conn = null)
        {
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            }

            var _log = new EntityLog<ReportItem>();
            try
            {
                _log.BizId = report.ReportId;
            }
            catch { }
            try
            {
                _log.BizKvid = report.Kvid;
            }
            catch { }
            try
            {
                _log.OwnerKvid = reportItem.Kvid;
            }
            catch { }
            try
            {
                _log.Source = reportItem.Title.Substring(0,49);
            }
            catch { }
            _log.BizType = "Kivii.Lac.Lims.Entities.Report";
            _log.Title = "收费更新日志";
            _log.Type = type;
            _log.Summary = $"检验费由{originalAmount}修改为{finalAmount}";
            conn.Insert(_log);
        }
        #endregion

        #region 标准值更新，添加日志记录的方法
        //标准值更新,创建日志记录
        public static void SetDetectionValueQualifiedLog(this ReportItem reportItem,string oldValue,string newValue, IDbConnection conn = null)
        {
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            }

            var _log = new EntityLog<ReportItem>();
            try
            {
                _log.OwnerKvid = reportItem.Kvid;
                _log.BizId = reportItem.Title;
                _log.BizKvid = reportItem.Kvid;
                _log.BizType = "Kivii.Lac.Lims.Entities.ReportItem";

            }
            catch { }
            _log.Title = "标准值操作日志";
            _log.Type = "OperateDetectionValueQualified";
            _log.Summary = $"标准值由“{oldValue}”修改为“{newValue}”";
            if(_log.Summary.Length>500)_log.Summary= _log.Summary.Substring(0, 450) + "···";
            conn.Insert(_log);
        }
        //客户要求标准值更新,创建日志记录
        public static void SetDetectionValueQualifiedCustomerRequiredLog(this ReportItem reportItem, string type, string updateContent, IDbConnection conn = null)
        {
            if (reportItem.Metadata == null) return;
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            }

            var _log = new EntityLog<ReportItem>();
            try
            {
                _log.OwnerKvid = reportItem.Kvid;
                _log.BizId = reportItem.Title;
                _log.BizKvid = reportItem.Kvid;
                _log.BizType = "Kivii.Lac.Lims.Entities.ReportItem";

            }
            catch { }
            _log.Title = "客户要求标准值操作日志";
            _log.Type = "OperateDetectionValueQualifiedCustomerRequired";
            if (type == "create")
            {
                updateContent = string.Empty;
                if (reportItem.Metadata.ContainsKey("DetectionValueCustomer") || !reportItem.Metadata["DetectionValueCustomer"].IsNullOrEmpty()) updateContent= reportItem.Metadata["DetectionValueCustomer"];
                updateContent = "标准值初始值：" + updateContent;
                _log.Summary = updateContent;
            }
            if (type == "update") _log.Summary = updateContent;
            if (_log.Summary.Length > 500) _log.Summary = _log.Summary.Substring(0, 450) + "···";
            conn.Insert(_log);
        }
        #endregion

        #region 取消标准值更新标记
        /// <summary>
        /// 数据录入下达时,取消标准值更新标记
        /// </summary>
        /// <param name="report"></param>
        /// <param name="conn"></param>
        public static void ResetTagUpdateDetectionValueQualified(this Report report, IDbConnection conn = null)
        {
            report.ThrowIfNull("未获取报告信息!");
            if (conn == null)
            {
                conn = KiviiContext.GetOpenedDbConnection<Report>();
            }
            var allReportItems = report.GetOriginalReportItems(conn);
            var queryTestRoles = conn.From<EntityDuty<Test>>();
            queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
            queryTestRoles.Select(o => o.RoleName);
            var testRoleNames = conn.Select(queryTestRoles).ConvertAll(o => o.RoleName);
            //所在组的全部项
            var ownerPrimaryItems = allReportItems.Where(o => o.WorkGroupKvid == KiviiContext.CurrentMember.DepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames)).ToList();
            //获取修改过标准值的数据
            ownerPrimaryItems = ownerPrimaryItems.Where(o=>o.Metadata!=null&&o.Metadata.ContainsKey("IsUpdateDetectionValueQualified")&&o.Metadata["IsUpdateDetectionValueQualified"]=="true").ToList();
            foreach (var reportItem in ownerPrimaryItems)
            {
                reportItem.RemoveAllOnlyProperties();
                reportItem.Metadata["IsUpdateDetectionValueQualified"]=null;
                reportItem.AddOnlyProperties(o=>o.Metadata);
                conn.UpdateOnly(reportItem);
            }
        }
        #endregion
    }
}
