﻿using Kivii.Lims.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lac.Lims.Extensions
{
    /// <summary>
    /// 报告明细格式化筛选扩展类，使用internal 类型，说明这些扩展方法只能在本动态库中使用
    /// </summary>
    internal static class ReportItemFormatterExtensions
    {
        /// <summary>
        /// 获取检验大项集合,不包含树形结构。
        /// </summary>
        /// <param name="allReportItems">必须是一个报告的全项</param>
        /// <returns></returns>
        public static List<ReportItem> GetMainTest(this List<ReportItem> allReportItems)
        {
            var rtns = new List<ReportItem>();
            if (allReportItems.IsNullOrEmpty()) return rtns;

            foreach (var item in allReportItems)
            {
                if (item.Type == ReportItemType.TestFolder)
                {
                    rtns.Add(item);
                    continue;
                }
                if (item.Type == ReportItemType.Test)
                {
                    if (item.ParentKvid == Guid.Empty)
                    {
                        rtns.Add(item);
                        continue;
                    }
                    var parentTest = allReportItems.FirstOrDefault(o => o.Kvid == item.ParentKvid);
                    ////如果原列表里未找到父级，需要去数据库里找到
                    if (parentTest == null)
                    {
                    }
                    //要是仍然为空再跳过
                    if (parentTest == null) continue;
                    if (parentTest.Type != ReportItemType.TestFolder)
                    {
                        rtns.Add(item);
                        continue;
                    }
                }
            }
            return rtns;
        }

        public static List<ReportItemEx> getTreeItemsEx(this List<ReportItemEx> datasource, Guid kvid)
        {
            var rtns = datasource.Where(o => o.ParentKvid == kvid).OrderBy(o => o.ReportKvid).OrderBy(o => o.SortId).ThenBy(o => o.SortIdEx).ThenBy(o => o.Title).ToList();
            if (rtns.IsNullOrEmpty()) return null;
            foreach (var rtn in rtns)
            {
                rtn.SubItemExs = getTreeItemsEx(datasource, rtn.Kvid);
            }
            return rtns;
        }
        public static void getPlatItemsEx(this List<ReportItemEx> treeItems, List<ReportItemEx> rtns)
        {
            if (treeItems.IsNullOrEmpty()) return;
            foreach (var item in treeItems)
            {
                rtns.Add(item);
                getPlatItemsEx(item.SubItemExs, rtns);
                item.SubItemExs = null;
            }
        }

    }
}
