using Kivii.Lims.Entities;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using Kivii.Linq;
using Kivii.Lac.Lims.Extensions;
using System.Linq.Expressions;
using System.IO;
using Kivii;
using System.Text.RegularExpressions;
using Kivii.Zip;
using Kivii.Lims;

namespace Kivii.Lac.Lims.Transforms
{
    #region CDUQD
    [Api(Description = "报告创建")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportCreate : RestfulExecution<Report>
    {
        public Report Item { get; set; }

        public List<Report> Items { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("Need Item Or Items!");

            if (Items.IsNullOrEmpty()) Items = new List<Report>();
            if (Item != null) Items.Add(Item);

            var rtns = new RestfulCreateResponse<Report>();
            rtns.Results = new List<Report>();

            //判断要创建的报告是否传入了样品号，如果传入了，检查样品信息
            var sampleKvids = Items.ConvertAll(o => o.SampleKvid);
            sampleKvids.RemoveAll(o => o == Guid.Empty);
            sampleKvids = sampleKvids.Distinct().ToList();

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var existSamples = conn.SelectByIds<Sample>(sampleKvids);
            IDbTransaction trans = null;

            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);

            try
            {
                foreach (var item in Items)
                {
                    //若存在样品信息则不创建新的样品,否则创建新的样品信息
                    var existSample = existSamples.FirstOrDefault(o => o.Kvid == item.SampleKvid);

                    if (existSample == null) {//需要提前给Report设置一个新的
                        item.SampleKvid = Guid.NewGuid();
                        item.AddOnlyProperties(o => o.SampleKvid);
                    }
                    conn.InsertOnly(item);//因为样品的编号是跟随ReportId的，所以先要插入报告，才能生成报告号。
                    if (existSample != null) continue;

                    //创建样品
                    var sample = new Sample();
                    sample.Kvid = item.SampleKvid;
                    sample.SourceKvid = item.SampleSourceKvid;
                    sample.RootKvid = sample.Kvid;
                    sample.ParentKvid = Guid.Empty;
                    sample.SourceBatchNumber = item.SampleSourceBatchNumber;
                    sample.Name = item.SampleName;
                    sample.Type = string.Empty;
                    sample.BatchNumber = item.SampleBatchNumber;
                    sample.SerialNumber = item.ReportId;
                    sample.Brand = item.SampleBrand;
                    sample.Specification = string.Empty;
                    sample.Model = item.SampleModel;
                    sample.Material = string.Empty;
                    sample.Grade = item.SampleGrade;
                    sample.GradeSafety = item.SampleGradeSafety;
                    sample.Quantity = item.SampleQuantity;
                    sample.QuantityUnit = item.SampleQuantityUnit;
                    sample.ManufacturerKvid = item.SampleManufacturerKvid;
                    sample.ManufacturerName = item.SampleManufacturerName;
                    sample.ManufacturerContactKvid = item.SampleManufacturerContactKvid;
                    sample.ManufacturerContactName = item.SampleManufacturerContactName;
                    sample.ManufacturerContactNumber = item.SampleManufacturerContactNumber;
                    sample.ManufacturerAddress = item.SampleManufacturerAddress;
                    conn.Insert(sample);
                    rtns.Results.Add(item);
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    //代码初审 张梦圆 20210904
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportDelete : RestfulDelete<Report>
    {
        IDbTransaction trans = null;
        public override bool OnPreRestfulDelete(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulDeleteResponse<Report> rtns)
        {
            //删除逻辑需要完善？？？？？？？？？？？？？？？？？？？？？？？？？
            #region 数据验证
            //只能删除委托状态报告
            if (dbConnection.Exists<Report>(o => Sql.In(o.Kvid, Kvids) && o.Status > (int)ReportStatusType.CommissionAccept)) throw new Exception("待删除报告中存在非委托状态的报告，禁止删除！");
            //获取待删除报告集合
            var queryReports = dbConnection.From<Report>();
            queryReports.Where(o => Sql.In(o.Kvid, Kvids));
            var reports = dbConnection.Select(queryReports);
            //获取待删除检验项目集合
            var queryAllReportItems = dbConnection.From<ReportItem>();
            queryAllReportItems.Where(o => Sql.In(o.ReportKvid, Kvids));
            var allReportItems = dbConnection.Select(queryAllReportItems);
            //检验项目删除验证
            foreach (var kvid in Kvids)
            {
                var reportItems = allReportItems.Where(o => o.ReportKvid == kvid).ToList();
                if (reportItems.IsNullOrEmpty()) continue;
                var report = reports.FirstOrDefault(o => o.Kvid == kvid);
                report.ThrowIfNull("未找到匹配报告,删除失败!");
                report.ValidReportItemDelete_Biz(reportItems);
            }
            //获取待删除关联样品集合
            //先判断是否存在引用了待删除报告样品号的报告,如果有,忽略更新样品信息
            var sampleKvids = reports.ConvertAll(o => o.SampleKvid);
            var existAllReports = dbConnection.Select<Report>(o => Sql.In(o.SampleKvid, sampleKvids));

            if (dbConnection is LinqConnection && (dbConnection as LinqConnection).DbTransaction == null)
            {
                trans = dbConnection.OpenTransaction(IsolationLevel.ReadUncommitted);
            }
            //trans = dbConnection.OpenTransaction(IsolationLevel.ReadUncommitted);
            if (existAllReports.Count == reports.Count)
            {
                var updateSampleLamda = dbConnection.From<Sample>();
                updateSampleLamda = updateSampleLamda.Update(o => o.Status);
                updateSampleLamda = updateSampleLamda.Where(o => Sql.In(o.RootKvid, sampleKvids));
                dbConnection.UpdateOnly(new Sample { Status = -1 }, updateSampleLamda);
            }
            else
            {
                foreach (var report in reports)
                {
                    var existReports = existAllReports.Where(o => o.SampleKvid == report.SampleKvid).ToList();
                    if (existReports.Count > 1) continue;
                    var updateSampleLamda = dbConnection.From<Sample>();
                    updateSampleLamda = updateSampleLamda.Update(o => o.Status);
                    updateSampleLamda = updateSampleLamda.Where(o => o.RootKvid == report.SampleKvid);
                    dbConnection.UpdateOnly(new Sample { Status = -1 }, updateSampleLamda);
                }
            }

            #endregion
            return base.OnPreRestfulDelete(req, res, dbConnection, rtns);
        }
        public override bool OnPostRestfulDelete(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulDeleteResponse<Report> rtns)
        {
            try
            {
                //删除ReportItem
                var updateReportItemLamda = dbConnection.From<ReportItem>();
                updateReportItemLamda = updateReportItemLamda.Update(o => o.Status);
                updateReportItemLamda = updateReportItemLamda.Where(o => Sql.In(o.ReportKvid, Kvids));
                dbConnection.UpdateOnly(new ReportItem { Status = -1 }, updateReportItemLamda);

                //修改图片集合-报告相关状态
                var updateReportFileLamda = dbConnection.From<EntityDbFile<Report>>();
                updateReportFileLamda = updateReportFileLamda.Update(o => o.Status);
                updateReportFileLamda = updateReportFileLamda.Where(o => Sql.In(o.OwnerKvid, Kvids));
                dbConnection.UpdateOnly(new EntityDbFile<Report> { Status = -1 }, updateReportFileLamda);

                trans?.Commit();
                return base.OnPostRestfulDelete(req, res, dbConnection, rtns);
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            finally
            {
                trans?.Dispose();
            }
        }
    }
    //代码初审 张梦圆 20210904
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportUpdate : RestfulUpdate<Report>
    {
        //修改报告号的时候,如果存在子样品,需要记录更改前的报告号
        public string originalReportId = string.Empty;
        public string originalReportRequied = string.Empty;
        //因加急类型需要记录日志，需要记录原加急类型，如果不一致才要记录日志
        public decimal originalUrgentRate = 0;
        public override bool OnPreRestfulUpdate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulUpdateResponse<Report> rtns)
        {
            var originalReport = dbConnection.SingleById<Report>(Item.Kvid);
            originalReport.ThrowIfNull("参数错误:未找到对应报告!");
            if (Item.OnlyPropertiesIsExist(o => o.ReportId)) originalReportId = originalReport.ReportId;
            if (originalReport.Metadata != null && originalReport.Metadata.ContainsKey("ReportRequire")) originalReportRequied = originalReport.Metadata["ReportRequire"];
            originalUrgentRate = originalReport.UrgentRate;
            return base.OnPreRestfulUpdate(req, res, dbConnection, rtns);
        }
        public override bool OnPostRestfulUpdate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulUpdateResponse<Report> rtns)
        {
            #region 数据验证
            var report = dbConnection.SingleById<Report>(Item.Kvid);
            report.ThrowIfNull("参数错误:未找到对应报告!");
            #endregion
            //提升为系统功能？？？？？？？？？？？？？？？？？？？？？？？？？？
            if (Item.OnlyPropertiesIsExist(o => o.UrgentRate) && originalUrgentRate != Item.UrgentRate)
            {
                var _log = new EntityLog<Report>();
                _log.OwnerKvid = Item.Kvid;
                _log.Title = "加急系数";
                _log.Type = "更新";
                _log.Summary = $"修改加急类型为{Item.UrgentRate}倍加急；";
                dbConnection.Insert(_log);
            }
            if (Item.OnlyPropertiesIsExist(o => o.Metadata) && Item.Metadata.ContainsKey("ReportRequire") && Item.Metadata["ReportRequire"] != originalReportRequied)
            {
                var _log = new EntityLog<Report>();
                _log.OwnerKvid = Item.Kvid;
                _log.Title = "报告要求";
                _log.Type = "更新";
                _log.Summary = $"修改报告要求为{Item.Metadata["ReportRequire"]}；";
                dbConnection.Insert(_log);
            }
            //当报告修了检验依据，因报告结论是根据检验依据生成的，所以自动清空结论。避免前后不一致，升级为标准版的功能。
            if (Item.OnlyPropertiesIsExist(o => o.TestStandards))
            {
                Item.Conclusion = string.Empty;
                Item.AddOnlyProperties(o => o.Conclusion);
            }
            //如果当前引用的样品存在其他报告引用,不允许修改
            var count = dbConnection.Scalar<Report, int>(o => Sql.Count(o.Kvid), o => o.SampleKvid == report.SampleKvid);
            if (count != 1) return base.OnPostRestfulUpdate(req, res, dbConnection, rtns);
            //更新样品信息
            Item.SampleKvid = report.SampleKvid;
            Item.UpdateSampleInfos(originalReportId, dbConnection);
            return base.OnPostRestfulUpdate(req, res, dbConnection, rtns);
        }
    }
    //代码初审 张梦圆 20210906
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportQuery : RestfulQuery<Report>
    {
        //是否根据当前人所在组筛选报告
        public bool isFilterByWorkGroup { get; set; }
        //是否过滤更改报告
        public bool isFilterByNotChange { get; set; }
        public ReportStatusType filterStatus { get; set; }
        public string filterStatusEx { get; set; }
        public override bool OnPreRestfulQuery(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Report> rtns, ref Expression<Func<Report, bool>> where)
        {
            if (isFilterByNotChange)
            {
                where = o => (o.BizType != "X" && o.BizType != "G0" && o.BizType != "G1") || o.BizType == null;
                return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
            }
            //获取当前人是否具有超级管理员的权限
            var session = Request.GetSession();
            var isAdministrator = session.HasRole(SystemRoles.Administrator, null);
            if (isFilterByWorkGroup)
            {
                if (isAdministrator) return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
                //获取当前登录人的部门Kvid
                var currentDepartmentKvid = KiviiContext.CurrentMember.DepartmentKvid;
                //查询数据录入报告Kvids集合
                var queryReportKvidsDataEntry = dbConnection.From<Report>();
                queryReportKvidsDataEntry.Where(o => o.Status == (int)ReportStatusType.DataEntry);
                queryReportKvidsDataEntry.Select(o => o.Kvid);
                //查询所有符合条件的报告Kvid
                var queryReportKvids = dbConnection.From<ReportItem>();
                queryReportKvids.Where(o => Sql.In(o.ReportKvid, queryReportKvidsDataEntry) && o.WorkGroupKvid == currentDepartmentKvid);
                queryReportKvids.SelectDistinct(o => o.ReportKvid);
                where = o => Sql.In(o.Kvid, queryReportKvids);
                return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
            }
            //DataEntry状态的报告的筛选条件需要单独处理
            if (filterStatus != ReportStatusType.Unsupported)
            {
                if (filterStatus != ReportStatusType.DataEntry) return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
                if (filterStatusEx == "All") return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
                var queryTestRoles = dbConnection.From<EntityDuty<Test>>();
                queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
                queryTestRoles.Select(o => o.RoleName);
                var testRoleNames = dbConnection.Select(queryTestRoles).ConvertAll(o => o.RoleName);

                //获取当前登录人的部门Kvid
                var currentDepartmentKvid = KiviiContext.CurrentMember.DepartmentKvid;
                var queryReportKvidsByReportItem = dbConnection.From<ReportItem>();
                queryReportKvidsByReportItem.Where(o => o.Type != ReportItemType.SampleSelf && o.Type != ReportItemType.SamplePart && o.Type != ReportItemType.Unsupported);
                queryReportKvidsByReportItem.SelectDistinct(o => o.ReportKvid);

                //根据打回记录查询打回的任务；
                var queryReportKvidsByReportNote = dbConnection.From<ReportNote>();
                queryReportKvidsByReportNote.Where(o => o.Title == "项目打回" && o.From == "项目检验" && o.To == "委托受理");
                queryReportKvidsByReportNote.SelectDistinct(o => o.ReportKvid);
                //分为所有任务（All）、待处理任务（Pending）、已处理任务（Processed）、已完成任务（Finished）、待复核任务（Reviewing）
                //2023年4月26新增数据录入已打回的任务查询：已打回任务（Callbacked）
                //待复核任务是指该角色所在组的报告的所有项目均已进入待复核状态或之后
                if (filterStatusEx == "Pending" || filterStatusEx.IsNullOrEmpty())
                {
                    if (!isAdministrator) queryReportKvidsByReportItem.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    queryReportKvidsByReportItem.And(o => (o.Status == (int)ReportItemStatusType.Testing || o.Status == (int)ReportItemStatusType.BeforeReview || o.Status == (int)ReportItemStatusType.TestFinished));
                    where = o => Sql.In(o.Kvid, queryReportKvidsByReportItem);
                }
                else if (filterStatusEx == "Processed")
                {
                    if (!isAdministrator) queryReportKvidsByReportItem.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    queryReportKvidsByReportItem.And(o => o.Status == int.MaxValue);
                    where = o => o.Status < int.MaxValue && (Sql.In(o.Kvid, queryReportKvidsByReportItem));
                }
                else if (filterStatusEx == "Finished")
                {
                    if (!isAdministrator) queryReportKvidsByReportItem.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    where = o => o.Status == int.MaxValue && Sql.In(o.Kvid, queryReportKvidsByReportItem);
                }
                else if (filterStatusEx == "Callbacked")
                {
                    if (!isAdministrator) queryReportKvidsByReportItem.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    where = o => o.Status == (int)ReportStatusType.CommissionAccept && Sql.In(o.Kvid, queryReportKvidsByReportItem) && Sql.In(o.Kvid, queryReportKvidsByReportNote);
                }
                //以下逻辑：只要有待复核的项目就能查出来报告，不符合实际要求
                else if (filterStatusEx == "Reviewing")
                {
                    var queryReportTestingKvids = dbConnection.From<ReportItem>();
                    queryReportTestingKvids.Where(o => o.Type != ReportItemType.SampleSelf && o.Type != ReportItemType.SamplePart && o.Type != ReportItemType.Unsupported);
                    queryReportTestingKvids.SelectDistinct(o => o.ReportKvid);
                    queryReportTestingKvids.Where(o => (o.Status == (int)ReportItemStatusType.Testing));
                    if (!isAdministrator)
                    {
                        queryReportKvidsByReportItem.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                        queryReportTestingKvids.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    }
                    queryReportKvidsByReportItem.And(o => (o.Status == (int)ReportItemStatusType.Testing || o.Status == (int)ReportItemStatusType.BeforeReview || o.Status == (int)ReportItemStatusType.TestFinished));
                    where = o => Sql.In(o.Kvid, queryReportKvidsByReportItem) && !Sql.In(o.Kvid, queryReportTestingKvids);

                }
                //实际要求：所有项目的状态都在待复核，或部分项目待复核，部分项目已完成的报告筛选；只要复核完报告就能点下达完成的报告筛选
                //定制需求，加入一种筛选，要求筛选出本组待处理任务，且若有纤维含量的任务，纤维含量已经完成的任务；
                else if (filterStatusEx == "FibreContentFinishedPending")
                {
                    //testRoleNames.Add("纤维含量组");
                    //var queryKvidsByReportItemFibre = dbConnection.From<ReportItem>();
                    //queryKvidsByReportItemFibre.Where(o => o.WorkGroupName == "纤维含量组" && o.Status == (int)ReportItemStatusType.TestCollected);
                    //queryKvidsByReportItemFibre.Select(o => o.Kvid);
                    if (!isAdministrator) queryReportKvidsByReportItem.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    queryReportKvidsByReportItem.And(o => (o.Status == (int)ReportItemStatusType.Testing || o.Status == (int)ReportItemStatusType.BeforeReview || o.Status == (int)ReportItemStatusType.TestFinished));
                    //queryReportKvidsByReportItem.And(o => Sql.In(o.Kvid, queryKvidsByReportItemFibre));
                    var queryResultKvid = dbConnection.From<ReportItem>();
                    queryResultKvid.Where(o => Sql.In(o.ReportKvid, queryReportKvidsByReportItem) && o.WorkGroupName == "纤维含量组" && o.Status == (int)ReportItemStatusType.TestCollected);
                    queryResultKvid.Select(o => o.ReportKvid);
                    where = o => Sql.In(o.Kvid, queryResultKvid);
                }
                else
                {
                    throw new Exception("filterStatusEx参数异常！");
                }
            }
            return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
        }
    }

    //[RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    //public class ReportQuery2 : RestfulQuery<Report>
    //{
    //    //是否根据当前人所在组筛选报告
    //    public bool isFilterByWorkGroup { get; set; }
    //    //是否过滤更改报告
    //    public bool isFilterByNotChange { get; set; }
    //    public ReportStatusType filterStatus { get; set; }
    //    public string filterStatusEx { get; set; }
    //    public override bool OnPreRestfulQuery(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Report> rtns, ref Expression<Func<Report, bool>> where)
    //    {
    //        if (isFilterByWorkGroup)
    //        {
    //            //判断当前登录人是否具有超级管理员的权限
    //            var currentMember = KiviiContext.CurrentMember;
    //            var roles = ((Kivii.AuthUserSession)currentMember).Roles;
    //            if (roles.Contains("C97F15E4-420D-0000-AF96-B04863289A4B"))
    //            {
    //                return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
    //            }
    //            //获取当前登录人的部门Kvid
    //            var currentDepartmentKvid = KiviiContext.CurrentMember.DepartmentKvid;
    //            //查询所有数据录入状态下的报告Kvid
    //            var queryAllReportKvids = dbConnection.From<Report>();
    //            queryAllReportKvids.Where(o => o.Status == 400);
    //            queryAllReportKvids.Select(o => o.Kvid);
    //            //查询所有符合条件的报告Kvid
    //            var queryReportKvids = dbConnection.From<ReportItem>();
    //            queryReportKvids.Where(o => Sql.In(o.ReportKvid, queryAllReportKvids) && o.WorkGroupKvid == currentDepartmentKvid);
    //            queryReportKvids.SelectDistinct(o => o.ReportKvid);
    //            where = o => Sql.In(o.Kvid, queryReportKvids);
    //        }
    //        if (isFilterByNotChange)
    //        {
    //            where = o => !o.ReportId.Contains("-X");
    //        }

    //        //DataEntry状态的报告的筛选条件需要单独处理
    //        if (filterStatus != ReportStatusType.Unsupported)
    //        {
    //            if (filterStatus != ReportStatusType.DataEntry) return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
    //            var currentMember = KiviiContext.CurrentMember;
    //            var roles = ((Kivii.AuthUserSession)currentMember).Roles;
    //            var queryTestRoles = dbConnection.From<EntityDuty<Test>>();
    //            queryTestRoles.Where(o => o.OwnerKvid == currentMember.Kvid);
    //            queryTestRoles.Select(o => o.RoleName);
    //            var testRoleNames = dbConnection.Select(queryTestRoles).ConvertAll(o => o.RoleName);

    //            //获取当前登录人的部门Kvid
    //            var currentDepartmentKvid = KiviiContext.CurrentMember.DepartmentKvid;
    //            var queryReportKvids = dbConnection.From<ReportItem>();
    //            queryReportKvids.Where(o => o.Type != ReportItemType.SampleSelf && o.Type != ReportItemType.SamplePart && o.Type != ReportItemType.Unsupported);
    //            queryReportKvids.SelectDistinct(o => o.ReportKvid);
    //            //分为所有任务（All）、待处理任务（Pending）、已处理任务（Processed）、已完成任务（Finished）、待复核任务（Reviewing）
    //            //待复核任务是指该角色所在组的报告的所有项目均已进入待复核状态或之后
    //            if (filterStatusEx == "All")
    //            {
    //                if (roles.Contains("C97F15E4-420D-0000-AF96-B04863289A4B"))
    //                {
    //                    queryReportKvids.Where(o => (o.Status >= 200));
    //                }
    //                else
    //                {
    //                    queryReportKvids.Where(o => (o.Status >= 200) && (o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames)));
    //                }
    //                where = o => (Sql.In(o.Kvid, queryReportKvids));
    //            }
    //            else if (filterStatusEx == "Pending" || filterStatusEx.IsNullOrEmpty())
    //            {
    //                if (roles.Contains("C97F15E4-420D-0000-AF96-B04863289A4B"))
    //                {
    //                    queryReportKvids.Where(o => (o.Status == 200 || o.Status == 250 || o.Status == 300));
    //                }
    //                else
    //                {
    //                    queryReportKvids.Where(o => (o.Status == 200 || o.Status == 250 || o.Status == 300) && (o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames)));
    //                }
    //                where = o => (Sql.In(o.Kvid, queryReportKvids));
    //            }
    //            else if (filterStatusEx == "Processed")
    //            {
    //                if (roles.Contains("C97F15E4-420D-0000-AF96-B04863289A4B"))
    //                {
    //                    queryReportKvids.Where(o => o.Status == int.MaxValue);
    //                }
    //                else
    //                {
    //                    queryReportKvids.Where(o => o.Status == int.MaxValue && (o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames)));
    //                }
    //                where = o => o.Status < int.MaxValue && (Sql.In(o.Kvid, queryReportKvids));
    //            }
    //            else if (filterStatusEx == "Finished")
    //            {
    //                if (!roles.Contains("C97F15E4-420D-0000-AF96-B04863289A4B"))
    //                {
    //                    queryReportKvids.Where(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
    //                }
    //                where = o => o.Status == int.MaxValue && (Sql.In(o.Kvid, queryReportKvids));
    //            }
    //            else if (filterStatusEx == "Reviewing")
    //            {
    //                if (roles.Contains("C97F15E4-420D-0000-AF96-B04863289A4B"))
    //                {
    //                    queryReportKvids.Where(o => (o.Status >= 250 && o.Status < 300));
    //                }
    //                else
    //                {
    //                    queryReportKvids.Where(o => (o.Status >= 250 && o.Status < 300) && (o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames)));
    //                }
    //                where = o => (Sql.In(o.Kvid, queryReportKvids));
    //            }
    //            else
    //            {
    //                throw new Exception("filterStatusEx参数异常！");
    //            }
    //        }
    //        return base.OnPreRestfulQuery(req, res, dbConnection, rtns, ref where);
    //    }
    //}
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportRead : RestfulRead<Report> { }

    #endregion

    #region 同步报告信息(金额+检验依据等信息)
    //代码初审 张梦圆 20210906
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "同步报告")]
    public class ReportSync : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            ReportKvid.ThrowIfEmpty("请传入要同步的报告!");
            #endregion
            //获取报告信息
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            if (report == null) throw new Exception();
            report.CalculateReportStandard(conn);
            report.CalculateReportAmount(conn);
            conn.UpdateOnly(report);
            report.RemoveAllOnlyProperties();
            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            rtns.Results.Add(report);
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 同步报告信息(金额)
    //代码初审 张梦圆 20210906
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "同步报告金额")]
    public class ReportAmountSync : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }

        public List<Guid> ReportKvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (ReportKvid == Guid.Empty && ReportKvids.IsNullOrEmpty()) throw new Exception("请传入要重新计算金额的报告!");

            if (ReportKvids.IsNullOrEmpty()) ReportKvids = new List<Guid>();
            if (ReportKvid != Guid.Empty) ReportKvids.Add(ReportKvid);

            //获取报告信息
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var reports = conn.SelectByIds<Report>(ReportKvids);
            reports.ThrowIfNullOrEmpty("未能找到传入的报告信息!");

            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            foreach (var item in reports)
            {
                item.CalculateReportAmount(conn);
                conn.UpdateOnly(item);
                item.RemoveAllOnlyProperties();
                rtns.Results.Add(item);
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 任务下达,标准值若修改过加以提示
    /// <summary>
    /// 仅处理数据录入任务下达的情况
    /// 下达时,若当前组存在标准值修改的情况,加提示
    /// </summary>
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "任务下达提示(标准值更新)")]
    public class ReportGetUpdateDetectionValueQualifiedInfos : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            (ReportKvid == Guid.Empty).ThrowIfTrue("未传入报告Kvid!");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var report = conn.SingleById<Report>(ReportKvid);
            (report == null).ThrowIfTrue("未捕获对应报告信息!");
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            //获取检验项目集合
            var reportItemsAll = report.GetOriginalReportItems(conn);
            //根据当前人的组别，只处理当前组的数据
            var queryTestRoles = conn.From<EntityDuty<Test>>();
            queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
            queryTestRoles.Select(o => o.RoleName);
            var testRoleNames = conn.Select(queryTestRoles).ConvertAll(o => o.RoleName);
            var ownerReportItems = reportItemsAll.Where(o => o.WorkGroupKvid == KiviiContext.CurrentMember.DepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames)).ToList();
            var reportItemsUpdateDetectionValueQualified = ownerReportItems.Where(o => o.Metadata.ContainsKey("IsUpdateDetectionValueQualified") && o.Metadata["IsUpdateDetectionValueQualified"] == "true").ToList();
            rtns.Results.AddRange(reportItemsUpdateDetectionValueQualified);
            rtns.Total = rtns.Results.Count;
            return rtns;
        }

    }
    #endregion

    #region 任务下达
    /// <summary>
    /// 不能删除该类，
    /// 针对任务下达定义一个区别于RestfulQueryResponse的响应类(不影响最后序列化后的数据结构)
    /// 这样做的目的是为了报告签发后 启用excel转PDF功能的全局监听对此响应进行捕获
    /// 要是都用RestfulQueryResponse全局监听会频繁触发
    /// </summary>
    //代码初审 张梦圆 20210906
    public class ReportAssignResponse : RestfulQueryResponse<Report>
    {

    }

    /// <summary>
    /// 参考 apps.jst-gov.com\Apps.Jst-Gov.Com.V3.5\Modules\Lims\Contracts\Themes\Contract-List.ascx.cs 方法ContractFlowGo_OnEvent
    /// </summary>
    //代码初审 张梦圆 20210906
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "任务下达")]
    public class ReportAssign : RestfulExecution<Report>, IReturn<ReportAssignResponse>
    {
        [ApiMember(Description = "")]
        public List<Guid> ReportKvids { get; set; }
        public string Description { get; set; }
        public ReportStatusType StatusType { get; set; }
        public bool UseThread { get; set; }
        public DateTime? IssueDate { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            StatusType.ThrowIfNull("参数错误");
            ReportKvids.ThrowIfNullOrEmpty("您当前还未选择需要处理的委托协议,请选择委托协议后重新操作！");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();

            var reports = conn.SelectByIds<Report>(ReportKvids);
            reports.ThrowIfNullOrEmpty("未找到所选委托协议信息!");
            reports.RemoveAll(o => o.Status == int.MaxValue);
            reports.ThrowIfNullOrEmpty("所选报告都为归档报告无需再下达!");
            

            var rtns = new ReportAssignResponse();
            rtns.Results = new List<Report>();
            foreach (var report in reports)
            {
                using (KiviiContext.Profiler($"下达：{report.ReportId}"))
                {
                    try
                    {
                        //初始化metadata的description为空，为了下达是否带上文本内容，记录日志
                        report.Metadata["LogDescription"] = "";
                        if (!Description.IsNullOrEmpty()) report.Metadata["LogDescription"] = Description;

                        (report.ReportId.IsNullOrEmpty() || report.DemanderName.IsNullOrEmpty() || report.SampleName.IsNullOrEmpty() || report.DeadDate == null || report.DealDate == null).ThrowIfTrue($"当前报告[{report.ReportId}]的基本信息填写不完整，请填写完必填信息后重新操作！");
                        var originalStatus = report.Status;
                        //report.beforeAsign(StatusType, conn);
                        switch (report.Status)
                        {
                            case (int)ReportStatusType.Unsupported:
                                report.Assign(StatusType);
                                rtns.Results.Add(report);
                                break;
                            case (int)ReportStatusType.CommissionAccept:
                                report.Assign(StatusType);
                                rtns.Results.Add(report);
                                break;
                            case (int)ReportStatusType.ContractReview:
                                report.Assign(StatusType);
                                rtns.Results.Add(report);
                                break;
                            case (int)ReportStatusType.TaskAssign:
                                report.Assign(StatusType);
                                rtns.Results.Add(report);
                                break;
                            case (int)ReportStatusType.DataEntry:
                                report.AssignTest(StatusType, null, UseThread);
                                rtns.Results.Add(report);
                                break;
                            case (int)ReportStatusType.ReportPreparation:
                                report.AssignPrepare(StatusType);
                                rtns.Results.Add(report);
                                break;
                            case (int)ReportStatusType.ReportReview:
                                report.AssignCheck(StatusType);
                                rtns.Results.Add(report);
                                break;
                            case (int)ReportStatusType.ReportIssue:
                                report.AssignIssue(StatusType, IssueDate);
                                rtns.Results.Add(report);
                                break;
                            case (int)ReportStatusType.ReportPrint:
                                report.AssignPrint(StatusType);
                                rtns.Results.Add(report);
                                break;
                            //case 999:
                            //    report.AssignManager(conn);
                            //    rtns.Results.Add(report);
                            //    break;
                            default:
                                break;
                        }

                        //是从报告打印下达就不走后面步骤了
                        if (originalStatus >= (int)ReportStatusType.ReportPrint) continue;
                        if (originalStatus == report.Status) continue;

                        //报告任务下达后需要生成对应excel文件
                        if (originalStatus >= (int)ReportStatusType.ReportIssue && originalStatus < (int)ReportStatusType.ReportPrint)
                        {
                            report.GenerateReport(Configs.RouteReportGenerateSourcePath, originalStatus, false);
                        }
                        else
                        {
                            report.GenerateReport(Configs.RouteReportGenerateSourcePath, originalStatus);
                        }


                        //如果是更改报告下达，需要生成报告附页
                        if (report.BizType == "X" && StatusType == ReportStatusType.CommissionAccept)
                        {
                            report.excuteGenerateX();
                        }
                        //G0更改报告首页信息；
                        if (report.BizType == "G0" && StatusType == ReportStatusType.CommissionAccept)
                        {
                            report.excuteGenerateX();
                        }

                        //签发动作 需要合并excel
                        if (originalStatus >= (int)ReportStatusType.ReportIssue && originalStatus < (int)ReportStatusType.ReportPrint) report.excuteCombine(Configs.RouteReportGenerateCombineSourcePath);
                    }
                    catch (Exception ex)
                    {
                        throw ex;
                    }
                }
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    [RequiresAnyRole(MemberRoles.Everyone)]
    [Api(Description = "任务下达2")]
    public class ReportAssign2 : RestfulExecution<Report>, IReturn<ReportAssignResponse>
    {
        public List<Guid> ReportKvids { get; set; }
        public string Description { get; set; }
        public ReportStatusType StatusType { get; set; }
        public DateTime? IssueDate { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            StatusType.ThrowIfNull("参数错误");
            ReportKvids.ThrowIfNullOrEmpty("您当前还未选择需要处理的委托协议,请选择委托协议后重新操作！");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();

            var reports = conn.SelectByIds<Report>(ReportKvids);
            reports.ThrowIfNullOrEmpty("未找到所选委托协议信息!");
            reports.RemoveAll(o => o.Status == int.MaxValue);
            reports.ThrowIfNullOrEmpty("所选报告都为归档报告无需再下达!");


            var rtns = new ReportAssignResponse();
            rtns.Results = new List<Report>();
            foreach (var report in reports)
            {
                using (KiviiContext.Profiler($"下达：{report.ReportId}"))
                {
                    try
                    {
                        //初始化metadata的description为空，为了下达是否带上文本内容，记录日志
                        report.Metadata["LogDescription"] = "";
                        if (!Description.IsNullOrEmpty()) report.Metadata["LogDescription"] = Description;

                        (report.ReportId.IsNullOrEmpty() || report.DemanderName.IsNullOrEmpty() || report.SampleName.IsNullOrEmpty() || report.DeadDate == null || report.DealDate == null).ThrowIfTrue($"当前报告[{report.ReportId}]的基本信息填写不完整，请填写完必填信息后重新操作！");
                        var originalStatus = report.Status;

                        report.Assign(StatusType);
                        rtns.Results.Add(report);
                        ////report.beforeAsign(StatusType, conn);
                        //switch (report.Status)
                        //{
                        //    case (int)ReportStatusType.Unsupported:
                        //        report.Assign(StatusType);
                        //        rtns.Results.Add(report);
                        //        break;
                        //    case (int)ReportStatusType.CommissionAccept:
                        //        report.Assign(StatusType);
                        //        rtns.Results.Add(report);
                        //        break;
                        //    case (int)ReportStatusType.ContractReview:
                        //        report.Assign(StatusType);
                        //        rtns.Results.Add(report);
                        //        break;
                        //    case (int)ReportStatusType.TaskAssign:
                        //        report.Assign(StatusType);
                        //        rtns.Results.Add(report);
                        //        break;
                        //    case (int)ReportStatusType.DataEntry:
                        //        report.AssignTest(StatusType, null, UseThread);
                        //        rtns.Results.Add(report);
                        //        break;
                        //    case (int)ReportStatusType.ReportPreparation:
                        //        report.AssignPrepare(StatusType);
                        //        rtns.Results.Add(report);
                        //        break;
                        //    case (int)ReportStatusType.ReportReview:
                        //        report.AssignCheck(StatusType);
                        //        rtns.Results.Add(report);
                        //        break;
                        //    case (int)ReportStatusType.ReportIssue:
                        //        report.AssignIssue(StatusType, IssueDate);
                        //        rtns.Results.Add(report);
                        //        break;
                        //    case (int)ReportStatusType.ReportPrint:
                        //        report.AssignPrint(StatusType);
                        //        rtns.Results.Add(report);
                        //        break;
                        //    //case 999:
                        //    //    report.AssignManager(conn);
                        //    //    rtns.Results.Add(report);
                        //    //    break;
                        //    default:
                        //        break;
                        //}

                        //是从报告打印下达就不走后面步骤了
                        if (originalStatus >= (int)ReportStatusType.ReportPrint) continue;
                        if (originalStatus == report.Status) continue;

                        //报告任务下达后需要生成对应excel文件
                        if (originalStatus >= (int)ReportStatusType.ReportIssue && originalStatus < (int)ReportStatusType.ReportPrint)
                        {
                            report.GenerateReport(Configs.RouteReportGenerateSourcePath, originalStatus, false);
                        }
                        else
                        {
                            report.GenerateReport(Configs.RouteReportGenerateSourcePath, originalStatus);
                        }

                        //签发动作 需要合并excel
                        if (originalStatus >= (int)ReportStatusType.ReportIssue && originalStatus < (int)ReportStatusType.ReportPrint) report.excuteCombine(Configs.RouteReportGenerateCombineSourcePath);
                    }
                    catch (Exception ex)
                    {
                        throw ex;
                    }
                }
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 任务回收
    //代码初审 张梦圆 20210906
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "任务回收")]
    public class ReportRecycle : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public int CurrentStatus { get; set; }
        public string Description { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("未传入报告Kvid！");
            CurrentStatus.ThrowIfNull("CurrentStatus参数不能为空！");
            Description.ThrowIfNullOrEmpty("回收报告必须输入原因！");
            var rtns = new RestfulUpdateResponse<Report>();
            rtns.Results = new List<Report>();

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到当前报告！");
            if (CurrentStatus == (int)ReportStatusType.CommissionAccept)
            {
                if (report.Status == (int)ReportStatusType.TaskAssign)
                {
                    report.AddReportNote("任务回收", "任务分派", "委托受理", Description, conn);
                    report.Status = (int)ReportStatusType.CommissionAccept;
                    report.AddOnlyProperties(o => o.Status);
                }
                else throw new Exception("当前报告无法回收！");
            }
            else if (report.Status == CurrentStatus + 100)
            {
                report.AddReportNote("任务回收", ReportExtension.AssignDescription(report.Status), ReportExtension.AssignDescription(CurrentStatus), Description, conn);
                report.Status = CurrentStatus;
                report.AddOnlyProperties(o => o.Status);
            }
            else
            {
                throw new Exception("当前报告无法回收！");
            }
            conn.UpdateOnly(report);
            report.RemoveAllOnlyProperties();
            rtns.Results.Add(report);
            return rtns;
        }
    }
    #endregion

    #region 报告任务打回
    //代码初审 张梦圆 20210906
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "任务打回")]
    public class ReportCallBack : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public string Description { get; set; }
        public int ToStatus { get; set; }
        public bool IsDirectReturn { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("未传入报告Kvid！");
            if (ToStatus == 0) throw new Exception("目的流程参数异常！");
            Description.ThrowIfNullOrEmpty("打回报告必须输入原因！");

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到当前报告！");

            //归档及打印状态的报告无法打回，需先调用召回接口
            if (report.StatusType == ReportStatusType.ReportCollected) throw new Exception("归档状态报告无法打回，请先召回报告！");
            if (report.StatusType == ReportStatusType.ReportPrint) throw new Exception("打印状态报告无法打回，请先召回报告！");
            if (report.Status == ToStatus) throw new Exception("请勿重复打回！");

            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();

            report.AddReportNote("任务打回", ReportExtension.AssignDescription(report.Status), ReportExtension.AssignDescription(ToStatus), Description, conn);
            if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
            report.Metadata["CallBackFrom"] = ((ReportStatusType)report.Status).ToString();
            report.Metadata["CallBackTo"] = ((ReportStatusType)ToStatus).ToString();
            //增加实验室打回报告时，记录下组别，用于查询实验室打回记录
            if (report.Status == (int)ReportStatusType.DataEntry)
            {
                report.AddDataEntryBackNote("任务打回", KiviiContext.CurrentMember.DepartmentName, ReportExtension.AssignDescription(ToStatus), Description, conn);
            }
            //增加适配，检验完成后打回，要记录，用于展示列表颜色
            if (report.Status > 400)
            {
                report.Metadata["IsCallbackAfterFinish"] = "true";
            }
            report.AddOnlyProperties(o => o.Metadata);
            if (IsDirectReturn)
            {
                report.Metadata["IsDirectReturn"] = "Yes";
                report.Metadata["DirectReturnStatus"] = report.Status.ToString();
                report.AddOnlyProperties(o => o.Metadata);
            }
            else//处理二次打回时，清空直接返回问题
            {
                report.Metadata["IsDirectReturn"] = null;
                report.Metadata["DirectReturnStatus"] = null;
                report.AddOnlyProperties(o => o.Metadata);
            }
            report.Status = ToStatus;
            report.AddOnlyProperties(o => o.Status);

            conn.UpdateOnly(report);
            report.RemoveAllOnlyProperties();
            rtns.Results.Add(report);
            return rtns;
        }
    }
    #endregion

    #region 生成报告首页、附页等
    //代码初审 张梦圆 20210906
    public class ReportGenerateResponse : RestfulQueryResponse<Report>
    {
        public bool IsSucceed { get; set; }
        public string Message { get; set; }
        public bool IsCombine { get; set; }
    }

    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "手动生成报告")]
    public class ReportGenerate : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public Guid ReportFileKvid { get; set; }
        public string SavePath { get; set; }
        public bool IsCombine { get; set; }
        public bool IsReGenerate { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 参数验证
            ReportKvid.ThrowIfEmpty("参数非法：ReportKvid不能为空！");
            SavePath.ThrowIfNullOrEmpty("参数非法：SavaPath不能为空！");
            #endregion
            var rtns = new ReportGenerateResponse();
            rtns.Results = new List<Report>();
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            var reportFile = conn.SingleById<ReportFile>(ReportFileKvid);
            try
            {
                report.ThrowIfNull("报告不存在！");
                report.TemplateKvid.ThrowIfEmpty("报告无模板，无法生成！");
                if (IsCombine)//如果是合并报告
                {
                    var queryReportFile = conn.From<ReportFile>();
                    queryReportFile.Where(o => o.ReportKvid == report.Kvid && o.TemplateKvid == report.TemplateKvid);
                    var rtnReportFiles = conn.Select(queryReportFile);
                    //若没有当前模板的ReportFile记录，则新建ReportFile记录
                    if (rtnReportFiles.Count == 0)
                    {
                        var queryTemplate = conn.From<Template>();
                        queryTemplate.Where(o => o.Kvid == report.TemplateKvid);
                        var rtnTemplate = conn.Select(queryTemplate).FirstOrDefault();
                        rtnTemplate.ThrowIfNull("未找到当前报告所对应模板！");

                        var queryTemplateFile = conn.From<TemplateFile>();
                        queryTemplateFile.Where(o => o.TemplateKvid == rtnTemplate.Kvid && o.Tags != "" && o.Tags != null);
                        var rtnTemplateFile = conn.Select(queryTemplateFile);
                        rtnTemplateFile.ThrowIfNull("当前报告所关联的模板无文件！");

                        foreach (var en in rtnTemplateFile)
                        {
                            var newReportFile = new ReportFile();
                            newReportFile.ReportKvid = report.Kvid;
                            newReportFile.TemplateKvid = en.TemplateKvid;
                            newReportFile.TemplateFileKvid = en.Kvid;
                            newReportFile.Name = en.Name;
                            newReportFile.Tags = en.Tags;
                            newReportFile.TemplateFilePath = en.Path;
                            newReportFile.SortID = en.SortId;
                            conn.Insert(newReportFile);
                            rtnReportFiles.Add(newReportFile);
                        }
                    }
                    report.excuteCombine(SavePath, conn);
                }
                else
                {
                    reportFile.ThrowIfNull("未找到对应的ReportFile记录");
                    var openPath = "";
                    if (IsReGenerate || reportFile.Path.IsNullOrEmpty())
                    {
                        if (reportFile.TemplateFilePath != null && reportFile.TemplateFilePath != "")
                        {
                            openPath = reportFile.TemplateFilePath;
                        }
                        else
                        {
                            var templateFile = conn.SingleById<TemplateFile>(reportFile.TemplateFileKvid);
                            templateFile.ThrowIfNull("未找到关联的模板文件，无法重新生成！");
                            openPath = templateFile.Path;
                        }
                    }
                    else
                    {
                        openPath = reportFile.Path;
                    }
                    var sPath = $"/Storages{SavePath}/{report.Kvid}/{report.ReportId}-{reportFile.Name}";
                    report.excuteGenerate(openPath, sPath, false);

                    reportFile.Path = sPath;
                    reportFile.AddOnlyProperties(o => o.Path);
                    conn.UpdateOnly(reportFile);

                }
                rtns.Results.Add(report);
                //var openPath = IsReGenerate == true ?
            }
            catch (Exception ex)
            {
                rtns.IsSucceed = false;
                rtns.IsCombine = false;
                rtns.Message = ex.Message;
                return rtns;
            }

            rtns.Total = rtns.Results.Count();
            rtns.IsSucceed = true;
            rtns.IsCombine = IsCombine;
            rtns.Message = "执行成功！";
            return rtns;
        }
    }
    #endregion

    #region 报告引用，生成检验费为0的对应的项目，应用于出合格报告，更改报告
    public class MatchKvids
    {
        public Guid OriginalKvid { get; set; }
        public Guid MatchKvid { get; set; }
    }
    //代码初审 张梦圆 20210907
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "报告引用")]
    public class ReportReference : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public Guid ReferenceReportKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (ReportKvid == Guid.Empty || ReferenceReportKvid == Guid.Empty) throw new Exception("参数异常，不能为空！");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到当前报告！");
            if (report.StatusType != ReportStatusType.CommissionAccept) throw new Exception("当前报告不在委托状态，无法引用");
            var referenceReport = conn.SingleById<Report>(ReferenceReportKvid);
            referenceReport.ThrowIfNull("未找到引用的报告！");
            //if (referenceReport.Status <= (int)ReportStatusType.ReportIssue) throw new Exception("报告还未签发，无法引用！");
            #endregion
            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();

            //获取当前报告的主样品信息
            var rootSample = conn.SingleById<Sample>(report.SampleKvid);
            rootSample.ThrowIfNull("当前报告未找到主样品,无法引用!");
            //获取引用报告的样品信息
            var querySamplesReference = conn.From<Sample>();
            querySamplesReference.Where(o => o.Kvid == referenceReport.SampleKvid || o.ParentKvid == referenceReport.SampleKvid);
            var samplesReference = conn.Select(querySamplesReference);
            //提取引用报告的主样品
            var rootSampleReference = samplesReference.FirstOrDefault(o => o.ParentKvid == Guid.Empty);
            rootSampleReference.ThrowIfNull("引用报告未找到主样品,无法引用!");
            //提取引用报告的子样品
            samplesReference.Remove(rootSampleReference);
            //建立样品的关联关系
            var samplesMatchKvids = new List<MatchKvids>();
            samplesMatchKvids.Add(new MatchKvids
            {
                OriginalKvid = Guid.Empty,
                MatchKvid = Guid.Empty
            });
            samplesMatchKvids.Add(new MatchKvids
            {
                OriginalKvid = rootSampleReference.Kvid,
                MatchKvid = rootSample.Kvid
            });

            //获取引用报告的所有检验项目
            var queryReportItemsReference = conn.From<ReportItem>();
            queryReportItemsReference.Where(o => o.ReportKvid == ReferenceReportKvid);
            var reportItemsReference = conn.Select(queryReportItemsReference);
            //获取检验项目新老Kvid集合
            var reportItemsKvids = reportItemsReference.Select(o => o.Kvid).ToList();
            var reportItemsMatchKvids = new List<MatchKvids>();
            reportItemsMatchKvids.Add(new MatchKvids
            {
                OriginalKvid = Guid.Empty,
                MatchKvid = Guid.Empty
            });
            foreach (var reportItemKvid in reportItemsKvids)
            {
                reportItemsMatchKvids.Add(new MatchKvids
                {
                    OriginalKvid = reportItemKvid,
                    MatchKvid = Guid.NewGuid()
                });
            }
            var samplesOperate = new List<Sample>();
            //获取待创建子样品集合
            foreach (var sampleReference in samplesReference)
            {
                var sampleChild = new Sample();
                sampleChild.PopulateWith(sampleReference);
                sampleChild.Kvid = Guid.NewGuid();
                sampleChild.RootKvid = rootSample.RootKvid;
                sampleChild.ParentKvid = rootSample.Kvid;
                sampleChild.SourceKvid = report.SampleSourceKvid;
                sampleChild.SourceBatchNumber = String.Empty;
                sampleChild.SerialNumber = sampleReference.SerialNumber.Replace(rootSampleReference.SerialNumber, rootSample.SerialNumber);
                samplesMatchKvids.Add(new MatchKvids
                {
                    OriginalKvid = sampleReference.Kvid,
                    MatchKvid = sampleChild.Kvid
                });
                samplesOperate.Add(sampleChild);
            }
            var reportItemsOperate = new List<ReportItem>();
            //获取待创建检验项目集合
            foreach (var reportItemReference in reportItemsReference)
            {
                var reportItem = new ReportItem();
                reportItem.PopulateWith(reportItemReference);
                reportItem.ReportKvid = report.Kvid;
                reportItem.SampleKvid = samplesMatchKvids.Single(o => o.OriginalKvid == reportItemReference.SampleKvid).MatchKvid;
                reportItem.SampleRootKvid = samplesMatchKvids.Single(o => o.OriginalKvid == reportItemReference.SampleRootKvid).MatchKvid;
                reportItem.RootKvid = reportItemsMatchKvids.Single(o => o.OriginalKvid == reportItemReference.RootKvid).MatchKvid;
                reportItem.ParentKvid = reportItemsMatchKvids.Single(o => o.OriginalKvid == reportItemReference.ParentKvid).MatchKvid;
                reportItem.Kvid = reportItemsMatchKvids.Single(o => o.OriginalKvid == reportItemReference.Kvid).MatchKvid;
                reportItem.AmountPlan = 0;
                reportItem.Amount = 0;
                reportItemsOperate.Add(reportItem);
            }
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                //创建子样品
                foreach (var sampleOperate in samplesOperate)
                {
                    conn.Insert(sampleOperate);
                }
                //创建检验项目
                foreach (var reportItemOperate in reportItemsOperate)
                {
                    conn.Insert(reportItemOperate);
                }
                //更新报告的检验日期、到样日期、以及Biz信息
                report.TestStartDate = referenceReport.TestStartDate;
                report.TestFinishDate = referenceReport.TestFinishDate;
                report.BizType = "H";
                report.BizId = referenceReport.ReportId;
                report.BizKvid = referenceReport.Kvid;
                report.AddOnlyProperties(o => new { o.BizKvid, o.BizId, o.BizType, o.TestStartDate, o.TestFinishDate });
                if (referenceReport.Metadata.ContainsKey("SampleEffectiveDate"))
                {
                    if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
                    report.Metadata["SampleEffectiveDate"] = referenceReport.Metadata["SampleEffectiveDate"];
                    report.AddOnlyProperties(o => o.Metadata);
                }
                conn.UpdateOnly(report);
                report.RemoveAllOnlyProperties();

                rtns.Results.Add(report);
                rtns.Total = rtns.Results.Count;
                trans?.Commit();
                return rtns;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }
    }
    //[RequiresAnyRole(SystemRoles.Everyone)]
    //public class ReportReference : RestfulExecution<Report>
    //{
    //    public Guid ReportKvid { get; set; }
    //    public Guid ReferenceReportKvid { get; set; }
    //    public override object OnExecution(IRequest req, IResponse res)
    //    {
    //        if (ReportKvid == Guid.Empty || ReferenceReportKvid == Guid.Empty) throw new Exception("参数异常，不能为空！");
    //        var conn = KiviiContext.GetOpenedDbConnection<Report>();
    //        var report = conn.SingleById<Report>(ReportKvid);
    //        report.ThrowIfNull("未找到当前报告！");
    //        if (report.StatusType != ReportStatusType.CommissionAccept) throw new Exception("当前报告不在委托状态，无法引用");
    //        var referenceReport = conn.SingleById<Report>(ReferenceReportKvid);
    //        referenceReport.ThrowIfNull("未找到引用的报告！");
    //        if (referenceReport.Status < 700) throw new Exception("报告还未签发，无法引用！");
    //        var queryReportItem = conn.From<ReportItem>();
    //        //queryReportItem.OrderBy(o => o.ParentKvid);
    //        queryReportItem.Where(o => o.ReportKvid == referenceReport.Kvid);
    //        var reportItem = conn.Select(queryReportItem);

    //        var rtns = new RestfulQueryResponse<ReportItem>();
    //        rtns.Results = new List<ReportItem>();
    //        var trans = conn.OpenTransaction();
    //        try
    //        {
    //            foreach (var en in reportItem)
    //            {
    //                var nodeReportItem = reportItem.Where(o => o.ParentKvid == en.Kvid).ToList();
    //                if (en.ParentKvid != Guid.Empty) continue;
    //                var newReportItem = new ReportItem();
    //                newReportItem.PopulateWith(en);
    //                newReportItem.Kvid = new Guid();
    //                newReportItem.ReportKvid = report.Kvid;
    //                newReportItem.AmountPlan = 0;
    //                newReportItem.Amount = 0;
    //                conn.Insert(newReportItem);
    //                rtns.Results.Add(newReportItem);
    //                if (!nodeReportItem.IsNullOrEmpty())
    //                {
    //                    foreach (var node in nodeReportItem)
    //                    {
    //                        node.ReportKvid = newReportItem.ReportKvid;
    //                        node.Amount = 0;
    //                        node.AmountPlan = 0;
    //                        node.ModifyRoot(newReportItem, conn);
    //                    }
    //                }

    //            }
    //            trans?.Commit();
    //        }
    //        catch (Exception ex)
    //        {
    //            trans?.Rollback();
    //            throw ex;
    //        }
    //        rtns.Total = rtns.Results.Count;
    //        return rtns;
    //    }
    //}
    #endregion

    #region 统计报告批次/费用
    //代码初审 张梦圆 20210906
    public class ReportStatisticalInfo
    {
        //报告份数
        public int Total { get; set; }
        //报告费
        public decimal Amount { get; set; }
        //更改费
        public decimal AmountChange { get; set; }
        //折后费
        public decimal AmountTestPlan { get; set; }
        //检验费
        public decimal AmountTest { get; set; }
        //套餐费
        public decimal QuotationAmount { get; set; }
        //大项总数
        public int TotalReportItems { get; set; }
    }
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportStatisticsQuery : RestfulExecution<Report>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public ReportStatusType filterStatus { get; set; }
        public string filterStatusEx { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Report>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);

            //DataEntry状态的报告的筛选条件需要单独处理
            if (filterStatus == ReportStatusType.DataEntry)
            {
                //获取当前人是否具有超级管理员的权限
                var session = Request.GetSession();
                var isAdministrator = session.HasRole(SystemRoles.Administrator, null);
                var queryTestRoles = conn.From<EntityDuty<Test>>();
                queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
                queryTestRoles.Select(o => o.RoleName);
                var testRoleNames = conn.Select(queryTestRoles).ConvertAll(o => o.RoleName);

                //获取当前登录人的部门Kvid
                var currentDepartmentKvid = KiviiContext.CurrentMember.DepartmentKvid;
                var queryReportKvids = conn.From<ReportItem>();
                queryReportKvids.Where(o => o.Type != ReportItemType.SampleSelf && o.Type != ReportItemType.SamplePart && o.Type != ReportItemType.Unsupported);
                queryReportKvids.SelectDistinct(o => o.ReportKvid);
                //分为所有任务（All）、待处理任务（Pending）、已处理任务（Processed）、已完成任务（Finished）、待复核任务（Reviewing）
                //待复核任务是指该角色所在组的报告的所有项目均已进入待复核状态或之后
                if (filterStatusEx == "All")
                {
                    queryReportKvids.Where(o => (o.Status >= (int)ReportItemStatusType.Testing));
                    if (!isAdministrator) queryReportKvids.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    sqlExpress.And(o => Sql.In(o.Kvid, queryReportKvids));
                }
                else if (filterStatusEx == "Pending" || filterStatusEx.IsNullOrEmpty())
                {
                    queryReportKvids.Where(o => (o.Status == (int)ReportItemStatusType.Testing || o.Status == (int)ReportItemStatusType.BeforeReview || o.Status == (int)ReportItemStatusType.TestFinished));
                    if (!isAdministrator) queryReportKvids.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    sqlExpress.And(o => Sql.In(o.Kvid, queryReportKvids));
                }
                else if (filterStatusEx == "Processed")
                {
                    queryReportKvids.Where(o => o.Status == int.MaxValue);
                    if (!isAdministrator) queryReportKvids.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    sqlExpress.And(o => Sql.In(o.Kvid, queryReportKvids) && o.Status < int.MaxValue);
                }
                else if (filterStatusEx == "Finished")
                {
                    if (!isAdministrator) queryReportKvids.Where(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    sqlExpress.And(o => Sql.In(o.Kvid, queryReportKvids) && o.Status == int.MaxValue);
                }
                else if (filterStatusEx == "Reviewing")
                {
                    queryReportKvids.Where(o => o.Status >= (int)ReportItemStatusType.BeforeReview && o.Status < (int)ReportItemStatusType.TestFinished);
                    if (!isAdministrator) queryReportKvids.And(o => o.WorkGroupKvid == currentDepartmentKvid || Sql.In(o.WorkGroupName, testRoleNames));
                    sqlExpress.And(o => Sql.In(o.Kvid, queryReportKvids));
                }
                else
                {
                    throw new Exception("filterStatusEx参数异常！");
                }
            }
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);

            var reportStatisticalInfo = new ReportStatisticalInfo();
            reportStatisticalInfo.Total = rtns.Total;
            reportStatisticalInfo.Amount = 0;
            reportStatisticalInfo.AmountChange = 0;
            reportStatisticalInfo.AmountTest = 0;
            reportStatisticalInfo.AmountTestPlan = 0;
            reportStatisticalInfo.TotalReportItems = 0;
            //获取报告Kvid集合
            //var reportKvids = rtns.Results.ConvertAll(o=>o.Kvid);
            var autoStatistic = Request.TryResolve<IAutoQueryDb>();
            autoStatistic.IncludeTotal = false;
            var requestNew = new RestfulQuery<Report>();
            requestNew = requestNew.PopulateWith(this);
            var queryLamada = autoStatistic.CreateQuery(Request, conn, requestNew, dynamicParams);
            queryLamada.Select(o => o.Kvid);
            queryLamada.Rows = null;
            var totalReportItems = conn.Scalar<ReportItem, int>(o => Sql.Count(o.Kvid), o => Sql.In(o.ReportKvid, queryLamada) && o.DetectionKvid != Guid.Empty && o.StandardTitle != null && o.StandardTitle != "");
            reportStatisticalInfo.TotalReportItems = totalReportItems;
            foreach (var report in rtns.Results)
            {
                reportStatisticalInfo.Amount += report.Amount;
                decimal amountChange = 0;
                if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
                if (report.Metadata.ContainsKey("AmountChange"))
                {
                    amountChange = report.Metadata["AmountChange"].ToDecimal();
                }
                reportStatisticalInfo.AmountChange += amountChange;
                reportStatisticalInfo.AmountTest += report.AmountTest;
                reportStatisticalInfo.AmountTestPlan += report.AmountTestPlan;
                reportStatisticalInfo.QuotationAmount += report.QuotationAmount;
            }
            return reportStatisticalInfo;
        }
    }
    public class ReportStatisticalInfoForWorkGroup
    {
        //报告份数
        public int ReportsTotal { get; set; }
        //检验费
        public decimal AmountTestPlan { get; set; }
        //折后费
        public decimal AmountTest { get; set; }
    }
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "报告数据统计-检验组")]

    public class ReportStatisticsForWorkGroup : RestfulExecution<Report>
    { 
        public DateTime? DeadDateBegin { get; set; }
        public DateTime? DeadDateEnd { get; set; }
        public List<string> WorkGroupName { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (DeadDateBegin == null && DeadDateEnd == null) throw new Exception("完成日期时间范围未选择!");
            if (WorkGroupName.IsNullOrEmpty()) throw new Exception("未传入检验组!");
            #endregion
            var rtns = new ReportStatisticalInfoForWorkGroup();
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var queryReportKvids = conn.From<Report>();
            queryReportKvids.Where(o=>o.DeadDate>=DeadDateBegin&&o.DeadDate<=DeadDateEnd&&o.Status==(int)ReportStatusType.ReportCollected);
            queryReportKvids.Select(o=>o.Kvid);
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o=>Sql.In(o.ReportKvid, queryReportKvids)&&WorkGroupName.Contains(o.WorkGroupName));
            queryReportItems.Select(o => new { o.ReportKvid,o.Amount,o.AmountPlan});
            var reportItems = conn.Select(queryReportItems);
            rtns.ReportsTotal = reportItems.ConvertAll(o=>o.ReportKvid).Distinct().ToList().Count;
            var amountTestPlan = reportItems.Sum(o=>o.AmountPlan);
            var amountTest= reportItems.Sum(o => o.Amount);
            rtns.AmountTestPlan = amountTestPlan;
            rtns.AmountTest = amountTest;
            return rtns;
        }
    }
    #endregion

    #region 创建更改报告(-X报告)
    //代码初审 张梦圆 20210907
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "生成X报告")]
    public class ReportChangeGenerate : RestfulExecution<Report>
    {
        public Report ChangeReport { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            ChangeReport.ThrowIfNull("未传入更改报告信息!");
            //需要根据原报告Kvid去创建对应关系
            ChangeReport.Kvid.ThrowIfEmpty("未找到对应原报告!");
            #endregion

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            //获取更改报告相关文件集合(样品图片等)
            var queryOriginalFiles = conn.From<EntityDbFile<Report>>();
            queryOriginalFiles.Where(o => o.OwnerKvid == ChangeReport.Kvid && o.DbFolderPath == "/Report/Images");
            var originalFiles = conn.Select(queryOriginalFiles);
            //获取更改报告检验项目集合
            var queryOriginalReportItems = conn.From<ReportItem>();
            queryOriginalReportItems.Where(o => o.ReportKvid == ChangeReport.Kvid);
            var originalReportItems = conn.Select(queryOriginalReportItems);
            //获取检验项目新老Kvid集合
            var reportItemsKvids = originalReportItems.Select(o => o.Kvid).ToList();
            var reportItemKvidsReplace = new List<MatchKvids>();
            reportItemKvidsReplace.Add(new MatchKvids
            {
                OriginalKvid = Guid.Empty,
                MatchKvid = Guid.Empty
            });
            foreach (var reportItemKvid in reportItemsKvids)
            {
                reportItemKvidsReplace.Add(new MatchKvids
                {
                    OriginalKvid = reportItemKvid,
                    MatchKvid = Guid.NewGuid()
                });
            }
            //1.获取待创建报告信息
            var reportIdPrefix = string.Empty;
            var index = ChangeReport.ReportId.LastIndexOf("-XW");
            if (index == -1)
            {
                //不包含XW
                var subIndex = ChangeReport.ReportId.LastIndexOf("-X");
                if (subIndex == -1)
                {
                    reportIdPrefix = ChangeReport.ReportId;
                }
                else
                {
                    reportIdPrefix = ChangeReport.ReportId.Substring(0, subIndex);
                }
            }
            else {
                var arrReportId = Regex.Split(ChangeReport.ReportId, "-X", RegexOptions.IgnoreCase);
                if (arrReportId.Length > 2)
                {
                    var subIndex = ChangeReport.ReportId.LastIndexOf("-X");
                    reportIdPrefix = ChangeReport.ReportId.Substring(0, subIndex);
                }
                else {
                    reportIdPrefix = ChangeReport.ReportId;
                }
            }
            //获取最新版本报告
            var ralateReport = conn.Select<Report>(o => o.ReportId.Contains(reportIdPrefix + "-X")).OrderByDescending(o => o.CreateTime).FirstOrDefault();
            //只有选择原报告时，才会用最新的X报告，其他情况用选择的报告的信息
            if (index == -1)
            {
                //不包含XW
                var subIndex = ChangeReport.ReportId.LastIndexOf("-X");
                if (subIndex != -1)
                {
                    ChangeReport = ralateReport;
                }
            }
            else
            {
                var arrReportId = Regex.Split(ChangeReport.ReportId, "-X", RegexOptions.IgnoreCase);
                if (arrReportId.Length > 2)
                {
                    ChangeReport = ralateReport;
                }
            }
            //if (ralateReport != null) ChangeReport = ralateReport;
            var reportOperate = new Report();
            reportOperate.Metadata = new Dictionary<string, string>();
            reportOperate.PopulateWith(ChangeReport);
            reportOperate.ReportId = reportIdPrefix + "-X{x}";
            reportOperate.BizKvid = ChangeReport.Kvid;
            reportOperate.BizType = "X";
            reportOperate.BizId = ChangeReport.ReportId;
            reportOperate.AmountTestPlan = 0;
            reportOperate.AmountTest = 0;
            reportOperate.AmountPlan = 0;
            reportOperate.AmountPayment = 0;
            reportOperate.AmountSettlement = 0;
            reportOperate.QuotationAmount = 0;
            reportOperate.IssuerKvid = Guid.Empty;
            reportOperate.IssuerName = String.Empty;
            reportOperate.IssueDate = null;
            reportOperate.Kvid = Guid.NewGuid();
            reportOperate.Status = (int)ReportStatusType.CommissionAccept;
            //2.获取待创建文件集合
            var filesOperate = new List<EntityDbFile<Report>>();
            foreach (var originalFile in originalFiles)
            {
                var file = new EntityDbFile<Report>();
                file.PopulateWith(originalFile);
                file.OwnerKvid = reportOperate.Kvid;
                file.Kvid = Guid.NewGuid();
                filesOperate.Add(file);
            }
            //3.获取待创建检验项目
            var reportItemsOperate = new List<ReportItem>();
            foreach (var originalReportItem in originalReportItems)
            {
                var reportItem = new ReportItem();
                reportItem.PopulateWith(originalReportItem);
                reportItem.Amount = 0;
                reportItem.AmountPlan = 0;
                reportItem.ReportKvid = reportOperate.Kvid;
                reportItem.ParentKvid = reportItemKvidsReplace.Where(o => o.OriginalKvid == originalReportItem.ParentKvid).FirstOrDefault().MatchKvid;
                reportItem.Status = int.MaxValue;
                reportItem.Kvid = reportItemKvidsReplace.Where(o => o.OriginalKvid == originalReportItem.Kvid).FirstOrDefault().MatchKvid;
                reportItemsOperate.Add(reportItem);
            }
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            try
            {
                conn.Insert(reportOperate);
                reportOperate.RemoveAllOnlyProperties();
                rtns.Results.Add(reportOperate);
                foreach (var fileOperate in filesOperate)
                {
                    conn.Insert(fileOperate);
                }
                foreach (var reportItemOperate in reportItemsOperate)
                {
                    conn.Insert(reportItemOperate);
                }
                trans?.Commit();
                rtns.Total = rtns.Results.Count;
                return rtns;
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
        }
    }
    //[RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    //[Api(Description = "生成X报告")]
    //public class ReportChangeCreate2 : RestfulExecution<Report>{
    //    public Report ChangeReport { get; set; }
    //    public override object OnExecution(IRequest req, IResponse res)
    //    {
    //        #region 数据验证
    //        ChangeReport.ThrowIfNull("未传入更改报告信息!");
    //        //需要根据原报告Kvid去创建对应关系
    //        ChangeReport.Kvid.ThrowIfEmpty("未找到对应原报告!");
    //        #endregion
    //        var conn = KiviiContext.GetOpenedDbConnection<Report>();
    //        var trans = conn.OpenTransaction();
    //        var rtns = new RestfulQueryResponse<Report>();
    //        rtns.Results = new List<Report>();
    //        try
    //        {
    //            var reportIdPrefix = Regex.Split(ChangeReport.ReportId, "-X", RegexOptions.IgnoreCase)[0];
    //            //1.创建报告
    //            var report = new Report();
    //            report.Metadata = new Dictionary<string, string>();
    //            report.PopulateWith(ChangeReport);
    //            report.ReportId = reportIdPrefix + "-X{?}";
    //            report.AmountTestPlan = 0;
    //            report.AmountTest = 0;
    //            report.AmountPlan = 0;
    //            report.Amount = 0;
    //            report.AmountPayment = 0;
    //            report.AmountSettlement = 0;
    //            report.IssuerKvid = Guid.Empty;
    //            report.IssuerName = String.Empty;
    //            report.IssueDate = null;
    //            report.Kvid = Guid.NewGuid();
    //            report.Status = (int)ReportStatusType.CommissionAccept;
    //            conn.Insert(report);
    //            report.RemoveAllOnlyProperties();
    //            rtns.Results.Add(report);
    //            //2.关联报告相关文件
    //            var queryOriginalFiles = conn.From<EntityDbFile<Report>>();
    //            queryOriginalFiles.Where(o=>o.OwnerKvid==ChangeReport.Kvid&&o.DbFolderPath== "/Report/Images");
    //            var originalFiles = conn.Select(queryOriginalFiles);
    //            foreach (var originalFile in originalFiles) {
    //                var file = new EntityDbFile<Report>();
    //                file.PopulateWith(originalFile);
    //                file.OwnerKvid = report.Kvid;
    //                file.Kvid = Guid.NewGuid();
    //                conn.Insert(file);
    //            }
    //            //3.创建检验项目
    //            var queryOriginalReportItems = conn.From<ReportItem>();
    //            queryOriginalReportItems.Where(o=>o.ReportKvid==ChangeReport.Kvid);
    //            var originalReportItems = conn.Select(queryOriginalReportItems);
    //            //获取检验项目新老Kvid集合
    //            var reportItemsKvids = originalReportItems.Select(o => o.Kvid).ToList();
    //            var reportItemKvidsReplace = new List<MatchKvids>();
    //            reportItemKvidsReplace.Add(new MatchKvids
    //            {
    //                OriginalKvid = Guid.Empty,
    //                MatchKvid = Guid.Empty
    //            });
    //            foreach (var reportItemKvid in reportItemsKvids)
    //            {
    //                reportItemKvidsReplace.Add(new MatchKvids
    //                {
    //                    OriginalKvid = reportItemKvid,
    //                    MatchKvid = Guid.NewGuid()
    //                });
    //            }
    //            foreach (var originalReportItem in originalReportItems) {
    //                var reportItem = new ReportItem();
    //                reportItem.PopulateWith(originalReportItem);
    //                reportItem.ReportKvid = report.Kvid;
    //                reportItem.ParentKvid=reportItemKvidsReplace.Where(o => o.OriginalKvid == originalReportItem.ParentKvid).FirstOrDefault().MatchKvid;
    //                reportItem.Status = int.MaxValue;
    //                reportItem.Kvid = reportItemKvidsReplace.Where(o => o.OriginalKvid == originalReportItem.Kvid).FirstOrDefault().MatchKvid;
    //                conn.Insert(reportItem);
    //            }
    //            trans?.Commit();
    //            rtns.Total = rtns.Results.Count;
    //            return rtns;
    //        }
    //        catch (Exception ex) {
    //            trans?.Rollback();
    //            throw ex;
    //        }
    //    }
    //}

    #endregion

    #region 报告下载
    [Api(Description = "报告下载")]
    [Authenticate]
    public class ReportFileDownlload : RestfulExecution<Report>
    {
        public List<Guid> Kvids { get; set; }

        public string FolderPath { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("请传入要下载的报告Kvids");
            FolderPath.ThrowIfNullOrEmpty("请指定要下载的目录地址");

            var conn = KiviiContext.GetOpenedDbConnection<EntityDbFile<Report>>();

            var queryPdfEntityFile = conn.From<EntityDbFile<Report>>();
            queryPdfEntityFile.Where(o => o.OwnerKvid != Guid.Empty && o.ParentKvid == Guid.Empty && o.DbFolderPath == FolderPath && o.Extension.ToLower() == ".pdf" && Sql.In(o.OwnerKvid, Kvids));
            var pdfEntityFiles = conn.Select(queryPdfEntityFile);
            if (pdfEntityFiles == null || pdfEntityFiles.Count == 0) return null;
            if (pdfEntityFiles.Count == 1)
            {
                var dbFile = pdfEntityFiles[0];
                var physicalStorageFilePath = dbFile.GetPhysicalPath();
                if (!File.Exists(physicalStorageFilePath)) throw new Exception("服务器端文件已被移除");
                FileInfo localFile = new FileInfo(physicalStorageFilePath);

                var log = new EntityLog<Report>();
                log.OwnerKvid = dbFile.OwnerKvid;
                log.BizId = dbFile.Name;
                log.BizType = typeof(Report).FullName;
                log.Type = "ReportDownload";
                log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                log.Summary = "报告下载";
                log.Remark = $"由 {KiviiContext.CurrentMember.FullName} 下载";
                conn.Insert(log);

                return new HttpResult(localFile, asAttachment: true, fileName: dbFile.Name);
            }
            var memoryStream = new MemoryStream();
            using (var zipFile = new ZipFile(Encoding.UTF8))
            {
                foreach (var dbFile in pdfEntityFiles)
                {
                    var physicalStorageFilePath = dbFile.GetPhysicalPath();
                    if (!File.Exists(physicalStorageFilePath))
                    {
                        zipFile.AddEntry(dbFile.Name + "(文件不存在).txt", "错误:文件不存在");
                        continue;
                    }
                    FileInfo localFile = new FileInfo(physicalStorageFilePath);
                    zipFile.AddEntry(dbFile.Name, localFile.ReadFully());

                    var log = new EntityLog<Report>();
                    log.OwnerKvid = dbFile.OwnerKvid;
                    log.BizId = dbFile.Name;
                    log.BizType = typeof(Report).FullName;
                    log.Type = "ReportDownload";
                    log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    log.Summary = "报告下载";
                    log.Remark = $"由 {KiviiContext.CurrentMember.FullName} 下载";
                    conn.Insert(log);
                }
                zipFile.Save(memoryStream);
            }
            return new HttpResult(memoryStream, "{0}等{1}个文件.zip".Fmt(Path.GetFileNameWithoutExtension(pdfEntityFiles[0].Name), pdfEntityFiles.Count), true);
        }
    }
    #endregion

    #region 报告强制删除
    [Api(Description = "报告强制删除")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportForceDelete : RestfulExecution<Report>
    {
        public List<Guid> Kvids { get; set; }
        public string Reason { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            Kvids.ThrowIfNullOrEmpty("参数错误:未传入要删除的报告Kvids");
            Reason.ThrowIfNullOrEmpty("必须输入删除原因！");

            var rtns = new RestfulDeleteResponse<Guid>();
            rtns.Results = new List<Guid>();

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var reports = conn.SelectByIds<Report>(Kvids);
            if (reports.IsNullOrEmpty()) return rtns;

            //待操作样品Kvids
            var sampleKvids = reports.ConvertAll(o => o.SampleKvid);

            //获取待删除关联样品集合
            //先判断是否存在引用了待删除报告样品号的报告,如果有,忽略更新样品信息
            var existAllReportsBySample = conn.Select<Report>(o => Sql.In(o.SampleKvid, sampleKvids));

            if (existAllReportsBySample.Count == reports.Count)
            {
                var updateSampleLamda = conn.From<Sample>();
                updateSampleLamda = updateSampleLamda.Update(o => o.Status);
                updateSampleLamda = updateSampleLamda.Where(o => Sql.In(o.RootKvid, sampleKvids));
                conn.UpdateOnly(new Sample { Status = -1 }, updateSampleLamda);
            }
            else
            {
                foreach (var report in reports)
                {
                    var existReports = existAllReportsBySample.Where(o => o.SampleKvid == report.SampleKvid).ToList();
                    if (existReports.Count > 1) continue;
                    var updateSampleLamda = conn.From<Sample>();
                    updateSampleLamda = updateSampleLamda.Update(o => o.Status);
                    updateSampleLamda = updateSampleLamda.Where(o => o.RootKvid == report.SampleKvid);
                    conn.UpdateOnly(new Sample { Status = -1 }, updateSampleLamda);
                }
            }
            //样品来源
            var sampleSourceKvids = reports.ConvertAll(o => o.SampleSourceKvid);

            //获取待删除关联样品来源集合
            //先判断是否存在引用了待删除报告样品来源的报告,如果有,忽略更新样品来源信息
            var existAllReportsBySampleSource = conn.Select<Report>(o => Sql.In(o.SampleSourceKvid, sampleSourceKvids));

            if (existAllReportsBySampleSource.Count == reports.Count)
            {
                var updateSampleSourceLamda = conn.From<SampleSource>();
                updateSampleSourceLamda = updateSampleSourceLamda.Update(o => o.Status);
                updateSampleSourceLamda = updateSampleSourceLamda.Where(o => Sql.In(o.Kvid, sampleSourceKvids));
                conn.UpdateOnly<SampleSource>(new SampleSource { Status = -1 }, updateSampleSourceLamda);
            }
            else
            {
                foreach (var report in reports)
                {
                    var existReports = existAllReportsBySampleSource.Where(o => o.SampleSourceKvid == report.SampleSourceKvid).ToList();
                    if (existReports.Count > 1) continue;
                    var updateSampleSourceLamda = conn.From<SampleSource>();
                    updateSampleSourceLamda = updateSampleSourceLamda.Update(o => o.Status);
                    updateSampleSourceLamda = updateSampleSourceLamda.Where(o => Sql.In(o.Kvid, sampleSourceKvids));
                    conn.UpdateOnly<SampleSource>(new SampleSource { Status = -1 }, updateSampleSourceLamda);
                }
            }

            var queryTestRootKvids = conn.From<ReportItem>();
            queryTestRootKvids.Where(o => Sql.In(o.ReportKvid, reports.ConvertAll(p => p.Kvid)));
            queryTestRootKvids.Select(o => o.TestRootKvid);
            //test项目
            var testRootKvids = conn.Select<Guid>(queryTestRootKvids);
            testRootKvids = testRootKvids.Distinct().ToList();

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);

            try
            {
                //删除Test
                var updateTestLamda = conn.From<Test>();
                updateTestLamda = updateTestLamda.Update(o => o.Status);
                updateTestLamda = updateTestLamda.Where(o => Sql.In(o.RootKvid, testRootKvids));
                conn.UpdateOnly<Test>(new Test { Status = -1 }, updateTestLamda);

                //删除Reports
                var updateReportLamda = conn.From<Report>();
                updateReportLamda = updateReportLamda.Update(o => o.Status);
                updateReportLamda = updateReportLamda.Where(o => Sql.In(o.Kvid, reports.ConvertAll(p => p.Kvid)));
                conn.UpdateOnly<Report>(new Report { Status = -1 }, updateReportLamda);

                // 删除ReportItems
                var updateReportItemLamda = conn.From<ReportItem>();
                updateReportItemLamda = updateReportItemLamda.Update(o => o.Status);
                updateReportItemLamda = updateReportItemLamda.Where(o => Sql.In(o.ReportKvid, reports.ConvertAll(p => p.Kvid)));
                conn.UpdateOnly<ReportItem>(new ReportItem { Status = -1 }, updateReportItemLamda);

                foreach (var item in reports)
                {
                    var log = new EntityLog<Report>();
                    log.OwnerKvid = item.Kvid;
                    log.BizId = item.ReportId;
                    log.BizKvid = item.Kvid;
                    log.BizType = typeof(Report).FullName;
                    log.Type = "报告强制删除";
                    log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    log.Summary = Reason;
                    log.Remark = $"[强制删除] 已成功将{item.ReportId}删除,操作人:{KiviiContext.CurrentMember.FullName},时间:{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}!";
                    conn.Insert(log);
                }

                rtns.Results.AddRange(reports.ConvertAll(o => o.Kvid));

                trans?.Commit();

            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            return rtns;
        }
    }
    #endregion

    #region 签发召回报告
    [Api(Description = "签发召回报告")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportIssuedCallBack : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public string Description { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("参数不能为空！");
            Description.ThrowIfNullOrEmpty("请输入召回原因！");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到当前报告！");

            var originalStatus = report.Status;

            if (report.Status <= 700) throw new Exception("当前报告尚未签发，无需召回");
            report.Status = 700;
            report.AddOnlyProperties(o => o.Status);
            conn.UpdateOnly(report);

            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            rtns.Results.Add(report);

            report.AddReportNote("报告召回", ReportExtension.AssignDescription(originalStatus), ReportExtension.AssignDescription(700), Description, conn);

            return rtns;
        }
    }
    #endregion

    #region 同步更新报告随附文件信息,处理异常数据
    [Api(Description = "同步随附文件信息")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportDescriptionSpecialSync : RestfulExecution<Report>
    {
        public string InternalCode { get; set; }
        public bool isUpdateAll { get; set; }
        public string ReportId { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            if (InternalCode.IsNullOrEmpty()) InternalCode = "report";
            if (!ReportId.IsNullOrEmpty())
            {
                var conn = KiviiContext.GetOpenedDbConnection<Report>();
                var queryReport = conn.From<Report>();
                queryReport.Where(o => o.ReportId == ReportId);
                var report = conn.Select(queryReport).FirstOrDefault();
                if (report == null) throw new Exception("未找到对应报告信息!");
                var queryReportNote = conn.From<EntityNote<Report>>();
                queryReportNote.Where(o => o.InternalCode == InternalCode && o.EntityKvid == report.Kvid).OrderByDescending(o => o.CreateTime);
                var reportNote = conn.Select(queryReportNote).FirstOrDefault();
                if (reportNote == null) throw new Exception("当前报告未维护随附文件信息,更新失败!");
                if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
                if (InternalCode == "report")
                {
                    if (report.Metadata.ContainsKey("DescriptionSpecial") && report.Metadata["DescriptionSpecial"] == reportNote.Remark) return rtns;
                    report.Metadata["DescriptionSpecial"] = reportNote.Remark;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                if (InternalCode == "reportupdate")
                {
                    if (report.Metadata.ContainsKey("DescriptionUpdate") && report.Metadata["DescriptionUpdate"] == reportNote.Remark) return rtns;
                    report.Metadata["DescriptionUpdate"] = reportNote.Remark;
                    report.AddOnlyProperties(o => o.Metadata);
                }
                conn.UpdateOnly(report);
                report.RemoveAllOnlyProperties();
                rtns.Results.Add(report);
            }
            else {
                if (!isUpdateAll) return rtns;
                var conn = KiviiContext.GetOpenedDbConnection<Report>();
                //获取所有报告最新随附文件信息
                var queryReportNotesAll = conn.From<EntityNote<Report>>();
                queryReportNotesAll.Where(o => o.InternalCode == InternalCode);
                queryReportNotesAll.OrderByDescending(o => new { o.EntityKvid, o.CreateTime });
                var reportNotesAll = conn.Select(queryReportNotesAll);
                //记录已存在报告的信息
                var entityKvidExist = new List<Guid>();
                var reportNotes = new List<EntityNote<Report>>();
                foreach (var reportNote in reportNotesAll)
                {
                    if (entityKvidExist.Contains(reportNote.EntityKvid)) continue;
                    entityKvidExist.Add(reportNote.EntityKvid);
                    reportNotes.Add(reportNote);
                }
                //获取需要修改特定字段的报告
                var queryReports = conn.From<Report>();
                queryReports.Where(o => Sql.In(o.Kvid, entityKvidExist));
                var reports = conn.Select(queryReports);
                foreach (var report in reports)
                {
                    var reportNote = reportNotesAll.FirstOrDefault(o => o.EntityKvid == report.Kvid);
                    if (reportNote == null) throw new Exception("未找到对应随附文件记录!");
                    if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
                    if (InternalCode == "report")
                    {
                        if (report.Metadata.ContainsKey("DescriptionSpecial") && report.Metadata["DescriptionSpecial"] == reportNote.Remark) continue;
                        report.Metadata["DescriptionSpecial"] = reportNote.Remark;
                        report.AddOnlyProperties(o => o.Metadata);
                    }
                    if (InternalCode == "reportupdate")
                    {
                        if (report.Metadata.ContainsKey("DescriptionUpdate") && report.Metadata["DescriptionUpdate"] == reportNote.Remark) continue;
                        report.Metadata["DescriptionUpdate"] = reportNote.Remark;
                        report.AddOnlyProperties(o => o.Metadata);
                    }
                    conn.UpdateOnly(report);
                    report.RemoveAllOnlyProperties();
                    rtns.Results.Add(report);
                }
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 委托报告不合格项目查询
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportUnQualifiedQuery : RestfulExecution<Report>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public string Title { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Report>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.Status == int.MaxValue && o.TestJudgement == "不合格" && (o.TypeEx == "W" || o.TypeEx == "Z"));
            if (!Title.IsNullOrEmpty()) sqlExpress.And(o => o.TestDetections.Contains(Title));
            //sqlExpress.Rows = null;
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            //先查出所有不合格的报告
            //查出所有不合格的项目
            var reports = new List<Report>();
            reports = rtns.Results;
            var reportKvids = reports.Select(o => o.Kvid);
            var queryAllReportItems = conn.From<ReportItem>();
            queryAllReportItems.Where(o => Sql.In(o.ReportKvid, reportKvids));
            var allReportItems = conn.Select(queryAllReportItems);
            List<Report> rtnReports = new List<Report>();
            //遍历所有报告，将不合格项目拼一个固定的格式
            foreach (var en in reports)
            {
                var currentReportItems = allReportItems.Where(o => o.ReportKvid == en.Kvid).ToList();
                currentReportItems.ThrowIfNullOrEmpty("未找到数据！");
                var currentMainUnQualfiedReportItems = currentReportItems.Where(o => o.Judgement == "不合格");
                // currentMainUnQualfiedReportItems = currentMainUnQualfiedReportItems.Where(o => o.Title.Contains(Title));
                en.SupplierContactAddress = "";
                var unQualifiedList = new List<string>();
                foreach (var kv in currentMainUnQualfiedReportItems)
                {
                    //只处理检测项，若为检测项，找大项
                    if (kv.Type != ReportItemType.Test) continue;
                    var parentReportItem = currentReportItems.Where(o => o.Kvid == kv.ParentKvid).FirstOrDefault();

                    //处理无子项的大项
                    if (parentReportItem == null || parentReportItem.Type != ReportItemType.TestFolder)
                    {
                        if (!Title.IsNullOrEmpty() && !kv.Title.ToUpper().Contains(Title.ToUpper())) continue;
                        unQualifiedList.Add(kv.Title + "(标准值：" + kv.DetectionValueQualified + " 实测值：" + kv.TestValue + ")");
                    }
                    else if (parentReportItem.Type == ReportItemType.TestFolder)
                    {
                        if (!Title.IsNullOrEmpty() && !parentReportItem.Title.ToUpper().Contains(Title.ToUpper())) continue;
                        unQualifiedList.Add(parentReportItem.Title + "-" + kv.Title + "(标准值：" + kv.DetectionValueQualified + " 实测值：" + kv.TestValue + ")");
                    }
                    else
                    {
                        continue;
                    }
                }
                en.SupplierContactAddress = string.Join("、", unQualifiedList);
                if (en.SupplierContactAddress != "") rtnReports.Add(en);
            }
            //当传入项目名称参数时，要过滤掉无符合结果的报告SupplierContactAddress为空
            rtns.Results = rtnReports;
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 生成合格报告
    public class ReportGenerateTypeH : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("参数不能为空！");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到报告，无法生成合格报告！");
            if (report.StatusType != ReportStatusType.ReportCollected) throw new Exception("非归档报告无法生成合格报告");

            var rtns = new RestfulCreateResponse<Report>();
            rtns.Results = new List<Report>();

            #region 生成report
            var newReport = new Report();
            newReport.PopulateWith(report);
            newReport.Kvid = Guid.NewGuid();
            newReport.ReportId = string.Empty;
            newReport.ReportIdNumber = 0;
            newReport.BizId = report.ReportId;
            newReport.BizType = "H";
            newReport.BizKvid = report.Kvid;
            newReport.TestJudgement = "";
            newReport.Conclusion = string.Empty;
            newReport.Status = 100;
            newReport.DeadDate = null;
            newReport.PreparationDate = null;
            newReport.ReviewDate = null;
            newReport.IssueDate = null;
            newReport.OperatorKvid = KiviiContext.CurrentMember.Kvid;
            newReport.OperatorName = KiviiContext.CurrentMember.FullName;
            newReport.ManagerKvid = Guid.Empty;
            newReport.ManagerName = String.Empty;
            newReport.PreparerKvid = Guid.Empty;
            newReport.PreparerName = String.Empty;
            newReport.ReviewerKvid = Guid.Empty;
            newReport.ReviewerName = String.Empty;
            newReport.IssuerKvid = Guid.Empty;
            newReport.IssuerName = String.Empty;
            #endregion

            #region 生成ReportItem
            //获取引用报告的所有检验项目
            var queryReportItemsReference = conn.From<ReportItem>();
            queryReportItemsReference.Where(o => o.ReportKvid == report.Kvid);
            var reportItemsReference = conn.Select(queryReportItemsReference);
            //获取检验项目新老Kvid集合
            var reportItemsKvids = reportItemsReference.Select(o => o.Kvid).ToList();
            var reportItemsMatchKvids = new List<MatchKvids>();
            reportItemsMatchKvids.Add(new MatchKvids
            {
                OriginalKvid = Guid.Empty,
                MatchKvid = Guid.Empty
            });
            foreach (var reportItemKvid in reportItemsKvids)
            {
                reportItemsMatchKvids.Add(new MatchKvids
                {
                    OriginalKvid = reportItemKvid,
                    MatchKvid = Guid.NewGuid()
                });
            }
            var reportItemsOperate = new List<ReportItem>();
            //获取待创建检验项目集合
            foreach (var reportItemReference in reportItemsReference)
            {
                var reportItem = new ReportItem();
                reportItem.PopulateWith(reportItemReference);
                reportItem.ReportKvid = newReport.Kvid;
                reportItem.RootKvid = reportItemsMatchKvids.Single(o => o.OriginalKvid == reportItemReference.RootKvid).MatchKvid;
                reportItem.ParentKvid = reportItemsMatchKvids.Single(o => o.OriginalKvid == reportItemReference.ParentKvid).MatchKvid;
                reportItem.Kvid = reportItemsMatchKvids.Single(o => o.OriginalKvid == reportItemReference.Kvid).MatchKvid;
                reportItem.AmountPlan = 0;
                reportItem.Amount = 0;
                reportItemsOperate.Add(reportItem);
            }
            #endregion

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                conn.Insert(newReport);
                foreach (var en in reportItemsOperate)
                {
                    conn.Insert(en);
                }
                trans?.Commit();
                rtns.Results.Add(newReport);
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }
            return rtns;
        }
    }
    #endregion

    #region 按组统计报告数量及待完成数量
    public class ReportPending
    {
        public string OwnerName { get; set; }
        public int ReportTotalAmount { get; set; }
        public int ReportPendingAmount { get; set; }
        public List<Guid> ReportKvids { get; set; }
        public int SortId { get; set; }
    }
    public class ReportPendingPivot : RestfulExecution<Report>
    {
        /// <summary>
        /// 报告的状态，不同状态的统计维度是不一样的
        /// </summary>
        public ReportStatusType ReportStatusType { get; set; }
        /// <summary>
        /// 根据报告的完成日期统计
        /// </summary>
        public DateTime DeadDateFrom { get; set; }
        public DateTime DeadDateTo { get; set; }
        /// <summary>
        /// 根据不同报告类型进行查询，报告的TypeEx字段
        /// </summary>
        public string ReportType { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportStatusType.ThrowIfNull("参数异常！");
            DeadDateFrom.ThrowIfNull("必须传入起始日期！");
            DeadDateTo.ThrowIfNull("必须传入终止日期！");
            var days = DeadDateTo.Subtract(DeadDateFrom).Days;
            if (days > 92) throw new Exception("最多查询三个月的数据，您已超出日期范围上线！");

            var rtns = new RestfulExecutionResponse<ReportPending>();
            rtns.Results = new List<ReportPending>();
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            //2022.4.1 统计数据录入的待处理
            if (ReportStatusType == ReportStatusType.DataEntry)
            {
                //根据条件查出所有报告的Kvid
                var queryReportKvid = conn.From<Report>();
                queryReportKvid.Select(o => o.Kvid);
                queryReportKvid.Where(o => o.DeadDate >= DeadDateFrom && o.DeadDate <= DeadDateTo);
                if (!ReportType.IsNullOrEmpty())
                {
                    queryReportKvid.And(o => o.TypeEx == ReportType);
                }

                //一次性查出日期范围内的报告的项目
                var queryReportItem = conn.From<ReportItem>();
                queryReportItem.Select(o => new { o.ReportKvid, o.WorkGroupName, o.Status, o.Type });
                queryReportItem.Where(o => Sql.In(o.ReportKvid, queryReportKvid));
                var reportItems = conn.Select(queryReportItem);
                reportItems = reportItems.filterReportItem();
                //根据检验组分组统计
                var groups = reportItems.GroupBy(o => o.WorkGroupName);
                foreach (var kv in groups)
                {
                    var reporPending = new ReportPending();
                    reporPending.OwnerName = kv.Key;
                    //先算每组的总批次数
                    var currentAll = reportItems.Where(o => o.WorkGroupName == kv.Key).Select(o => o.ReportKvid).ToList().Distinct();
                    var currentPending = reportItems.Where(o => o.WorkGroupName == kv.Key && o.Status >= 200 && o.Status <= 300).Select(o => o.ReportKvid).ToList().Distinct();
                    var totalAmount = currentAll.Count();
                    var totalPendingAmount = currentPending.Count();
                    reporPending.ReportTotalAmount = totalAmount;
                    reporPending.ReportPendingAmount = totalPendingAmount;
                    var pendingReportKvids = new List<Guid>();
                    pendingReportKvids.AddRange(currentPending);
                    reporPending.ReportKvids = pendingReportKvids;
                    rtns.Results.Add(reporPending);
                }
            }
            //按照指定顺序排序
            foreach (var rtn in rtns.Results)
            {
                if (rtn.OwnerName == "安全组") rtn.SortId = 1;
                else if (rtn.OwnerName == "纤维含量组") rtn.SortId = 2;
                else if (rtn.OwnerName == "微生物组") rtn.SortId = 3;
                else if (rtn.OwnerName == "物理组") rtn.SortId = 4;
                else if (rtn.OwnerName == "色牢度组") rtn.SortId = 5;
                else if (rtn.OwnerName == "开样组") rtn.SortId = 6;
                else if (rtn.OwnerName == "功能组") rtn.SortId = 7;
                else if (rtn.OwnerName == "轻工组") rtn.SortId = 8;
                else if (rtn.OwnerName == "纤检组") rtn.SortId = 9;
                else if (rtn.OwnerName == "羽绒组") rtn.SortId = 10;
                else rtn.SortId = 99;
            }
            rtns.Results = rtns.Results.OrderBy(o => o.SortId).ToList();
            return rtns;
        }
    }
    #endregion

    #region 标记报告为待复核及重置
    [Api(Description = "报告标记待复核及重置")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportRemarkReviewingChange : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("参数ReportKvid不能为空");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到当前报告！");

            //根据当前人的角色
            var queryTestRoles = conn.From<EntityDuty<Test>>();
            queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
            queryTestRoles.Select(o => o.RoleName);
            var testRoleNames = conn.Select(queryTestRoles).ConvertAll(o => o.RoleName);

            var testRoleNameString = testRoleNames.Join(",");
            //如果当前角色无检验组，则以组织机构的部门为准
            if(testRoleNameString.IsNullOrEmpty()) testRoleNameString= KiviiContext.CurrentMember.DepartmentName;
            //获取当前人的组别、以及否待复核，从而判别是进入待复核还是取消待复核
            //var currentDepartment = KiviiContext.CurrentMember.DepartmentName;
            if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
            if (report.Metadata.ContainsKey("ReviewingGroup"))
            {
                if (report.Metadata["ReviewingGroup"].Contains(testRoleNameString))
                {
                    report.Metadata["ReviewingGroup"] = report.Metadata["ReviewingGroup"].Replace(testRoleNameString, string.Empty);
                }
                else
                {
                    report.Metadata["ReviewingGroup"] += testRoleNameString;
                }
            }
            else
            {
                report.Metadata["ReviewingGroup"] = testRoleNameString;
            }
            report.AddOnlyProperties(o => o.Metadata);
            conn.UpdateOnly(report);
            var rtns = new RestfulUpdateResponse<Report>();
            rtns.Results = new List<Report>();
            rtns.Results.Add(report);
            return rtns;
        }
    }
    #endregion

    #region 同款号报告提示
    public class ReportSampleBatchNumberValidResponse
    {
        public bool IsExist { get; set; }
        public string Tips { get; set; }
    }
    [Api(Description = "同款号报告提示")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportSampleBatchNumberValid : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("参数ReportKvid不能为空");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到当前报告！");

            var rtns = new ReportSampleBatchNumberValidResponse();

            var querySameSampleBatchNumberReport = conn.From<Report>();
            querySameSampleBatchNumberReport.Select(o => o.ReportId);
            querySameSampleBatchNumberReport.Where(o => o.Status == int.MaxValue && o.SampleBatchNumber != string.Empty && o.SampleBatchNumber != null && o.SampleBatchNumber == report.SampleBatchNumber);
            querySameSampleBatchNumberReport.OrderByDescending(o => o.CreateTime);
            var SameSampleBatchNumberReports = conn.Select(querySameSampleBatchNumberReport);
            if (SameSampleBatchNumberReports.Count == 0)
            {
                rtns.IsExist = false;
            }
            else
            {
                rtns.IsExist = true;
                foreach (var en in SameSampleBatchNumberReports)
                {
                    rtns.Tips = rtns.Tips + en.ReportId + "\n";
                }
            }
            return rtns;
        }
    }
    #endregion

    #region 报告查询-App使用
    /// <summary>
    /// 报告查询
    /// </summary>
    public class EntityForApp
    {
        public string DisplayName { get; set; }
        public string FullName { get; set; }
        public string InternalCode { get; set; }
        public string Path { get; set; }
        public string DateInfo { get; set; }
        public string OperatorInfo { get; set; }
        public string Remark { get; set; }
        public string FollowType { get; set; }
        public string NavigateTitle { get; set; }
        public string NavigateUrl { get; set; }
        public string ShareTitle { get; set; }
        public string ShareUrl { get; set; }
        public string ShowUrl { get; set; }
        public Guid Kvid { get; set; }
    }
    //代码初审 张梦圆 20210906
    [Route("/Restful/Kivii.Lac.Lims.Entities.Report/EntityForAppQuery")]
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportEntityForAppQuery : RestfulQuery<Report>
    {
        public override object OnRestfulQueryResponse(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulQueryResponse<Report> rtns)
        {
            var rtnsTrans = new RestfulQueryResponse<EntityForApp>();
            rtnsTrans.Results = new List<EntityForApp>();
            var reports = rtns.Results.OrderByDescending(o => o.DealDate).ToList();
            foreach (var result in reports)
            {
                var kvidPrefix = result.Kvid.ToString().Substring(0, 4);
                var newResult = new EntityForApp();
                newResult.DisplayName = $"{result.SampleBrand + result.SampleName}";
                newResult.FullName = result.ReportId;
                newResult.DateInfo = result.DealDate != null ? $"发布日期:{result.DealDate.Value:yyyy-MM-dd}" : "";
                newResult.Remark = $"检测结果:{result.TestJudgement}";
                newResult.Kvid = result.Kvid;
                newResult.FollowType = "Kivii.Lac.Lims.Entities.Report";
                newResult.NavigateTitle = $"当前跳转至:{result.ReportId}";
                newResult.NavigateUrl = $"http://www.jst-gov.com/Url/{kvidPrefix}/{result.ReportId}";
                newResult.ShareTitle = $"您的好友向您分享了{result.ReportId}";
                newResult.ShareUrl = $"/report/detail/{result.Kvid}";
                newResult.ShowUrl = $"/report/detail/{result.Kvid}";
                rtnsTrans.Results.Add(newResult);
            }
            return rtnsTrans;
        }
    }

    [Route("/report/detail/{kvid}")]
    [Route("/report/detail/{kvid}.html")]
    //代码初审 张梦圆 20210906
    public class ReportEntityForAppRead : RestfulExecution<Report>
    {
        public Guid Kvid { get; set; }
        public string Template { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var rtns = new Report();
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            rtns = conn.SingleById<Report>(Kvid);
            rtns.ThrowIfNull("未找到对应报告!");
            var view = "TemplateCommonDetailAppReport";
            if (!Template.IsNullOrEmpty()) view = Template;
            return new HttpResult
            {
                Response = rtns,
                View = view
            };
        }
    }

    [Authenticate]
    [Api(Description = "报告查询-标记附加信息")]
    public class ReportExQuery : RestfulExecution<Report>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Report>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.And(o => o.OperatorName == KiviiContext.CurrentMember.FullName);
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            rtns.Results.Each(o => o.TestNeedJudge = false);
            //组织是否上传样品图片的信息
            var sampleKvids = rtns.Results.ConvertAll(o => o.SampleKvid);
            var sampleFiles = conn.Select<EntityDbFile<Sample>>(o => Sql.In(o.OwnerKvid, sampleKvids));
            var sampleKvidsWithSampleFile = sampleFiles.ConvertAll(o => o.OwnerKvid).Distinct().ToList();
            var reportsWithSampleFile = rtns.Results.Where(o => Sql.In(o.SampleKvid, sampleKvidsWithSampleFile)).ToList();
            reportsWithSampleFile.Each(o => o.TestNeedJudge = true);
            return rtns;
        }
    }

    #endregion

    #region 报告统计-大屏使用

    public class ReportsStatisticsInfos
    {
        //组别
        public string WorkGroupName { get; set; }
        //待处理报告总数
        public int ReportsPending { get; set; }
        //待处理报告(打回)
        public int ReportsPendingCallBack { get; set; }
        //待处理报告(超期)
        public int ReportsPendingOverdue { get; set; }
        //待处理报告(加急)
        public int ReportsPendingUrgent { get; set; }
        //今日报告总数
        public int ReportsToday { get; set; }
        //今日待处理报告总数
        public int ReportsPendingToday { get; set; }
        //今日已完成报告总数
        public int ReportsFinishedToday { get; set; }
        //今日待处理报告(打回)
        public int ReportsPendingCallBackToday { get; set; }
        //今日待处理报告(加急)
        public int ReportsPendingUrgentToday { get; set; }
        //今日待处理项目数
        public int ReportItemsPendingToday { get; set; }
        //报告未完成率
        public double ReportsPendingRateToday { get; set; }
        //报告完成率
        public double ReportsFinishedRateToday { get; set; }
    }

    public class ReportsProcessInfos
    {
        //报告统计信息
        public ReportsStatisticsInfos ReportsStatisticsInfos { get; set; }
        //待处理报告列表(超期)--委托
        public List<Report> ReportsPeddingOverdueEntrustToday { get; set; }
        //待处理报告列表(超期)--监督
        public List<Report> ReportsPeddingOverdueSuperviseToday { get; set; }
        //今日待处理报告列表
        public List<Report> ReportsPeddingToday { get; set; }
        //今日待处理报告列表(打回)
        public List<Report> ReportsPeddingCallBackToday { get; set; }
        //今日待处理报告列表(加急)
        public List<Report> ReportsPeddingUrgentToday { get; set; }

    }
    /// <summary>
    /// 数据录入
    /// </summary>
    [Route("/reportsProcess/{workGroupName}")]
    public class ReportsProcessForDataEntry : RestfulExecution<Report>
    {
        public string WorkGroupName { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            if (WorkGroupName.IsNullOrEmpty()) return false;
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = $"{KiviiContext.GetUrnKey("reports:")}cachedReportsProcessForScreen_{WorkGroupName}";
            var rtns = cache.Get<ReportsProcessInfos>(cacheKey);
            if (rtns != null) return rtns;
            rtns = new ReportsProcessInfos();
            rtns.ReportsStatisticsInfos = new ReportsStatisticsInfos();
            rtns.ReportsPeddingToday = new List<Report>();
            rtns.ReportsPeddingOverdueEntrustToday = new List<Report>();
            rtns.ReportsPeddingOverdueSuperviseToday = new List<Report>();
            rtns.ReportsPeddingUrgentToday = new List<Report>();
            rtns.ReportsPeddingCallBackToday = new List<Report>();
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            //获取今日时间节点
            var beginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day);
            var endTime = beginTime.AddDays(1);
            rtns.ReportsStatisticsInfos.WorkGroupName = WorkGroupName;
            //获取待处理报告
            var queryReportKvidsPending = conn.From<ReportItem>();
            queryReportKvidsPending.Where(o => (o.Status == (int)ReportItemStatusType.Testing || o.Status == (int)ReportItemStatusType.BeforeReview || o.Status == (int)ReportItemStatusType.TestFinished) && o.WorkGroupName == WorkGroupName);
            queryReportKvidsPending.SelectDistinct(o => o.ReportKvid);
            var dealDateBegin = new DateTime(2021, 09, 22);
            var queryReportsPending = conn.From<Report>();
            queryReportsPending.Where(o => o.DealDate >= dealDateBegin && Sql.In(o.Kvid, queryReportKvidsPending));
            queryReportsPending.OrderBy(o => o.DeadDate);
            queryReportsPending.Select(o => new { o.ReportId, o.SampleName, o.SampleBatchNumber, o.TestNeedJudge, o.TestJudgement, o.DealDate, o.DeadDate, o.TestStartDate, o.TestFinishDate, o.Status, o.UrgentRate, o.Metadata, o.Kvid });
            var reportsPending = conn.Select(queryReportsPending);
            //待处理报告总数
            rtns.ReportsStatisticsInfos.ReportsPending = reportsPending.Count;
            //待处理报告(打回)
            var reportsPendingCallBack = reportsPending.Where(o => o.Metadata.Count > 0 && ((o.Metadata.Keys.Contains("FlowCallBack") && o.Metadata["FlowCallBack"] != "0") || (o.Metadata.Keys.Contains("CallBackTo") && o.Metadata["CallBackTo"] == "DataEntry"))).ToList();
            rtns.ReportsStatisticsInfos.ReportsPendingCallBack = reportsPendingCallBack.Count;
            //待处理报告(超期)
            var reportsPendingOverdue = reportsPending.Where(o => o.DeadDate < beginTime && o.Status < (int)ReportStatusType.ReportCollected).ToList();
            rtns.ReportsStatisticsInfos.ReportsPendingOverdue = reportsPendingOverdue.Count;
            //待处理报告列表(超期)--委托
            rtns.ReportsPeddingOverdueEntrustToday = reportsPendingOverdue.Where(o => o.Type != "F" && o.Type != "K" && o.TypeEx != "Z" && o.TypeEx != "W" && o.TypeEx != "J").ToList();
            //待处理报告列表(超期)--监督
            rtns.ReportsPeddingOverdueSuperviseToday = reportsPendingOverdue.Where(o => o.Type != "F" && o.Type != "K" && o.TypeEx == "J").ToList();
            //待处理报告(加急)
            var reportsPendingUrgent = reportsPending.Where(o => o.UrgentRate > 1).ToList();
            rtns.ReportsStatisticsInfos.ReportsPendingUrgent = reportsPendingUrgent.Count;
            //今日待处理报告
            var reportsPendingToday = reportsPending.Where(o => o.DeadDate < endTime).ToList();
            rtns.ReportsStatisticsInfos.ReportsPendingToday = reportsPendingToday.Count;
            //今日待处理报告列表
            rtns.ReportsPeddingToday = reportsPendingToday;
            //今日已完成报告
            var queryReportKvidsFinishedToday = conn.From<ReportItem>();
            queryReportKvidsFinishedToday.Where(o => o.Status == (int)ReportItemStatusType.TestCollected && o.WorkGroupName == WorkGroupName);
            queryReportKvidsFinishedToday.SelectDistinct(o => o.ReportKvid);
            rtns.ReportsStatisticsInfos.ReportsFinishedToday = conn.Scalar<Report, int>(o => Sql.Count(o.Kvid), o => o.DeadDate >= beginTime && o.DeadDate < endTime && Sql.In(o.Kvid, queryReportKvidsFinishedToday));
            //今日报告总数
            rtns.ReportsStatisticsInfos.ReportsToday = rtns.ReportsStatisticsInfos.ReportsPendingToday + rtns.ReportsStatisticsInfos.ReportsFinishedToday;
            //今日报告未完成率
            var reportsPendingRateToday = Math.Round((double)reportsPendingToday.Count / rtns.ReportsStatisticsInfos.ReportsToday * 100, 2);
            rtns.ReportsStatisticsInfos.ReportsPendingRateToday = reportsPendingRateToday;
            //今日报告完成率
            rtns.ReportsStatisticsInfos.ReportsFinishedRateToday = 100- reportsPendingRateToday;
            //今日待处理报告列表(打回)
            var reportsPendingCallBackToday = reportsPendingCallBack.Where(o => o.DeadDate < endTime).ToList();
            rtns.ReportsStatisticsInfos.ReportsPendingCallBackToday = reportsPendingCallBackToday.Count;
            rtns.ReportsPeddingCallBackToday = reportsPendingCallBackToday;
            //今日待处理报告(加急)
            var reportsPendingUrgentToday = reportsPendingUrgent.Where(o => o.DeadDate < endTime).ToList();
            rtns.ReportsStatisticsInfos.ReportsPendingUrgentToday = reportsPendingUrgentToday.Count;
            rtns.ReportsPeddingUrgentToday = reportsPendingUrgentToday;
            //今日待处理任务数
            var reportsTodayKvids = reportsPendingToday.ConvertAll(o => o.Kvid);
            var reportItemsPendingToday = conn.Scalar<ReportItem, int>(o => Sql.Count(o.Kvid), o => Sql.In(o.ReportKvid, reportsTodayKvids) && o.WorkGroupName == WorkGroupName && o.DetectionKvid != Guid.Empty && o.StandardTitle != null && o.StandardTitle != "");
            rtns.ReportsStatisticsInfos.ReportItemsPendingToday = reportItemsPendingToday;
            cache.Set(cacheKey, rtns, TimeSpan.FromMinutes(5));
            return rtns;
        }
    }

    /// <summary>
    /// 暂未使用该接口
    /// </summary>
    //[Route("/reportsPreparation")]
    //public class ReportsProcessForPreparation : RestfulExecution<Report>
    //{
    //    public override object OnExecution(IRequest req, IResponse res)
    //    {
    //        var cache = KiviiContext.GetCacheClient();
    //        var cacheKey = $"{KiviiContext.GetUrnKey("reports:")}cachedReportsPreparationForScreen";
    //        var rtns = cache.Get<ReportsProcessInfos>(cacheKey);
    //        if (rtns != null) return rtns;
    //        rtns = new ReportsProcessInfos();
    //        rtns.ReportsStatisticsInfos = new ReportsStatisticsInfos();
    //        rtns.ReportsPeddingToday = new List<Report>();
    //        rtns.ReportsPeddingUrgentToday = new List<Report>();
    //        rtns.ReportsPeddingCallBackToday = new List<Report>();
    //        var conn = KiviiContext.GetOpenedDbConnection<Report>();
    //        //获取今日时间节点
    //        var beginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day);
    //        var endTime = beginTime.AddDays(1);
    //        //获取待处理报告
    //        var dealDateBegin = new DateTime(2021, 09, 22);
    //        var queryReportsPending = conn.From<Report>();
    //        queryReportsPending.Where(o => o.DealDate > dealDateBegin && o.Status==(int)ReportStatusType.ReportPreparation);
    //        queryReportsPending.OrderBy(o => o.DeadDate);
    //        queryReportsPending.Select(o => new { o.ReportId, o.SampleName, o.SampleBatchNumber, o.TestNeedJudge, o.TestJudgement, o.DealDate, o.DeadDate, o.TestStartDate, o.TestFinishDate, o.Status, o.UrgentRate, o.Metadata, o.Kvid });
    //        var reportsPending = conn.Select(queryReportsPending);
    //        //待处理报告总数
    //        rtns.ReportsStatisticsInfos.ReportsPending = reportsPending.Count;
    //        //待处理报告(打回)
    //        var reportsPendingCallBack = reportsPending.Where(o => o.Metadata.Count > 0 && ((o.Metadata.Keys.Contains("FlowCallBack") && o.Metadata["FlowCallBack"] != "0") || (o.Metadata.Keys.Contains("CallBackTo") && o.Metadata["CallBackTo"] == "DataEntry"))).ToList();
    //        rtns.ReportsStatisticsInfos.ReportsPendingCallBack = reportsPendingCallBack.Count;
    //        //待处理报告(超期)
    //        var reportsPendingOverdue = reportsPending.Where(o => o.DeadDate < beginTime).ToList();
    //        rtns.ReportsStatisticsInfos.ReportsPendingOverdue = reportsPendingOverdue.Count;
    //        rtns.ReportsPeddingOverdue = reportsPendingOverdue;
    //        //待处理报告(加急)
    //        var reportsPendingUrgent = reportsPending.Where(o => o.UrgentRate > 1).ToList();
    //        rtns.ReportsStatisticsInfos.ReportsPendingUrgent = reportsPendingUrgent.Count;
    //        //今日待处理报告
    //        var reportsPendingToday = reportsPending.Where(o => o.DeadDate < endTime).ToList();
    //        rtns.ReportsStatisticsInfos.ReportsPendingToday = reportsPendingToday.Count;
    //        //今日待处理报告列表
    //        rtns.ReportsPeddingToday = reportsPendingToday;
    //        //今日已完成报告;
    //        rtns.ReportsStatisticsInfos.ReportsFinishedToday = conn.Scalar<Report, int>(o => Sql.Count(o.Kvid), o => o.DeadDate > beginTime && o.DeadDate < endTime && o.Status>(int)ReportStatusType.ReportPreparation);
    //        //今日报告总数
    //        rtns.ReportsStatisticsInfos.ReportsToday = rtns.ReportsStatisticsInfos.ReportsPendingToday + rtns.ReportsStatisticsInfos.ReportsFinishedToday;
    //        //今日待处理报告列表(打回)
    //        var reportsPendingCallBackToday = reportsPendingCallBack.Where(o => o.DeadDate < endTime).ToList();
    //        rtns.ReportsStatisticsInfos.ReportsPendingCallBackToday = reportsPendingCallBackToday.Count;
    //        rtns.ReportsPeddingCallBackToday = reportsPendingCallBackToday;
    //        //今日待处理报告(加急)
    //        var reportsPendingUrgentToday = reportsPendingUrgent.Where(o => o.DeadDate < endTime).ToList();
    //        rtns.ReportsStatisticsInfos.ReportsPendingUrgentToday = reportsPendingUrgentToday.Count;
    //        rtns.ReportsPeddingUrgentToday = reportsPendingUrgentToday;
    //        //今日待处理任务数
    //        var reportsTodayKvids = reportsPendingToday.ConvertAll(o => o.Kvid);
    //        var reportItemsPendingToday = conn.Scalar<ReportItem, int>(o => Sql.Count(o.Kvid), o => Sql.In(o.ReportKvid, reportsTodayKvids)&& o.DetectionKvid != Guid.Empty && o.StandardTitle != null && o.StandardTitle != "");
    //        rtns.ReportsStatisticsInfos.ReportItemsPendingToday = reportItemsPendingToday;
    //        cache.Set(cacheKey, rtns, TimeSpan.FromMinutes(5));
    //        return rtns;
    //    }
    //}

    public class ReportsStatisticsInfosAll
    {
        //今日待处理报告总数
        public int ReportsPendingToday { get; set; }
        //今日待处理报告(加急)
        public int ReportsPendingUrgentToday { get; set; }
        //今日待处理报告(超期)
        public int ReportsPendingOverdueToday { get; set; }
    }
    /// <summary>
    /// 全组统计：今日待处理报告数量、加急报告数量、超期报告数量。
    /// </summary>
    [Route("/reportsStatistics")]
    public class ReportsStatistics : RestfulExecution<Report>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = $"{KiviiContext.GetUrnKey("reports:")}cachedReportsStatisticsForScreen";
            var rtns = cache.Get<ReportsStatisticsInfosAll>(cacheKey);
            if (rtns != null) return rtns;
            rtns = new ReportsStatisticsInfosAll();
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            //获取今日时间节点
            var beginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day);
            var endTime = beginTime.AddDays(1).AddSeconds(-1);
            //获取待处理报告
            var queryReportKvidsPending = conn.From<ReportItem>();
            queryReportKvidsPending.Where(o => (o.Status == (int)ReportItemStatusType.Testing || o.Status == (int)ReportItemStatusType.BeforeReview || o.Status == (int)ReportItemStatusType.TestFinished));
            queryReportKvidsPending.SelectDistinct(o => o.ReportKvid);
            var dealDateBegin = new DateTime(2021, 09, 22);
            var queryReportsPendingToday = conn.From<Report>();
            queryReportsPendingToday.Where(o => o.DealDate >= dealDateBegin && o.DeadDate < endTime && Sql.In(o.Kvid, queryReportKvidsPending));
            queryReportsPendingToday.OrderBy(o => o.DeadDate);
            queryReportsPendingToday.Select(o => new { o.ReportId, o.DealDate, o.DeadDate, o.Status, o.UrgentRate, o.Kvid });
            var reportsPendingToday = conn.Select(queryReportsPendingToday);
            //今日待处理报告总数
            rtns.ReportsPendingToday = reportsPendingToday.Count;
            //今日待处理报告(加急)
            var reportsPendingUrgent = reportsPendingToday.Where(o => o.UrgentRate > 1).ToList();
            rtns.ReportsPendingUrgentToday = reportsPendingUrgent.Count;
            //今日待处理报告(超期)
            var reportsPendingOverdue = reportsPendingToday.Where(o => o.DeadDate < beginTime && o.Status < (int)ReportStatusType.ReportCollected).ToList();
            rtns.ReportsPendingOverdueToday = reportsPendingOverdue.Count;
            cache.Set(cacheKey, rtns, TimeSpan.FromMinutes(5));
            return rtns;
        }
    }
    public class ReportsStatisticsByWorkGroupName
    {
        public List<ReportsTotalInfos> Month { get; set; }
        public List<ReportsTotalInfos> Week { get; set; }
    }
    public class ReportsTotalInfos
    {
        //类型
        public string Category { get; set; }
        //报告数量
        public int ReportsTotal { get; set; }
    }
    [Route("/reportsStatisticsByWorkGroupName/{WorkGroupNameList}")]
    public class ReportsStatisticsByDepartment : RestfulExecution<Report>
    {
        public List<string> WorkGroupNameList { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (WorkGroupNameList == null || WorkGroupNameList.Count == 0) return false;
            #endregion
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = $"{KiviiContext.GetUrnKey("reports:")}cachedReportsStatisticsByWorkGroupNameForScreen";
            var rtns = cache.Get<ReportsStatisticsByWorkGroupName>(cacheKey);
            if (rtns != null) return rtns;
            rtns = new ReportsStatisticsByWorkGroupName();
            rtns.Month = new List<ReportsTotalInfos>();
            rtns.Week = new List<ReportsTotalInfos>();
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            #region 日期范围获取
            //获取本月日期
            var currentDate = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day, 0, 0, 0);
            //本月月初
            var monthBegin = currentDate.AddDays(1 - currentDate.Day);
            //本月月末
            var monthEnd = monthBegin.AddMonths(1).AddSeconds(-1);
            var currentDateOfWeek = Convert.ToInt32(DateTime.Now.DayOfWeek.ToString("d"));
            //本周周一
            var weekBegin = DateTime.Now.AddDays(1 - ((currentDateOfWeek == 0) ? 7 : currentDateOfWeek)).Date;
            //本周周日
            var weekEnd = weekBegin.AddDays(7).AddSeconds(-1);
            #endregion
            //获取本月完成报告数据集合
            //需先从报告中获取所有完成日期在本月范围内(-7天,处理本周跨月的情况)的数据,再根据组信息进一步组合
            var queryReportKvidsAll = conn.From<Report>();
            queryReportKvidsAll.Where(o => o.IssueDate >= monthBegin.AddDays(-7) && o.IssueDate <= monthEnd && o.Status == (int)ReportStatusType.ReportCollected);
            queryReportKvidsAll.Select(o => o.Kvid);
            //根据组信息提取相关报告数据
            var queryReportKvidsFinish = conn.From<ReportItem>();
            queryReportKvidsFinish.Where(o => Sql.In(o.ReportKvid, queryReportKvidsAll) && o.Status == (int)ReportItemStatusType.TestCollected && WorkGroupNameList.Contains(o.WorkGroupName));
            queryReportKvidsFinish.SelectDistinct(o => o.ReportKvid);
            var reports = conn.Select<Report>(o => Sql.In(o.Kvid, queryReportKvidsFinish));
            //if (reports.Count==0) return rtns;
            //组合月汇总数据
            var category = string.Empty;
            //筛选时间节点
            var dateTimeBegin = monthBegin.AddDays(-1);
            var dateTimeEnd = monthBegin;
            var days = DateTime.DaysInMonth(currentDate.Year, currentDate.Month);
            var reportsTotal = 0;
            for (var d = 0; d < days; d++)
            {
                var reportsTotalInfos = new ReportsTotalInfos();
                dateTimeBegin = dateTimeBegin.AddDays(1);
                dateTimeEnd = dateTimeBegin.AddDays(1).AddSeconds(-1);
                reportsTotal = reports.Where(o => o.IssueDate >= dateTimeBegin && o.IssueDate <= dateTimeEnd).ToList().Count;
                category = d + 1 + "号";
                reportsTotalInfos.ReportsTotal = reportsTotal;
                reportsTotalInfos.Category = category;
                rtns.Month.Add(reportsTotalInfos);
            }
            //组合周汇总数据
            dateTimeBegin = weekBegin.AddDays(-1);
            for (var w = 0; w < 7; w++)
            {
                dateTimeBegin = dateTimeBegin.AddDays(1);
                dateTimeEnd = dateTimeBegin.AddDays(1).AddSeconds(-1);
                reportsTotal = reports.Where(o => o.IssueDate >= dateTimeBegin && o.IssueDate <= dateTimeEnd).ToList().Count;
                var reportsTotalInfos = new ReportsTotalInfos();
                switch (w)
                {
                    case 0:
                        category = "周一";
                        break;
                    case 1:
                        category = "周二";
                        break;
                    case 2:
                        category = "周三";
                        break;
                    case 3:
                        category = "周四";
                        break;
                    case 4:
                        category = "周五";
                        break;
                    case 5:
                        category = "周六";
                        break;
                    case 6:
                        category = "周日";
                        break;
                    default:
                        break;
                }
                reportsTotalInfos.ReportsTotal = reportsTotal;
                reportsTotalInfos.Category = category;
                rtns.Week.Add(reportsTotalInfos);
            }
            cache.Set(cacheKey, rtns, TimeSpan.FromMinutes(5));
            return rtns;
        }
    }


    public class ReportsStatisticsOverall
    {
        //报告数量统计
        //今日报告
        public int TotalToday { get; set; }
        //委托受理报告
        public int TotalCommissionAccept { get; set; }
        //任务分派报告
        public int TotalTaskAssign { get; set; }
        //数据录入报告
        public int TotalDataEntry { get; set; }
        //编制中报告
        public int TotalReportPreparation { get; set; }
        //签发中报告
        public int TotalReportIssue { get; set; }
        //已签发报告
        public int TotalReportIssued { get; set; }
        //数据录入各组报告数量统计
        //安全组总数
        public int TotalSaftyGroup { get; set; }
        public int CurrentSaftyGroup { get; set; }
        //微生物组总数
        public int TotalMicrobeGroup { get; set; }
        public int CurrentMicrobeGroup { get; set; }
        //纤维含量组总数
        public int TotalFiberContentGroup { get; set; }
        public int CurrentFiberContentGroup { get; set; }
        //功能组总数
        public int TotalFunctionGroup { get; set; }
        public int CurrentFunctionGroup { get; set; }
        //轻工组总数
        public int TotalLightIndustryGroup { get; set; }
        public int CurrentLightIndustryGroup { get; set; }
        //开样组总数
        public int TotalSampleOpeningGroup { get; set; }
        public int CurrentSampleOpeningGroup { get; set; }
        //物理组总数
        public int TotalPhysicsGroup { get; set; }
        public int CurrentPhysicsGroup { get; set; }
        //纤检组总数
        public int TotalFibroscopyGroup { get; set; }
        public int CurrentFibroscopyGroup { get; set; }
        //羽绒组总数
        public int TotalEiderdownGroup { get; set; }
        public int CurrentEiderdownGroup { get; set; }
        //色牢度组总数
        public int TotalColorFastnessGroup { get; set; }
        public int CurrentColorFastnessGroup { get; set; }
    }
    [Route("/reportsStatisticsOverall")]
    public class ReportStatisticsOverallByToday : RestfulExecution<Report>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            var rtns = new ReportsStatisticsOverall();
            //获取今日报告
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            //获取今日时间节点
            var beginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day);
            var endTime = beginTime.AddDays(1);
            //获取今日报告
            var queryReports = conn.From<Report>();
            queryReports.Where(o => o.DeadDate >= beginTime && o.DeadDate < endTime);
            var reports = conn.Select(queryReports);
            rtns.TotalToday = reports.Count;
            //委托受理报告
            var reportsCommissionAccept = reports.Where(o => o.Status == (int)ReportStatusType.CommissionAccept).ToList();
            rtns.TotalCommissionAccept = reportsCommissionAccept.Count;
            //任务分派报告
            var reportsTaskAssign = reports.Where(o => o.Status == (int)ReportStatusType.TaskAssign).ToList();
            rtns.TotalTaskAssign = reportsTaskAssign.Count;
            //数据录入报告
            var reportsDataEntry = reports.Where(o => o.Status == (int)ReportStatusType.DataEntry).ToList();
            rtns.TotalDataEntry = reportsDataEntry.Count;
            //编制中
            var reportsPreparation = reports.Where(o => o.Status == (int)ReportStatusType.ReportPreparation).ToList();
            rtns.TotalReportPreparation = reportsPreparation.Count;
            //签发中
            var reportsIssue = reports.Where(o => o.Status == (int)ReportStatusType.ReportIssue).ToList();
            rtns.TotalReportIssue = reportsIssue.Count;
            //签发完成
            var reportsIssued = reports.Where(o => o.Status == (int)ReportStatusType.ReportPrint || o.Status == (int)ReportStatusType.ReportCollected).ToList();
            rtns.TotalReportIssued = reportsIssued.Count;
            //获取今日报告检验项目集合
            queryReports.Select(o=>o.Kvid);
            var queryReportItemsTotal = conn.From<ReportItem>();
            queryReportItemsTotal.Where(o=>Sql.In(o.ReportKvid, queryReports)&&o.WorkGroupKvid!=Guid.Empty);
            queryReportItemsTotal.Select(o=>new { o.Status,o.WorkGroupName, o.ReportKvid});
            var reportItemsTotal = conn.Select(queryReportItemsTotal);
            //获取数据录入阶段检验项目
            var reportItemsCurrent = reportItemsTotal.Where(o=>o.Status == (int)ReportItemStatusType.Testing || o.Status == (int)ReportItemStatusType.BeforeReview || o.Status == (int)ReportItemStatusType.TestFinished).ToList();
            //安全组
            var totalReportItemsSafty= reportItemsTotal.Where(o => o.WorkGroupName == "安全组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.TotalSaftyGroup = totalReportItemsSafty.Count;
            var currentReportItemsSafty = reportItemsCurrent.Where(o => o.WorkGroupName == "安全组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.CurrentSaftyGroup = currentReportItemsSafty.Count;
            //微生物组
            var totalReportItemsMicrobe = reportItemsTotal.Where(o => o.WorkGroupName == "微生物组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.TotalMicrobeGroup = totalReportItemsMicrobe.Count;
            var currentReportItemsMicrobe = reportItemsCurrent.Where(o => o.WorkGroupName == "微生物组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.CurrentMicrobeGroup = currentReportItemsMicrobe.Count;
            //纤维含量组
            var totalReportItemsFiberContent = reportItemsTotal.Where(o => o.WorkGroupName == "纤维含量组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.TotalFiberContentGroup = totalReportItemsFiberContent.Count;
            var currentReportItemsFiberContent = reportItemsCurrent.Where(o => o.WorkGroupName == "纤维含量组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.CurrentFiberContentGroup = currentReportItemsFiberContent.Count;
            //功能组
            var totalReportItemsFunction = reportItemsTotal.Where(o => o.WorkGroupName == "功能组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.TotalFunctionGroup = totalReportItemsFunction.Count;
            var currentReportItemsFunction = reportItemsCurrent.Where(o => o.WorkGroupName == "功能组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.CurrentFunctionGroup = currentReportItemsFunction.Count;
            //轻工组
            var totalReportItemsLightIndustry = reportItemsTotal.Where(o => o.WorkGroupName == "轻工组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.TotalLightIndustryGroup = totalReportItemsLightIndustry.Count;
            var currentReportItemsLightIndustry = reportItemsCurrent.Where(o => o.WorkGroupName == "轻工组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.CurrentLightIndustryGroup = currentReportItemsLightIndustry.Count;
            //开样组
            var totalReportItemsSampleOpening = reportItemsTotal.Where(o => o.WorkGroupName == "开样组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.TotalSampleOpeningGroup = totalReportItemsSampleOpening.Count;
            var currentReportItemsSampleOpening = reportItemsCurrent.Where(o => o.WorkGroupName == "开样组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.CurrentSampleOpeningGroup = currentReportItemsSampleOpening.Count;
            //物理组
            var totalReportItemsPhysics = reportItemsTotal.Where(o => o.WorkGroupName == "物理组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.TotalPhysicsGroup = totalReportItemsPhysics.Count;
            var currentReportItemsPhysics = reportItemsCurrent.Where(o => o.WorkGroupName == "物理组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.CurrentPhysicsGroup = currentReportItemsPhysics.Count;
            //纤检组
            var totalReportItemsFibroscopy = reportItemsTotal.Where(o => o.WorkGroupName == "纤检组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.TotalFibroscopyGroup = totalReportItemsFibroscopy.Count;
            var currentReportItemsFibroscopy = reportItemsCurrent.Where(o => o.WorkGroupName == "纤检组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.CurrentFibroscopyGroup = currentReportItemsFibroscopy.Count;
            //羽绒组
            var totalReportItemsEiderdown = reportItemsTotal.Where(o => o.WorkGroupName == "羽绒组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.TotalEiderdownGroup = totalReportItemsEiderdown.Count;
            var currentReportItemsEiderdown = reportItemsCurrent.Where(o => o.WorkGroupName == "羽绒组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.CurrentEiderdownGroup = currentReportItemsEiderdown.Count;
            //色牢度组
            var totalReportItemsColorFastness = reportItemsTotal.Where(o => o.WorkGroupName == "色牢度组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.TotalColorFastnessGroup = totalReportItemsColorFastness.Count;
            var currentReportItemsColorFastness = reportItemsCurrent.Where(o => o.WorkGroupName == "色牢度组").GroupBy(p => p.ReportKvid).Select(q => q.First()).ToList();
            rtns.CurrentColorFastnessGroup = currentReportItemsColorFastness.Count;
            return rtns;
        }
    }

    /// <summary>
    /// 事业发展科看板接口
    /// </summary>
    public class ReportsStatisticsInfosForCA
    {
        //报告总数
        public int Total { get; set; }
        //业务受理
        public int QuantityCommissionAccept { get; set; }
        //任务分派
        public int QuantityTaskAssign { get; set; }
        //数据录入
        public int QuantityDataEntry { get; set; }
        //报告编制
        public int QuantityReportPreparation { get; set; }
        //报告签发
        public int QuantityReportIssue { get; set; }
        //报告归档
        public int QuantityReportCollected { get; set; }
    }
    public class ReportsStatisticsInfosForCAToday
    {
        public ReportsStatisticsInfosForCA ReportsToday { get; set; }
        public ReportsStatisticsInfosForCA ReportsOverdue { get; set; }
    }
    [Route("/reportsProcess/commissionAccept")]
    public class ReportsProcessForCommissionAccept : RestfulExecution<Report>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = $"{KiviiContext.GetUrnKey("reports:")}cachedReportsProcessForCommissionAccept";
            var rtns = cache.Get<ReportsStatisticsInfosForCAToday>(cacheKey);
            if (rtns != null) return rtns;
            rtns = new ReportsStatisticsInfosForCAToday();
            rtns.ReportsToday = new ReportsStatisticsInfosForCA();
            rtns.ReportsOverdue = new ReportsStatisticsInfosForCA();
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            //获取今日时间节点
            var beginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day);
            var endTime = beginTime.AddDays(1);
            //获取今日待处理报告
            var queryReportsPendingToday = conn.From<Report>();
            var typeList = new List<string>();
            typeList.Add("A");
            typeList.Add("B");
            typeList.Add("C");
            typeList.Add("D");
            typeList.Add("F");
            typeList.Add("X");
            typeList.Add("Y");
            typeList.Add("T");
            queryReportsPendingToday.Where(o => o.DeadDate >= beginTime && o.DeadDate < endTime && o.TypeEx != "J" && typeList.Contains(o.Type));
            queryReportsPendingToday.Select(o => new { o.Status, o.Kvid });
            var reportsPendingToday = conn.Select(queryReportsPendingToday);
            rtns.ReportsToday.Total = reportsPendingToday.Count;
            rtns.ReportsToday.QuantityCommissionAccept = reportsPendingToday.Where(o => o.Status == (int)ReportStatusType.CommissionAccept).ToList().Count;
            rtns.ReportsToday.QuantityTaskAssign = reportsPendingToday.Where(o => o.Status == (int)ReportStatusType.TaskAssign).ToList().Count;
            rtns.ReportsToday.QuantityDataEntry = reportsPendingToday.Where(o => o.Status == (int)ReportStatusType.DataEntry).ToList().Count;
            rtns.ReportsToday.QuantityReportPreparation = reportsPendingToday.Where(o => o.Status == (int)ReportStatusType.ReportPreparation).ToList().Count;
            rtns.ReportsToday.QuantityReportIssue = reportsPendingToday.Where(o => o.Status == (int)ReportStatusType.ReportIssue).ToList().Count;
            rtns.ReportsToday.QuantityReportCollected = reportsPendingToday.Where(o => o.Status == (int)ReportStatusType.ReportCollected).ToList().Count;
            //获取超期报告
            var queryReportsOverdue = conn.From<Report>();
            //超期起始时间前推3个月
            queryReportsOverdue.Where(o => o.DeadDate < beginTime && o.DeadDate > beginTime.AddMonths(-3) && o.Status < (int)ReportStatusType.ReportCollected && o.TypeEx != "J" && typeList.Contains(o.Type));
            queryReportsOverdue.And(o => o.Category.ToUpper() != "TEST" || o.Category == string.Empty || o.Category == null);
            queryReportsOverdue.Select(o => new { o.Status, o.Kvid });
            var reportsOverdue = conn.Select(queryReportsOverdue);
            rtns.ReportsOverdue.Total = reportsOverdue.Count;
            rtns.ReportsOverdue.QuantityCommissionAccept = reportsOverdue.Where(o => o.Status == (int)ReportStatusType.CommissionAccept).ToList().Count;
            rtns.ReportsOverdue.QuantityTaskAssign = reportsOverdue.Where(o => o.Status == (int)ReportStatusType.TaskAssign).ToList().Count;
            rtns.ReportsOverdue.QuantityDataEntry = reportsOverdue.Where(o => o.Status == (int)ReportStatusType.DataEntry).ToList().Count;
            rtns.ReportsOverdue.QuantityReportPreparation = reportsOverdue.Where(o => o.Status == (int)ReportStatusType.ReportPreparation).ToList().Count;
            rtns.ReportsOverdue.QuantityReportIssue = reportsOverdue.Where(o => o.Status == (int)ReportStatusType.ReportIssue).ToList().Count;
            cache.Set(cacheKey, rtns, TimeSpan.FromMinutes(5));
            return rtns;
        }
    }

    public class ReportsAmountInfosForCA
    {
        //今日报告批次
        public int ReportTotalToday { get; set; }
        //今日收费
        public decimal AmountToday { get; set; }
        //当月报告批次
        public int ReportTotalCurrentMonth { get; set; }
        //当月收费
        public decimal AmountCurrentMonth { get; set; }
        //当年报告批次
        public int ReportTotalCurrentYear { get; set; }
        //当年收费
        public decimal AmountCurrentYear { get; set; }
    }
    [Route("/reportsAmount/commissionAccept")]
    public class ReportsAmountForCommissionAccept : RestfulExecution<Report>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            var cache = KiviiContext.GetCacheClient();
            var cacheKey = $"{KiviiContext.GetUrnKey("reports:")}cachedReportsAmountForCommissionAccept";
            var rtns = cache.Get<ReportsAmountInfosForCA>(cacheKey);
            if (rtns != null) return rtns;
            rtns = new ReportsAmountInfosForCA();
            //获取今日时间节点
            var beginTimeToday = new DateTime(DateTime.Now.Year, DateTime.Now.Month, DateTime.Now.Day);
            var endTimeToday = beginTimeToday.AddDays(1);
            //获取当月1号时间节点
            var beginTimeCurrentMonth = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            var endTimeCurrentMonth = beginTimeCurrentMonth.AddMonths(1);
            //获取当年1月1号时间节点
            var beginTimeCurrentYear = new DateTime(DateTime.Now.Year, 1, 1);
            var endTimeCurrentYear = beginTimeCurrentYear.AddYears(1);
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var typeList = new List<string>();
            typeList.Add("A");
            typeList.Add("B");
            typeList.Add("C");
            typeList.Add("D");
            typeList.Add("F");
            typeList.Add("X");
            typeList.Add("Y");
            typeList.Add("T");
            var queryReportsCurrentYear = conn.From<Report>();
            queryReportsCurrentYear.Where(o => o.DealDate >= beginTimeCurrentYear && o.DealDate < endTimeCurrentYear && o.TypeEx != "J");
            queryReportsCurrentYear.And(o => typeList.Contains(o.Type));
            queryReportsCurrentYear.And(o => o.Category.ToUpper() != "TEST" || o.Category == string.Empty || o.Category == null);
            queryReportsCurrentYear.Select(o => new { o.Amount, o.AmountTest, o.QuotationAmount, o.UrgentRate, o.DealDate, o.Kvid });
            //获取当年报告数据
            var reportsCurrentYear = conn.Select(queryReportsCurrentYear);
            foreach (var report in reportsCurrentYear)
            {
                if (report.QuotationAmount <= 0) continue;
                report.AmountTest = 0;
            }
            rtns.ReportTotalCurrentYear = reportsCurrentYear.Count;
            rtns.AmountCurrentYear = Math.Round(reportsCurrentYear.Sum(o => o.AmountTest), 2) + Math.Round(reportsCurrentYear.Sum(o => o.QuotationAmount), 2) + Math.Round(reportsCurrentYear.Sum(o => (o.AmountTest + o.QuotationAmount) * (o.UrgentRate - 1)), 2) + Math.Round(reportsCurrentYear.Sum(o => o.Amount), 2);
            //获取当天报告数据
            var reportsToday = reportsCurrentYear.Where(o => o.DealDate >= beginTimeToday && o.DealDate < endTimeToday).ToList();
            rtns.ReportTotalToday = reportsToday.Count;
            rtns.AmountToday = Math.Round(reportsToday.Sum(o => o.Amount) + reportsToday.Sum(o => o.AmountTest) + reportsToday.Sum(o => o.QuotationAmount) + reportsToday.Sum(o => (o.AmountTest + o.QuotationAmount) * (o.UrgentRate - 1)), 2);
            //获取当月报告数据
            var reportsCurrentMonth = reportsCurrentYear.Where(o => o.DealDate >= beginTimeCurrentMonth && o.DealDate < endTimeCurrentMonth).ToList();
            rtns.ReportTotalCurrentMonth = reportsCurrentMonth.Count;
            rtns.AmountCurrentMonth = Math.Round(reportsCurrentMonth.Sum(o => o.Amount) + reportsCurrentMonth.Sum(o => o.AmountTest) + reportsCurrentMonth.Sum(o => o.QuotationAmount) + reportsCurrentMonth.Sum(o => (o.AmountTest + o.QuotationAmount) * (o.UrgentRate - 1)), 2);
            cache.Set(cacheKey, rtns, TimeSpan.FromMinutes(5));
            return rtns;
        }
    }
    #endregion

    #region 创建更改报告-G
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "生成更改报告（-G）")]
    public class ReportChangeExecute : RestfulExecution<Report>
    {
        public ReportChange ReportChangeItem { get; set; }
        //public Guid reportKvid { get; set; }
        ///// <summary>
        ///// 更改类型
        ///// 区分更改的是实验信息还是非实验信息
        ///// 以区分不同的处理流程
        ///// </summary>
        //public string changeType { get; set; }
        ///// <summary>
        ///// 更改内容
        ///// </summary>
        //public string changeReason { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportChangeItem.ThrowIfNull("参数异常！");
            var reportKvid = ReportChangeItem.ReportKvid;
            var changeType = ReportChangeItem.Type;
            var changeReason = ReportChangeItem.Content;
            var handlingPersonName = ReportChangeItem.OperatorName;
            var responsibleName = ReportChangeItem.PersonLiableName;
            #region 数据验证
            reportKvid.ThrowIfEmpty("未传入更改报告信息!");
            changeType.ThrowIfNull("未传入更改报告类型!");
            (changeType != "首页" && changeType != "附页").ThrowIfTrue("更改类型只能为<首页>或<附页>");
            #endregion

            #region 获取原报告的相关联的信息
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var originalReport = conn.SingleById<Report>(reportKvid);
            originalReport.ThrowIfNull("未查询到原报告，无法更改！");
            (originalReport.StatusType != ReportStatusType.ReportCollected).ThrowIfTrue("报告未归档，无法更改报告！");
            //获取更改报告相关文件集合(样品图片等)
            var queryOriginalFiles = conn.From<EntityDbFile<Report>>();
            queryOriginalFiles.Where(o => o.OwnerKvid == originalReport.Kvid && o.DbFolderPath == "/Report/Images");
            var originalFiles = conn.Select(queryOriginalFiles);
            //获取更改报告检验项目集合
            var queryOriginalReportItems = conn.From<ReportItem>();
            queryOriginalReportItems.Where(o => o.ReportKvid == originalReport.Kvid);
            var originalReportItems = conn.Select(queryOriginalReportItems);
            //获取检验项目新老Kvid集合
            var reportItemsKvids = originalReportItems.Select(o => o.Kvid).ToList();
            var reportItemKvidsReplace = new List<MatchKvids>();
            reportItemKvidsReplace.Add(new MatchKvids
            {
                OriginalKvid = Guid.Empty,
                MatchKvid = Guid.Empty
            });
            foreach (var reportItemKvid in reportItemsKvids)
            {
                reportItemKvidsReplace.Add(new MatchKvids
                {
                    OriginalKvid = reportItemKvid,
                    MatchKvid = Guid.NewGuid()
                });
            }
            //1.获取待创建报告信息
            var reportIdPrefix = string.Empty;
            var subIndex = originalReport.ReportId.LastIndexOf("-G");
            if (subIndex == -1)
            {
                reportIdPrefix = originalReport.ReportId;
            }
            else
            {
                reportIdPrefix = originalReport.ReportId.Substring(0, subIndex);
            }
            #endregion

            #region 获取最新版本报告,更改报告只能对最新的报告进行更改
            var latestReport = conn.Select<Report>(o => o.ReportId.Contains(reportIdPrefix + "-G")).OrderByDescending(o => o.CreateTime).FirstOrDefault();
            #endregion
            #region 生成新的-G报告
            if (latestReport == null) latestReport = originalReport;
            //更改报告逻辑修改：根据是否存在未归档的更改报告，进行分类讨论，若存在，则对该报告进行更新，若不存在，则创建更改报告。
            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            if (latestReport.StatusType != ReportStatusType.ReportCollected)
            {
                #region 创建更改记录
                var updateLog = new EntityNote<Report>();
                updateLog.InternalCode = "reportupdate";
                updateLog.EntityKvid = latestReport.Kvid;
                updateLog.Remark = changeReason.Substring(500);
                conn.Insert(updateLog);
                #endregion
                //创建初始化的日志
                latestReport.AddReportNote("报告更改", "更改报告修改", "更改报告修改", changeReason, conn, ReportChangeItem.OperatorName);
                rtns.Results.Add(latestReport);
            }
            else
            {
                var reportOperate = new Report();
                reportOperate.Metadata = new Dictionary<string, string>();
                reportOperate.PopulateWith(latestReport);
                if (subIndex == -1)
                {
                    reportOperate.Remark = $"本报告是对原报告（{reportIdPrefix}）的更改，原报告作废。\n" + reportOperate.Remark;
                }
                reportOperate.ReportId = reportIdPrefix + "-G{x}";
                reportOperate.BizKvid = latestReport.Kvid;
                reportOperate.BizType = changeType == "首页" ? "G0" : "G1";
                reportOperate.BizId = latestReport.ReportId;
                reportOperate.Amount = ReportChangeItem.Amount;
                reportOperate.AmountTestPlan = 0;
                reportOperate.AmountTest = 0;
                reportOperate.AmountPlan = 0;
                reportOperate.AmountPayment = 0;
                reportOperate.AmountSettlement = 0;
                reportOperate.QuotationAmount = 0;
                reportOperate.IssuerKvid = Guid.Empty;
                reportOperate.IssuerName = String.Empty;
                reportOperate.IssueDate = null;
                reportOperate.Kvid = Guid.NewGuid();
                reportOperate.Metadata["HandlingPersonName"] = handlingPersonName;
                reportOperate.Metadata["ResponsibleName"] = responsibleName;
                //reportOperate.Tag = KiviiContext.EntityTagForDbSync;
                //reportOperate.CreatorName = handlingPersonName;
                //reportOperate.CreatorKvid = ReportChangeItem.CreatorKvid;
                //根据更改类型初始化报告的状态
                reportOperate.Status = changeType == "首页" ? (int)ReportStatusType.CommissionAccept : (int)ReportStatusType.ReportPreparation;
                //2.获取待创建文件集合
                var filesOperate = new List<EntityDbFile<Report>>();
                foreach (var originalFile in originalFiles)
                {
                    var file = new EntityDbFile<Report>();
                    file.PopulateWith(originalFile);
                    file.OwnerKvid = reportOperate.Kvid;
                    file.Kvid = Guid.NewGuid();
                    filesOperate.Add(file);
                }
                //3.获取待创建检验项目
                var reportItemsOperate = new List<ReportItem>();
                foreach (var originalReportItem in originalReportItems)
                {
                    var reportItem = new ReportItem();
                    reportItem.PopulateWith(originalReportItem);
                    reportItem.TestOriginalFilePath = string.Empty;
                    reportItem.TestKvid = Guid.Empty;
                    reportItem.TestRootKvid = Guid.Empty;
                    reportItem.Amount = 0;
                    reportItem.AmountPlan = 0;
                    reportItem.ReportKvid = reportOperate.Kvid;
                    try
                    {
                        reportItem.RootKvid = reportItemKvidsReplace.Where(o => o.OriginalKvid == originalReportItem.RootKvid).FirstOrDefault().MatchKvid;
                    }
                    catch { }
                    reportItem.ParentKvid = reportItemKvidsReplace.Where(o => o.OriginalKvid == originalReportItem.ParentKvid).FirstOrDefault().MatchKvid;
                    reportItem.Status = int.MaxValue;
                    reportItem.Kvid = reportItemKvidsReplace.Where(o => o.OriginalKvid == originalReportItem.Kvid).FirstOrDefault().MatchKvid;
                    reportItemsOperate.Add(reportItem);
                }
                #endregion

            #region 创建更改记录
            var changeLog = new EntityNote<Report>();
            changeLog.InternalCode = "reportupdate";
            changeLog.EntityKvid = reportOperate.Kvid;
            changeLog.Remark = changeReason;
            conn.Insert(changeLog);
            #endregion

            #region 开启事务，处理数据
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                conn.Insert(reportOperate);
                reportOperate.RemoveAllOnlyProperties();
                rtns.Results.Add(reportOperate);
                foreach (var fileOperate in filesOperate)
                {
                    conn.Insert(fileOperate);
                }
                foreach (var reportItemOperate in reportItemsOperate)
                {
                    conn.Insert(reportItemOperate);
                }
                //创建初始化的日志
                reportOperate.AddReportNote("报告更改", "更改报告审批", changeType == "首页" ? "委托受理" : "报告编制", changeReason, conn, ReportChangeItem.OperatorName);
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            #endregion
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region (临时接口)系统异常报告查询，报告在录入，项目全完成
    [RequiresAnyRole(MemberRoles.Administrator)]
    public class ReportTempQuery : RestfulExecution<Report>
    {
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var queryReport = conn.From<Report>();
            queryReport.Where(o => o.Status == 400);
            var report = conn.Select(queryReport);

            var queryReportItem = conn.From<ReportItem>();
            queryReportItem.Where(o => o.Status < int.MaxValue && o.Status > 150 && o.GoodsKvid != Guid.Empty);
            var reportItems = conn.Select(queryReportItem);

            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            foreach (var en in report)
            {
                if (!reportItems.Exists(o => o.ReportKvid == en.Kvid)) rtns.Results.Add(en);
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 标记报告为含量已核查
    [Api(Description = "报告含量标记已核查及重置")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportRemarkFiberContentCheckChange : RestfulExecution<Report>
    {
        public List<Guid> ReportKvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvids.ThrowIfNullOrEmpty("参数ReportKvid不能为空");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var queryReport = conn.From<Report>();
            queryReport.Where(o => Sql.In(o.Kvid, ReportKvids));
            var reports = conn.Select(queryReport);
            reports.ThrowIfNull("未找到当前报告！");

            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            foreach (var report in reports)
            {
                if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
                if (report.Metadata.ContainsKey("IsFiberContentCheck"))
                {
                    if (report.Metadata["IsFiberContentCheck"] == "True")
                    {
                        report.Metadata["IsFiberContentCheck"] = "False";
                    }
                    else
                    {
                        report.Metadata["IsFiberContentCheck"] = "True";
                        report.Metadata["FiberContentCheckName"] = KiviiContext.CurrentMember.FullName;
                        report.Metadata["FiberContentCheckTime"] = DateTime.Now.ToString();
                    }
                }
                else
                {
                    report.Metadata.Add("IsFiberContentCheck", "True");
                    report.Metadata.Add("FiberContentCheckName", KiviiContext.CurrentMember.FullName);
                    report.Metadata.Add("FiberContentCheckTime", DateTime.Now.ToString());
                }
                report.AddOnlyProperties(o => o.Metadata);
                conn.UpdateOnly(report);
                rtns.Results.Add(report);
            }
            return rtns;
        }
    }
    #endregion

    #region 标记报告为已核查
    //2023新增需求报告核查标记，不仅仅是含量核查
    [Api(Description = "报告核查标记及重置")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportRemarkCheckChange : RestfulExecution<Report>
    {
        public List<Guid> ReportKvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvids.ThrowIfNullOrEmpty("参数ReportKvid不能为空");
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var queryReport = conn.From<Report>();
            queryReport.Where(o => Sql.In(o.Kvid, ReportKvids));
            var reports = conn.Select(queryReport);
            reports.ThrowIfNull("未找到当前报告！");

            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            foreach (var report in reports)
            {
                if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
                if (report.Metadata.ContainsKey("IsCheck"))
                {
                    if (report.Metadata["IsCheck"] == "True")
                    {
                        report.Metadata["IsCheck"] = "False";
                    }
                    else
                    {
                        report.Metadata["IsCheck"] = "True";
                        report.Metadata["CheckName"] = KiviiContext.CurrentMember.FullName;
                        report.Metadata["CheckTime"] = DateTime.Now.ToString();
                    }
                }
                else
                {
                    report.Metadata.Add("IsCheck", "True");
                    report.Metadata.Add("CheckName", KiviiContext.CurrentMember.FullName);
                    report.Metadata.Add("CheckTime", DateTime.Now.ToString());
                }
                report.AddOnlyProperties(o => o.Metadata);
                conn.UpdateOnly(report);
                rtns.Results.Add(report);
            }
            return rtns;
        }
    }
    #endregion

    #region 签报告查询
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportIssueQuery : RestfulExecution<Report>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }
        public double IntervalDays { get; set; } = 60;//默认间隔天数最大60天

        public override object OnExecution(IRequest req, IResponse res)
        {
            int monthDay = DateTime.DaysInMonth(DateTime.Now.Year, DateTime.Now.Month);
            if (BeginTime == DateTime.MinValue) BeginTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, 1);
            if (EndTime == DateTime.MinValue) EndTime = new DateTime(DateTime.Now.Year, DateTime.Now.Month, monthDay);

            BeginTime = DateTime.Parse(BeginTime.ToString("yyyy-MM-dd"));
            EndTime = DateTime.Parse(EndTime.ToString("yyyy-MM-dd"));
            if (EndTime < BeginTime) throw new Exception("查询结束日期不可小于开始日期!");
            var span = new TimeSpan(EndTime.Ticks - BeginTime.Ticks);
            if (span.TotalDays > IntervalDays) throw new Exception($"查询时间间隔最大{IntervalDays}");

            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Report>();
            request = request.PopulateWith(this);

            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.Status == (int)ReportStatusType.ReportCollected && o.IssueDate >= BeginTime && o.IssueDate < EndTime);
            //sqlExpress.OrderBy(o => new { o.ReportId, o.IssueDate });
            sqlExpress.Select(o => new { o.ReportId, o.IssueDate, o.Status });
            sqlExpress.Rows = null;
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }
    #endregion

    #region 委托报告含量项目查询
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportFibreQuery : RestfulExecution<Report>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public string Title { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<Report>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress.Where(o => o.Status == int.MaxValue && (o.TypeEx == "W" || o.TypeEx == "Z"));
            if (!Title.IsNullOrEmpty()) sqlExpress.And(o => o.TestDetections.Contains(Title));
            //sqlExpress.Rows = null;
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            //查出所有不合格的项目
            var reports = new List<Report>();
            reports = rtns.Results;
            var reportKvids = reports.Select(o => o.Kvid);
            var queryAllReportItems = conn.From<ReportItem>();
            queryAllReportItems.Where(o => Sql.In(o.ReportKvid, reportKvids));
            var allReportItems = conn.Select(queryAllReportItems);
            List<Report> rtnReports = new List<Report>();
            //遍历所有报告，将不合格项目拼一个固定的格式
            foreach (var en in reports)
            {
                var currentReportItems = allReportItems.Where(o => o.ReportKvid == en.Kvid).ToList();
                currentReportItems.ThrowIfNullOrEmpty("未找到数据！");
                var currentMainUnQualfiedReportItems = currentReportItems.Where(o => o.WorkGroupName == "纤维含量组");
                // currentMainUnQualfiedReportItems = currentMainUnQualfiedReportItems.Where(o => o.Title.Contains(Title));
                en.SupplierContactAddress = "";
                var unQualifiedList = new List<string>();
                foreach (var kv in currentMainUnQualfiedReportItems)
                {
                    //只处理检测项，若为检测项，找大项
                    if (kv.Type != ReportItemType.Test) continue;
                    var parentReportItem = currentReportItems.Where(o => o.Kvid == kv.ParentKvid).FirstOrDefault();

                    //处理无子项的大项
                    if (parentReportItem == null || parentReportItem.Type != ReportItemType.TestFolder)
                    {
                        if (!Title.IsNullOrEmpty() && !kv.Title.ToUpper().Contains(Title.ToUpper())) continue;
                        unQualifiedList.Add(kv.Title + "(标准值：" + kv.DetectionValueQualified + " 实测值：" + kv.TestValue + ")");
                    }
                    else if (parentReportItem.Type == ReportItemType.TestFolder)
                    {
                        if (!Title.IsNullOrEmpty() && !parentReportItem.Title.ToUpper().Contains(Title.ToUpper())) continue;
                        unQualifiedList.Add(parentReportItem.Title + "-" + kv.Title + "(标准值：" + kv.DetectionValueQualified + " 实测值：" + kv.TestValue + ")");
                    }
                    else
                    {
                        continue;
                    }
                }
                en.SupplierContactAddress = string.Join("、", unQualifiedList);
                if (en.SupplierContactAddress != "") rtnReports.Add(en);
            }
            //当传入项目名称参数时，要过滤掉无符合结果的报告SupplierContactAddress为空
            rtns.Results = rtnReports;
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 任务分派工作量批次统计，根据流转记录去查
    public class ReportTaskAssignWorkPivot : RestfulExecution<Report>
    {
        public DateTime? DeadDateFrom { get; set; }
        public DateTime? DeadDateTo { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            DeadDateFrom.ThrowIfNull("必须传入起始日期！");
            DeadDateTo.ThrowIfNull("必须传入终止日期！");

            var rtns = new RestfulQueryResponse<ReportNote>();
            rtns.Results = new List<ReportNote>();
            var conn = KiviiContext.GetOpenedDbConnection<ReportNote>();
            var query = conn.From<ReportNote>();
            query.Where(o => o.OperateTime >= DeadDateFrom && o.OperateTime <= DeadDateTo && o.Title == "任务下达" && o.From == "任务分派" && o.To == "项目检验");
            query.GroupBy(o => o.OperatorName);
            query.Select(o => new
            {
                o.OperatorName,
                Status = Sql.Count(o.OperatorName)
            });
            var results = conn.Select(query).ToList();
            rtns.Results = results;
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 含量已完成的待处理任务查询，用于查询各检验组的待处理任务，其中若有纤维含量项目，含量项目需要是已完成状态 

    #endregion

    #region 数据导出-供纺检使用
    public class ReportData
    {
        public string ReportId { get; set; }
        public string SampleName { get; set; }
        public string TechParameter { get; set; }
        public string ReportItemTitle { get; set; }
        public string ReportItemUnit { get; set; }
        public string ReportItemDetectionValueQualified { get; set; }
        public string ReportItemTestValue { get; set; }
        public string ReportItemDetectionCodeActual { get; set; }
    }

    public class ReportDataExport : RestfulExecution<Report>
    {
        public string StandardCode { get; set; }
        public List<string> WorkGroupNameList { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (StandardCode.IsNullOrEmpty()) throw new Exception("未传入检验依据!");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var beginDate = new DateTime(DateTime.Today.Year,1,1);
            var endDate = DateTime.Now;
            //获取报告信息
            var queryReportsAll = conn.From<Report>();
            queryReportsAll.Where(o=>o.DealDate>=beginDate&&o.DealDate < endDate&&o.Status==(int)ReportStatusType.ReportCollected&&o.TestStandards.Contains(StandardCode));
            queryReportsAll.OrderBy(o=>o.DealDate);
            queryReportsAll.Select(o => new { o.Kvid,o.ReportId,o.SampleName,o.Metadata});
            var reportsAll = conn.Select(queryReportsAll);
            Configs.Logger.Info($"共查询报告:{reportsAll.Count}份");
            //获取检验项目信息
            var queryReportKvidsAll = conn.From<Report>();
            queryReportKvidsAll.Where(o => o.DealDate >= beginDate && o.DealDate < endDate && o.Status == (int)ReportStatusType.ReportCollected && o.TestStandards.Contains(StandardCode));
            queryReportKvidsAll.Select(o => o.Kvid);
            var queryReportItemsAll = conn.From<ReportItem>();
            queryReportItemsAll.Where(o=>Sql.In(o.ReportKvid, queryReportKvidsAll) &&(o.Type==ReportItemType.TestFolder||o.Type==ReportItemType.Test));
            queryReportItemsAll.OrderBy(o=>o.SortId);
            queryReportItemsAll.Select(o => new { o.ReportKvid,o.ParentKvid,o.Kvid,o.StandardCode,o.WorkGroupName,o.Type,o.Title,o.DetectionUnit,o.DetectionValueQualified,o.TestValue,o.DetectionCodeActual});
            var reportItemsAll = conn.Select(queryReportItemsAll);
            //获取涉及检验项目
            var reportItemsParentAll = reportItemsAll.Where(o => o.ParentKvid == Guid.Empty && o.StandardCode.Contains(StandardCode)).ToList();
            if (!WorkGroupNameList.IsNullOrEmpty())
            {
                reportItemsParentAll = reportItemsParentAll.Where(o => WorkGroupNameList.Contains(o.WorkGroupName)).ToList();
            }
            Configs.Logger.Info($"共查询检验项目大项:{reportItemsParentAll.Count}个");
            var rtns = new List<ReportData>();
            foreach (var report in reportsAll)
            {
                var reportItemsParent = reportItemsParentAll.Where(o => o.ReportKvid == report.Kvid);
                foreach (var reportItemParent in reportItemsParent)
                {
                    var reportDataParent = new ReportData();
                    reportDataParent.ReportId = report.ReportId;
                    reportDataParent.SampleName = report.SampleName;
                    if (report.Metadata != null && report.Metadata.ContainsKey("TechParameter")) reportDataParent.TechParameter = report.Metadata["TechParameter"];
                    reportDataParent.ReportItemTitle = reportItemParent.Title;
                    reportDataParent.ReportItemUnit = reportItemParent.DetectionUnit;
                    reportDataParent.ReportItemDetectionValueQualified=reportItemParent.DetectionValueQualified;
                    reportDataParent.ReportItemTestValue = reportItemParent.TestValue;
                    reportDataParent.ReportItemDetectionCodeActual = reportItemParent.DetectionCodeActual;
                    rtns.Add(reportDataParent);
                    if (reportItemParent.Type == ReportItemType.Test) continue;
                    var reportItemsChildren = reportItemsAll.Where(o=>o.ParentKvid==reportItemParent.Kvid).OrderBy(o=>o.SortId).ToList();
                    foreach (var reportItemChild in reportItemsChildren)
                    {
                        var reportDataChild = new ReportData();
                        reportDataChild.ReportId = report.ReportId;
                        reportDataChild.SampleName = report.SampleName;
                        if (report.Metadata != null && report.Metadata.ContainsKey("TechParameter")) reportDataChild.TechParameter = report.Metadata["TechParameter"];
                        reportDataChild.ReportItemTitle =$"{reportItemParent.Title}-{reportItemChild.Title}" ;
                        reportDataChild.ReportItemUnit = reportItemChild.DetectionUnit;
                        reportDataChild.ReportItemDetectionValueQualified = reportItemChild.DetectionValueQualified;
                        reportDataChild.ReportItemTestValue = reportItemChild.TestValue;
                        reportDataChild.ReportItemDetectionCodeActual = reportItemChild.DetectionCodeActual;
                        rtns.Add(reportDataChild);
                    }
                }
            }
            Configs.Logger.Info($"共组织数据:{rtns.Count}条");
            return rtns;
        }
    }
    #endregion

    #region 同步报告金额，根据实测值特殊值进行自动更新价格
    public class ReportSyncAmountByKey : RestfulExecution<Report>
    {
        public Guid ReportKvid { get; set; }

        public List<Guid> ReportKvids { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            if (ReportKvid == Guid.Empty && ReportKvids.IsNullOrEmpty()) throw new Exception("请传入要重新计算金额的报告!");

            if (ReportKvids.IsNullOrEmpty()) ReportKvids = new List<Guid>();
            if (ReportKvid != Guid.Empty) ReportKvids.Add(ReportKvid);

            //获取报告信息
            var conn = KiviiContext.GetOpenedDbConnection<Report>();
            var reports = conn.SelectByIds<Report>(ReportKvids);
            reports.ThrowIfNullOrEmpty("未能找到传入的报告信息!");

            var rtns = new RestfulQueryResponse<Report>();
            rtns.Results = new List<Report>();
            foreach (var item in reports)
            {
                item.AutoSetReportItemAmount(conn);
                item.CalculateReportAmount(conn);
                conn.UpdateOnly(item);
                item.RemoveAllOnlyProperties();
                rtns.Results.Add(item);
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 监督报告导出任务单
    public class ReportWithWorkGroupName
    { 
        public string ReportId { get; set; }
        public string WorkGroupName { get; set; }
    }
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "监督报告导出任务单")]
    public class ReportGenarateTask : RestfulExecution<Report>
    {
        public List<Guid> ReportKvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            if (ReportKvids.IsNullOrEmpty()) throw new Exception("报告Kvids为空!");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var queryReports = conn.From<Report>();
            queryReports.Where(o=>Sql.In(o.Kvid, ReportKvids));
            queryReports.Select(o => new { o.Kvid,o.ReportId});
            var reports = conn.Select(queryReports);
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o=>Sql.In(o.ReportKvid, ReportKvids)&&o.StandardCode!=null);
            queryReportItems.Select(o => new { o.ReportKvid,o.WorkGroupName});
            var reportItems = conn.Select(queryReportItems);
            var groupedData = reportItems
                .GroupBy(item => new {
                    item.ReportKvid,
                    item.WorkGroupName
                })
                .Select(group => new ReportItem
                {
                    ReportKvid = group.Key.ReportKvid,
                    WorkGroupName = group.Key.WorkGroupName
                }).ToList();
            var rtns = new RestfulQueryResponse<ReportWithWorkGroupName>();
            rtns.Results = new List<ReportWithWorkGroupName>();
            foreach (var reportKvid in ReportKvids)
            {
                var currentReport = reports.FirstOrDefault(o=>o.Kvid==reportKvid);
                if (currentReport == null) throw new Exception("未能匹配对应报告!");
                var currentReportItems = groupedData.Where(o => o.ReportKvid == reportKvid).OrderBy(o=>o.WorkGroupName).ToList();
                foreach (var currentReportItem in currentReportItems)
                {
                    var reportWithWorkGroupName = new ReportWithWorkGroupName();
                    reportWithWorkGroupName.ReportId = currentReport.ReportId;
                    reportWithWorkGroupName.WorkGroupName = currentReportItem.WorkGroupName;
                    rtns.Results.Add(reportWithWorkGroupName);
                }
                var reportWithWorkGroupNameSpecial = new ReportWithWorkGroupName();
                reportWithWorkGroupNameSpecial.ReportId = currentReport.ReportId;
                reportWithWorkGroupNameSpecial.WorkGroupName = "抽样单";
                rtns.Results.Add(reportWithWorkGroupNameSpecial);
            }
            return rtns;
        }
    }
    #endregion
}
