﻿using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lac.Lims.Transforms
{
    #region CDUQD
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportChangeCreate : RestfulCreate<ReportChange> 
    {
        public override bool OnPreRestfulCreate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulCreateResponse<ReportChange> rtns)
        {
            Item.Status = 100;
            Item.AddOnlyProperties(o => o.Status);
            return base.OnPreRestfulCreate(req, res, dbConnection, rtns);
        }
    }

    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportChangeDelete : RestfulDelete<ReportChange>
    {
        public override bool OnPreRestfulDelete(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulDeleteResponse<ReportChange> rtns)
        {
            var queryReportChanges = dbConnection.From<ReportChange>();
            queryReportChanges.Where(o => Sql.In(o.Kvid, Kvids));
            var reportChanges = dbConnection.Select(queryReportChanges);
            if (reportChanges.Exists(o => o.Status > 100)) throw new Exception("待删除的更改报告存在非草稿状态的数据，无法删除，请检查后操作！");
            return base.OnPreRestfulDelete(req, res, dbConnection, rtns);
        }
    }

    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportChangeUpdate : RestfulUpdate<ReportChange>
    {
        public override bool OnPreRestfulUpdate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulUpdateResponse<ReportChange> rtns)
        {
            var reportChange = dbConnection.SingleById<ReportChange>(Item.Kvid);
            if (reportChange.Status > 100) throw new Exception("当前更改报告非草稿状态无法更改！");
            return base.OnPreRestfulUpdate(req, res, dbConnection, rtns);
        }
    }

    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportChangeQuery : RestfulQuery<ReportChange> { }

    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportChangeRead : RestfulRead<ReportChange> { }
    #endregion

    #region 统计信息
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportChangeStatisticsQuery : RestfulExecution<ReportChange>
    {
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<ReportChange>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<ReportChange>();
            request = request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            var reportChangeStatisticalInfo = new ReportStatisticalInfo();
            reportChangeStatisticalInfo.Total = rtns.Total;
            reportChangeStatisticalInfo.Amount = 0;
            foreach (var reportChange in rtns.Results)
            {
                reportChangeStatisticalInfo.Amount += reportChange.Amount;
            }
            return reportChangeStatisticalInfo;
        }
    }
    #endregion

    #region 手动建立更改报告的审批流程
    //申请审批 Status->150
    [RequiresAnyRole(MemberRoles.Administrator,MemberRoles.Manager,MemberRoles.Configuration)]
    [Api(Description = "申请审批")]
    public class ReportChangeApply : RestfulExecution<ReportChange>
    {
        public Guid Kvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<ReportChange>();
            var reportChange = conn.SingleById<ReportChange>(Kvid);
            var rtns = new RestfulUpdateResponse<ReportChange>();
            rtns.Results = new List<ReportChange>();
            reportChange.ThrowIfNull("未查到该更改报告申请！");
            if (reportChange.Status > 100) throw new Exception("当前更改报告申请非待审批状态，无法申请审批！");
            //重新申请，清空打回标记
            if (reportChange.Metadata != null && reportChange.Metadata.ContainsKey("CallBackTo") && reportChange.Metadata["CallBackTo"] == "100")
            {
                reportChange.Metadata["CallBackTo"] = null;
                reportChange.AddOnlyProperties(o => o.Metadata);
            }

            reportChange.Status = 150;
            reportChange.AddOnlyProperties(o => o.Status);
            conn.UpdateOnly(reportChange);
            reportChange.RemoveAllOnlyProperties();
            rtns.Results.Add(reportChange);
            #region 创建操作日志
            var _log = new EntityLog<ReportChange>();
            _log.OwnerKvid = Kvid;
            _log.Title = "更改报告";
            _log.Type = "流转";
            _log.Summary = "提交了更改报告申请";
            conn.Insert(_log);
            #endregion
            return rtns;
        }
    }

    //申请驳回 Status->150
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "驳回审批")]
    public class ReportChangeRefuse : RestfulExecution<ReportChange>
    {
        public Guid Kvid { get; set; }
        public string Remark { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<ReportChange>();
            var reportChange = conn.SingleById<ReportChange>(Kvid);
            var rtns = new RestfulUpdateResponse<ReportChange>();
            rtns.Results = new List<ReportChange>();
            reportChange.ThrowIfNull("未查到该更改报告申请");
            if (reportChange.Status != 150) throw new Exception("当前更改报告申请非审批状态，无法驳回审批！");
            if (reportChange.Metadata == null) reportChange.Metadata = new Dictionary<string, string>();
            reportChange.Metadata["CallBackFrom"] = "150";
            reportChange.Metadata["CallBackTo"] = "100";
            reportChange.AddOnlyProperties(o => o.Metadata);
            reportChange.Status = 100;
            reportChange.AddOnlyProperties(o => o.Status);
            conn.UpdateOnly(reportChange);
            reportChange.RemoveAllOnlyProperties();
            rtns.Results.Add(reportChange);
            #region 创建操作日志
            var _log = new EntityLog<ReportChange>();
            _log.OwnerKvid = Kvid;
            _log.Title = "更改报告";
            _log.Type = "流转";
            _log.Summary = "驳回了更改报告申请审批";
            _log.Remark = Remark;
            conn.Insert(_log);
            #endregion
            return rtns;
        }
    }

    //更改报告申请同意
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "同意")]
    public class ReportChangeFinish : RestfulExecution<ReportChange>
    {
        public Guid Kvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<ReportChange>();
            var reportChange = conn.SingleById<ReportChange>(Kvid);
            var rtns = new RestfulUpdateResponse<ReportChange>();
            rtns.Results = new List<ReportChange>();
            reportChange.ThrowIfNull("未查到该更改报告申请单");
            //验证报告是否归档，报告未归档，不允许更改
            var queryOriginalReport = conn.From<Report>();
            queryOriginalReport.Where(o => o.ReportId == reportChange.ReportId);
            var originalReport = conn.Select(queryOriginalReport).FirstOrDefault();
            originalReport.ThrowIfNull("未查询到原报告！");
            if (originalReport.StatusType != ReportStatusType.ReportCollected) throw new Exception("原报告未归档，无法更改！");
            if (reportChange.Status != 150 ) throw new Exception("当前更改报告申请单非审批状态，无法启用！");
            reportChange.Status = 200;
            reportChange.AddOnlyProperties(o => o.Status);
            conn.UpdateOnly(reportChange);
            reportChange.RemoveAllOnlyProperties();
            rtns.Results.Add(reportChange);
            #region 创建操作日志
            var _log = new EntityLog<ReportChange>();
            _log.OwnerKvid = Kvid;
            _log.Title = "更改报告";
            _log.Type = "流转";
            _log.Summary = "同意了更改报告申请";
            conn.Insert(_log);
            #endregion
            return rtns;
        }
    }

    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "初始化")]
    public class ReportChangeInit : RestfulExecution<ReportChange>
    {
        public Guid Kvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<ReportChange>();
            var reportChange = conn.SingleById<ReportChange>(Kvid);
            var rtns = new RestfulUpdateResponse<ReportChange>();
            rtns.Results = new List<ReportChange>();
            reportChange.ThrowIfNull("未查到该更改报告申请");
            if (reportChange.Status == 100) throw new Exception("当前更改报告申请单已是草稿状态，无需初始化！");
            reportChange.Status = 100;
            reportChange.AddOnlyProperties(o => o.Status);
            conn.UpdateOnly(reportChange);
            reportChange.RemoveAllOnlyProperties();
            rtns.Results.Add(reportChange);
            #region 创建操作日志
            var _log = new EntityLog<ReportChange>();
            _log.OwnerKvid = Kvid;
            _log.Title = "更改报告";
            _log.Type = "流转";
            _log.Summary = "初始化至草稿状态";
            conn.Insert(_log);
            #endregion
            return rtns;
        }
    }

    //审批中未审核通过的报告进行召回
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "召回")]
    public class ReportChangeBack : RestfulExecution<ReportChange>
    {
        public Guid Kvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<ReportChange>();
            var reportChange = conn.SingleById<ReportChange>(Kvid);
            var rtns = new RestfulUpdateResponse<ReportChange>();
            rtns.Results = new List<ReportChange>();
            reportChange.ThrowIfNull("未查到该更改报告申请");
            if (reportChange.Status != 150) throw new Exception("当前更改报告申请单状态禁止召回！");
            reportChange.Status = 100;
            reportChange.AddOnlyProperties(o => o.Status);
            conn.UpdateOnly(reportChange);
            reportChange.RemoveAllOnlyProperties();
            rtns.Results.Add(reportChange);
            #region 创建操作日志
            var _log = new EntityLog<ReportChange>();
            _log.OwnerKvid = Kvid;
            _log.Title = "更改报告";
            _log.Type = "流转";
            _log.Summary = "召回至草稿状态";
            conn.Insert(_log);
            #endregion
            return rtns;
        }
    }

    #endregion
}
