﻿using Kivii.Lims.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Lac.Lims.Transforms
{
    #region CUDQR
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportNoteQuery:RestfulQuery<ReportNote> { }
    #endregion

    #region 报告打回及返工任务查询
    [Api(Description = "报告打回及返工任务查询")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportNoteQueryEx : RestfulExecution<ReportNote>
    {
        //参数名不能与返回的字段同名
        public string ReportIdFilter { get; set; }
        public string ReportOperatorNameFilter { get; set; }
        public DateTime? DeadDateGreaterThanOrEqualTo { get; set; }
        public DateTime? DeadDateLessThanOrEqualTo { get; set; }

        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }
        #endregion
        public override object OnExecution(IRequest req, IResponse res)
        {
            //返工的title要提前组织好
            List<string> titles = new List<string>();
            titles.Add("任务打回");
            titles.Add("项目打回");
            titles.Add("回收");
            titles.Add("报告召回");
            var conn = KiviiContext.GetOpenedDbConnection<ReportNote>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<ReportNoteEx>();
            request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress
                .LeftJoin<Report>((reportNoteEx, report) => reportNoteEx.ReportKvid == report.Kvid)
                .Select<ReportNote, Report>((reportNote, report) => new
                {
                    T = Sql.AllFields(reportNote),
                    report.ReportId,
                    ReportOperatorName = report.OperatorName,
                    ReportPreparerName = report.PreparerName
                });
            sqlExpress.And<ReportNote>(o => Sql.In(o.Title, titles));
            if (!ReportIdFilter.IsNullOrEmpty())
            {
                sqlExpress.And<Report>(o => o.ReportId.Contains(ReportIdFilter));
            }
            if (!ReportOperatorNameFilter.IsNullOrEmpty())
            {
                sqlExpress.And<Report>(o => o.OperatorName.Contains(ReportOperatorNameFilter));
            }
            if (DeadDateGreaterThanOrEqualTo != null)
            {
                sqlExpress.And<Report>(o => o.DeadDate >= DeadDateGreaterThanOrEqualTo);
            }
            if (DeadDateLessThanOrEqualTo != null)
            {
                sqlExpress.And<Report>(o => o.DeadDate <= DeadDateLessThanOrEqualTo);
            }
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }
    #endregion
}
