﻿
using Kivii;
using Kivii.Lims.Entities;
using Kivii.Lac.Lims.Extensions;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using static Kivii.Lac.Lims.Extensions.ReportExtension;

namespace Kivii.Lac.Lims.Transforms
{
    #region CRUDQ
    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "创建")]
    public class ReportItemCreate : RestfulCreate<ReportItem>
    {
        public override bool OnPreRestfulCreate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulCreateResponse<ReportItem> rtns)
        {
            #region 项目类型判断
            //规则只有在检测项目录（TestFolder）、色号（SampleSelf）、部位（SamplePart）下才能建立子项
            if (Item.ParentKvid != Guid.Empty)
            {
                var parentNode = dbConnection.SingleById<ReportItem>(Item.ParentKvid);
                if (parentNode != null)
                {
                    (parentNode.Type == ReportItemType.Unsupported || parentNode.Type == ReportItemType.Folder).ThrowIfTrue("未知项下不可建立子项!");

                    //若在类型为Test的项目下建立子项，则自动将该类型更新为TestFolder
                    if (parentNode.Type == ReportItemType.Test)
                    {
                        (Item.Type != ReportItemType.Test).ThrowIfTrue($"此项下不可建立当前类型{Item.Type}!");
                        parentNode.Type = ReportItemType.TestFolder;
                        parentNode.AddOnlyProperties(o => o.Type);
                        dbConnection.UpdateOnly(parentNode);
                    }

                    if (parentNode.Type == ReportItemType.TestFolder)
                    {
                        (Item.Type == ReportItemType.TestFolder || Item.Type == ReportItemType.SamplePart || Item.Type == ReportItemType.SampleSelf).ThrowIfTrue($"检测目录下不能建立当前类型项目{Item.Type}!");
                    }

                    if (parentNode.Type == ReportItemType.SamplePart)
                    {
                        (Item.Type == ReportItemType.SamplePart || Item.Type == ReportItemType.SampleSelf).ThrowIfTrue($"此项下不能建立当前类型项目{Item.Type}!");
                    }

                    if (parentNode.Type == ReportItemType.SampleSelf)
                    {
                        (Item.Type == ReportItemType.SampleSelf).ThrowIfTrue($"此项下不能建立当前类型项目{Item.Type}!");
                    }
                }
                else Item.ParentKvid = Guid.Empty;
            }
            #endregion
            Item.AddOnlyProperties(o => o.Status);
            return base.OnPreRestfulCreate(req, res, dbConnection, rtns);
        }
    }

    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "读取")]
    public class ReportItemRead : RestfulRead<ReportItem>
    { }

    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "更新")]
    public class ReportItemUpdate : RestfulUpdate<ReportItem>
    {
        public decimal AmountPlan = 0;
        //用于记录检验项目修改内容(目前应用于:客户要求标准值)
        public ReportItem preItem = null;
        public string updateContent = string.Empty;
        public override bool OnPreRestfulUpdate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulUpdateResponse<ReportItem> rtns)
        {
            (Item.Status == int.MaxValue).ThrowIfTrue("当前检测为归档状态，无权限修改！");
            var reportItem = dbConnection.SingleById<ReportItem>(Item.Kvid);
            preItem = reportItem;
            AmountPlan = reportItem.AmountPlan;
            if (Item.OnlyPropertiesIsExist(o => o.DetectionValueQualified))
            {
                reportItem.DetectionValueQualified = Item.DetectionValueQualified;
                var report = dbConnection.SingleById<Report>(reportItem.ReportKvid);
                if (report != null)
                {
                    if (reportItem.CalculateJudgement2(report))
                    {
                        Item.Judgement = reportItem.Judgement;
                        Item.AddOnlyProperties(o => o.Judgement);
                    }
                }
            }
            if (Item.Metadata != null && Item.Metadata.ContainsKey("DetectionValueCustomer"))
            {
                var newValue = Item.Metadata["DetectionValueCustomer"];
                var oldValue = string.Empty;
                if (preItem.Metadata != null && preItem.Metadata.ContainsKey("DetectionValueCustomer")) oldValue = preItem.Metadata["DetectionValueCustomer"];
                if(oldValue!=newValue) updateContent = $"标准值由“{oldValue}”修改为“{newValue}”";
            }
            return base.OnPreRestfulUpdate(req, res, dbConnection, rtns);
        }
        public override bool OnPostRestfulUpdate(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulUpdateResponse<ReportItem> rtns)
        {
            if (Item.OnlyPropertiesIsExist(o => o.AmountPlan) && AmountPlan != Item.AmountPlan)
            {
                var report = dbConnection.SingleById<Report>(rtns.Results[0].ReportKvid);
                if (report != null)
                {
                    rtns.Results[0].SetAmountLog(report, "ReportItemUpdate", AmountPlan, rtns.Results[0].AmountPlan, dbConnection);
                }
            }
            //使用preItem是为了日志记录的更详细,能定位到项目名称
            if (!updateContent.IsNullOrEmpty()) preItem.SetDetectionValueQualifiedCustomerRequiredLog("update",updateContent,dbConnection);
            return base.OnPostRestfulUpdate(req, res, dbConnection, rtns);
        }
    }

    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "更新扩展,录入实测值文件路径等")]
    public class ReportItemUpdateEx2 : RestfulExecution<ReportItem>
    {
        public ReportItem Item { get; set; }

        public List<ReportItem> Items { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null & Items.IsNullOrEmpty()).ThrowIfTrue("请传入要更新的信息!");
            if (Items.IsNullOrEmpty()) Items = new List<ReportItem>();
            if (Item != null)
            {
                if (!Items.Exists(o => o.Kvid == Item.Kvid)) Items.Add(Item);
            }

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var reportItems = conn.SelectByIds<ReportItem>(Items.ConvertAll(o => o.Kvid));
            reportItems.ThrowIfNullOrEmpty("未找到要更新的项目信息!");

            var parentReportItems = conn.Select<ReportItem>(o => Sql.In(o.Kvid, reportItems.ConvertAll(p => p.ParentKvid)) & o.Type == ReportItemType.TestFolder);

            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            var trans = conn.OpenTransaction();

            try
            {
                foreach (var _reportItem in reportItems)
                {
                    var item = Items.FirstOrDefault(o => o.Kvid == _reportItem.Kvid);
                    if (item == null) continue;
                    item.RemoveAllOnlyProperties();
                    item.AddOnlyProperties(o => o.Status);
                    item.AddOnlyProperties(o => o.StatusType);
                    if (item.OnlyProperties.Count <= 0) continue;
                    conn.UpdateOnly(item);
                    rtns.Results.Add(item);

                    //更改实测值执行  要是实测值相同  就不执行下面操作
                    if (item.OnlyPropertiesIsExist(o => o.TestValue))
                    {
                        //判断是否需要更新大项
                        var parentReportItem = parentReportItems.FirstOrDefault(o => o.Kvid == _reportItem.ParentKvid & o.Type == ReportItemType.TestFolder);
                        if (parentReportItem != null)
                        {
                            if (parentReportItem.CalculateJudgement(conn))
                            {
                                parentReportItem.StatusType = ReportItemStatusType.BeforeReview;
                                parentReportItem.AddOnlyProperties(o => o.Status);
                                parentReportItem.AddOnlyProperties(o => o.Judgement);
                                parentReportItem.AddOnlyProperties(o => o.TestCreateTime);
                                parentReportItem.AddOnlyProperties(o => o.TestCreatorKvid);
                                parentReportItem.AddOnlyProperties(o => o.TestCreatorName);

                                //要是有更新TestValue 复核人全部清空
                                parentReportItem.TestReviewerKvid = Guid.Empty;
                                parentReportItem.AddOnlyProperties(o => o.TestReviewerKvid);
                                parentReportItem.TestReviewerName = string.Empty;
                                parentReportItem.AddOnlyProperties(o => o.TestReviewerName);
                                parentReportItem.TestReviewTime = null;
                                parentReportItem.AddOnlyProperties(o => o.TestReviewTime);

                                conn.UpdateOnly(parentReportItem);
                                rtns.Results.Add(parentReportItem);
                            }
                        }
                    }
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    //代码初审 任天宇 20210904

    public enum OperateType
    {
        Unsupported = 0,

        /// <summary>
        /// 录入
        /// </summary>
        DataEntry = 1,

        /// <summary>
        /// 复核
        /// </summary>
        Review = 2,

        /// <summary>
        /// 复测，对原检测再次录入数据；只记录Test信息，不更新ReportItem数据
        /// </summary>
        ReTest = 3,

        /// <summary>
        /// 修改标准值
        /// </summary>
        AltervalueQualified = 4
    }
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "更新扩展,录入实测值文件路径等")]
    public class ReportItemUpdateEx : RestfulExecution<ReportItem>
    {
        public Guid ReportKvid { get; set; }
        public ReportItem Item { get; set; }
        public List<ReportItem> Items { get; set; }
        public OperateType OperateType { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据有效性验证
            if (OperateType == OperateType.Unsupported) throw new Exception("传入OperateType类型错误，不支持的操作！");
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("请传入要更新的信息!");
            ReportKvid.ThrowIfEmpty("need reportkvid");
            #endregion

            if (Items.IsNullOrEmpty()) Items = new List<ReportItem>();
            if (Item != null)
            {
                if (!Items.Exists(o => o.Kvid == Item.Kvid)) Items.Add(Item);
            }
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            Report report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未查询到对应的检测报告！");

            //指定报告的全部ReportItem
            var allReportItmes = conn.Select<ReportItem>(o => o.ReportKvid == ReportKvid);

            var reportItemsKvids = Items.ConvertAll(o => o.Kvid);
            var reportItems = allReportItmes.Where(o => reportItemsKvids.Contains(o.Kvid)).ToList();
            (reportItems.Count == reportItemsKvids.Count).ThrowIfFalse("未找到要更新的项目信息!");
            //只取选中项目的大项及子项
            reportItems = reportItems.Where(o => o.Type == ReportItemType.TestFolder || o.Type == ReportItemType.Test).ToList();

            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            IDbTransaction trans = null;

            //如果是复核，验证所有的reportitems的检验人和复核人
            if (OperateType == OperateType.Review)
            {
                if (reportItems.Exists(o => o.TestCreatorKvid == KiviiContext.CurrentMember.Kvid)) throw new Exception("存在录入人与复核人相同，无法执行！");
                if (reportItems.Exists(o => o.StatusType != ReportItemStatusType.BeforeReview && o.StatusType != ReportItemStatusType.TestFinished)) throw new Exception("存在不在待复核状态的项目!");

                var testKvids = reportItems.ConvertAll(o => o.TestKvid);
                var queryTest = conn.From<Test>();
                queryTest.Where(o => Sql.In(o.Kvid, testKvids));
                var tests = conn.Select(queryTest);
                //此处应判断Test的数量

                //此处启动事务，使用批处理更新ReportItem和Test
                trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
                try
                {
                    //更新ReportItems
                    var updateReportItems = conn.From<ReportItem>();
                    updateReportItems = updateReportItems.Update(o => new { o.TestReviewerKvid, o.TestReviewerName, o.TestReviewTime, o.Status });
                    updateReportItems = updateReportItems.Where(o => Sql.In(o.Kvid, reportItems.ConvertAll(p => p.Kvid)));
                    conn.UpdateOnly<ReportItem>(new ReportItem { TestReviewerKvid = KiviiContext.CurrentMember.Kvid, TestReviewerName = KiviiContext.CurrentMember.FullName, TestReviewTime = DateTime.Now, Status = (int)ReportItemStatusType.TestFinished }, updateReportItems);

                    //上面批量更新 数据没有反写,方便返回值
                    reportItems.ForEach(o => { o.TestReviewerKvid = KiviiContext.CurrentMember.Kvid; o.TestReviewerName = KiviiContext.CurrentMember.FullName; o.TestReviewTime = DateTime.Now; o.Status = (int)ReportItemStatusType.TestFinished; o.RemoveAllOnlyProperties(); });

                        //更新Tests
                        var updateTests = conn.From<Test>();
                    updateTests = updateTests.Update(o => new { o.ReviewerKvid, o.ReviewerName, o.ReviewTime });
                    updateTests = updateTests.Where(o => Sql.In(o.Kvid, tests.ConvertAll(p => p.Kvid)));
                    conn.UpdateOnly<Test>(new Test { ReviewerKvid = KiviiContext.CurrentMember.Kvid, ReviewerName = KiviiContext.CurrentMember.FullName, ReviewTime = DateTime.Now }, updateTests);

                    rtns.Results.AddRange(reportItems);

                    trans?.Commit();
                }
                catch (Exception ex)
                {
                    trans?.Rollback();
                    throw ex;
                }
                return rtns;
            }

            if (OperateType == OperateType.DataEntry)
            {
                if (reportItems.Exists(o => o.StatusType == ReportItemStatusType.TestCollected)) throw new Exception("存在项目已归档，无法录入数据或重录！");

                var reportItemParentKvids = reportItems.Select(o => o.ParentKvid).Distinct();
                var parentReportItems = allReportItmes.Where(o => o.Type == ReportItemType.TestFolder && reportItemParentKvids.Contains(o.Kvid));

                //定义好待更新的ReportItem和Test集合，数据处理完毕后，批量执行更新操作
                var reportItemUpdatePool = new List<ReportItem>();
                var testInsertPool = new List<Test>();
                var testUpdatePool = new List<Test>();

                var originalTests = conn.SelectByIds<Test>(reportItems.ConvertAll(o => o.TestRootKvid));
                foreach (var _reportItem in reportItems)
                {
                    var item = Items.FirstOrDefault(o => o.Kvid == _reportItem.Kvid);
                    if (item == null) continue;
                    //使用_reportitem进行更新
                    if (!item.Title.IsNullOrEmpty())
                    {
                        _reportItem.Title = item.Title;
                        _reportItem.AddOnlyProperties(o => o.Title);
                    }
                    if (!item.DetectionCodeActual.IsNullOrEmpty())
                    {
                        _reportItem.DetectionCodeActual = item.DetectionCodeActual;
                        _reportItem.AddOnlyProperties(o => o.DetectionCodeActual);
                    }
                    
                    if (!item.TestOriginalFilePath.IsNullOrEmpty())
                    {
                        _reportItem.TestOriginalFilePath = item.TestOriginalFilePath;
                        _reportItem.AddOnlyProperties(o => o.TestOriginalFilePath);
                    }

                    //情况1 实测值和原始记录都为空就抛出异常
                    if (item.TestValue.IsNullOrEmpty() && item.TestOriginalFilePath.IsNullOrEmpty()) throw new Exception("类型为DataEntry的操作，必须传入实测值或原始记录路径！");
                    if (_reportItem.Type == ReportItemType.Test)
                    {
                        //经过计算，若完成判定则更新判定结果
                        _reportItem.StatusType = ReportItemStatusType.BeforeReview;
                        _reportItem.AddOnlyProperties(o => o.Status);
                        _reportItem.TestValue = item.TestValue;

                        //优化CalculateJudgement方法，传入报告和检测项目集合

                        if (_reportItem.CalculateJudgement2(report))
                        {
                            _reportItem.AddOnlyProperties(o => o.Judgement);
                        }
                        var originalTest = originalTests.FirstOrDefault(o => o.Kvid == _reportItem.TestKvid);
                        //情况2  实测值和原始记录都传了 就创建新的Test
                        if (!item.TestValue.IsNullOrEmpty() && !item.TestOriginalFilePath.IsNullOrEmpty())
                        {

                            #region 创建一个Test
                            var _test = new Test();
                            _test.StandardKvid = _reportItem.StandardKvid;
                            _test.DetectionKvid = _reportItem.DetectionKvid;
                            _test.Kvid = Guid.NewGuid();
                            _test.RootKvid = originalTest == null ? _test.Kvid : originalTest.RootKvid;
                            _test.SampleKvid = _reportItem.SampleKvid;
                            _test.SampleRootKvid = _reportItem.SampleRootKvid;
                            _test.CreateTime = DateTime.Now;
                            if (!item.TestValue.IsNullOrEmpty()) _test.Value = item.TestValue;
                            if (!item.TestOriginalFilePath.IsNullOrEmpty()) _test.OriginalFilePath = item.TestOriginalFilePath;
                            testInsertPool.Add(_test);
                            #endregion

                            #region ReportItem 判定及更新Test相关字段
                            _reportItem.TestKvid = _test.Kvid;
                            _reportItem.AddOnlyProperties(o => o.TestKvid);
                            _reportItem.TestRootKvid = _test.RootKvid;
                            _reportItem.AddOnlyProperties(o => o.TestRootKvid);
                            if (!_reportItem.TestValue.IsNullOrEmpty()) _reportItem.AddOnlyProperties(o => o.TestValue);
                            //if (!_reportItem.TestOriginalFilePath.IsNullOrEmpty()) _reportItem.AddOnlyProperties(o => o.TestOriginalFilePath);
                            #endregion
                        }
                        //情况3 只传了实测值或者原始记录路径,则只更新当前Test(要是存在Test,不然就创建一个没有原始记录的Test)
                        else if (!item.TestValue.IsNullOrEmpty() || !item.TestOriginalFilePath.IsNullOrEmpty())
                        {
                            //原本就没有Test则创建一个
                            if (originalTest == null)
                            {
                                var _test = new Test();
                                _test.StandardKvid = _reportItem.StandardKvid;
                                _test.DetectionKvid = _reportItem.DetectionKvid;
                                _test.Kvid = Guid.NewGuid();
                                _test.RootKvid = _test.Kvid;
                                _test.SampleKvid = _reportItem.SampleKvid;
                                _test.SampleRootKvid = _reportItem.SampleRootKvid;
                                _test.CreateTime = DateTime.Now;
                                if (!item.TestValue.IsNullOrEmpty()) _test.Value = item.TestValue;
                                if (!item.TestOriginalFilePath.IsNullOrEmpty()) _test.OriginalFilePath = item.TestOriginalFilePath;
                                testInsertPool.Add(_test);

                                //_reportItem.TestOriginalFilePath = item.TestOriginalFilePath;
                                _reportItem.TestKvid = _test.Kvid;
                                _reportItem.AddOnlyProperties(o => o.TestKvid);
                                _reportItem.TestRootKvid = _test.RootKvid;
                                _reportItem.AddOnlyProperties(o => o.TestRootKvid);
                            }
                            //要是有test了 就更新
                            else
                            {
                                if (!item.TestValue.IsNullOrEmpty())
                                {
                                    originalTest.Value = item.TestValue;
                                    originalTest.AddOnlyProperties(o => o.Value);
                                }
                                if (!item.TestOriginalFilePath.IsNullOrEmpty())
                                {
                                    originalTest.OriginalFilePath = item.TestOriginalFilePath;
                                    originalTest.AddOnlyProperties(o => o.OriginalFilePath);
                                }
                                testUpdatePool.Add(originalTest);
                            }

                            #region ReportItem 判定及更新Test相关字段
                            if (!_reportItem.TestValue.IsNullOrEmpty()) _reportItem.AddOnlyProperties(o => o.TestValue);
                            //if (!_reportItem.TestOriginalFilePath.IsNullOrEmpty()) _reportItem.AddOnlyProperties(o => o.TestOriginalFilePath);
                            #endregion
                        }
                        //要是有更新TestValue 复核人全部清空
                        _reportItem.TestReviewerKvid = Guid.Empty;
                        _reportItem.AddOnlyProperties(o => o.TestReviewerKvid);
                        _reportItem.TestReviewerName = string.Empty;
                        _reportItem.AddOnlyProperties(o => o.TestReviewerName);
                        _reportItem.TestReviewTime = null;
                        _reportItem.AddOnlyProperties(o => o.TestReviewTime);
                        _reportItem.TestCreateTime = DateTime.Now;
                        _reportItem.AddOnlyProperties(o => o.TestCreateTime);
                        _reportItem.TestCreatorKvid = KiviiContext.CurrentMember.Kvid;
                        _reportItem.AddOnlyProperties(o => o.TestCreatorKvid);
                        _reportItem.TestCreatorName = KiviiContext.CurrentMember.FullName;
                        _reportItem.AddOnlyProperties(o => o.TestCreatorName);
                        _reportItem.StatusType = ReportItemStatusType.BeforeReview;
                        _reportItem.AddOnlyProperties(o => o.Status);
                        reportItemUpdatePool.Add(_reportItem);
                        //判断是否需要更新大项
                        var parentReportItem = parentReportItems.FirstOrDefault(o => o.Kvid == _reportItem.ParentKvid && o.Type == ReportItemType.TestFolder);
                        if (parentReportItem != null)
                        {
                            //大项是TestFolder 判定的话 需要把大项下面的子项传给判定方法
                            //allNodes是此大项的全部子项  nodes是待处理的子项 两者合并后传个方法进行大项判定
                            //2023/0909改写法
                            var allNodes = allReportItmes.Where(o => o.ParentKvid == parentReportItem.Kvid).ToList();
                            //var nodes = reportItemUpdatePool.Where(o => o.ParentKvid == parentReportItem.Kvid).ToList();
                            //foreach (var all in allNodes)
                            //{
                            //    if (nodes.Exists(o => o.Kvid == all.Kvid)) continue;
                            //    nodes.Add(all);
                            //}
                            if (parentReportItem.CalculateJudgement2(report, allNodes)) parentReportItem.AddOnlyProperties(o => o.Judgement);
                            parentReportItem.StatusType = ReportItemStatusType.BeforeReview;
                            parentReportItem.AddOnlyProperties(o => o.Status);
                            parentReportItem.TestCreateTime = DateTime.Now;
                            parentReportItem.AddOnlyProperties(o => o.TestCreateTime);
                            parentReportItem.TestCreatorKvid = KiviiContext.CurrentMember.Kvid;
                            parentReportItem.AddOnlyProperties(o => o.TestCreatorKvid);
                            parentReportItem.TestCreatorName = KiviiContext.CurrentMember.FullName;
                            parentReportItem.AddOnlyProperties(o => o.TestCreatorName);

                            var parentItem = Items.FirstOrDefault(o => o.Kvid == parentReportItem.Kvid);
                            if (parentItem != null)
                            {
                                parentReportItem.TestOriginalFilePath = parentItem.TestOriginalFilePath;
                                parentReportItem.AddOnlyProperties(o => o.TestOriginalFilePath);
                            }
                            //要是有更新TestValue 复核人全部清空
                            parentReportItem.TestReviewerKvid = Guid.Empty;
                            parentReportItem.AddOnlyProperties(o => o.TestReviewerKvid);
                            parentReportItem.TestReviewerName = string.Empty;
                            parentReportItem.AddOnlyProperties(o => o.TestReviewerName);
                            parentReportItem.TestReviewTime = null;
                            parentReportItem.AddOnlyProperties(o => o.TestReviewTime);

                            if (reportItemUpdatePool.Exists(o => o.Kvid == parentReportItem.Kvid)) reportItemUpdatePool.RemoveAll(o => o.Kvid == parentReportItem.Kvid);
                            reportItemUpdatePool.Add(parentReportItem);
                        }
                    }
                    if (_reportItem.Type == ReportItemType.TestFolder)
                    {
                        //当如果是大项ReportItem,分两种情况要是传来的数据中有此大项的子项,就直接跳过,因为上一个步骤会连同对应大项一起判定或者记录原始记录路径, 要是传过来的数据中没有此大项的子项,说明此大项需要更新原始记录路径!
                        var existNodes = Items.Where(o => o.ParentKvid == _reportItem.Kvid).ToList();
                        if (!existNodes.IsNullOrEmpty()) continue;
                        _reportItem.TestOriginalFilePath = item.TestOriginalFilePath;
                        _reportItem.AddOnlyProperties(o => o.TestOriginalFilePath);
                        if (reportItemUpdatePool.Exists(o => o.Kvid == _reportItem.Kvid)) reportItemUpdatePool.RemoveAll(o => o.Kvid == _reportItem.Kvid);
                        reportItemUpdatePool.Add(_reportItem);
                    }
                }

                trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
                try
                {
                    foreach (var reportItem in reportItemUpdatePool)
                    {
                        conn.UpdateOnly(reportItem);
                        reportItem.RemoveAllOnlyProperties();//这句为了返回完整ReportItem数据,否则只会返回更新的字段数据
                        rtns.Results.Add(reportItem);
                    }
                    foreach (var testInsert in testInsertPool)
                    {
                        conn.Insert(testInsert);
                    }
                    foreach (var testUpdate in testUpdatePool)
                    {
                        conn.UpdateOnly(testUpdate);
                    }
                    trans?.Commit();

                }
                catch (Exception ex)
                {
                    trans?.Rollback();
                    throw ex;
                }
            }

            if (OperateType == OperateType.ReTest)
            {
                if (reportItems.Exists(o => o.StatusType == ReportItemStatusType.TestCollected)) throw new Exception("存在项目已归档，无法复测！");

                var reportItemParentKvids = reportItems.Select(o => o.ParentKvid).Distinct();
                var parentReportItems = allReportItmes.Where(o => o.Type == ReportItemType.TestFolder && reportItemParentKvids.Contains(o.Kvid));

                //定义好待更新的ReportItem和Test集合，数据处理完毕后，批量执行更新操作
                var reportItemUpdatePool = new List<ReportItem>();
                var testInsertPool = new List<Test>();
                var testUpdatePool = new List<Test>();

                var originalTests = conn.SelectByIds<Test>(reportItems.ConvertAll(o => o.TestRootKvid));
                foreach (var _reportItem in reportItems)
                {
                    var item = Items.FirstOrDefault(o => o.Kvid == _reportItem.Kvid);
                    if (item == null) continue;
                    //情况1 实测值和原始记录都为空就抛出异常
                    if (item.TestOriginalFilePath.IsNullOrEmpty()) throw new Exception("类型为复测的操作，必须传入原始记录路径！");
                    if (_reportItem.Type == ReportItemType.Test)
                    {
                        var originalTest = originalTests.FirstOrDefault(o => o.Kvid == _reportItem.TestKvid);
                        //情况2  实测值和原始记录都传了 就创建新的Test
                        if (!item.TestValue.IsNullOrEmpty())
                        {

                            #region 创建一个Test
                            var _test = new Test();
                            _test.StandardKvid = _reportItem.StandardKvid;
                            _test.DetectionKvid = _reportItem.DetectionKvid;
                            _test.Kvid = Guid.NewGuid();
                            _test.RootKvid = originalTest == null ? _test.Kvid : originalTest.RootKvid;
                            _test.SampleKvid = _reportItem.SampleKvid;
                            _test.SampleRootKvid = _reportItem.SampleRootKvid;
                            _test.CreateTime = DateTime.Now;
                            if (!item.TestValue.IsNullOrEmpty()) _test.Value = item.TestValue;
                            _test.OriginalFilePath = item.TestOriginalFilePath;
                            testInsertPool.Add(_test);
                            #endregion

                            #region ReportItem 更新Metadata，记录复测的原始记录路径
                            if (_reportItem.Metadata == null) _reportItem.Metadata = new Dictionary<string, string>();
                            _reportItem.Metadata["ReTestSdmsFilePath"] = _test.OriginalFilePath;
                            _reportItem.AddOnlyProperties(o => o.Metadata);
                            reportItemUpdatePool.Add(_reportItem);
                            #endregion
                        }
                        //判断是否需要更新大项
                        var parentReportItem = parentReportItems.FirstOrDefault(o => o.Kvid == _reportItem.ParentKvid && o.Type == ReportItemType.TestFolder);
                        if (parentReportItem != null)
                        {
                            //大项是TestFolder 判定的话 需要把大项下面的子项传给判定方法
                            //allNodes是此大项的全部子项  nodes是待处理的子项 两者合并后传个方法进行大项判定
                            var allNodes = allReportItmes.Where(o => o.ParentKvid == parentReportItem.Kvid).ToList();
                            var nodes = reportItemUpdatePool.Where(o => o.ParentKvid == parentReportItem.Kvid).ToList();
                            foreach (var all in allNodes)
                            {
                                if (nodes.Exists(o => o.Kvid == all.Kvid)) continue;
                                nodes.Add(all);
                            }

                            var parentItem = Items.FirstOrDefault(o => o.Kvid == parentReportItem.Kvid);
                            if (parentItem != null)
                            {
                                #region ReportItem 更新Metadata，记录复测的原始记录路径
                                if (parentReportItem.Metadata == null) parentReportItem.Metadata = new Dictionary<string, string>();
                                parentReportItem.Metadata["ReTestSdmsFilePath"] = parentItem.TestOriginalFilePath;
                                parentReportItem.AddOnlyProperties(o => o.Metadata);
                                reportItemUpdatePool.Add(parentReportItem);
                                #endregion
                            }
                        }
                    }
                    if (_reportItem.Type == ReportItemType.TestFolder)
                    {
                        //当如果是大项ReportItem,分两种情况要是传来的数据中有此大项的子项,就直接跳过,因为上一个步骤会连同对应大项一起判定或者记录原始记录路径, 要是传过来的数据中没有此大项的子项,说明此大项需要更新原始记录路径!
                        var existNodes = Items.Where(o => o.ParentKvid == _reportItem.Kvid).ToList();
                        if (!existNodes.IsNullOrEmpty()) continue;

                        #region ReportItem 更新Metadata，记录复测的原始记录路径
                        if (_reportItem.Metadata == null) _reportItem.Metadata = new Dictionary<string, string>();
                        _reportItem.Metadata["ReTestSdmsFilePath"] = item.TestOriginalFilePath;
                        _reportItem.AddOnlyProperties(o => o.Metadata);
                        #endregion
                        if (reportItemUpdatePool.Exists(o => o.Kvid == _reportItem.Kvid)) reportItemUpdatePool.RemoveAll(o => o.Kvid == _reportItem.Kvid);
                        reportItemUpdatePool.Add(_reportItem);
                    }
                }

                trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
                try
                {
                    foreach (var reportItem in reportItemUpdatePool)
                    {
                        conn.UpdateOnly(reportItem);
                        reportItem.RemoveAllOnlyProperties();//这句为了返回完整ReportItem数据,否则只会返回更新的字段数据
                        rtns.Results.Add(reportItem);
                    }
                    foreach (var testInsert in testInsertPool)
                    {
                        conn.Insert(testInsert);
                    }
                    foreach (var testUpdate in testUpdatePool)
                    {
                        conn.UpdateOnly(testUpdate);
                    }
                    trans?.Commit();

                }
                catch (Exception ex)
                {
                    trans?.Rollback();
                    throw ex;
                }
            }

            if (OperateType == OperateType.AltervalueQualified)
            {
                if (reportItems.Exists(o => o.StatusType == ReportItemStatusType.TestCollected)) throw new Exception("存在项目已归档，禁止修改标准值！");
                var reportItemParentKvids = reportItems.Select(o => o.ParentKvid).Distinct();
                var parentReportItems = allReportItmes.Where(o => o.Type == ReportItemType.TestFolder && reportItemParentKvids.Contains(o.Kvid));
                //定义好待更新的ReportItem集合，数据处理完毕后，批量执行更新操作
                var reportItemUpdatePool = new List<ReportItem>();
                foreach (var _reportItem in reportItems)
                {
                    var item = Items.FirstOrDefault(o => o.Kvid == _reportItem.Kvid);
                    _reportItem.DetectionValueQualified = item.DetectionValueQualified;
                    _reportItem.AddOnlyProperties(o => o.DetectionValueQualified);
                    if (_reportItem.CalculateJudgement2(report))
                    {
                        _reportItem.AddOnlyProperties(o => o.Judgement);
                    }
                    reportItemUpdatePool.Add(_reportItem);
                    var parentReportItem = parentReportItems.FirstOrDefault(o => o.Kvid == _reportItem.ParentKvid && o.Type == ReportItemType.TestFolder);
                    if (parentReportItem != null)
                    {
                        //大项是TestFolder 判定的话 需要把大项下面的子项传给判定方法
                        //allNodes是此大项的全部子项  nodes是待处理的子项 两者合并后传个方法进行大项判定
                        var allNodes = allReportItmes.Where(o => o.ParentKvid == parentReportItem.Kvid).ToList();
                        //var nodes = reportItemUpdatePool.Where(o => o.ParentKvid == parentReportItem.Kvid).ToList();
                        //foreach (var all in allNodes)
                        //{
                        //    if (nodes.Exists(o => o.Kvid == all.Kvid)) continue;
                        //    nodes.Add(all);
                        //}
                        if (parentReportItem.CalculateJudgement2(report, allNodes)) parentReportItem.AddOnlyProperties(o => o.Judgement);
                        reportItemUpdatePool.Add(parentReportItem);
                    }
                    trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
                    try
                    {
                        foreach (var reportItem in reportItemUpdatePool)
                        {
                            conn.UpdateOnly(reportItem);
                            reportItem.RemoveAllOnlyProperties();//这句为了返回完整ReportItem数据,否则只会返回更新的字段数据
                            rtns.Results.Add(reportItem);
                        }
                        trans?.Commit();

                    }
                    catch (Exception ex)
                    {
                        trans?.Rollback();
                        throw ex;
                    }

                }
            }
            return rtns;
        }
    }

    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "删除")]
    public class ReportItemDelete : RestfulDelete<ReportItem>
    {
        private IDbTransaction trans;
        public override bool OnPreRestfulDelete(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulDeleteResponse<ReportItem> rtns)
        {
            var queryReportItem = dbConnection.From<ReportItem>();
            queryReportItem.Where(o => Sql.In(o.Kvid, Kvids));
            var reportItems = dbConnection.Select(queryReportItem);
            if (reportItems.IsNullOrEmpty()) throw new Exception("未找到待删除的项目");

            var report = dbConnection.SingleById<Report>(reportItems.FirstOrDefault().ReportKvid);
            report.ThrowIfNull("未找到报告");

            var queryAllReportItems = dbConnection.From<ReportItem>();
            queryAllReportItems.Where(o => o.ReportKvid == reportItems.FirstOrDefault().ReportKvid);
            var allReportItems = dbConnection.Select(queryAllReportItems);

            //2021-10-20若待删除的项目中存在色号或部位，且该色号或部位存在非待删除的项目，移除待删除项目中的色号或部位
            //var removeReportItemKvids = new List<Guid>();
            //var SelfOrPartReportItems = reportItems.Where(o => o.Type == ReportItemType.SamplePart || o.Type == ReportItemType.SampleSelf).ToList();
            //foreach (var SelfOrPartReportItem in SelfOrPartReportItems)
            //{
            //    var subDeleteItemCount = reportItems.Count(o => o.ParentKvid == SelfOrPartReportItem.Kvid && o.Type != ReportItemType.SamplePart);
            //    var subAllItemCount = allReportItems.Count(o => o.ParentKvid == SelfOrPartReportItem.Kvid && o.Type != ReportItemType.SamplePart);
            //    if (subDeleteItemCount < subAllItemCount) removeReportItemKvids.Add(SelfOrPartReportItem.Kvid);
            //    //如果待删除色号下子项的数量和所有项目中该色号下子项数量相等，有两种情况
            //    else if(subDeleteItemCount == subAllItemCount)
            //    {
            //        //1、色号下的所有项目包括部位都是待删除的
            //        //2、例：色号下有一个部位，部位下有两个项目，但是只选中了一个项目
            //        //只需要比较samplekvid为此项目的数量即可
            //        if (SelfOrPartReportItem.Type != ReportItemType.SampleSelf) continue;
            //        var childDeleteCount = reportItems.Count(o => o.SampleKvid == SelfOrPartReportItem.Kvid);
            //        var childAllCount = allReportItems.Count(o => o.SampleKvid == SelfOrPartReportItem.Kvid);
            //        if (childDeleteCount != childAllCount) removeReportItemKvids.Add(SelfOrPartReportItem.Kvid);
            //    }
            //}
            //验证数据结构和业务结构
            reportItems.ValidReportItemDelete_Stract(allReportItems);
            report.ValidReportItemDelete_Biz(reportItems);

            var updateReportItems = new List<ReportItem>();

            var deleteParentKvids = reportItems.Where(o => o.ParentKvid != Guid.Empty).Select(o => o.ParentKvid).Distinct().ToList();
            var queryParentReportItems = dbConnection.From<ReportItem>();
            queryParentReportItems.Where(o => Sql.In(o.Kvid, deleteParentKvids));
            var parentReportItems = dbConnection.Select(queryParentReportItems);

            //parentReportItems.RemoveAll(o=>Sql.In(o.Kvid, removeReportItemKvids));

            //判断若删除的项目的父项是检测项目录类型（TestFolder），更新此项的类型为Test
            foreach (var pReportItem in parentReportItems)
            {
                if (pReportItem.Type != ReportItemType.TestFolder) continue;
                //var count = dbConnection.Scalar<ReportItem, int>(o => Sql.Count(o.Kvid), o => o.ParentKvid == pReportItem.Kvid);
                //if (count != 1) continue;
                //若删除后，父项无子项，更新父项类型
                var subReportItems = allReportItems.Where(o => o.ParentKvid == pReportItem.Kvid).ToList();
                var subDeleteReportItems = allReportItems.Where(o => o.ParentKvid == pReportItem.Kvid && Sql.In(o.Kvid, Kvids)).ToList();
                if(subReportItems.Count == subDeleteReportItems.Count)
                {
                    pReportItem.Type = ReportItemType.Test;
                    pReportItem.AddOnlyProperties(o => o.Type);
                    updateReportItems.Add(pReportItem);
                }
            }

            //如果待删除的项目中包含子色号，需要同时删除子样品
            var selfReportItemSample = reportItems.Where(o => o.Type == ReportItemType.SampleSelf && o.SampleKvid != o.SampleRootKvid).ToList();
            var selfReportItemSampleKvids = selfReportItemSample.Select(o => o.SampleKvid).ToList();
            //定义待更新的样品Kvid集合
            var updateSelfReportItemSampleKvids = new List<Guid>();

            if (selfReportItemSample != null)
            {
                //获取待删除的项目的所有的样品下的所有检验项目（包含一个样品下的多个报告下的检验项目）
                var queryAllSampleReportItems = dbConnection.From<ReportItem>();
                queryAllSampleReportItems.Where(o => Sql.In(o.SampleKvid, selfReportItemSampleKvids));
                var allSampleReportItems = dbConnection.Select(queryAllSampleReportItems);
                foreach (var selfSample in selfReportItemSample)
                {
                    //获取当前待删除的色号下的项目（包含一个样品下的多个报告下的检验项目）
                    var currentSampleReportItems = allSampleReportItems.Where(o => o.SampleKvid == selfSample.Kvid);
                    //获取是几份报告下的项目
                    var countReport = currentSampleReportItems.Select(o => o.ReportKvid).Distinct().ToList().Count;
                    if (countReport == 1) updateSelfReportItemSampleKvids.Add(selfSample.Kvid);
                }
            }



            if (dbConnection is LinqConnection && (dbConnection as LinqConnection).DbTransaction == null)
            {
                trans = dbConnection.OpenTransaction(IsolationLevel.ReadUncommitted);
            }
            try
            {
                foreach (var en in updateReportItems)
                {
                    dbConnection.UpdateOnly(en);
                }
                if (updateSelfReportItemSampleKvids != null)
                {
                    //删除Sample
                    var updateSampleLamda = dbConnection.From<Sample>();
                    updateSampleLamda = updateSampleLamda.Update(o => o.Status);
                    updateSampleLamda = updateSampleLamda.Where(o => Sql.In(o.Kvid, updateSelfReportItemSampleKvids));
                    dbConnection.UpdateOnly<Sample>(new Sample { Status = -1 }, updateSampleLamda);
                }
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            return base.OnPreRestfulDelete(req, res, dbConnection, rtns);
        }
        public override bool OnPostRestfulDelete(IRequest req, IResponse res, IDbConnection dbConnection, IRestfulDeleteResponse<ReportItem> rtns)
        {
            trans?.Commit();
            return base.OnPostRestfulDelete(req, res, dbConnection, rtns);
        }

    }

    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "查询")]
    public class ReportItemQuery : RestfulQuery<ReportItem>
    {
    }
    #endregion

    #region 实测值置空
    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "实测值置空")]
    public class ReportItemTestValueReset : RestfulExecution<ReportItem>
    {
        public ReportItem Item { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据录入
            (Item == null || Item.Kvid == Guid.Empty).ThrowIfTrue("未传入更新对象,更新失败!");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var reportItem = conn.SingleById<ReportItem>(Item.Kvid);
            if (reportItem == null) throw new Exception("未传入更新对象,更新失败!");
            (reportItem.Type != ReportItemType.Test).ThrowIfTrue("更新对象非检测项,禁止更新!");
            (reportItem.StatusType == ReportItemStatusType.TestCollected).ThrowIfTrue("项目已归档，无法修改实测值！");
            if (reportItem.TestCreatorKvid == Guid.Empty) return null;
            //允许本组的人删除
            var queryTestRoles = conn.From<EntityDuty<Test>>();
            queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
            queryTestRoles.Select(o => o.RoleName);
            var testRoleNames = conn.Select(queryTestRoles).ConvertAll(o => o.RoleName);
            if (!testRoleNames.Contains(reportItem.WorkGroupName) && KiviiContext.CurrentMember.Kvid != reportItem.TestCreatorKvid) throw new Exception("存在非本组人员权限内的项目，无法删除数据！");
            //reportItem.Exists(o => !testRoleNames.Contains(o.WorkGroupName)).ThrowIfTrue("存在非本组人员权限内的项目，无法删除数据！");
            //只允许录入人和主检人清空实测值
            //if (KiviiContext.CurrentMember.Kvid != reportItem.TestCreatorKvid) throw new Exception("只允许录入人清空实测值！");
            //获取操作项的关联数据(根据RootKvid查询)
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == reportItem.ReportKvid && o.RootKvid == reportItem.RootKvid);
            var reportItems = conn.Select(queryReportItems);
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            //定义待更新的项目集合
            var updateReportItems = new List<ReportItem>();
            var updateTests = new List<Test>();

            //更新实测值及复核人信息
            reportItem.TestValue = string.Empty;
            reportItem.AddOnlyProperties(o => o.TestValue);
            reportItem.Judgement = string.Empty;
            reportItem.AddOnlyProperties(o => o.Judgement);
            reportItem.TestCreatorKvid = Guid.Empty;
            reportItem.AddOnlyProperties(o => o.TestCreatorKvid);
            reportItem.TestCreatorName = string.Empty;
            reportItem.AddOnlyProperties(o => o.TestCreatorName);
            reportItem.TestCreateTime = null;
            reportItem.AddOnlyProperties(o => o.TestCreateTime);
            reportItem.TestReviewerKvid = Guid.Empty;
            reportItem.AddOnlyProperties(o => o.TestReviewerKvid);
            reportItem.TestReviewerName = string.Empty;
            reportItem.AddOnlyProperties(o => o.TestReviewerName);
            reportItem.TestReviewTime = null;
            reportItem.AddOnlyProperties(o => o.TestReviewTime);
            reportItem.StatusType = ReportItemStatusType.BeforeReview;
            reportItem.AddOnlyProperties(o => o.Status);
            updateReportItems.Add(reportItem);
            //获取父项并更新
            var reportItemParent = reportItems.FirstOrDefault(o => o.Kvid == reportItem.ParentKvid && o.Type == ReportItemType.TestFolder);
            if (reportItemParent != null)
            {
                reportItemParent.Judgement = string.Empty;
                reportItemParent.AddOnlyProperties(o => o.Judgement);
                reportItemParent.TestReviewerKvid = Guid.Empty;
                reportItemParent.AddOnlyProperties(o => o.TestReviewerKvid);
                reportItemParent.TestReviewerName = string.Empty;
                reportItemParent.AddOnlyProperties(o => o.TestReviewerName);
                reportItemParent.TestReviewTime = null;
                reportItemParent.AddOnlyProperties(o => o.TestReviewTime);
                reportItemParent.StatusType = ReportItemStatusType.BeforeReview;
                reportItemParent.AddOnlyProperties(o => o.Status);
                updateReportItems.Add(reportItemParent);
            }
            //获取对应Test信息,并更新实测值
            if (reportItem.TestKvid != Guid.Empty)
            {
                var test = conn.SingleById<Test>(reportItem.TestKvid);
                test.ThrowIfNull("未找到对应检测数据!");
                test.Value = string.Empty;
                test.AddOnlyProperties(o => o.Value);
                updateTests.Add(test);
            }
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                foreach (var en in updateReportItems)
                {
                    conn.UpdateOnly(en);
                    en.RemoveAllOnlyProperties();
                    rtns.Results.Add(en);
                }
                foreach (var en in updateTests)
                {
                    conn.UpdateOnly(en);
                }
                trans?.Commit();

            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion


    #region 色号、部位增删改相关
    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "检测项生成色号")]
    public class ReportItemCreateSelf : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }

        public List<string> SelfValues { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("请传入检验项目!");
            SelfValues.RemoveAll(o => o.IsNullOrEmpty());
            SelfValues.ThrowIfNullOrEmpty("请传入要生成的色号信息!");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            //所选项目
            var selectedReportItems = conn.SelectByIds<ReportItem>(ReportItemKvids);
            selectedReportItems.ThrowIfNullOrEmpty("未找到所选检验项目!");
            var reportKvids = selectedReportItems.ConvertAll(o => o.ReportKvid).Distinct().ToList();
            (reportKvids.Count != 1).ThrowIfTrue("所选检验项不在同一报告下!");
            var report = conn.SingleById<Report>(reportKvids.FirstOrDefault());
            report.ThrowIfNull("未找到项目对应报告!");
            (report.Status != (int)ReportStatusType.CommissionAccept).ThrowIfTrue($"当前报告{report.ReportId},不在委托受理状态,无法生成色号!");
            (report.StatusType != ReportStatusType.CommissionAccept).ThrowIfTrue($"当前报告{report.ReportId},不在委托受理状态,无法生成色号!");

            var sample = conn.SingleById<Sample>(report.SampleKvid);
            sample.ThrowIfNull("未找到报告对应样品信息!");

            //整个报告的全部项目
            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == report.Kvid);
            //全项中的大项
            var allMainReportItems = allReportItems.filterReportItem();
            allMainReportItems.ThrowIfNullOrEmpty("此报告全部检验项目未找到检验大项,结构异常,请联系管理员!");

            //传入项的大项
            var selectedMainReportItems = selectedReportItems.Where(o => Sql.In(o.Kvid, allMainReportItems.ConvertAll(p => p.Kvid))).ToList();
            selectedMainReportItems.ThrowIfNullOrEmpty("请选择检验大项进行生成色号!");

            var rtns = new RestfulCreateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            var insertReportItems = new List<ReportItem>();
            var insertSamples = new List<Sample>();
            var updateSamples = new List<Sample>();
            var firstReportItemKvid = Guid.Empty;

            //原报告检验项目 不存在色号信息
            if (!allReportItems.Exists(o => o.Type == ReportItemType.SampleSelf))
            {
                //原报告检验项目 不存在色号信息时，生成色号依然要验证项目的状态
                allMainReportItems.Exists(o => o.StatusType != ReportItemStatusType.BeforeTest).ThrowIfTrue($"当前报告{report.ReportId},不在委托受理状态,无法生成色号!");
                //if (selectedMainReportItems.Exists(o => o.TestKvid != Guid.Empty)) throw new Exception("存在项目有检测记录，无法生成部位！");
                var firstSelf = SelfValues.FirstOrDefault();
                firstSelf.ThrowIfNullOrEmpty("第一位色号信息不能为空!");
                //将第一个色号信息移除 特殊处理
                SelfValues.Remove(firstSelf);

                #region 全项目生成第一个色号
                var firstReportItem = new ReportItem();
                firstReportItem.Kvid = Guid.NewGuid();
                firstReportItem.RootKvid = firstReportItem.Kvid;
                firstReportItem.ParentKvid = Guid.Empty;
                firstReportItem.ReportKvid = report.Kvid;
                firstReportItem.Type = ReportItemType.SampleSelf;
                firstReportItem.Title = firstSelf;
                firstReportItem.SampleKvid = sample.Kvid;
                firstReportItem.SampleRootKvid = sample.RootKvid;
                firstReportItem.SortIdEx = 0;
                firstReportItem.SortId = 0;
                firstReportItem.StatusType = ReportItemStatusType.BeforeTest;
                insertReportItems.Add(firstReportItem);
                firstReportItemKvid = firstReportItem.Kvid;
                rtns.Results.Add(firstReportItem);
                #endregion

                #region 更新对应Sample的规格字段
                sample.Specification = firstSelf;
                sample.AddOnlyProperties(o => o.Specification);
                updateSamples.Add(sample);
                #endregion
            }
            //若SelfValue不为空 则循环生成子样品
            var maxSort = 0;
            var sortList = allReportItems.Where(o => o.Type == ReportItemType.SampleSelf).ToList();
            if (sortList.Count != 0) maxSort = sortList.Select(o => o.SortId).Max();
            if (!SelfValues.IsNullOrEmpty())
            {
                foreach (var self in SelfValues)
                {
                    #region 创建子样品
                    var nodeSample = new Sample();
                    nodeSample.PopulateWith(sample);
                    nodeSample.Kvid = Guid.NewGuid();
                    nodeSample.RootKvid = sample.RootKvid;
                    nodeSample.ParentKvid = sample.Kvid;
                    nodeSample.SerialNumber = sample.SerialNumber + "-{?}";
                    nodeSample.Name = sample.Name;
                    nodeSample.Specification = self;
                    insertSamples.Add(nodeSample);

                    int index = SelfValues.IndexOf(self) + 1 + maxSort;

                    var selfReportItem = new ReportItem();
                    selfReportItem.Kvid = Guid.NewGuid();
                    selfReportItem.RootKvid = selfReportItem.Kvid;
                    selfReportItem.ParentKvid = Guid.Empty;
                    selfReportItem.ReportKvid = report.Kvid;
                    selfReportItem.Type = ReportItemType.SampleSelf;
                    selfReportItem.Title = self;
                    selfReportItem.SampleKvid = nodeSample.Kvid;
                    selfReportItem.SampleRootKvid = nodeSample.RootKvid;
                    selfReportItem.SortIdEx = index;
                    selfReportItem.SortId = index;
                    selfReportItem.StatusType = ReportItemStatusType.BeforeTest;
                    insertReportItems.Add(selfReportItem);
                    rtns.Results.Add(selfReportItem);
                    #endregion

                    #region 创建新的项目(若传入了部位信息则也一同创建)
                    //选中的主项有部位的
                    var preRemoveSelectedMainReportItemWithPartKvids = new List<Guid>();

                    //选中的部位项目
                    var parentItems = allReportItems.Where(o => Sql.In(o.Kvid, selectedMainReportItems.ConvertAll(p => p.ParentKvid)) && o.Type == ReportItemType.SamplePart).ToList();
                    //存在选中部位项目的大项 进行处理生成色号,同时会从数组中移除,若最后还剩余则要进行下一步处理
                    foreach (var parent in parentItems)
                    {
                        var nodeSelectedMainReportItems = selectedMainReportItems.Where(o => o.ParentKvid == parent.Kvid).ToList();
                        if (nodeSelectedMainReportItems.IsNullOrEmpty()) continue;

                        var newParentItem = new ReportItem();
                        newParentItem.PopulateWith(parent);
                        newParentItem.Kvid = Guid.NewGuid();
                        newParentItem.RootKvid = selfReportItem.RootKvid;
                        newParentItem.ParentKvid = selfReportItem.Kvid;
                        newParentItem.ReportKvid = report.Kvid;
                        newParentItem.SampleKvid = nodeSample.Kvid;
                        newParentItem.SampleRootKvid = nodeSample.RootKvid;
                        insertReportItems.Add(newParentItem);
                        rtns.Results.Add(newParentItem);

                        foreach (var nodeSelectedMain in nodeSelectedMainReportItems)
                        {
                            //selectedMainReportItems.Remove(nodeSelectedMain);
                            preRemoveSelectedMainReportItemWithPartKvids.Add(nodeSelectedMain.Kvid);

                            var newNodeSelectedMain = new ReportItem();
                            newNodeSelectedMain.PopulateWith(nodeSelectedMain);
                            newNodeSelectedMain.Kvid = Guid.NewGuid();
                            newNodeSelectedMain.RootKvid = newParentItem.RootKvid;
                            newNodeSelectedMain.ParentKvid = newParentItem.Kvid;
                            newNodeSelectedMain.ReportKvid = report.Kvid;
                            newNodeSelectedMain.SampleKvid = nodeSample.Kvid;
                            newNodeSelectedMain.SampleRootKvid = nodeSample.RootKvid;

                            //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                            newNodeSelectedMain.TestKvid = Guid.Empty;
                            newNodeSelectedMain.TestValue = string.Empty;
                            newNodeSelectedMain.TestOriginalFilePath = string.Empty;
                            newNodeSelectedMain.TestCreatorName = string.Empty;
                            newNodeSelectedMain.TestCreatorKvid = Guid.Empty;
                            newNodeSelectedMain.TestCreateTime = null;
                            newNodeSelectedMain.TestReviewerKvid = Guid.Empty;
                            newNodeSelectedMain.TestReviewerName = string.Empty;
                            newNodeSelectedMain.TestReviewTime = null;
                            newNodeSelectedMain.TestRootKvid = Guid.Empty;
                            newNodeSelectedMain.Status = 100;

                            insertReportItems.Add(newNodeSelectedMain);
                            rtns.Results.Add(newNodeSelectedMain);

                            var nodes = allReportItems.Where(o => o.ParentKvid == nodeSelectedMain.Kvid).ToList();
                            foreach (var node in nodes)
                            {
                                var newNode = new ReportItem();
                                newNode.PopulateWith(node);
                                newNode.Kvid = Guid.NewGuid();
                                newNode.RootKvid = newNodeSelectedMain.RootKvid;
                                newNode.ParentKvid = newNodeSelectedMain.Kvid;
                                newNode.ReportKvid = report.Kvid;
                                newNode.SampleKvid = nodeSample.Kvid;
                                newNode.SampleRootKvid = nodeSample.RootKvid;

                                //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                                newNode.TestKvid = Guid.Empty;
                                newNode.TestValue = string.Empty;
                                newNode.TestOriginalFilePath = string.Empty;
                                newNode.TestCreatorName = string.Empty;
                                newNode.TestCreatorKvid = Guid.Empty;
                                newNode.TestCreateTime = null;
                                newNode.TestReviewerKvid = Guid.Empty;
                                newNode.TestReviewerName = string.Empty;
                                newNode.TestReviewTime = null;
                                newNode.TestRootKvid = Guid.Empty;
                                newNode.Status = 100;

                                insertReportItems.Add(newNode);
                                rtns.Results.Add(newNode);
                            }
                        }
                    }

                    //要是选中的大项还有剩余说明 是没有部位信息的大项 需要进行直接生成色号
                    var selectedMainReportItemWithoutParts = selectedMainReportItems.Where(o => !Sql.In(o.Kvid, preRemoveSelectedMainReportItemWithPartKvids)).ToList();
                    if (!selectedMainReportItemWithoutParts.IsNullOrEmpty())
                    {
                        foreach (var nodeSelectedMain in selectedMainReportItemWithoutParts)
                        {
                            var newNodeSelectedMain = new ReportItem();
                            newNodeSelectedMain.PopulateWith(nodeSelectedMain);
                            newNodeSelectedMain.Kvid = Guid.NewGuid();
                            newNodeSelectedMain.RootKvid = selfReportItem.RootKvid;
                            newNodeSelectedMain.ParentKvid = selfReportItem.Kvid;
                            newNodeSelectedMain.ReportKvid = report.Kvid;
                            newNodeSelectedMain.SampleKvid = nodeSample.Kvid;
                            newNodeSelectedMain.SampleRootKvid = nodeSample.RootKvid;

                            //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                            newNodeSelectedMain.TestKvid = Guid.Empty;
                            newNodeSelectedMain.TestValue = string.Empty;
                            newNodeSelectedMain.TestOriginalFilePath = string.Empty;
                            newNodeSelectedMain.TestCreatorName = string.Empty;
                            newNodeSelectedMain.TestCreatorKvid = Guid.Empty;
                            newNodeSelectedMain.TestCreateTime = null;
                            newNodeSelectedMain.TestReviewerKvid = Guid.Empty;
                            newNodeSelectedMain.TestReviewerName = string.Empty;
                            newNodeSelectedMain.TestReviewTime = null;
                            newNodeSelectedMain.TestRootKvid = Guid.Empty;
                            newNodeSelectedMain.Status = 100;

                            insertReportItems.Add(newNodeSelectedMain);
                            rtns.Results.Add(newNodeSelectedMain);

                            var nodes = allReportItems.Where(o => o.ParentKvid == nodeSelectedMain.Kvid).ToList();
                            foreach (var node in nodes)
                            {
                                var newNode = new ReportItem();
                                newNode.PopulateWith(node);
                                newNode.Kvid = Guid.NewGuid();
                                newNode.RootKvid = newNodeSelectedMain.RootKvid;
                                newNode.ParentKvid = newNodeSelectedMain.Kvid;
                                newNode.ReportKvid = report.Kvid;
                                newNode.SampleKvid = nodeSample.Kvid;
                                newNode.SampleRootKvid = nodeSample.RootKvid;

                                //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                                newNode.TestKvid = Guid.Empty;
                                newNode.TestValue = string.Empty;
                                newNode.TestOriginalFilePath = string.Empty;
                                newNode.TestCreatorName = string.Empty;
                                newNode.TestCreatorKvid = Guid.Empty;
                                newNode.TestCreateTime = null;
                                newNode.TestReviewerKvid = Guid.Empty;
                                newNode.TestReviewerName = string.Empty;
                                newNode.TestReviewTime = null;
                                newNode.TestRootKvid = Guid.Empty;
                                newNode.Status = 100;

                                insertReportItems.Add(newNode);
                                rtns.Results.Add(newNode);
                            }
                        }
                    }
                    #endregion
                }
            }
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                if (firstReportItemKvid != Guid.Empty)
                {
                    //全项的RootKvid更新
                    var updateRootLamda = conn.From<ReportItem>();
                    updateRootLamda = updateRootLamda.Update(o => o.RootKvid);
                    updateRootLamda = updateRootLamda.Where(o => Sql.In(o.Kvid, allReportItems.ConvertAll(p => p.Kvid)));
                    conn.UpdateOnly(new ReportItem { RootKvid = firstReportItemKvid }, updateRootLamda);

                    //全项中ParentKvid为空的更新
                    var updateParentLamda = conn.From<ReportItem>();
                    updateParentLamda = updateParentLamda.Update(o => o.ParentKvid);
                    updateParentLamda = updateParentLamda.Where(o => Sql.In(o.Kvid, allReportItems.Where(i => i.ParentKvid == Guid.Empty).ToList().ConvertAll(p => p.Kvid)));
                    conn.UpdateOnly(new ReportItem { ParentKvid = firstReportItemKvid }, updateParentLamda);
                }
                foreach (var en in insertReportItems)
                {
                    conn.Insert(en);
                }
                foreach (var en in insertSamples)
                {
                    conn.Insert(en);
                }
                foreach (var en in updateSamples)
                {
                    conn.Update(en);
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "检测项生成部位")]
    public class ReportItemCreatePart : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }

        public List<string> PartValues { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("请传入检验项目!");
            PartValues.RemoveAll(o => o.IsNullOrEmpty());
            PartValues.ThrowIfNullOrEmpty("请传入要生成的部位信息!");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            //所选项目 SelectByIds此方法不过滤status<0的数据
            var selectedReportItems = conn.SelectByIds<ReportItem>(ReportItemKvids);
            selectedReportItems.ThrowIfNullOrEmpty("未找到所选检验项目!");

            var reportKvids = selectedReportItems.ConvertAll(o => o.ReportKvid).Distinct().ToList();
            (reportKvids.Count != 1).ThrowIfTrue("所选检验项不在同一报告下!");
            var report = conn.SingleById<Report>(reportKvids.FirstOrDefault());
            report.ThrowIfNull("未找到项目对应报告!");
            (report.StatusType != ReportStatusType.CommissionAccept).ThrowIfTrue($"当前报告{report.ReportId},不在委托受理状态,无法生成部位!");

            //整个报告的全部项目
            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == report.Kvid);

            //全项中的大项
            var allMainReportItems = allReportItems.filterReportItem();
            allMainReportItems.ThrowIfNullOrEmpty("此报告全部检验项目未找到检验大项,结构异常,请联系管理员!");

            //传入项的大项
            var selectedMainReportItems = selectedReportItems.Where(o => Sql.In(o.Kvid, allMainReportItems.ConvertAll(p => p.Kvid))).ToList();
            selectedMainReportItems.ThrowIfNullOrEmpty("请选择检验大项进行生成部位!");

            //查看是否已经存在色号
            bool existSelf = allReportItems.Exists(o => o.Type == ReportItemType.SampleSelf);

            var rtns = new RestfulCreateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            var insertReportItems = new List<ReportItem>();
            var updateReportItems = new List<ReportItem>();

            //存在色号 则生成的部位全部需要先查到对应色号信息,要是没有找到则说明数据问题 直接跳过
            if (existSelf)
            {
                var groupRoot = selectedMainReportItems.GroupBy(o => o.RootKvid);

                foreach (var kv in groupRoot)
                {
                    foreach (var part in PartValues)
                    {
                        var selfParent = allReportItems.FirstOrDefault(o => o.Kvid == kv.Key);
                        if (selfParent == null) continue;

                        int index = PartValues.IndexOf(part);

                        var partReportItem = new ReportItem();
                        partReportItem.Kvid = Guid.NewGuid();
                        partReportItem.RootKvid = selfParent.RootKvid;
                        partReportItem.ParentKvid = selfParent.Kvid;
                        partReportItem.ReportKvid = report.Kvid;
                        partReportItem.Type = ReportItemType.SamplePart;
                        partReportItem.Title = part;
                        partReportItem.SampleKvid = selfParent.SampleKvid;
                        partReportItem.SampleRootKvid = selfParent.SampleRootKvid;
                        partReportItem.SortIdEx = selectedMainReportItems[0].SortIdEx + index;
                        partReportItem.SortId = selectedMainReportItems[0].SortId;
                        partReportItem.StatusType = ReportItemStatusType.BeforeTest;
                        insertReportItems.Add(partReportItem);
                        rtns.Results.Add(partReportItem);

                        foreach (var primary in kv)
                        {
                            var partParent = allReportItems.FirstOrDefault(o => o.Kvid == primary.ParentKvid && o.Type == ReportItemType.SamplePart);

                            var newNodePrimary = new ReportItem();
                            newNodePrimary.PopulateWith(primary);
                            newNodePrimary.Kvid = Guid.NewGuid();
                            newNodePrimary.RootKvid = partReportItem.RootKvid;
                            newNodePrimary.ParentKvid = partReportItem.Kvid;
                            newNodePrimary.ReportKvid = report.Kvid;
                            newNodePrimary.SampleKvid = partReportItem.SampleKvid;
                            newNodePrimary.SampleRootKvid = partReportItem.SampleRootKvid;
                            newNodePrimary.SortIdEx += index;
                            newNodePrimary.Status = 100;

                            //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                            //2024-03-11 接口升级：生成部位时，默认第一个部位为原项目，不移除检验项目中的检测信息
                            if(index != 0)
                            {
                                newNodePrimary.TestKvid = Guid.Empty;
                                newNodePrimary.TestValue = string.Empty;
                                newNodePrimary.TestOriginalFilePath = string.Empty;
                                newNodePrimary.TestCreatorName = string.Empty;
                                newNodePrimary.TestCreatorKvid = Guid.Empty;
                                newNodePrimary.TestCreateTime = null;
                                newNodePrimary.TestReviewerKvid = Guid.Empty;
                                newNodePrimary.TestReviewerName = string.Empty;
                                newNodePrimary.TestReviewTime = null;
                                newNodePrimary.TestRootKvid = Guid.Empty;
                            }

                            insertReportItems.Add(newNodePrimary);
                            rtns.Results.Add(newNodePrimary);

                            //没有找到父项 则说明原项目没有部位  要在生成部位后删除原项目
                            if (partParent == null)
                            {
                                selectedReportItems.Exists(o => o.Status != (int)ReportItemStatusType.BeforeTest && o.Status > -1).ThrowIfTrue($"当前操作的项目,不在委托受理状态,无法生成部位!");
                                //if (selectedMainReportItems.Exists(o => o.TestKvid != Guid.Empty)) throw new Exception("存在项目有检测记录，无法生成部位！");
                                primary.Status = -1;
                                primary.AddOnlyProperties(o => o.Status);
                                updateReportItems.Add(primary);
                            }

                            var nodes = allReportItems.Where(o => o.ParentKvid == primary.Kvid).ToList();
                            foreach (var node in nodes)
                            {
                                var newNode = new ReportItem();
                                newNode.PopulateWith(node);
                                newNode.Kvid = Guid.NewGuid();
                                newNode.RootKvid = newNodePrimary.RootKvid;
                                newNode.ParentKvid = newNodePrimary.Kvid;
                                newNode.ReportKvid = report.Kvid;
                                newNode.SampleKvid = newNodePrimary.SampleKvid;
                                newNode.SampleRootKvid = newNodePrimary.SampleRootKvid;
                                newNode.SortIdEx += index;
                                newNode.Status = 100;

                                //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                                //2024-03-11 接口升级：生成部位时，默认第一个部位为原项目，不移除检验项目中的检测信息
                                if (index != 0)
                                {
                                    newNode.TestKvid = Guid.Empty;
                                    newNode.TestValue = string.Empty;
                                    newNode.TestOriginalFilePath = string.Empty;
                                    newNode.TestCreatorName = string.Empty;
                                    newNode.TestCreatorKvid = Guid.Empty;
                                    newNode.TestCreateTime = null;
                                    newNode.TestReviewerKvid = Guid.Empty;
                                    newNode.TestReviewerName = string.Empty;
                                    newNode.TestReviewTime = null;
                                    newNode.TestRootKvid = Guid.Empty;
                                }                                    

                                insertReportItems.Add(newNode);
                                rtns.Results.Add(newNode);

                                //没有找到父项 则说明原项目没有部位  要在生成部位后删除原项目
                                if (partParent == null)
                                {
                                    selectedReportItems.Exists(o => o.Status != (int)ReportItemStatusType.BeforeTest && o.Status > -1).ThrowIfTrue($"当前操作的项目,不在委托受理状态,无法生成部位!");
                                    node.Status = -1;
                                    node.AddOnlyProperties(o => o.Status);
                                    updateReportItems.Add(node);
                                }
                            }
                        }
                    }
                }
            }
            //没有存在色号 则生成的部位就是当前项目的根项目
            else
            {
                foreach (var part in PartValues)
                {
                    int index = PartValues.IndexOf(part);
                    var partReportItem = new ReportItem();
                    partReportItem.Kvid = Guid.NewGuid();
                    partReportItem.RootKvid = partReportItem.Kvid;
                    partReportItem.ParentKvid = Guid.Empty;
                    partReportItem.ReportKvid = report.Kvid;
                    partReportItem.Type = ReportItemType.SamplePart;
                    partReportItem.Title = part;
                    partReportItem.SampleKvid = selectedMainReportItems[0].SampleKvid;
                    partReportItem.SampleRootKvid = selectedMainReportItems[0].SampleRootKvid;
                    partReportItem.SortIdEx = selectedMainReportItems[0].SortIdEx + index;
                    partReportItem.SortId = selectedMainReportItems[0].SortId;
                    partReportItem.StatusType = ReportItemStatusType.BeforeTest;
                    insertReportItems.Add(partReportItem);
                    rtns.Results.Add(partReportItem);

                    foreach (var primary in selectedMainReportItems)
                    {
                        var partParent = allReportItems.FirstOrDefault(o => o.Kvid == primary.ParentKvid && o.Type == ReportItemType.SamplePart);

                        #region 新建部位ReportItem及检验项目
                        var newNodePrimary = new ReportItem();
                        newNodePrimary.PopulateWith(primary);
                        newNodePrimary.Kvid = Guid.NewGuid();
                        newNodePrimary.RootKvid = partReportItem.RootKvid;
                        newNodePrimary.ParentKvid = partReportItem.Kvid;
                        newNodePrimary.ReportKvid = report.Kvid;
                        newNodePrimary.SampleKvid = partReportItem.SampleKvid;
                        newNodePrimary.SampleRootKvid = partReportItem.SampleRootKvid;
                        newNodePrimary.SortIdEx += index;
                        newNodePrimary.Status = 100;

                        //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                        //2024-03-11 接口升级：生成部位时，默认第一个部位为原项目，不移除检验项目中的检测信息
                        if (index != 0)
                        {
                            newNodePrimary.TestKvid = Guid.Empty;
                            newNodePrimary.TestValue = string.Empty;
                            newNodePrimary.TestOriginalFilePath = string.Empty;
                            newNodePrimary.TestCreatorName = string.Empty;
                            newNodePrimary.TestCreatorKvid = Guid.Empty;
                            newNodePrimary.TestCreateTime = null;
                            newNodePrimary.TestReviewerKvid = Guid.Empty;
                            newNodePrimary.TestReviewerName = string.Empty;
                            newNodePrimary.TestReviewTime = null;
                            newNodePrimary.TestRootKvid = Guid.Empty;
                        }
                        insertReportItems.Add(newNodePrimary);
                        rtns.Results.Add(newNodePrimary);

                        //没有找到父项 则说明原项目没有部位  要在生成部位后删除原项目
                        if (partParent == null)
                        {
                            //生成多部位时，只需要验证第一次的status为受理状态
                            selectedReportItems.Exists(o => o.Status != (int)ReportItemStatusType.BeforeTest && o.Status > -1).ThrowIfTrue($"当前操作的项目,不在委托受理状态,无法生成部位!");
                            //if (selectedMainReportItems.Exists(o => o.TestKvid != Guid.Empty)) throw new Exception("存在项目有检测记录，无法生成部位！");
                            primary.Status = -1;
                            primary.AddOnlyProperties(o => o.Status);
                            updateReportItems.Add(primary);
                        }

                        var nodes = allReportItems.Where(o => o.ParentKvid == primary.Kvid).ToList();
                        foreach (var node in nodes)
                        {
                            var newNode = new ReportItem();
                            newNode.PopulateWith(node);
                            newNode.Kvid = Guid.NewGuid();
                            newNode.RootKvid = newNodePrimary.RootKvid;
                            newNode.ParentKvid = newNodePrimary.Kvid;
                            newNode.ReportKvid = report.Kvid;
                            newNode.SampleKvid = newNodePrimary.SampleKvid;
                            newNode.SampleRootKvid = newNodePrimary.SampleRootKvid;
                            newNode.SortIdEx += index;
                            newNode.Status = 100;

                            //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                            if (index != 0)
                            {
                                newNodePrimary.TestKvid = Guid.Empty;
                                newNodePrimary.TestValue = string.Empty;
                                newNodePrimary.TestOriginalFilePath = string.Empty;
                                newNodePrimary.TestCreatorName = string.Empty;
                                newNodePrimary.TestCreatorKvid = Guid.Empty;
                                newNodePrimary.TestCreateTime = null;
                                newNodePrimary.TestReviewerKvid = Guid.Empty;
                                newNodePrimary.TestReviewerName = string.Empty;
                                newNodePrimary.TestReviewTime = null;
                                newNodePrimary.TestRootKvid = Guid.Empty;
                            }
                            insertReportItems.Add(newNode);
                            rtns.Results.Add(newNode);

                            //没有找到父项 则说明原项目没有部位  要在生成部位后删除原项目
                            if (partParent == null)
                            {
                                selectedReportItems.Exists(o => o.Status != (int)ReportItemStatusType.BeforeTest && o.Status > -1).ThrowIfTrue($"当前操作的项目,不在委托受理状态,无法生成部位!");
                                node.Status = -1;
                                node.AddOnlyProperties(o => o.Status);
                                updateReportItems.Add(node);
                            }
                        }
                        #endregion
                    }
                }
            }
            //开启事务，执行sql
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                foreach (var en in insertReportItems)
                {
                    conn.Insert(en);
                }
                foreach (var en in updateReportItems)
                {
                    conn.UpdateOnly(en);
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

     [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "检测项更新色号")]
    public class ReportItemUpdateSelf : RestfulExecution<ReportItem>
    {
        public Guid ReportItemKvid { get; set; }

        public string SelfValue { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvid.ThrowIfEmpty("请传入要更新的色号信息!");
            SelfValue.ThrowIfNullOrEmpty("色号信息不能为空!");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var reportItem = conn.SingleById<ReportItem>(ReportItemKvid);
            reportItem.ThrowIfNull("未找到所选色号信息!");
            (reportItem.Type != ReportItemType.SampleSelf).ThrowIfTrue("所选项不为色号项目,请选择正确色号项!");
            (reportItem.Status != (int)ReportItemStatusType.BeforeTest).ThrowIfTrue($"当前报告不在委托受理状态,无法更新色号!");
            (reportItem.StatusType != ReportItemStatusType.BeforeTest).ThrowIfTrue($"当前报告不在委托受理状态,无法更新色号!");

            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            if (reportItem.Title == SelfValue)
            {
                rtns.Results.Add(reportItem);
                return rtns;
            }

            var sample = conn.SingleById<Sample>(reportItem.SampleKvid);
            sample.ThrowIfNull("未找到对应样品信息!");

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);

            try
            {
                #region 更新ReportItem和Sample
                reportItem.Title = SelfValue;
                reportItem.AddOnlyProperties(o => o.Title);
                conn.UpdateOnly(reportItem);
                rtns.Results.Add(reportItem);

                sample.Specification = SelfValue;
                sample.AddOnlyProperties(o => o.Specification);
                conn.UpdateOnly(sample);
                #endregion
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    /// <summary>
    /// 与上一接口区别，只更新选中项的部位
    /// </summary>
    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "检测项更新部位")]
    public class ReportItemUpdatePart : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }

        public string PartValue { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("请传入检验项目!");
            PartValue.ThrowIfNullOrEmpty("部位信息不能为空!");

            //去重
            ReportItemKvids = ReportItemKvids.Distinct().ToList();

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            var selectedReportItems = conn.SelectByIds<ReportItem>(ReportItemKvids);
            selectedReportItems.ThrowIfNullOrEmpty("未找到所选部位项信息!");

            var reportKvids = selectedReportItems.ConvertAll(o => o.ReportKvid).Distinct().ToList();
            (reportKvids.Count != 1).ThrowIfTrue("所选检验项不在同一报告下!");
            var queryAllReportItems = conn.From<ReportItem>();
            queryAllReportItems.Where(o => o.ReportKvid == reportKvids.FirstOrDefault());
            var allReportItems = conn.Select(queryAllReportItems);

            selectedReportItems.Exists(o => o.StatusType != ReportItemStatusType.BeforeTest).ThrowIfTrue($"当前报告不在委托受理状态,无法更新部位!");

            var mainReportItems = selectedReportItems.filterReportItem();
            mainReportItems.Exists(o => o.ParentKvid == Guid.Empty).ThrowIfTrue("存在无部位的项目，无法修改部位！");
            var mainReportItemParentKvids = mainReportItems.Select(o => o.ParentKvid).Distinct().ToList();
            //验证选中项的父项是否都为部位
            var mainParentReportItems = conn.SelectByIds<ReportItem>(mainReportItemParentKvids);
            mainParentReportItems.Exists(o => o.Type != ReportItemType.SamplePart).ThrowIfTrue("存在选中项无部位的项目，无法修改部位！");

            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);

            try
            {
                foreach (var partReportItem in mainParentReportItems)
                {
                    //如果更新的是部位下的所有项目，直接更新部位即可
                    var countAllSubReportItems = allReportItems.Count(o => o.ParentKvid == partReportItem.Kvid);
                    var countSelectReportItems = mainReportItems.Count(o => o.ParentKvid == partReportItem.Kvid);
                    if (countAllSubReportItems == countSelectReportItems)
                    {
                        partReportItem.Title = PartValue;
                        partReportItem.AddOnlyProperties(o => o.Title);
                        conn.UpdateOnly(partReportItem);
                    }
                    else
                    {
                        var currentSelectedMainReportItems = mainReportItems.Where(o => o.ParentKvid == partReportItem.Kvid).ToList();
                        ////若当前报告的项目下已经有该部位了，直接挂到该部位下面
                        ////取消考虑此种情况，因为可能有多个色号。
                        //var queryCurrentPartReportItem = conn.From<ReportItem>();
                        //queryCurrentPartReportItem.Where(o => o.Type == ReportItemType.SamplePart && o.Title == PartValue);
                        //var alterPartReportItem = conn.Select(queryCurrentPartReportItem).FirstOrDefault();
                        //if(alterPartReportItem != null)
                        //{
                        //    var updateReportItemLamda = conn.From<ReportItem>();
                        //    updateReportItemLamda = updateReportItemLamda.Update(o => o.ParentKvid);
                        //    updateReportItemLamda = updateReportItemLamda.Where(o => Sql.In(o.Kvid, currentSelectedMainReportItems.ConvertAll(p => p.Kvid)));
                        //    conn.UpdateOnly<ReportItem>(new ReportItem { ParentKvid = alterPartReportItem.Kvid }, updateReportItemLamda);
                        //    continue;
                        //}
                        var newPartReportItem = new ReportItem();
                        newPartReportItem.PopulateWith(partReportItem);
                        newPartReportItem.Kvid = Guid.NewGuid();
                        newPartReportItem.Title = PartValue;

                        var updateReportItemLamda2 = conn.From<ReportItem>();
                        updateReportItemLamda2 = updateReportItemLamda2.Update(o => new { o.ParentKvid });
                        updateReportItemLamda2 = updateReportItemLamda2.Where(o => Sql.In(o.Kvid, currentSelectedMainReportItems.ConvertAll(p => p.Kvid)));
                        conn.UpdateOnly<ReportItem>(new ReportItem { ParentKvid = newPartReportItem.Kvid }, updateReportItemLamda2);

                        //若部位不是根节点
                        if (partReportItem.RootKvid != partReportItem.Kvid)
                        {
                            conn.Insert(newPartReportItem);
                        }
                        else
                        {
                            //若是根节点,需要同时处理所有子项
                            newPartReportItem.RootKvid = newPartReportItem.Kvid;
                            conn.Insert(newPartReportItem);
                            var kvids = selectedReportItems.Where(o => o.RootKvid == partReportItem.RootKvid).Select(o => o.Kvid).ToList();

                            var updateReportItemLamda3 = conn.From<ReportItem>();
                            updateReportItemLamda3 = updateReportItemLamda3.Update(o => new { o.RootKvid });
                            updateReportItemLamda3 = updateReportItemLamda3.Where(o => Sql.In(o.Kvid, kvids));
                            conn.UpdateOnly<ReportItem>(new ReportItem { RootKvid = newPartReportItem.RootKvid }, updateReportItemLamda3);

                            //用上面的代码优化写法
                            //foreach (var en in currentSelectedMainReportItems)
                            //{
                            //    //先更新选中项大项的root
                            //    var updateReportItemLamda3 = conn.From<ReportItem>();
                            //    updateReportItemLamda3 = updateReportItemLamda3.Update(o => new { o.RootKvid });
                            //    updateReportItemLamda3 = updateReportItemLamda3.Where(o => Sql.In(o.Kvid, currentSelectedMainReportItems.ConvertAll(p => p.Kvid)));
                            //    conn.UpdateOnly<ReportItem>(new ReportItem { RootKvid = newPartReportItem.RootKvid }, updateReportItemLamda3);

                            //    var subReportItems = selectedReportItems.Where(o => o.ParentKvid == en.Kvid).ToList();
                            //    if (subReportItems != null)
                            //    {
                            //        //更新子项的root
                            //        var updateReportItemLamda4 = conn.From<ReportItem>();
                            //        updateReportItemLamda4 = updateReportItemLamda4.Update(o => new { o.RootKvid });
                            //        updateReportItemLamda4 = updateReportItemLamda4.Where(o => Sql.In(o.Kvid, subReportItems.ConvertAll(p => p.Kvid)));
                            //        conn.UpdateOnly<ReportItem>(new ReportItem { RootKvid = newPartReportItem.RootKvid}, updateReportItemLamda4);
                            //    }
                            //}
                        }
                    }
                }

                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "检测项移除色号")]
    public class ReportItemDeleteSelf : RestfulExecution<ReportItem>
    {
        public Guid ReportItemKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvid.ThrowIfEmpty("请传入要移除的色号信息!");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var reportItem = conn.SingleById<ReportItem>(ReportItemKvid);
            reportItem.ThrowIfNull("未找到所选色号信息!");
            (reportItem.Type != ReportItemType.SampleSelf).ThrowIfTrue("所选项不为色号项目,请选择正确色号项!");

            //var countSampleSelf = conn.Count<ReportItem>(o => o.ReportKvid == reportItem.ReportKvid && o.Type == ReportItemType.SampleSelf);

            var sample = conn.SingleById<Sample>(reportItem.SampleKvid);
            sample.ThrowIfNull("未找到报告对应样品信息!");

            //整个报告的全部项目
            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == reportItem.ReportKvid);
            allReportItems.Exists(o => o.StatusType != ReportItemStatusType.BeforeTest).ThrowIfTrue("当前报告存在非委托受理状态的项目,无法移除色号!");

            //定义待更新的项目的集合，数据处理完毕后统一更新
            var updateReportItems = new List<ReportItem>();

            //删除色号项目
            reportItem.Status = -1;
            reportItem.AddOnlyProperties(o => o.Status);
            updateReportItems.Add(reportItem);

            var rtns = new RestfulDeleteResponse<ReportItem>();
            rtns.Results = new List<Guid>();

            //分类讨论，分两种情况，1、只有一个色号；2、存在多色号
            //1、先找主色号
            var mainSelfReportItems = allReportItems.Where(o => o.Type == ReportItemType.SampleSelf && o.SampleKvid == o.SampleRootKvid).ToList();
            if (mainSelfReportItems.Count() != 1) throw new Exception("数据异常，请联系管理员！");
            var mainSelfReportItem = mainSelfReportItems.FirstOrDefault();
            if (mainSelfReportItem == null || mainSelfReportItem.Type != ReportItemType.SampleSelf) throw new Exception("数据异常，请联系管理员！");

            //2、根据要删除的色号是否为主色号，分类讨论
            //2021-12-03只考虑删除主色号，其他情况，直接删除；
            var isDeleteMainSelf = mainSelfReportItem.Kvid == reportItem.Kvid;
            if (!isDeleteMainSelf) throw new Exception("该接口只能处理主色号；如需删除其他色号，请使用删除功能");
            var allSelfSubReportItems = new List<ReportItem>();
            if (isDeleteMainSelf)
            {
                //删除主色号时，报告只能有一个色号项目
                var selfReportItems = allReportItems.Where(o => o.Type == ReportItemType.SampleSelf).ToList();
                if (selfReportItems.Count() != 1) throw new Exception("当前报告项目存在多个色号，或找不到色号项目，无法删除主色号！");
                sample.Specification = string.Empty;
                sample.AddOnlyProperties(o => o.Specification);
            }
            else
            {
                //筛选出主项下的所有大项，以便过滤重复项目
                allSelfSubReportItems = allReportItems.Where(o => o.ParentKvid == mainSelfReportItem.Kvid).ToList();
                if (allSelfSubReportItems == null) throw new Exception("主色号下无项目，请联系管理员！");
                sample.Status = -1;
                sample.AddOnlyProperties(o => o.Status);
            }

            var parentItems = allReportItems.Where(o => o.ParentKvid == reportItem.Kvid).ToList();
            if (!parentItems.IsNullOrEmpty())
            {
                foreach (var parent in parentItems)
                {

                    if (isDeleteMainSelf)
                    {
                        parent.RootKvid = parent.Kvid;
                        parent.ParentKvid = Guid.Empty;
                    }
                    else
                    {
                        //过滤掉重复的项目，部位忽略--删除
                        if (parent.Type != ReportItemType.SamplePart && allSelfSubReportItems.Exists(o => o.DetectionKvid == parent.DetectionKvid))
                        {
                            parent.Status = -1;
                            parent.AddOnlyProperties(o => o.Status);
                        }

                        parent.RootKvid = mainSelfReportItem.Kvid;
                        parent.ParentKvid = mainSelfReportItem.Kvid;
                    }
                    parent.AddOnlyProperties(o => o.RootKvid);
                    parent.AddOnlyProperties(o => o.ParentKvid);
                    if (parent.SampleKvid != reportItem.SampleRootKvid)
                    {
                        parent.SampleKvid = reportItem.SampleRootKvid;
                        parent.AddOnlyProperties(o => o.SampleKvid);
                    }
                    updateReportItems.Add(parent);

                    var secdItems = allReportItems.Where(o => o.ParentKvid == parent.Kvid).ToList();
                    if (!secdItems.IsNullOrEmpty())
                    {
                        foreach (var secd in secdItems)
                        {
                            if (isDeleteMainSelf)
                            {
                                secd.RootKvid = parent.Kvid;
                            }
                            else
                            {
                                if (parent.Status == -1)
                                {
                                    secd.Status = -1;
                                    secd.AddOnlyProperties(o => o.Status);
                                }
                                secd.RootKvid = mainSelfReportItem.Kvid;
                            }
                            secd.AddOnlyProperties(o => o.RootKvid);
                            if (secd.SampleKvid != reportItem.SampleRootKvid)
                            {
                                secd.SampleKvid = reportItem.SampleRootKvid;
                                secd.AddOnlyProperties(o => o.SampleKvid);
                            }
                            updateReportItems.Add(secd);

                            var thirItems = allReportItems.Where(o => o.ParentKvid == secd.Kvid).ToList();
                            if (!thirItems.IsNullOrEmpty())
                            {
                                foreach (var third in thirItems)
                                {
                                    if (isDeleteMainSelf)
                                    {
                                        third.RootKvid = parent.Kvid;
                                    }
                                    else
                                    {
                                        if (secd.Status == -1)
                                        {
                                            third.Status = -1;
                                            third.AddOnlyProperties(o => o.Status);
                                            updateReportItems.Add(third);
                                            continue;
                                        }
                                        third.RootKvid = mainSelfReportItem.Kvid;
                                    }
                                    third.AddOnlyProperties(o => o.RootKvid);
                                    if (third.SampleKvid != reportItem.SampleRootKvid)
                                    {
                                        third.SampleKvid = reportItem.SampleRootKvid;
                                        third.AddOnlyProperties(o => o.SampleKvid);
                                    }
                                    updateReportItems.Add(third);
                                }
                            }
                        }
                    }
                }
            }

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                foreach (var en in updateReportItems)
                {
                    conn.UpdateOnly(en);
                }

                //更新样品信息
                conn.UpdateOnly(sample);

                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            return rtns;
        }
    }

    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "检测项移除项目的部位")]
    public class ReportItemDeletePart : RestfulExecution<ReportItem>
    {
        public List<ReportItem> ReportItems { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItems.ThrowIfNullOrEmpty("参数为空，无法处理！");
            var mainReportItems = ReportItems.filterReportItem();
            mainReportItems.Exists(o => o.StatusType != ReportItemStatusType.BeforeTest).ThrowIfTrue($"当前报告不在委托受理状态,无法移除部位!");
            if (mainReportItems.Exists(o => o.PartKvid == Guid.Empty || o.PartKvid == null)) throw new Exception("存在无部位的项目，无法移除部位");
            if (ReportItems.Exists(o => o.ReportKvid == Guid.Empty || o.ReportKvid == null)) throw new Exception("存在无归属报告的项目，无法移除部位");

            var reportKvids = ReportItems.Select(o => o.ReportKvid).Distinct().ToList();
            if (reportKvids.Count > 1) throw new Exception("存在多个报告的项目，无法删除部位");
            var reportKvid = reportKvids.FirstOrDefault();
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var queryAllReportItmes = conn.From<ReportItem>();
            queryAllReportItmes.Where(o => o.ReportKvid == reportKvid);
            var allReportItems = conn.Select(queryAllReportItmes);

            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            //定义待更新的项目集合
            var updateReportItems = new List<ReportItem>();
            var deleteReportItems = new List<Guid>();
            foreach (var reportItem in mainReportItems)
            {
                //验证传入的ReportItems中的PartKvid是否是其部位的Kvid
                var parentReportItem = allReportItems.Where(o => o.Kvid == reportItem.PartKvid).FirstOrDefault();
                if (parentReportItem == null || parentReportItem.Type != ReportItemType.SamplePart) throw new Exception("存在有不对应部位的项目，无法移除部位");

                #region 如果待删除的部位删除后，无子项了，连部位一起删除
                //验证传入的项目的当前部位的子项数量，和数据库中当前项目的部位的子项数量
                var countSelected = mainReportItems.Count(o => o.PartKvid == parentReportItem.Kvid);
                var countRemote = allReportItems.Count(o => o.ParentKvid == parentReportItem.Kvid);
                if (countSelected == countRemote) deleteReportItems.Add(parentReportItem.Kvid);
                #endregion

                var remoteReportItem = allReportItems.Where(o => o.Kvid == reportItem.Kvid).FirstOrDefault();
                if (remoteReportItem == null) throw new Exception("未找到对应的项目");
                //处理删除的部位为root的情况
                if(remoteReportItem.RootKvid == parentReportItem.Kvid)
                {
                    remoteReportItem.RootKvid = remoteReportItem.Kvid;
                    remoteReportItem.AddOnlyProperties(o => o.RootKvid);
                    //处理子项的RootKvid
                    var subReportItems = allReportItems.Where(o => o.ParentKvid == remoteReportItem.Kvid).ToList();
                    if (subReportItems != null)
                    {
                        foreach(var subItem in subReportItems)
                        {
                            subItem.RootKvid = remoteReportItem.RootKvid;
                            subItem.AddOnlyProperties(o => o.RootKvid);
                            updateReportItems.Add(subItem);
                        }
                    }
                }
                remoteReportItem.ParentKvid = parentReportItem.ParentKvid;
                remoteReportItem.AddOnlyProperties(o => o.ParentKvid);
                updateReportItems.Add(remoteReportItem);
            }

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                foreach (var en in updateReportItems)
                {
                    conn.UpdateOnly(en);
                }
                if (!deleteReportItems.IsNullOrEmpty())
                {
                    deleteReportItems = deleteReportItems.Distinct().ToList();
                    var updateReportItemLamda = conn.From<ReportItem>();
                    updateReportItemLamda = updateReportItemLamda.Update(o => o.Status);
                    updateReportItemLamda = updateReportItemLamda.Where(o => Sql.In(o.Kvid, deleteReportItems));
                    conn.UpdateOnly<ReportItem>(new ReportItem { Status = -1 }, updateReportItemLamda);
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            rtns.Results = updateReportItems;
            rtns.Total = rtns.Results.Count();
            return rtns;
        }
    }
    #endregion

    #region 项目置顶
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportItemTopping: RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("请传入检验项目!");
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            //所选项目
            var selectedReportItems = conn.SelectByIds<ReportItem>(ReportItemKvids);
            selectedReportItems.ThrowIfNullOrEmpty("未找到所选检验项目!");
            if (selectedReportItems.Exists(o => o.StatusType != ReportItemStatusType.BeforeTest)) throw new Exception("存在项目不在受理状态，无法置顶");
            var reportKvids = selectedReportItems.ConvertAll(o => o.ReportKvid).Distinct().ToList();
            (reportKvids.Count != 1).ThrowIfTrue("所选检验项不在同一报告下!");
            var report = conn.SingleById<Report>(reportKvids.FirstOrDefault());
            report.ThrowIfNull("未找到项目对应报告!");
            (report.Status != (int)ReportStatusType.CommissionAccept).ThrowIfTrue($"当前报告{report.ReportId},不在委托受理状态,无法置顶!");
            (report.StatusType != ReportStatusType.CommissionAccept).ThrowIfTrue($"当前报告{report.ReportId},不在委托受理状态,无法置顶!");

            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == report.Kvid);
            //验证是否为根目录，只允许根目录置顶
            var rootSelectItems = selectedReportItems.Where(o=>o.ParentKvid == Guid.Empty).OrderByDescending(o=>o.SortId).ThenByDescending(o => o.SortIdEx).ThenByDescending(o => o.Kvid).ToList();
            if (rootSelectItems.IsEmpty()) throw new Exception("未选中任何根节点，无法置顶");

            var minSortId = allReportItems.Where(o=>o.ParentKvid == Guid.Empty).OrderBy(o=>o.SortId).ThenBy(o => o.SortIdEx).ThenBy(o => o.Kvid).FirstOrDefault().SortId;

            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            //定义待更新的项目集合
            var updateReportItems = new List<ReportItem>();
            foreach(var root in rootSelectItems)
            {
                root.SortId = minSortId - 1;
                minSortId = root.SortId;
                root.AddOnlyProperties(o => o.SortId);
                updateReportItems.Add(root);
            }
            foreach (var en in updateReportItems)
            {
                conn.UpdateOnly(en);
            }
            rtns.Results = updateReportItems;
            return rtns;
        }
    }
    #endregion

    #region 检验项目自动判定
    /// <summary>
    /// 根据规则对比实测值和标准值，更新当前项目的判定结果
    /// </summary>
    //代码初审 任天宇 20210904
    [RequiresAnyRole(MemberRoles.Everyone)]
    public class ReportItemJudgement : RestfulExecution<ReportItem>
    {
        public Guid ReportItemKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvid.ThrowIfNull("未传入ReportItemKvid");

            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            var conn = KiviiContext.GetOpenedDbConnection<Test>();
            var reportItem = conn.SingleById<ReportItem>(ReportItemKvid);
            reportItem.ThrowIfNull("未找到对应的检测项目！");
            var report = conn.SingleById<Report>(reportItem.ReportKvid);
            report.ThrowIfNull("未找到对应报告，无法自动判定");
            if (reportItem.Type != ReportItemType.Test) throw new Exception("当前项目非检测项，无法自动判定");
            //if (reportItem.StatusType != ReportItemStatusType.Testing && reportItem.StatusType != ReportItemStatusType.BeforeReview && reportItem.StatusType != ReportItemStatusType.TestFinished) throw new Exception("当前项目非检验状态，无法判定");
            var valueQualified = reportItem.DetectionValueQualified;
            var testValue = reportItem.TestValue;

            var updateReportItems = new List<ReportItem>();

            //如果标准值或实测值为空，则不作判定，返回原数据！
            //if (valueQualified.IsNullOrEmpty() || testValue.IsNullOrEmpty())
            //{
            //    rtns.Results.Add(reportItem);
            //    rtns.Total = rtns.Results.Count;
            //    return rtns;
            //}

            //经过计算，若完成判定则更新判定结果
            if (reportItem.CalculateJudgement2(report))
            {
                reportItem.AddOnlyProperties(o => o.Judgement);
                updateReportItems.Add(reportItem);
                rtns.Results.Add(reportItem);
            }

            //判断是否需要更新大项
            var parentReportItem = conn.Single<ReportItem>(o => o.Kvid == reportItem.ParentKvid && o.Type == ReportItemType.TestFolder);
            if (parentReportItem != null)
            {
                //大项是TestFolder 判定的话 需要把大项下面的子项传给判定方法
                //allNodes是此大项的全部子项  nodes是待处理的子项 两者合并后传个方法进行大项判定
                var allNodes = conn.Select<ReportItem>(o => o.ParentKvid == reportItem.ParentKvid && o.Type == ReportItemType.Test);
                //从服务器获取的子项的判定结果还未更新，需要手动赋值
                foreach ( var node in allNodes )
                {
                    if (node.Kvid == reportItem.Kvid) node.Judgement = reportItem.Judgement;
                }
                //var nodes = updateReportItems.Where(o => o.ParentKvid == parentReportItem.Kvid).ToList();
                //foreach (var all in allNodes)
                //{
                //    if (nodes.Exists(o => o.Kvid == all.Kvid)) continue;
                //    nodes.Add(all);
                //}

                if (parentReportItem.CalculateJudgement2(report, allNodes))
                {
                    parentReportItem.StatusType = ReportItemStatusType.BeforeReview;
                    parentReportItem.AddOnlyProperties(o => o.Status);
                    parentReportItem.AddOnlyProperties(o => o.Judgement);
                    updateReportItems.Add(parentReportItem);
                    rtns.Results.Add(parentReportItem);
                }
            }

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                foreach (var en in updateReportItems)
                {
                    conn.UpdateOnly(en);
                    en.RemoveAllOnlyProperties();
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }

    //2021-08-24 code review replaced up
    //[RequiresAnyRole(SystemRoles.Everyone)]
    //public class ReportItemJudgement : RestfulExecution<ReportItem>
    //{
    //    public Guid ReportItemKvid { get; set; }
    //    public override object OnExecution(IRequest req, IResponse res)
    //    {
    //        ReportItemKvid.ThrowIfNull("未传入ReportItemKvid");

    //        var rtns = new RestfulQueryResponse<ReportItem>();
    //        rtns.Results = new List<ReportItem>();
    //        var conn = KiviiContext.GetOpenedDbConnection<Test>();
    //        var reportItem = conn.SingleById<ReportItem>(ReportItemKvid);
    //        reportItem.ThrowIfNull("未找到对应的检测项目！");
    //        if (reportItem.StatusType != ReportItemStatusType.Testing && reportItem.StatusType != ReportItemStatusType.BeforeReview && reportItem.StatusType != ReportItemStatusType.TestFinished) throw new Exception("当前项目非检验状态，无法判定");
    //        var valueQualified = reportItem.DetectionValueQualified;
    //        var testValue = reportItem.TestValue;

    //        //如果标准值或实测值为空，则不作判定，返回原数据！
    //        if (valueQualified.IsNullOrEmpty() || testValue.IsNullOrEmpty())
    //        {
    //            rtns.Results.Add(reportItem);
    //            rtns.Total = rtns.Results.Count;
    //            return rtns;
    //        }

    //        var trans = conn.OpenTransaction();
    //        try
    //        {
    //            //经过计算，若完成判定则更新判定结果
    //            if (reportItem.CalculateJudgement(conn))
    //            {
    //                reportItem.AddOnlyProperties(o => o.Judgement);
    //                conn.UpdateOnly(reportItem);
    //                reportItem.RemoveAllOnlyProperties();
    //                rtns.Results.Add(reportItem);
    //            }

    //            //判断是否需要更新大项
    //            var parentReportItem = conn.Single<ReportItem>(o => o.Kvid == reportItem.ParentKvid & o.Type == ReportItemType.TestFolder);
    //            if (parentReportItem != null)
    //            {
    //                if (parentReportItem.CalculateJudgement(conn))
    //                {
    //                    parentReportItem.StatusType = ReportItemStatusType.BeforeReview;
    //                    parentReportItem.AddOnlyProperties(o => o.Status);
    //                    parentReportItem.AddOnlyProperties(o => o.Judgement);
    //                    conn.UpdateOnly(parentReportItem);
    //                    parentReportItem.RemoveAllOnlyProperties();
    //                    rtns.Results.Add(parentReportItem);
    //                }
    //            }
    //            trans?.Commit();
    //        }
    //        catch (Exception ex)
    //        {
    //            trans?.Rollback();
    //            throw ex;
    //        }
    //        rtns.Total = rtns.Results.Count;
    //        return rtns;
    //    }
    //}
    #endregion

    #region 检验项目分组设置
    //代码初审 任天宇 20210905
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "项目分组")]
    public class ReportItemGroupSet : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }
        public Guid WorkGroupKvid { get; set; }
        public string WorkGroupName { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            ReportItemKvids.ThrowIfNull("未传入检测项目！");
            WorkGroupKvid.ThrowIfNull("未设置组Kvid！");
            WorkGroupName.ThrowIfNull("未设置组名称！");
            #endregion
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            var updateReportItems = new List<ReportItem>();

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => Sql.In(o.Kvid, ReportItemKvids) && (o.Type == ReportItemType.TestFolder || o.Type == ReportItemType.Test));
            var reportItems = conn.Select(queryReportItems);
            (reportItems.Exists(o => o.StatusType != ReportItemStatusType.Asigning)).ThrowIfTrue("当前状态不可修改检验组信息!");

            foreach (var reportItem in reportItems)
            {
                if (reportItem.Type == ReportItemType.TestFolder || reportItem.ParentKvid == Guid.Empty)
                {
                    reportItem.WorkGroupKvid = WorkGroupKvid;
                    reportItem.AddOnlyProperties(o => o.WorkGroupKvid);
                    reportItem.WorkGroupName = WorkGroupName;
                    reportItem.AddOnlyProperties(o => o.WorkGroupName);
                    updateReportItems.Add(reportItem);
                    rtns.Results.Add(reportItem);
                    continue;
                }
                var reportItemParent = conn.SingleById<ReportItem>(reportItem.ParentKvid);
                reportItemParent.ThrowIfNull("数据错误!未找到上级目录!");
                if (reportItemParent.Type == ReportItemType.TestFolder) continue;
                reportItem.WorkGroupKvid = WorkGroupKvid;
                reportItem.AddOnlyProperties(o => o.WorkGroupKvid);
                reportItem.WorkGroupName = WorkGroupName;
                reportItem.AddOnlyProperties(o => o.WorkGroupName);
                updateReportItems.Add(reportItem);
                rtns.Results.Add(reportItem);
            }
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                foreach (var en in updateReportItems)
                {
                    conn.UpdateOnly(en);
                    en.RemoveAllOnlyProperties();
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            rtns.Total = rtns.Results.Count;
            return rtns;

        }
    }
    #endregion

    #region 筛选(已分组,未分组,已判定,未判定)
    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "ReportItem筛选")]
    public class ReportItemQueryEx : RestfulExecution<ReportItem>
    {
        /// <summary>
        /// 要查看的项目报告Kvid,有部位信息则合并到大项中PartKvid和TitlePart
        /// </summary>
        public Guid ReportKvid { get; set; }

        /// <summary>
        /// 是否仅筛选传入的检验小组的检验项目,默认筛选全部项目
        /// </summary>
        public string WorkGroupName { get; set; }

        /// <summary>
        /// 是否是登录人所在组,默认false全部组
        /// </summary>
        public bool Owned { get; set; }

        /// <summary>
        /// 是否已经分组,默认不筛选
        /// </summary>
        public bool? Grouped { get; set; }

        /// <summary>
        /// 是否已经判定,默认不筛选
        /// </summary>
        public bool? Judged { get; set; }

        /// <summary>
        /// 仅显示主项,默认false,注:true时 上述分类无效
        /// 目前在报告结算时 明细清单使用到
        /// </summary>
        public bool OnlyMainItem { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            //ReportKvid.ThrowIfEmpty("请指定要查看项目的报告Kvid!");

            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            if (ReportKvid == Guid.Empty) return rtns;//为什么????

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            //整个报告的全部项目
            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == ReportKvid);
            if (allReportItems.IsNullOrEmpty()) return rtns;

            //全项中的大项
            var primaryAllReportItems = allReportItems.filterReportItem();

            List<ReportItem> finalResults = new List<ReportItem>();
            finalResults.AddRange(allReportItems);

            //只筛选主项出来
            if (OnlyMainItem)
            {
                finalResults = new List<ReportItem>();
                finalResults.AddRange(primaryAllReportItems);
            }
            else
            {
                //只查看登录人所在组
                if (Owned || !WorkGroupName.IsNullOrEmpty())
                {
                    finalResults = new List<ReportItem>();
                    //所在组的全部大项
                    List<ReportItem> ownerPrimaryItems = null;
                    if (!WorkGroupName.IsNullOrEmpty()) ownerPrimaryItems = primaryAllReportItems.Where(o => o.WorkGroupName == WorkGroupName).ToList();
                    else if (Owned)
                    {
                        //获取duty表中对应的角色
                        var queryTestRoles = conn.From<EntityDuty<Test>>();
                        queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
                        queryTestRoles.Select(o => o.RoleName);
                        var testRoleNames = conn.Select(queryTestRoles).ConvertAll(o => o.RoleName);
                        ownerPrimaryItems = primaryAllReportItems.Where(o => o.WorkGroupKvid == KiviiContext.CurrentMember.DepartmentKvid || testRoleNames.Contains(o.WorkGroupName)).ToList();
                    }

                    //没找到直接返回了 
                    if (ownerPrimaryItems.IsNullOrEmpty()) return rtns;
                    //ownerPrimaryItems.ForEach(o =>
                    //{
                    //if (!finalResults.Exists(p => p.Kvid == o.Kvid)) finalResults.Add(o);
                    //});
                    finalResults.AddRange(ownerPrimaryItems);

                    var ownerNodeItems = allReportItems.Where(o => ownerPrimaryItems.ConvertAll(p => p.Kvid).Contains(o.ParentKvid)).ToList();
                    if (!ownerNodeItems.IsNullOrEmpty()) finalResults.AddRange(ownerNodeItems);

                    var ownerPartItems = allReportItems.Where(o => o.Type == ReportItemType.SamplePart && ownerPrimaryItems.ConvertAll(p => p.ParentKvid).Contains(o.Kvid)).ToList();

                    if (!ownerPartItems.IsNullOrEmpty())
                    {
                        finalResults.AddRange(ownerPartItems);

                        var ownerRootItems = allReportItems.Where(o => o.Type == ReportItemType.SampleSelf && (ownerPartItems.ConvertAll(p => p.ParentKvid).Contains(o.Kvid) || ownerPrimaryItems.ConvertAll(p => p.ParentKvid).Contains(o.Kvid))).ToList();
                        if (!ownerRootItems.IsNullOrEmpty()) finalResults.AddRange(ownerRootItems);
                    }
                    else
                    {
                        var ownerRootItems = allReportItems.Where(o => o.Type == ReportItemType.SampleSelf && ownerPrimaryItems.ConvertAll(p => p.ParentKvid).Contains(o.Kvid)).ToList();
                        if (!ownerRootItems.IsNullOrEmpty()) finalResults.AddRange(ownerRootItems);
                    }
                }
                //筛选是否已分组
                if (Grouped != null)
                {
                    var primaryItems = finalResults.filterReportItem();
                    if (!primaryItems.IsNullOrEmpty())
                    {
                        //只查已分组
                        if (Grouped.Value)
                        {
                            //只留下未分组的项目
                            primaryItems.RemoveAll(o => !o.WorkGroupName.IsNullOrEmpty());
                        }
                        //只查未分组
                        else
                        {
                            //只留下已分组的项目
                            primaryItems.RemoveAll(o => o.WorkGroupName.IsNullOrEmpty());
                        }
                        //待移除的大项
                        var preRemoveKvids = primaryItems.ConvertAll(p => p.Kvid);
                        finalResults.RemoveAll(o => preRemoveKvids.Contains(o.Kvid));
                        //也移除下面的子项
                        finalResults.RemoveAll(o => preRemoveKvids.Contains(o.ParentKvid));
                    }
                }
                //筛选是否已判定
                if (Judged != null)
                {
                    var primaryItems = finalResults.filterReportItem();
                    if (!primaryItems.IsNullOrEmpty())
                    {
                        //只查已判定
                        if (Judged.Value)
                        {
                            //只留下未判定的项目
                            primaryItems.RemoveAll(o => !o.Judgement.IsNullOrEmpty());
                        }
                        //只查未判定
                        else
                        {
                            //只留下已判定的项目
                            primaryItems.RemoveAll(o => o.Judgement.IsNullOrEmpty());
                        }
                        //待移除的大项
                        var preRemoveKvids = primaryItems.ConvertAll(p => p.Kvid);
                        finalResults.RemoveAll(o => preRemoveKvids.Contains(o.Kvid));
                        //也移除下面的子项
                        finalResults.RemoveAll(o => preRemoveKvids.Contains(o.ParentKvid));
                    }
                }
            }

            #region 清理只有色号或部位的项
            //要是都没有检验项目 则清除当前Root下面的项目
            var group = finalResults.GroupBy(o => o.RootKvid);
            foreach (var kv in group)
            {
                if (!kv.ToList().Exists(o => o.Type == ReportItemType.Test || o.Type == ReportItemType.TestFolder)) finalResults.RemoveAll(o => o.RootKvid == kv.Key);
            }
            #endregion

            #region 合并部位信息
            rtns.Results.AddRange(finalResults);
            var sampleParts = rtns.Results.Where(o => o.Type == ReportItemType.SamplePart).ToList();
            if (sampleParts.IsNullOrEmpty())
            {
                //要是最后都没有检验项目返回了 则清除全部项目
                if (!rtns.Results.Exists(o => o.Type == ReportItemType.Test || o.Type == ReportItemType.TestFolder)) rtns.Results.Clear();
                //var newList2 = new List<ReportItem>();
                //foreach (var item in rtns.Results)
                //{
                //    if (newList2.Exists(o => o.Kvid == item.Kvid)) continue;
                //    newList2.Add(item);
                //}
                //rtns.Results = newList2;
                rtns.Total = rtns.Results.Count;

                return rtns;
            }

            foreach (var samplePart in sampleParts)
            {
                var tests = rtns.Results.Where(o => o.ParentKvid == samplePart.Kvid).ToList();
                foreach (var test in tests)
                {
                    test.TitlePart = samplePart.Title;
                    test.ParentKvid = samplePart.ParentKvid;
                    test.PartKvid = samplePart.Kvid;
                }
            }

            rtns.Results.RemoveAll(o => sampleParts.ConvertAll(p => p.Kvid).Contains(o.Kvid));
            #endregion

            //要是最后都没有检验项目返回了 则清除全部项目
            if (!rtns.Results.Exists(o => o.Type == ReportItemType.Test || o.Type == ReportItemType.TestFolder)) rtns.Results.Clear();

            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 检验项目打回
    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "项目打回")]
    public class ReportItemCallBack : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }
        public string Description { get; set; }
        public int FromStatus { get; set; }
        public int ToStatus { get; set; }
        public bool IsReportSync { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("参数ReportItems不能为空！");
            Description.ThrowIfNullOrEmpty("打回项目必须输入原因！");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => Sql.In(o.Kvid, ReportItemKvids));
            var rtnReportItems = conn.Select(queryReportItems);
            rtnReportItems.ThrowIfNull("未找到所需打回的项目！");
            var reportKvid = rtnReportItems.FirstOrDefault().ReportKvid;
            if (rtnReportItems.Exists(o => o.ReportKvid != reportKvid)) throw new Exception("只允许批量打回同一份报告的项目！");

            var report = conn.SingleById<Report>(reportKvid);
            report.ThrowIfNull("未找到当前项目所对应报告！");
            if (report.StatusType == ReportStatusType.ReportCollected) throw new Exception("报告已归档，无法打回项目");
            if (report.StatusType == ReportStatusType.ReportPrint) throw new Exception("报告处于打印状态，无法打回项目");
            //if (rtnReportItems.filterReportItem().Exists(o => o.Status < ToStatus)) throw new Exception("存在目的流程异常！");
            //项目打回时，若报告状态大于传入的FromStatus，则不允许打回
            //报告状态可以小于FromStatus值，因为在数据录入的时候，报告可能是小于数据录入的状态值的。
            if(report.Status>FromStatus) throw new Exception("当前报告状态禁止打回！");
            //查询所有的项目
            var queryAllReportItems = conn.From<ReportItem>();
            queryAllReportItems.Where(o => o.ReportKvid == reportKvid);
            var allReportItems = conn.Select(queryAllReportItems);


            //定义待更新的集合
            var updateReportItem = new List<ReportItem>();
            //筛选出打回项目中的大项
            var mainSelectReportItems = rtnReportItems.filterReportItem();
            mainSelectReportItems.ThrowIfNullOrEmpty("未选择大项，无法打回项目！");
            foreach (var en in mainSelectReportItems)
            {
                if (en.Status <= ToStatus) continue;
                updateReportItem.Add(en);
                var subReportItems = allReportItems.Where(o => o.ParentKvid == en.Kvid);
                updateReportItem.AddRange(subReportItems);
            }

            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                //遍历待更新集合，处理数据
                foreach (var en in updateReportItem)
                {
                    en.Status = ToStatus;
                    en.AddOnlyProperties(o => o.Status);
                    conn.UpdateOnly(en);
                    en.RemoveAllOnlyProperties();
                    rtns.Results.Add(en);
                }
                report.AddReportNote("项目打回", ReportExtension.AssignDescription(report.Status), ReportItemExtension.StatusDescription(ToStatus), Description, conn);
                //若时从实验室打回受理的，记录实验室打回记录
                if(FromStatus == (int)ReportStatusType.DataEntry&&ToStatus == (int)ReportStatusType.CommissionAccept)
                {
                    report.AddDataEntryBackNote("项目打回", KiviiContext.CurrentMember.DepartmentName, ReportExtension.AssignDescription(ToStatus), Description, conn);
                }
                //若打回的目的流程为实验室，记录打回的组别
                if (ToStatus == 300)
                {
                    //处理选择项目的所有组别，用于创建项目的流转记录；打回的目的流程体现组别，以逗号分隔，如“物理组、色牢度组”
                    var groupList = mainSelectReportItems.Select(o => o.WorkGroupName).ToList().Distinct();
                    var groups = string.Join("、", groupList);
                    report.AddReportItemNote("项目打回", ReportExtension.AssignDescription(report.Status), groups, Description, conn);
                }

                //项目打回，是否报告同步打回
                if (IsReportSync)
                {
                    //如果报告不在当前状态，不处理
                    if (report.Status == FromStatus)
                    {
                        //打回项目如果要打回报告，给打回标记
                        if (report.Metadata == null) report.Metadata = new Dictionary<string, string>();
                        report.Metadata["CallBackFrom"] = ((ReportStatusType)report.Status).ToString();
                        if (ToStatus == 100)
                        {
                            report.Metadata["CallBackTo"] = ((ReportStatusType)100).ToString();
                            report.Status = 100;
                            //若是从检验打回受理，则要记录打回的检验组
                            if(FromStatus == 400)
                            {
                                //若本身有打回组，则追加
                                if (report.Metadata.ContainsKey("report.Metadata"))
                                {
                                    report.Metadata["CallBackFromGroups"]+= KiviiContext.CurrentMember.DepartmentName;
                                }
                                else
                                {
                                    report.Metadata["CallBackFromGroups"] = KiviiContext.CurrentMember.DepartmentName;
                                }
                            }
                        }

                        if (ToStatus == 150)
                        {
                            report.Metadata["CallBackTo"] = ((ReportStatusType)300).ToString();
                            report.Status = 300;
                        }
                        if (ToStatus == 300)
                        {
                            report.Metadata["CallBackTo"] = ((ReportStatusType)400).ToString();
                            report.Status = 400;
                        }
                        //2023/09/19 升级当报告从检验完成后打回至前置节点的，均要加上记录，用于报告颜色显示区分
                        if(FromStatus > 400)
                        {
                            report.Metadata["IsCallbackAfterFinish"] = "true";
                        }
                        //若直接返回经过数据录入，且项目打回时，直接返回不生效
                        report.Metadata["IsDirectReturn"] = null;
                        report.Metadata["DirectReturnStatus"] = null;
                        report.AddOnlyProperties(o => o.Metadata);
                    }
                    //增加项目打回受理，报告强制同步打回受理，避免编制打回项目时，报告在数据录入场景
                    else if(ToStatus == 100)
                    {
                        report.Metadata["CallBackTo"] = ((ReportStatusType)100).ToString();
                        report.Status = 100;
                    }
                    report.AddOnlyProperties(o => o.Status);
                    conn.UpdateOnly(report);

                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            return rtns;
        }
    }
    #endregion

    #region 粘贴项目,已替换,待删除
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "拷贝粘贴项目")]
    public class ReportItemPaste1 : RestfulExecution<ReportItem>
    {
        public Guid FromReportKvid { get; set; }
        public Guid ToReportKvid { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            FromReportKvid.ThrowIfEmpty("参数错误，FromReportKvid不能为空");
            ToReportKvid.ThrowIfEmpty("参数错误，ToReportKvid不能为空");
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            var toReport = conn.SingleById<Report>(ToReportKvid);
            toReport.ThrowIfNull("未找到目的报告，无法粘贴！");
            if (toReport.Status != 100) throw new Exception("目的报告不在委托受理状态，无法粘贴项目！");
            var fromReport = conn.SingleById<Report>(FromReportKvid);
            fromReport.ThrowIfNull("未找到待拷贝的报告，无法粘贴！");

            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => o.ReportKvid == FromReportKvid);
            var reportItems = conn.Select(queryReportItems);
            reportItems.ThrowIfNullOrEmpty("当前拷贝的报告无检测项目，无法粘贴项目！");

            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            #region 以字典对象保存所有test的新的kvid
            var tempIndex = new Dictionary<Guid, Guid>();
            foreach (var reportItem in reportItems)
            {
                var kvid = Guid.NewGuid();
                tempIndex.Add(reportItem.Kvid, kvid);
            }
            #endregion
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            #region 以ParentKvid分组遍历，创建新的Test记录
            try
            {
                var groupTest = reportItems.GroupBy(o => o.ParentKvid);
                foreach (var group in groupTest)
                {
                    var dealTests = reportItems.Where(o => Sql.In(o.ParentKvid, group.Key)).ToList();
                    foreach (var test in dealTests)
                    {
                        var newReportItem = new ReportItem();
                        newReportItem.ReportKvid = toReport.Kvid;
                        newReportItem.SampleKvid = toReport.SampleKvid;
                        newReportItem.StandardKvid = test.StandardKvid;
                        newReportItem.StandardCode = test.StandardCode;
                        newReportItem.StandardTitle = test.StandardTitle;
                        newReportItem.DetectionKvid = test.DetectionKvid;
                        newReportItem.DetectionTitle = test.DetectionTitle;
                        newReportItem.DetectionCodeActual = test.DetectionCodeActual;
                        newReportItem.DetectionUnit = test.DetectionUnit;
                        newReportItem.DetectionValueQualified = test.DetectionValueQualified;
                        newReportItem.DetectionReferenceKvid = test.DetectionReferenceKvid;
                        newReportItem.Currency = test.Currency;
                        newReportItem.GoodsKvid = test.GoodsKvid;
                        newReportItem.PackageKvid = test.PackageKvid;
                        newReportItem.PackageUnit = test.PackageUnit;
                        newReportItem.AmountPlan = test.AmountPlan;
                        //设置理论收费和实际收费都为原价
                        newReportItem.Amount = test.AmountPlan;
                        newReportItem.Type = test.Type;
                        newReportItem.Title = test.Title;
                        newReportItem.SortIdEx = test.SortIdEx;
                        newReportItem.WorkGroupKvid = test.WorkGroupKvid;
                        newReportItem.WorkGroupName = test.WorkGroupName;
                        newReportItem.WorkerKvid = test.WorkerKvid;
                        newReportItem.WorkerName = test.WorkerName;
                        newReportItem.StatusType = ReportItemStatusType.BeforeTest;//BeforeTest状态
                        newReportItem.Kvid = tempIndex[test.Kvid];
                        if (group.Key != Guid.Empty)
                        {
                            newReportItem.ParentKvid = tempIndex[test.ParentKvid];
                        }
                        conn.Insert(newReportItem);
                        newReportItem.RemoveAllOnlyProperties();
                        rtns.Results.Add(newReportItem);
                    }
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            #endregion
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 检验项目召回
    //代码初审 任天宇 20210904
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "项目召回")]
    public class ReportItemRecyle : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }
        public string Description { get; set; }
        public ReportItemStatusType ToStatusType { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("参数ReportItems不能为空！");
            Description.ThrowIfNullOrEmpty("召回项目必须输入原因！");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => Sql.In(o.Kvid, ReportItemKvids));
            var rtnReportItems = conn.Select(queryReportItems);
            rtnReportItems.ThrowIfNull("未找到所需召的项目！");
            var reportKvid = rtnReportItems.FirstOrDefault().ReportKvid;
            var report = conn.SingleById<Report>(reportKvid);
            report.ThrowIfNull("未找到报告信息！");
            if (rtnReportItems.Exists(o => o.ReportKvid != reportKvid)) throw new Exception("只允许召回同一份报告的项目！");


            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                report.AddReportNote("项目召回", ReportExtension.AssignDescription(report.Status), ReportItemExtension.StatusDescription((int)ToStatusType), Description, conn);
                if (ToStatusType == ReportItemStatusType.BeforeTest)
                {
                    if (rtnReportItems.Exists(o => o.StatusType != ReportItemStatusType.Asigning && o.Type != ReportItemType.SampleSelf && o.Type != ReportItemType.SamplePart)) throw new Exception("所要召回的项目不在分派，无法召回");
                    var updateReportItems = conn.From<ReportItem>();
                    updateReportItems = updateReportItems.Update(o => o.Status);
                    updateReportItems = updateReportItems.Where(o => Sql.In(o.Kvid, rtnReportItems.ConvertAll(p => p.Kvid)));
                    conn.UpdateOnly(new ReportItem { Status = 100 }, updateReportItems);
                    rtnReportItems.ForEach(o => { o.Status = 100; });
                    rtns.Results.AddRange(rtnReportItems);
                    report.Status = 100;
                    report.AddOnlyProperties(o => o.Status);
                    conn.UpdateOnly(report);
                    //受理召回项目同时召回报告
                }
                else if (ToStatusType == ReportItemStatusType.Asigning)
                {
                    if (rtnReportItems.Exists(o => o.StatusType != ReportItemStatusType.Testing && o.Type != ReportItemType.SampleSelf && o.Type != ReportItemType.SamplePart)) throw new Exception("所要召回的项目，无法召回");
                    var updateReportItems = conn.From<ReportItem>();
                    updateReportItems = updateReportItems.Update(o => o.Status);
                    updateReportItems = updateReportItems.Where(o => Sql.In(o.Kvid, rtnReportItems.ConvertAll(p => p.Kvid)));
                    conn.UpdateOnly(new ReportItem { Status = 150 }, updateReportItems);
                    rtnReportItems.ForEach(o => { o.Status = 150; });
                    rtns.Results.AddRange(rtnReportItems);
                    report.Status = 300;
                    report.AddOnlyProperties(o => o.Status);
                    conn.UpdateOnly(report);
                }

                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
    #endregion

    [Api(Description = "批量修改原价")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportItemMulitAmount : RestfulExecution<ReportItem>
    {
        public Guid ReportKvid { get; set; }

        public List<Guid> ReportItemKvids { get; set; }

        public decimal Amount { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("need reportkvid");
            ReportItemKvids.ThrowIfNullOrEmpty("传入需要改价项目!");
            (Amount < 0).ThrowIfTrue("请填写要修改的价格!");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到此报告!");
            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == ReportKvid);
            var reportItems = allReportItems.Where(o => ReportItemKvids.Contains(o.Kvid)).ToList();

            reportItems.ThrowIfNullOrEmpty("未找要修改的项目!");
            (reportItems.Count != ReportItemKvids.Count).ThrowIfTrue("所选项目和查询后项目数量不一致!");

            var mains = allReportItems.filterReportItem();

            var selects = mains.Where(o => ReportItemKvids.Contains(o.Kvid)).ToList();

            decimal discountRate = 1;
            if (report.QuotationDiscountRate != null)
            {
                if (report.QuotationDiscountRate > 0 && report.QuotationDiscountRate <= 1) discountRate = report.QuotationDiscountRate.Value;
            }
            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            foreach (var item in selects)
            {
                item.SetAmountLog(report, "MultiUpdateAmountPlan", item.AmountPlan, Amount, conn);
                item.AmountPlan = Amount;
                item.AddOnlyProperties(o => o.AmountPlan);
                item.Amount = item.AmountPlan * discountRate;
                item.AddOnlyProperties(o => o.Amount);
                conn.UpdateOnly(item);
                item.RemoveAllOnlyProperties();
                rtns.Results.Add(item);

            }

            return rtns;
        }
    }

    [Api(Description = "批量手动判定")]
    [RequiresAnyRole(SystemRoles.Everyone)]
    public class ReportItemMulitJudgement : RestfulExecution<ReportItem>
    {
        public Guid ReportKvid { get; set; }

        public List<Guid> ReportItemKvids { get; set; }
        public string Judge { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("need reportkvid");
            ReportItemKvids.ThrowIfNullOrEmpty("传入需要改判定项目!");
            Judge.ThrowIfNull("请传入判定结果");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到此报告!");
            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == ReportKvid);
            var reportItems = allReportItems.Where(o => ReportItemKvids.Contains(o.Kvid)).ToList();

            reportItems.ThrowIfNullOrEmpty("未找要修改的项目!");
            (reportItems.Count != ReportItemKvids.Count).ThrowIfTrue("所选项目和查询后项目数量不一致!");

            //选中的测试项目
            var selectTests = reportItems.Where(o => o.Type == ReportItemType.Test).ToList();

            var parentKvids = selectTests.ConvertAll(o => o.ParentKvid).Distinct().ToList();
            parentKvids.RemoveAll(o => o == Guid.Empty);


            //选中的测试大项
            var selectFolders = reportItems.Where(o => o.Type == ReportItemType.TestFolder).ToList();
            //牵涉到大项
            var containFolders = allReportItems.Where(o => o.Type == ReportItemType.TestFolder && parentKvids.Contains(o.Kvid)).ToList();

            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            //检测项先更新判定
            foreach (var item in selectTests)
            {
                item.Judgement = Judge;
                item.AddOnlyProperties(o => o.Judgement);
                conn.UpdateOnly(item);
                item.RemoveAllOnlyProperties();
                rtns.Results.Add(item);
            }

            //合并选中和牵涉到的大项
            var folders = new List<ReportItem>();
            if (!selectFolders.IsNullOrEmpty()) folders.AddRange(selectFolders);
            foreach (var item in containFolders)
            {
                if (folders.Exists(o => o.Kvid == item.Kvid)) continue;
                folders.Add(item);
            }

            //大项通过判定方法进行自动判定
            foreach (var item in folders)
            {
                var allNodes = allReportItems.Where(o => o.ParentKvid == item.Kvid && o.Type == ReportItemType.Test).ToList();
                var nodes = reportItems.Where(o => o.ParentKvid == item.Kvid).ToList();
                foreach (var all in allNodes)
                {
                    if (nodes.Exists(o => o.Kvid == all.Kvid)) continue;
                    nodes.Add(all);
                }
                item.CalculateJudgement2(report, nodes);
                item.AddOnlyProperties(o => o.Judgement);
                conn.UpdateOnly(item);
                item.RemoveAllOnlyProperties();
                rtns.Results.Add(item);
            }

            return rtns;
        }
    }

    [Api(Description = "报告项目强制删除")]
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    public class ReportItemForceDelete : RestfulExecution<ReportItem>
    {
        public Guid ReportKvid { get; set; }
        public List<Guid> Kvids { get; set; }
        public string Reason { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportKvid.ThrowIfEmpty("请传入报告Kvid!");
            Kvids.ThrowIfNullOrEmpty("参数错误:未传入要删除的报告Kvids");

            var rtns = new RestfulDeleteResponse<Guid>();
            rtns.Results = new List<Guid>();

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var report = conn.SingleById<Report>(ReportKvid);
            report.ThrowIfNull("未找到此报告!");
            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == report.Kvid);
            if (allReportItems.IsNullOrEmpty()) return rtns;
            var reportItems = allReportItems.Where(o => Kvids.Contains(o.Kvid)).ToList();
            if (reportItems.Count != Kvids.Count) throw new Exception("请在同份报告下操作!");
            var existSelfs = reportItems.Where(o => o.Type == ReportItemType.SampleSelf).ToList();
            if (!existSelfs.IsNullOrEmpty())
            {
                var allNodes = allReportItems.Where(o => existSelfs.ConvertAll(p => p.RootKvid).Contains(o.RootKvid)).ToList();
                var selectNodes = reportItems.Where(o => existSelfs.ConvertAll(p => p.RootKvid).Contains(o.RootKvid)).ToList();
                if (allNodes.Count != selectNodes.Count) throw new Exception("选择大项后请选中其全部子项以删除!");
            }
            var existParts = reportItems.Where(o => o.Type == ReportItemType.SamplePart).ToList();
            if (!existParts.IsNullOrEmpty())
            {
                var allNodes = allReportItems.Where(o => existParts.ConvertAll(p => p.Kvid).Contains(o.ParentKvid)).ToList();
                var selectNodes = reportItems.Where(o => existParts.ConvertAll(p => p.Kvid).Contains(o.ParentKvid)).ToList();
                if (allNodes.Count != selectNodes.Count) throw new Exception("选择大项后请选中其全部子项以删除!");
            }
            var existFolders = reportItems.Where(o => o.Type == ReportItemType.TestFolder).ToList();
            if (!existFolders.IsNullOrEmpty())
            {
                var allNodes = allReportItems.Where(o => existFolders.ConvertAll(p => p.Kvid).Contains(o.ParentKvid)).ToList();
                var selectNodes = reportItems.Where(o => existFolders.ConvertAll(p => p.Kvid).Contains(o.ParentKvid)).ToList();
                if (allNodes.Count != selectNodes.Count) throw new Exception("选择大项后请选中其全部子项以删除!");
            }
            var queryTestKvids = conn.From<ReportItem>();
            queryTestKvids.Where(o => Sql.In(o.Kvid, reportItems.ConvertAll(p => p.Kvid)));
            queryTestKvids.Select(o => o.TestKvid);
            //test项目
            var testKvids = conn.Select<Guid>(queryTestKvids);

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                //删除Test
                var updateTestLamda = conn.From<Test>();
                updateTestLamda = updateTestLamda.Update(o => o.Status);
                updateTestLamda = updateTestLamda.Where(o => Sql.In(o.Kvid, testKvids));
                conn.UpdateOnly<Test>(new Test { Status = -1 }, updateTestLamda);

                // 删除ReportItems
                var updateReportItemLamda = conn.From<ReportItem>();
                updateReportItemLamda = updateReportItemLamda.Update(o => o.Status);
                updateReportItemLamda = updateReportItemLamda.Where(o => Sql.In(o.Kvid, reportItems.ConvertAll(p => p.Kvid)));
                conn.UpdateOnly<ReportItem>(new ReportItem { Status = -1 }, updateReportItemLamda);

                foreach (var item in reportItems)
                {
                    var log = new EntityLog<Report>();
                    log.OwnerKvid = item.Kvid;
                    log.BizId = item.Title;
                    log.BizKvid = item.Kvid;
                    log.BizType = typeof(ReportItem).FullName;
                    log.Type = "报告项目强制删除";
                    log.Title = DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss");
                    log.Summary = Reason;
                    log.Remark = $"[强制删除] 已成功将{report.ReportId}中的[{item.Title}]删除,操作人:{KiviiContext.CurrentMember.FullName},时间:{DateTime.Now.ToString("yyyy-MM-dd HH:mm:ss")}!";
                    conn.Insert(log);
                }
                rtns.Results.AddRange(reportItems.ConvertAll(o => o.Kvid));

                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }

    }

    #region 纤维含量费用核查
    public class ReportItemFiberContentQuery : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportKvids { get; set; }
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<ReportItemEx>();
            request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress
                .LeftJoin<Report>((reportItemEx, report) => reportItemEx.ReportKvid == report.Kvid)
                .Select<ReportItemEx, Report>((reportItemEx, report) => new
                {
                    T = Sql.AllFields(reportItemEx),
                    report.ReportId,
                    ReportDemanderName = report.DemanderName,
                    ReportSampleManufacturerName = report.SampleManufacturerName,
                    ReportTestDetections = report.TestDetections,
                    ReportAmountTest = report.AmountTest,
                    ReportAmountTestPlan = report.AmountTestPlan,
                    ReportAmount = report.Amount,
                    ReportStatus = report.Status,
                    //ReportAttachDocument = report.Metadata.ContainsKey("DescriptionSpecial") ? report.Metadata["DescriptionSpecial"] : "",
                });
            if (!ReportKvids.IsNullOrEmpty())
            {
                sqlExpress.And<Report>(o => Sql.In(o.Kvid, ReportKvids) && (o.Status == int.MaxValue||o.Status == (int)ReportStatusType.ReportPrint) && o.AmountSettlement == 0 && (o.BizType != "H" || o.BizType == null));
            }
            sqlExpress.And<ReportItemEx>(o => (o.Title.Contains("纤维含量") || o.Title.Contains("Fibre")) && o.Status == int.MaxValue);
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }
    #endregion

    #region 标记费用核查
    //public class ReportItemFiberContentCheckMark : RestfulExecution<ReportItem>
    //{
    //    public List<Guid> Kvids { get; set; }
    //    public override object OnExecution(IRequest req, IResponse res)
    //    {
    //        Kvids.ThrowIfNullOrEmpty();
    //        var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
    //        var queryReportItem = conn.From<ReportItem>();
    //        queryReportItem.Where(o => Sql.In(o.Kvid, Kvids));
    //        var reportItems = conn.Select(queryReportItem);
    //        if (reportItems == null) throw new Exception("无数据！");
    //        var rtns = new RestfulQueryResponse<ReportItem>();
    //        rtns.Results = new List<ReportItem>();
    //        foreach(var reportItem in reportItems)
    //        {
    //            if (reportItem.Metadata == null) reportItem.Metadata = new Dictionary<string, string>();
    //            if (reportItem.Metadata.ContainsKey("IsFiberContentCheck"))
    //            {
    //                if (reportItem.Metadata["IsFiberContentCheck"]=="True")
    //                {
    //                    reportItem.Metadata["IsFiberContentCheck"] = "False";
    //                }
    //                else
    //                {
    //                    reportItem.Metadata["IsFiberContentCheck"] = "True";
    //                    reportItem.Metadata["FiberContentCheckName"] = KiviiContext.CurrentMember.FullName;
    //                    reportItem.Metadata["FiberContentCheckTime"] = DateTime.Now.ToString();
    //                }
    //            }
    //            else
    //            {
    //                reportItem.Metadata.Add("IsFiberContentCheck", "True");
    //                reportItem.Metadata.Add("FiberContentCheckName", KiviiContext.CurrentMember.FullName);
    //                reportItem.Metadata.Add("FiberContentCheckTime", DateTime.Now.ToString());
    //            }
    //            reportItem.AddOnlyProperties(o => o.Metadata);
    //            conn.UpdateOnly(reportItem);
    //            rtns.Results.Add(reportItem);
    //        }
    //        return rtns;
    //    }
    //}
    #endregion

    #region 判定结果修改，支持批量修改
    public class ReportItemJudgementAlter : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }
        public string Judgement { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("未传入要判定的项目！");
            if (Judgement.IsNullOrEmpty()) Judgement = string.Empty;
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var queryReportItem = conn.From<ReportItem>();
            queryReportItem.Where(o => Sql.In(o.Kvid, ReportItemKvids));
            var reportItems = conn.Select(queryReportItem);
            reportItems.ThrowIfNullOrEmpty("未查到任何数据！");
            if (reportItems.Exists(o => o.StatusType == ReportItemStatusType.TestCollected)) throw new Exception("存在归档的项目，无法修改判定结果！");
            if (reportItems.Exists(o => o.Status < (int)ReportItemStatusType.Testing)) throw new Exception("存在项目不在检测状态，无法判定");
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            //只更新选中项的检测项的判断，且只能同时处理一个项目的判断
            var testReportItems = reportItems.Where(o => o.Type == ReportItemType.Test).ToList();
            var parentKvids = testReportItems.Select(o => o.ParentKvid).Distinct().ToList();

            if (parentKvids.Count > 1) throw new Exception("只能处理同一个项目的判定数据");
            if (parentKvids.FirstOrDefault() == Guid.Empty && reportItems.Count != 1) throw new Exception("只能处理同一个大项的判定数据！");
            //单独处理只更新大项判断的方法
            if (reportItems.Count == 1 && reportItems.FirstOrDefault().Type == ReportItemType.TestFolder)
            {
                var onlyItem = reportItems.FirstOrDefault();
                onlyItem.Judgement = Judgement;
                onlyItem.AddOnlyProperties(o => o.Judgement);
                conn.UpdateOnly(onlyItem);
                onlyItem.RemoveAllOnlyProperties();
                rtns.Results.Add(onlyItem);

            }
            //更新全部选中项的检测项为传入值
            var updateReportItemLamda = conn.From<ReportItem>();
            updateReportItemLamda = updateReportItemLamda.Update(o => o.Judgement);
            updateReportItemLamda = updateReportItemLamda.Where(o => Sql.In(o.Kvid, testReportItems.Select(p => p.Kvid)));
            conn.UpdateOnly<ReportItem>(new ReportItem { Judgement = Judgement }, updateReportItemLamda);

            foreach (var en in testReportItems)
            {
                en.Judgement = Judgement;
                rtns.Results.Add(en);
            }

            var parent = conn.SingleById<ReportItem>(parentKvids.FirstOrDefault());
            //只有当存在父项，且类型为检测项大项时，才进行判定
            if (parent != null && parent.Type == ReportItemType.TestFolder)
            {
                //找到此父项下的全部项目，进行判定
                var querySubItems = conn.From<ReportItem>();
                querySubItems.Where(o => o.ParentKvid == parent.Kvid);
                var subItems = conn.Select(querySubItems);
                subItems.ThrowIfNullOrEmpty("数据异常");
                var judgeValues = subItems.Select(o => o.Judgement).Distinct().ToList();
                //如果子项的判断都一样，直接赋值给大项
                if (judgeValues.Count == 1) parent.Judgement = judgeValues.FirstOrDefault();
                //20230426新增当子项只有“/”和“合格”，判定合格
                //20230909新增当子项只有“/”或“合格”或“—”，判定合格
                else if (judgeValues.Count == 2 && judgeValues.Contains("合格") && (judgeValues.Contains("/")|| judgeValues.Contains("—"))) parent.Judgement = "合格";
                else if (judgeValues.Count == 3 && judgeValues.Contains("合格") && judgeValues.Contains("/") && judgeValues.Contains("—")) parent.Judgement = "合格";
                else if (judgeValues.Contains("不合格")) parent.Judgement = "不合格";
                else parent.Judgement = "";
                parent.AddOnlyProperties(o => o.Judgement);
                conn.UpdateOnly(parent);
                parent.RemoveAllOnlyProperties();
                rtns.Results.Add(parent);
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 委托报告不合格项目导出
    public class ReportItemByJudgementQuery : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportKvids { get; set; }
        public string Judgement { get; set; }
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<ReportItemEx>();
            request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress
                .LeftJoin<Report>((reportItemEx, report) => reportItemEx.ReportKvid == report.Kvid)
                .Select<ReportItemEx, Report>((reportItemEx, report) => new
                {
                    T = Sql.AllFields(reportItemEx),
                    report.ReportId,
                    ReportSampleName = report.SampleName,
                    ReportDemanderName = report.DemanderName,
                    ReportSampleManufacturerName = report.SampleManufacturerName,
                    ReportSampleBatchNumber = report.SampleBatchNumber,
                    ReportTestStandards = report.TestStandards,
                    ReportRemark = report.Remark
                    //ReportAttachDocument = report.Metadata.ContainsKey("DescriptionSpecial") ? report.Metadata["DescriptionSpecial"] : "",
                });
            if (!ReportKvids.IsNullOrEmpty())
            {
                sqlExpress.And<Report>(o => Sql.In(o.Kvid, ReportKvids) && o.Status == int.MaxValue);
            }
            sqlExpress.And<ReportItemEx>(o => (o.Judgement == Judgement) && o.Status == int.MaxValue);
            //sqlExpress.OrderBy(o => o.ReportKvid);
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            return rtns;
        }
    }
    #endregion

    #region 工作量统计,用于实验室查看完成日期范围中总批次数和待处理批次数
    public class ReportItemPivotByGroupResponse : RestfulQueryResponse<ReportItemResult> { }
    public class ReportItemResult
    {
        public string WortGroupName { get; set; }
        public int ReportCount { get; set; }
        public int ReportCountDelay { get; set; }
        public string ReportIds { get; set; }
    }
    public class ReportItemPivotByGroup : RestfulExecution<ReportItem>
    {
        public DateTime ReportDeadDateFrom { get; set; }
        public DateTime ReportDeadDateTo { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportDeadDateFrom.ThrowIfNull("未传入完成日期从！");
            ReportDeadDateTo.ThrowIfNull("未传入完成日期至！");
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            return base.OnExecution(req, res);
        }
    }
    #endregion

    #region 项目加急与取消
    public class ReportItemUrgent : RestfulExecution<ReportItem>
    {
        public Guid ReportItemKvids { get; set; }
        public bool IsCancle { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNull("参数为空，接口异常");
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => Sql.In(o.Kvid, ReportItemKvids));
            var reportItems = conn.Select(queryReportItems);
            reportItems.ThrowIfNull("未查询到数据！");
            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            foreach(var en in reportItems)
            {
                if (en.Metadata == null) en.Metadata = new Dictionary<string, string>();
                en.Metadata["Urgent"] = IsCancle?"":"True";
                en.AddOnlyProperties(o => o.Metadata);
                conn.UpdateOnly(en);
                en.RemoveAllOnlyProperties();
                rtns.Results.Add(en);
            }
            return rtns;
        }
    }
    #endregion

    #region 数据录入手动清除实验数据
    [RequiresAnyRole(SystemRoles.Administrator, SystemRoles.Manager, SystemRoles.Configuration)]
    [Api(Description = "手动清除实验数据")]
    public class ReportItemClearTest : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("参数为空，接口异常");
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => Sql.In(o.Kvid, ReportItemKvids));
            var reportItems = conn.Select(queryReportItems);

            var validReportItems = new List<ReportItem>();
            validReportItems.AddRange(reportItems);
            //validReportItems.PopulateWith(reportItems);
            validReportItems.RemoveAll(o => o.TestKvid == Guid.Empty);
            validReportItems.ThrowIfNullOrEmpty("所选项目均无检测记录，无需清除数据！");
            validReportItems.Exists(o => o.StatusType == ReportItemStatusType.TestCollected).ThrowIfTrue("存在归档项目，无法移除实验数据");
            validReportItems.Exists(o => o.Status < 200).ThrowIfTrue("存在非实验状态的数据，无法移除实验数据");
            //允许本组的人删除
            var queryTestRoles = conn.From<EntityDuty<Test>>();
            queryTestRoles.Where(o => o.OwnerKvid == KiviiContext.CurrentMember.Kvid);
            queryTestRoles.Select(o => o.RoleName);
            var testRoleNames = conn.Select(queryTestRoles).ConvertAll(o => o.RoleName);
            (validReportItems.Exists(o => !testRoleNames.Contains(o.WorkGroupName)&& validReportItems.Exists(p => p.TestCreatorKvid != KiviiContext.CurrentMember.Kvid && o.TestCreatorKvid != Guid.Empty))).ThrowIfTrue("存在非本组人员权限内的项目，无法删除数据！");
            //validReportItems.Exists(o => o.TestCreatorKvid != KiviiContext.CurrentMember.Kvid&& o.TestCreatorKvid != Guid.Empty).ThrowIfTrue("只允许录入人清空实验数据！");
            var reportItemKvids = reportItems.Select(o => o.Kvid);
            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            var updateReportItemLamda = conn.From<ReportItem>();
            updateReportItemLamda = updateReportItemLamda.Update(o => new
            {
                o.TestKvid,
                o.TestValue,
                o.TestOriginalFilePath,
                o.TestCreatorName,
                o.TestCreatorKvid,
                o.TestCreateTime,
                o.TestReviewerKvid,
                o.TestReviewerName,
                o.TestReviewTime,
                o.TestRootKvid,
                o.Status,
                o.Judgement
            });
            updateReportItemLamda = updateReportItemLamda.Where(o => Sql.In(o.Kvid, reportItemKvids));
            conn.UpdateOnly<ReportItem>(new ReportItem
            {
                TestKvid = Guid.Empty,
                TestValue = string.Empty,
                TestOriginalFilePath = string.Empty,
                TestCreatorName = string.Empty,
                TestCreatorKvid = Guid.Empty,
                TestCreateTime = null,
                TestReviewerKvid = Guid.Empty,
                TestReviewerName = string.Empty,
                TestReviewTime = null,
                TestRootKvid = Guid.Empty,
                Judgement = string.Empty,
                Status = 200
            }, updateReportItemLamda);

            //手动组织返回值
            foreach(var reportItem in reportItems)
            {
                reportItem.TestKvid = Guid.Empty;
                reportItem.TestValue = string.Empty;
                reportItem.TestOriginalFilePath = string.Empty;
                reportItem.TestCreatorName = string.Empty;
                reportItem.TestCreatorKvid = Guid.Empty;
                reportItem.TestCreateTime = null;
                reportItem.TestReviewerKvid = Guid.Empty;
                reportItem.TestReviewerName = string.Empty;
                reportItem.TestReviewTime = null;
                reportItem.TestRootKvid = Guid.Empty;
                reportItem.Status = 200;
                reportItem.Judgement = string.Empty;
                rtns.Results.Add(reportItem);
            }
            return rtns;
        }
    }
    #endregion

    #region -G报告手动复核报告
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "手动复核实验数据")]
    public class ReportItemReviewTestG : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("参数为空，接口异常");
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var queryReportItems = conn.From<ReportItem>();
            queryReportItems.Where(o => Sql.In(o.Kvid, ReportItemKvids));
            var reportItems = conn.Select(queryReportItems);
            reportItems.ThrowIfNullOrEmpty("未查询到数据");
            //验证是否为-G报告
            var report = conn.SingleById<Report>(reportItems.FirstOrDefault().ReportKvid);
            report.ThrowIfNull("数据异常！");
            (report.BizType != "G0" && report.BizType != "G1").ThrowIfTrue("只允许复核-G报告的数据！");
            var validReportItems = new List<ReportItem>();
            validReportItems.AddRange(reportItems);
            //validReportItems.PopulateWith(reportItems);
            validReportItems.Exists(o => o.StatusType == ReportItemStatusType.TestCollected).ThrowIfTrue("存在归档项目，无法复核实验数据");
            validReportItems.Exists(o => o.Status < 200).ThrowIfTrue("存在非实验状态的数据，无法复核实验数据");
            var reportItemKvids = reportItems.Select(o => o.Kvid);
            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            var updateReportItemLamda = conn.From<ReportItem>();
            updateReportItemLamda = updateReportItemLamda.Update(o=>o.Status);
            updateReportItemLamda = updateReportItemLamda.Where(o => Sql.In(o.Kvid, reportItemKvids));
            conn.UpdateOnly<ReportItem>(new ReportItem
            {
                Status = 300
            }, updateReportItemLamda);

            //手动组织返回值
            foreach (var reportItem in reportItems)
            {
                reportItem.Status = 300;
                rtns.Results.Add(reportItem);
            }
            return rtns;
        }
    }

    #endregion

    #region 项目合格率统计
    public class ReportItemJudgementPivot : RestfulExecution<ReportItem>
    {
        public DateTime? DealDateFrom { get; set; }
        public DateTime? DealDateTo { get; set; }
        public string Judgement { get; set; } = "不合格";
        public string Title { get; set; }
        #region QueryArgs
        public virtual int? Skip { get; set; }

        public virtual int? Take { get; set; }

        public virtual string OrderBy { get; set; }

        public string OrderByDesc { get; set; }

        public virtual string Include { get; set; }

        public virtual string Fields { get; set; }

        public string QueryKeys { get; set; }

        public string QueryValues { get; set; }

        #endregion
        public override object OnExecution(IRequest req, IResponse res)
        {
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var dynamicParams = Request.GetRequestParams();
            var autoQuery = Request.TryResolve<IAutoQueryDb>();
            autoQuery.IncludeTotal = true;
            var request = new RestfulQuery<ReportItemEx>();
            request.PopulateWith(this);
            var sqlExpress = autoQuery.CreateQuery(Request, conn, request, dynamicParams);
            sqlExpress
                .LeftJoin<Report>((reportItemEx, report) => reportItemEx.ReportKvid == report.Kvid)
                .Select<ReportItemEx, Report>((reportItemEx, report) => new
                {
                    T = Sql.AllFields(reportItemEx),
                    report.ReportId,
                    ReportSampleName = report.SampleName,
                    ReportDemanderName = report.DemanderName,
                    ReportSampleManufacturerName = report.SampleManufacturerName,
                    ReportSampleBatchNumber = report.SampleBatchNumber,
                    ReportTestStandards = report.TestStandards,
                    ReportRemark = report.Remark
                    //ReportAttachDocument = report.Metadata.ContainsKey("DescriptionSpecial") ? report.Metadata["DescriptionSpecial"] : "",
                });
            sqlExpress.And<Report>(o => o.TypeEx == "W" || o.TypeEx == "Z");
            if (DealDateFrom != null)
            {
                sqlExpress.And<Report>(o => o.DealDate >= DealDateFrom && o.Status == int.MaxValue);
            }
            if (DealDateTo != null)
            {
                sqlExpress.And<Report>(o => o.DealDate <= DealDateTo && o.Status == int.MaxValue);
            }
            sqlExpress.And<ReportItemEx>(o => (o.Judgement == Judgement) && o.Status == int.MaxValue);
            //sqlExpress.OrderBy(o => o.ReportKvid);
            var rtns = autoQuery.Execute(Request, conn, request, sqlExpress);
            //如果是搜索特定项目，则计算合格率
            //if (!Title.IsNullOrEmpty())
            //{
            //    rtns.Meta = new Dictionary<string, string>();
            //    rtns.Meta["不合格项目数"] = conn.Count<ReportItem>(o => o.Title.Contains(Title) && o.Status == int.MaxValue).ToString();
            //    rtns.Meta["总项目数"] = "";
            //    rtns.Meta["不合格率"] = "";
            //    rtns.Meta["合格率"] = "";
            //}
            return rtns;
        }
    }
    #endregion

    #region 批量修改标准值
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "批量修改标准值")]
    public class ReportItemUpdateDetectionValueQualifiedMulti : RestfulExecution<ReportItem>
    {
        public Report Report { get; set; }
        public List<ReportItem> ReportItems { get; set; }
        public string DetectionValueQualified { get; set; }
        public bool IsFilterByReport { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            (ReportItems.Count == 0).ThrowIfTrue("未传入检验项目集合!");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            //设置状态范围
            var statusScope=new List<int>();
            if (IsFilterByReport == true)
            {
                (Report == null).ThrowIfTrue("未传入报告信息!");
                if (Report.StatusType == ReportStatusType.CommissionAccept) statusScope.Add((int)ReportItemStatusType.BeforeTest);
            }
            else {
                //存在报告状态为委托受理的情况,但项目已经到达录入
                //该情况仅允许修改状态值为200~300之间的数据
                statusScope.Add((int)ReportItemStatusType.Testing);
                statusScope.Add((int)ReportItemStatusType.BeforeReview);
                statusScope.Add((int)ReportItemStatusType.TestFinished);
            }
            if (statusScope.Count == 0) {
                rtns.Total = rtns.Results.Count;
                return rtns;
            }
            //1.更新标准值
            foreach (var reportItem in ReportItems)
            {
                //非可修改状态范围内的数据禁止修改
                if (!statusScope.Exists(o => o == reportItem.Status)) continue;
                //标准值未发生变化的项目无需操作
                if (reportItem.DetectionValueQualified == DetectionValueQualified) continue;
                var oldValue = reportItem.DetectionValueQualified;
                reportItem.RemoveAllOnlyProperties();
                reportItem.DetectionValueQualified = DetectionValueQualified;
                reportItem.AddOnlyProperties(o=>o.DetectionValueQualified);
                conn.UpdateOnly(reportItem);
                reportItem.RemoveAllOnlyProperties();
                rtns.Results.Add(reportItem);
                //2.创建标准值修改日志
                reportItem.SetDetectionValueQualifiedLog(oldValue, DetectionValueQualified,conn);
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

    #region 批量修改标准值
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration)]
    [Api(Description = "批量修改标准值2")]
    public class ReportItemUpdateDetectionValueQualifiedMulti2 : RestfulExecution<ReportItem>
    {
        public Report Report { get; set; }
        public List<ReportItem> ReportItems { get; set; }
        public string DetectionValueQualified { get; set; }
        public bool IsFilterByReport { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            (ReportItems.Count == 0).ThrowIfTrue("未传入检验项目集合!");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            //设置状态范围
            var statusScope = new List<int>();
            if (IsFilterByReport == true)
            {
                (Report == null).ThrowIfTrue("未传入报告信息!");
                if (Report.StatusType == ReportStatusType.CommissionAccept) statusScope.Add((int)ReportItemStatusType.BeforeTest);
            }
            else
            {
                //存在报告状态为委托受理的情况,但项目已经到达录入
                //该情况仅允许修改状态值为200~300之间的数据
                statusScope.Add((int)ReportItemStatusType.Testing);
                statusScope.Add((int)ReportItemStatusType.BeforeReview);
                statusScope.Add((int)ReportItemStatusType.TestFinished);
            }
            if (statusScope.Count == 0)
            {
                rtns.Total = rtns.Results.Count;
                return rtns;
            }

            List<ReportItem> updateReportItems = new List<ReportItem>();
            //1.更新标准值
            foreach (var reportItem in ReportItems)
            {
                //非可修改状态范围内的数据禁止修改
                if (!statusScope.Exists(o => o == reportItem.Status)) continue;
                //标准值未发生变化的项目无需操作
                if (reportItem.DetectionValueQualified == DetectionValueQualified) continue;
                var oldValue = reportItem.DetectionValueQualified;
                reportItem.RemoveAllOnlyProperties();
                reportItem.DetectionValueQualified = DetectionValueQualified;
                reportItem.AddOnlyProperties(o => o.DetectionValueQualified);
                //2.创建标准值修改日志
                reportItem.SetDetectionValueQualifiedLog(oldValue, DetectionValueQualified, conn);

                //经过计算，若完成判定则更新判定结果
                if (reportItem.CalculateJudgement2(Report))
                {
                    reportItem.AddOnlyProperties(o => o.Judgement);
                }
                updateReportItems.Add(reportItem);

                //判断是否需要更新大项
                var parentReportItem = conn.Single<ReportItem>(o => o.Kvid == reportItem.ParentKvid && o.Type == ReportItemType.TestFolder);
                if (parentReportItem != null)
                {
                    //大项是TestFolder 判定的话 需要把大项下面的子项传给判定方法
                    //allNodes是此大项的全部子项  nodes是待处理的子项 两者合并后传个方法进行大项判定
                    var allNodes = conn.Select<ReportItem>(o => o.ParentKvid == reportItem.ParentKvid && o.Type == ReportItemType.Test);
                    //从服务器获取的子项的判定结果还未更新，需要手动赋值
                    foreach (var node in allNodes)
                    {
                        if (node.Kvid == reportItem.Kvid) node.Judgement = reportItem.Judgement;
                    }
                    //var nodes = updateReportItems.Where(o => o.ParentKvid == parentReportItem.Kvid).ToList();
                    //foreach (var all in allNodes)
                    //{
                    //    if (nodes.Exists(o => o.Kvid == all.Kvid)) continue;
                    //    nodes.Add(all);
                    //}

                    if (parentReportItem.CalculateJudgement2(Report, allNodes))
                    {
                        parentReportItem.StatusType = ReportItemStatusType.BeforeReview;
                        parentReportItem.AddOnlyProperties(o => o.Status);
                        parentReportItem.AddOnlyProperties(o => o.Judgement);
                        updateReportItems.Add(parentReportItem);
                    }
                }

            }
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                foreach (var en in updateReportItems)
                {
                    conn.UpdateOnly(en);
                    en.RemoveAllOnlyProperties();
                    rtns.Results.Add(en);
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            return rtns;
        }
    }
    #endregion

    #region 待删除，增加色号、部位接口备份
    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "检测项生成部位")]
    public class ReportItemCreatePart2 : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }

        public List<string> PartValues { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("请传入检验项目!");
            PartValues.RemoveAll(o => o.IsNullOrEmpty());
            PartValues.ThrowIfNullOrEmpty("请传入要生成的部位信息!");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            //所选项目 SelectByIds此方法不过滤status<0的数据
            var selectedReportItems = conn.SelectByIds<ReportItem>(ReportItemKvids);
            selectedReportItems.ThrowIfNullOrEmpty("未找到所选检验项目!");

            var reportKvids = selectedReportItems.ConvertAll(o => o.ReportKvid).Distinct().ToList();
            (reportKvids.Count != 1).ThrowIfTrue("所选检验项不在同一报告下!");
            var report = conn.SingleById<Report>(reportKvids.FirstOrDefault());
            report.ThrowIfNull("未找到项目对应报告!");
            (report.StatusType != ReportStatusType.CommissionAccept).ThrowIfTrue($"当前报告{report.ReportId},不在委托受理状态,无法生成部位!");

            //整个报告的全部项目
            var allReportItems = conn.Select<ReportItem>(o => o.ReportKvid == report.Kvid);

            //全项中的大项
            var allMainReportItems = allReportItems.filterReportItem();
            allMainReportItems.ThrowIfNullOrEmpty("此报告全部检验项目未找到检验大项,结构异常,请联系管理员!");

            //传入项的大项
            var selectedMainReportItems = selectedReportItems.Where(o => Sql.In(o.Kvid, allMainReportItems.ConvertAll(p => p.Kvid))).ToList();
            selectedMainReportItems.ThrowIfNullOrEmpty("请选择检验大项进行生成部位!");

            //查看是否已经存在色号
            bool existSelf = allReportItems.Exists(o => o.Type == ReportItemType.SampleSelf);

            var rtns = new RestfulCreateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            var insertReportItems = new List<ReportItem>();
            var updateReportItems = new List<ReportItem>();

            //存在色号 则生成的部位全部需要先查到对应色号信息,要是没有找到则说明数据问题 直接跳过
            if (existSelf)
            {
                var groupRoot = selectedMainReportItems.GroupBy(o => o.RootKvid);

                foreach (var kv in groupRoot)
                {
                    foreach (var part in PartValues)
                    {
                        var selfParent = allReportItems.FirstOrDefault(o => o.Kvid == kv.Key);
                        if (selfParent == null) continue;

                        int index = PartValues.IndexOf(part);

                        var partReportItem = new ReportItem();
                        partReportItem.Kvid = Guid.NewGuid();
                        partReportItem.RootKvid = selfParent.RootKvid;
                        partReportItem.ParentKvid = selfParent.Kvid;
                        partReportItem.ReportKvid = report.Kvid;
                        partReportItem.Type = ReportItemType.SamplePart;
                        partReportItem.Title = part;
                        partReportItem.SampleKvid = selfParent.SampleKvid;
                        partReportItem.SampleRootKvid = selfParent.SampleRootKvid;
                        partReportItem.SortIdEx = selectedMainReportItems[0].SortIdEx + index;
                        partReportItem.SortId = selectedMainReportItems[0].SortId;
                        partReportItem.StatusType = ReportItemStatusType.BeforeTest;
                        insertReportItems.Add(partReportItem);
                        rtns.Results.Add(partReportItem);

                        foreach (var primary in kv)
                        {
                            var partParent = allReportItems.FirstOrDefault(o => o.Kvid == primary.ParentKvid && o.Type == ReportItemType.SamplePart);

                            var newNodePrimary = new ReportItem();
                            newNodePrimary.PopulateWith(primary);
                            newNodePrimary.Kvid = Guid.NewGuid();
                            newNodePrimary.RootKvid = partReportItem.RootKvid;
                            newNodePrimary.ParentKvid = partReportItem.Kvid;
                            newNodePrimary.ReportKvid = report.Kvid;
                            newNodePrimary.SampleKvid = partReportItem.SampleKvid;
                            newNodePrimary.SampleRootKvid = partReportItem.SampleRootKvid;
                            newNodePrimary.SortIdEx += index;
                            newNodePrimary.Status = 100;

                            //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                            newNodePrimary.TestKvid = Guid.Empty;
                            newNodePrimary.TestValue = string.Empty;
                            newNodePrimary.TestOriginalFilePath = string.Empty;
                            newNodePrimary.TestCreatorName = string.Empty;
                            newNodePrimary.TestCreatorKvid = Guid.Empty;
                            newNodePrimary.TestCreateTime = null;
                            newNodePrimary.TestReviewerKvid = Guid.Empty;
                            newNodePrimary.TestReviewerName = string.Empty;
                            newNodePrimary.TestReviewTime = null;
                            newNodePrimary.TestRootKvid = Guid.Empty;

                            insertReportItems.Add(newNodePrimary);
                            rtns.Results.Add(newNodePrimary);

                            //没有找到父项 则说明原项目没有部位  要在生成部位后删除原项目
                            if (partParent == null)
                            {
                                selectedReportItems.Exists(o => o.Status != (int)ReportItemStatusType.BeforeTest && o.Status > -1).ThrowIfTrue($"当前操作的项目,不在委托受理状态,无法生成部位!");
                                //if (selectedMainReportItems.Exists(o => o.TestKvid != Guid.Empty)) throw new Exception("存在项目有检测记录，无法生成部位！");
                                primary.Status = -1;
                                primary.AddOnlyProperties(o => o.Status);
                                updateReportItems.Add(primary);
                            }

                            var nodes = allReportItems.Where(o => o.ParentKvid == primary.Kvid).ToList();
                            foreach (var node in nodes)
                            {
                                var newNode = new ReportItem();
                                newNode.PopulateWith(node);
                                newNode.Kvid = Guid.NewGuid();
                                newNode.RootKvid = newNodePrimary.RootKvid;
                                newNode.ParentKvid = newNodePrimary.Kvid;
                                newNode.ReportKvid = report.Kvid;
                                newNode.SampleKvid = newNodePrimary.SampleKvid;
                                newNode.SampleRootKvid = newNodePrimary.SampleRootKvid;
                                newNode.SortIdEx += index;
                                newNode.Status = 100;

                                //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                                newNode.TestKvid = Guid.Empty;
                                newNode.TestValue = string.Empty;
                                newNode.TestOriginalFilePath = string.Empty;
                                newNode.TestCreatorName = string.Empty;
                                newNode.TestCreatorKvid = Guid.Empty;
                                newNode.TestCreateTime = null;
                                newNode.TestReviewerKvid = Guid.Empty;
                                newNode.TestReviewerName = string.Empty;
                                newNode.TestReviewTime = null;
                                newNode.TestRootKvid = Guid.Empty;

                                insertReportItems.Add(newNode);
                                rtns.Results.Add(newNode);

                                //没有找到父项 则说明原项目没有部位  要在生成部位后删除原项目
                                if (partParent == null)
                                {
                                    selectedReportItems.Exists(o => o.Status != (int)ReportItemStatusType.BeforeTest && o.Status > -1).ThrowIfTrue($"当前操作的项目,不在委托受理状态,无法生成部位!");
                                    node.Status = -1;
                                    node.AddOnlyProperties(o => o.Status);
                                    updateReportItems.Add(node);
                                }
                            }
                        }
                    }
                }
            }
            //没有存在色号 则生成的部位就是当前项目的根项目
            else
            {
                foreach (var part in PartValues)
                {
                    int index = PartValues.IndexOf(part);
                    var partReportItem = new ReportItem();
                    partReportItem.Kvid = Guid.NewGuid();
                    partReportItem.RootKvid = partReportItem.Kvid;
                    partReportItem.ParentKvid = Guid.Empty;
                    partReportItem.ReportKvid = report.Kvid;
                    partReportItem.Type = ReportItemType.SamplePart;
                    partReportItem.Title = part;
                    partReportItem.SampleKvid = selectedMainReportItems[0].SampleKvid;
                    partReportItem.SampleRootKvid = selectedMainReportItems[0].SampleRootKvid;
                    partReportItem.SortIdEx = selectedMainReportItems[0].SortIdEx + index;
                    partReportItem.SortId = selectedMainReportItems[0].SortId;
                    partReportItem.StatusType = ReportItemStatusType.BeforeTest;
                    insertReportItems.Add(partReportItem);
                    rtns.Results.Add(partReportItem);

                    foreach (var primary in selectedMainReportItems)
                    {
                        var partParent = allReportItems.FirstOrDefault(o => o.Kvid == primary.ParentKvid && o.Type == ReportItemType.SamplePart);

                        #region 新建部位ReportItem及检验项目
                        var newNodePrimary = new ReportItem();
                        newNodePrimary.PopulateWith(primary);
                        newNodePrimary.Kvid = Guid.NewGuid();
                        newNodePrimary.RootKvid = partReportItem.RootKvid;
                        newNodePrimary.ParentKvid = partReportItem.Kvid;
                        newNodePrimary.ReportKvid = report.Kvid;
                        newNodePrimary.SampleKvid = partReportItem.SampleKvid;
                        newNodePrimary.SampleRootKvid = partReportItem.SampleRootKvid;
                        newNodePrimary.SortIdEx += index;
                        newNodePrimary.Status = 100;

                        //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                        newNodePrimary.TestKvid = Guid.Empty;
                        newNodePrimary.TestValue = string.Empty;
                        newNodePrimary.TestOriginalFilePath = string.Empty;
                        newNodePrimary.TestCreatorName = string.Empty;
                        newNodePrimary.TestCreatorKvid = Guid.Empty;
                        newNodePrimary.TestCreateTime = null;
                        newNodePrimary.TestReviewerKvid = Guid.Empty;
                        newNodePrimary.TestReviewerName = string.Empty;
                        newNodePrimary.TestReviewTime = null;
                        newNodePrimary.TestRootKvid = Guid.Empty;

                        insertReportItems.Add(newNodePrimary);
                        rtns.Results.Add(newNodePrimary);

                        //没有找到父项 则说明原项目没有部位  要在生成部位后删除原项目
                        if (partParent == null)
                        {
                            //生成多部位时，只需要验证第一次的status为受理状态
                            selectedReportItems.Exists(o => o.Status != (int)ReportItemStatusType.BeforeTest && o.Status > -1).ThrowIfTrue($"当前操作的项目,不在委托受理状态,无法生成部位!");
                            //if (selectedMainReportItems.Exists(o => o.TestKvid != Guid.Empty)) throw new Exception("存在项目有检测记录，无法生成部位！");
                            primary.Status = -1;
                            primary.AddOnlyProperties(o => o.Status);
                            updateReportItems.Add(primary);
                        }

                        var nodes = allReportItems.Where(o => o.ParentKvid == primary.Kvid).ToList();
                        foreach (var node in nodes)
                        {
                            var newNode = new ReportItem();
                            newNode.PopulateWith(node);
                            newNode.Kvid = Guid.NewGuid();
                            newNode.RootKvid = newNodePrimary.RootKvid;
                            newNode.ParentKvid = newNodePrimary.Kvid;
                            newNode.ReportKvid = report.Kvid;
                            newNode.SampleKvid = newNodePrimary.SampleKvid;
                            newNode.SampleRootKvid = newNodePrimary.SampleRootKvid;
                            newNode.SortIdEx += index;
                            newNode.Status = 100;

                            //2021-10-10接口升级支归档报告增加部位（前提是原来分过部位）,需要重置检测相关信息
                            newNodePrimary.TestKvid = Guid.Empty;
                            newNodePrimary.TestValue = string.Empty;
                            newNodePrimary.TestOriginalFilePath = string.Empty;
                            newNodePrimary.TestCreatorName = string.Empty;
                            newNodePrimary.TestCreatorKvid = Guid.Empty;
                            newNodePrimary.TestCreateTime = null;
                            newNodePrimary.TestReviewerKvid = Guid.Empty;
                            newNodePrimary.TestReviewerName = string.Empty;
                            newNodePrimary.TestReviewTime = null;
                            newNodePrimary.TestRootKvid = Guid.Empty;

                            insertReportItems.Add(newNode);
                            rtns.Results.Add(newNode);

                            //没有找到父项 则说明原项目没有部位  要在生成部位后删除原项目
                            if (partParent == null)
                            {
                                selectedReportItems.Exists(o => o.Status != (int)ReportItemStatusType.BeforeTest && o.Status > -1).ThrowIfTrue($"当前操作的项目,不在委托受理状态,无法生成部位!");
                                node.Status = -1;
                                node.AddOnlyProperties(o => o.Status);
                                updateReportItems.Add(node);
                            }
                        }
                        #endregion
                    }
                }
            }
            //开启事务，执行sql
            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);
            try
            {
                foreach (var en in insertReportItems)
                {
                    conn.Insert(en);
                }
                foreach (var en in updateReportItems)
                {
                    conn.UpdateOnly(en);
                }
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    [RequiresAnyRole(SystemRoles.Everyone)]
    [Api(Description = "检测项更新色号")]
    public class ReportItemUpdateSelf2 : RestfulExecution<ReportItem>
    {
        public Guid ReportItemKvid { get; set; }

        public string SelfValue { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvid.ThrowIfEmpty("请传入要更新的色号信息!");
            SelfValue.ThrowIfNullOrEmpty("色号信息不能为空!");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var reportItem = conn.SingleById<ReportItem>(ReportItemKvid);
            reportItem.ThrowIfNull("未找到所选色号信息!");
            (reportItem.Type != ReportItemType.SampleSelf).ThrowIfTrue("所选项不为色号项目,请选择正确色号项!");
            (reportItem.Status != (int)ReportItemStatusType.BeforeTest).ThrowIfTrue($"当前报告不在委托受理状态,无法更新色号!");
            (reportItem.StatusType != ReportItemStatusType.BeforeTest).ThrowIfTrue($"当前报告不在委托受理状态,无法更新色号!");

            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            if (reportItem.Title == SelfValue)
            {
                rtns.Results.Add(reportItem);
                return rtns;
            }

            var sample = conn.SingleById<Sample>(reportItem.SampleKvid);
            sample.ThrowIfNull("未找到对应样品信息!");

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);

            try
            {
                #region 更新ReportItem和Sample
                reportItem.Title = SelfValue;
                reportItem.AddOnlyProperties(o => o.Title);
                conn.UpdateOnly(reportItem);
                rtns.Results.Add(reportItem);

                sample.Specification = SelfValue;
                sample.AddOnlyProperties(o => o.Specification);
                conn.UpdateOnly(sample);
                #endregion
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }

    #region 更新一个项目的部位，全部更新(暂停使用)
    [RequiresAnyRole(SystemRoles.Administrator)]
    [Api(Description = "检测项更新部位V1.0")]
    public class ReportItemUpdatePartEx : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }

        public string PartValue { get; set; }

        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNullOrEmpty("请传入检验项目!");
            PartValue.ThrowIfNullOrEmpty("部位信息不能为空!");

            //去重
            ReportItemKvids = ReportItemKvids.Distinct().ToList();

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();

            var reportItems = conn.SelectByIds<ReportItem>(ReportItemKvids);
            reportItems.ThrowIfNullOrEmpty("未找到所选部位项信息!");

            var reportKvids = reportItems.ConvertAll(o => o.ReportKvid).Distinct().ToList();
            (reportKvids.Count != 1).ThrowIfTrue("所选检验项不在同一报告下!");

            reportItems.Exists(o => o.Type != ReportItemType.SamplePart).ThrowIfTrue("所选项不是部位信息!");
            reportItems.Exists(o => o.Status != (int)ReportItemStatusType.BeforeTest).ThrowIfTrue($"当前报告不在委托受理状态,无法更新部位!");
            reportItems.Exists(o => o.StatusType != ReportItemStatusType.BeforeTest).ThrowIfTrue($"当前报告不在委托受理状态,无法更新部位!");

            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);

            try
            {
                foreach (var item in reportItems)
                {
                    if (item.Title == PartValue)
                    {
                        rtns.Results.Add(item);
                        continue;
                    }
                    item.Title = PartValue;
                    item.AddOnlyProperties(o => o.Title);
                    conn.UpdateOnly(item);
                }

                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
    #endregion

    #region <作废>检验项目重置，重录
    public class ReportItemReset : RestfulExecution<ReportItem>
    {
        public List<Guid> ReportItemKvids { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            ReportItemKvids.ThrowIfNull("请传入要重录的项目ReportItemKvids！");

            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var reportItems = conn.SelectByIds<ReportItem>(ReportItemKvids);
            reportItems.ThrowIfNullOrEmpty("找不到检测项！");
            //验证项目状态，归档项目不允许重录
            if (reportItems.Exists(o => o.StatusType == ReportItemStatusType.TestCollected)) throw new Exception("项目已归档，无法重录！");

            //项目中的大项
            //此处要主要 传过来的ReportItemKvids要是完整的树形结构
            //不然filterReportItem()可能回传不准确 因为此方法是根据传入的列表进行筛选 不去数据库中查询完整结构数据
            var primaryReportItems = reportItems.filterReportItem();

            primaryReportItems.ThrowIfNull("请选择对应主项进行重录！");

            var allNodeItems = conn.Select<ReportItem>(o => o.Type == ReportItemType.Test && Sql.In(o.ParentKvid, primaryReportItems.ConvertAll(p => p.Kvid)));

            var rtns = new RestfulUpdateResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();

            IDbTransaction trans = null;
            trans = conn.OpenTransaction(IsolationLevel.ReadUncommitted);

            try
            {
                #region 只针对大项处理
                foreach (var item in primaryReportItems)
                {
                    //大项不为这两个类型就跳过
                    if (item.Type != ReportItemType.TestFolder && item.Type != ReportItemType.Test) continue;
                    item.TestValue = string.Empty;
                    item.AddOnlyProperties(o => o.TestValue);
                    item.TestOriginalFilePath = string.Empty;
                    item.AddOnlyProperties(o => o.TestOriginalFilePath);
                    item.TestCreateTime = null;
                    item.AddOnlyProperties(o => o.TestCreateTime);
                    item.TestCreatorKvid = Guid.Empty;
                    item.AddOnlyProperties(o => o.TestCreatorKvid);
                    item.TestCreatorName = string.Empty;
                    item.AddOnlyProperties(o => o.TestCreatorName);
                    item.TestReviewTime = null;
                    item.AddOnlyProperties(o => o.TestReviewTime);
                    item.TestReviewerKvid = Guid.Empty;
                    item.AddOnlyProperties(o => o.TestReviewerKvid);
                    item.TestReviewerName = string.Empty;
                    item.AddOnlyProperties(o => o.TestReviewerName);
                    item.TestKvid = Guid.Empty;
                    item.AddOnlyProperties(o => o.TestKvid);
                    item.StatusType = ReportItemStatusType.Testing;
                    item.AddOnlyProperties(o => o.Status);
                    item.Judgement = string.Empty;
                    item.AddOnlyProperties(o => o.Judgement);
                    conn.UpdateOnly(item);

                    rtns.Results.Add(item);

                    //未找到子项信息直接跳过
                    if (allNodeItems.IsNullOrEmpty()) continue;

                    var nodes = allNodeItems.Where(o => o.ParentKvid == item.Kvid).ToList();
                    if (nodes.IsNullOrEmpty()) continue;
                    foreach (var node in nodes)
                    {
                        if (node.Type != ReportItemType.Test) continue;
                        node.TestValue = string.Empty;
                        node.AddOnlyProperties(o => o.TestValue);
                        node.TestOriginalFilePath = string.Empty;
                        node.AddOnlyProperties(o => o.TestOriginalFilePath);
                        node.TestCreateTime = null;
                        node.AddOnlyProperties(o => o.TestCreateTime);
                        node.TestCreatorKvid = Guid.Empty;
                        node.AddOnlyProperties(o => o.TestCreatorKvid);
                        node.TestCreatorName = string.Empty;
                        node.AddOnlyProperties(o => o.TestCreatorName);
                        node.TestReviewTime = null;
                        node.AddOnlyProperties(o => o.TestReviewTime);
                        node.TestReviewerKvid = Guid.Empty;
                        node.AddOnlyProperties(o => o.TestReviewerKvid);
                        node.TestReviewerName = string.Empty;
                        node.AddOnlyProperties(o => o.TestReviewerName);
                        node.TestKvid = Guid.Empty;
                        node.AddOnlyProperties(o => o.TestKvid);
                        node.StatusType = ReportItemStatusType.Testing;
                        node.AddOnlyProperties(o => o.Status);
                        node.Judgement = string.Empty;
                        node.AddOnlyProperties(o => o.Judgement);
                        conn.UpdateOnly(node);

                        rtns.Results.Add(node);
                    }

                }
                #endregion
                trans?.Commit();
            }
            catch (Exception ex)
            {
                trans?.Rollback();
                throw ex;
            }
            return rtns;
        }
    }
    #endregion

    #endregion

    #region 多行设置
    /// <summary>
    /// 多行设置
    /// </summary>
    [RequiresAnyRole(MemberRoles.Administrator, MemberRoles.Manager, MemberRoles.Configuration, LimsRoles.Manager, LimsRoles.Inspector)]
    [Api(Description = "多行设置")]
    public class ReportItemUpdateMulti : RestfulExecution<ReportItem>
    {
        //修改的实体集合
        public List<ReportItem> Items { get; set; }
        //修改属性：项目名称，单位等
        public string PropertyName { get; set; }
        //修改内容，允许为空，存在置空操作
        public string Content { get; set; }
        public override object OnExecution(IRequest req, IResponse res)
        {
            #region 数据验证
            (Items.Count == 0).ThrowIfTrue("未传入待更新数据集合！");
            PropertyName.ThrowIfNullOrEmpty("未传入更新属性！");
            #endregion
            var conn = KiviiContext.GetOpenedDbConnection<ReportItem>();
            var report = conn.SingleById<Report>(Items[0].ReportKvid);
            if (report == null) throw new Exception("未获取报告信息");
            //获取所有检验项目集合
            var reportItemsAll = conn.Select<ReportItem>(o => o.ReportKvid == Items[0].ReportKvid);
            var rtns = new RestfulQueryResponse<ReportItem>();
            rtns.Results = new List<ReportItem>();
            foreach (var item in Items)
            {
                //获取更新标准值前项目信息
                var reportItemAll = reportItemsAll.FirstOrDefault(o => o.Kvid == item.Kvid);
                (reportItemAll == null).ThrowIfTrue("未能捕获检验项目!");
                if (reportItemAll.StatusType == ReportItemStatusType.TestCollected) continue;
                item.RemoveAllOnlyProperties();
                item.GetType().GetProperty(PropertyName).SetValue(item, Content);
                item.AddOnlyProperties(new string[] { PropertyName });
                conn.UpdateOnly(item);
                item.RemoveAllOnlyProperties();
                rtns.Results.Add(item);
                //更新项目名称时需要创建日志
                if (PropertyName == "Title")
                {
                    //记录日志
                    var reportItemTitle = reportItemAll.Title;
                    //获取溯源信息
                    if (reportItemAll.ParentKvid != Guid.Empty)
                    {
                        var parent = reportItemsAll.FirstOrDefault(o => o.Kvid == reportItemAll.ParentKvid);
                        if (parent != null)
                        {
                            if (parent.Type == ReportItemType.TestFolder)
                            {
                                reportItemTitle = $"{parent.Title}_{reportItemTitle}";
                                if (parent.ParentKvid != Guid.Empty) parent = reportItemsAll.FirstOrDefault(o => o.Kvid == parent.ParentKvid);
                                if (parent != null)
                                {
                                    if (parent.Type == ReportItemType.SamplePart)
                                    {
                                        reportItemTitle = $"{reportItemTitle}({parent.Title})";
                                        if (parent.ParentKvid != Guid.Empty) parent = reportItemsAll.FirstOrDefault(o => o.Kvid == parent.ParentKvid);
                                        if (parent != null && parent.Type == ReportItemType.SampleSelf) reportItemTitle = $"{reportItemTitle}[{parent.Title}]";
                                    }
                                    else if (parent.Type == ReportItemType.SampleSelf)
                                    {
                                        reportItemTitle = $"{reportItemTitle}[{parent.Title}]";
                                    }
                                }
                            }
                            else if (parent.Type == ReportItemType.SamplePart)
                            {
                                reportItemTitle = $"{reportItemTitle}({parent.Title})";
                                if (parent.ParentKvid != Guid.Empty) parent = reportItemsAll.FirstOrDefault(o => o.Kvid == parent.ParentKvid);
                                if (parent != null && parent.Type == ReportItemType.SampleSelf) reportItemTitle = $"{reportItemTitle}[{parent.Title}]";
                            }
                            else if (parent.Type == ReportItemType.SampleSelf)
                            {
                                reportItemTitle = $"{reportItemTitle}[{parent.Title}]";
                            }
                        }
                    }
                    var updateContent = $"由“{reportItemAll.Title}” 改为 “{item.Title}” 。";
                    reportItemAll.Title = reportItemTitle.Replace(reportItemAll.Title, item.Title);
                    if (reportItemAll.Title != item.Title) report.CreateLog(reportItemAll, LogType.ReportItemUpdateTitle, updateContent, conn);
                }
            }
            rtns.Total = rtns.Results.Count;
            return rtns;
        }
    }
    #endregion

}
